"use strict";
appServices.service('DBServices', ['Utils', function (Utils) {
  var _DB;
  var dbIsOpenAndLoaded = false;

  function isUsable() {
    return dbIsOpenAndLoaded;
  }

  var _onDBUsable;

  function init() {
    return Utils.$q(function (resolve, reject) {
      _openDB().then(function (data) {
          console.log("OPEN CONNECTION BEFORE CHECK DB", data);
          loadDB().then(function () {
          dbIsOpenAndLoaded = true;
          console.log("OPEN AND LOAD DB");
          setTimeout(function () {
            (_onDBUsable || angular.noop)(true);
          }, 2000);

          //just for debugging...
          /*_showDBTables().then(function (tables) {
            Utils.alert.show("DB Tables"+tables);
          }, function (error) {
            console.log("Can't get DB Tables", error);
            Utils.alert.show("Can't get DB Tables");
          });*/

          resolve(_DB);
        }, function (error) {
            console.log("ERROR LOAD DB", error);
          reject(error)
        })
      }, function (error) {
        reject(error);
      })
    })
  }

  /**
   * @return Promise
   * @private
   */
  function loadDB() {
    return Utils.$q(function (resolve, reject) {
        console.log("BEFORE STATEMENT");
      var sqlStatements = [
        'CREATE TABLE IF NOT EXISTS userSession (userSubsId INTEGER PRIMARY KEY, userSubsInstId INTEGER, userName TEXT, onBoard STRING)',
        'CREATE TABLE IF NOT EXISTS userLogCount (id INTEGER PRIMARY KEY, name TEXT, count INTEGER)',
        'CREATE TABLE IF NOT EXISTS tutorials (id INTEGER PRIMARY KEY, name TEXT, visited TEXT)',
        'CREATE TABLE IF NOT EXISTS chat_contact_list ("id" INTEGER PRIMARY KEY NOT NULL, "name" TEXT(255) NOT NULL, "avatar" TEXT, "description" TEXT, "descrUpdatedAt" TEXT, "status" INTEGER NOT NULL, "lastConnAt" TEXT, "notifType" INTEGER(1), "customNotif" TEXT)',
        'CREATE TABLE IF NOT EXISTS chat_message_list ("id" INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL, "uniqueId" TEXT, "contactId" INTEGER, "groupId" INTEGER, "text" TEXT(500), "fileId" INTEGER, "extraData" TEXT, "createdAt" TEXT, "processedAt" TEXT, "receivedAt" TEXT, "isMine" INTEGER(1), "status" INTEGER(1) NOT NULL)',
        'CREATE TABLE IF NOT EXISTS chat_group_list ("id" INTEGER PRIMARY KEY NOT NULL, "createdAt" TEXT, "name" TEXT, "avatar" TEXT, "description" TEXT, "descrUpdatedAt" TEXT, "status" INTEGER(1), "notifType" INTEGER(1), "customNotif" TEXT)',
        'CREATE TABLE IF NOT EXISTS chat_group_contact_rel ("groupId" INTEGER, "contactId" INTEGER, "role" INTEGER, "status" INTEGER)',
        'CREATE TABLE IF NOT EXISTS chat_file_list ("id" INTEGER PRIMARY KEY NOT NULL, "type" INTEGER, "mimeType" TEXT, "extension" TEXT, "src" TEXT)'
      ];

      batch(sqlStatements).then(function (/*nothing*/) {
        console.log("CONSOLE DB BATCH EXEC");
        /*/
        var date = new Date().toISOString();
        sqlStatements = [
        ];

        batch(sqlStatements).then(function () {
          resolve();
        }, function (error) {
          console.error("Can't add test records to DB", error);
          reject(error);
        });//*/

        resolve();
      }, function (error) {
        console.error("Couldn't initialize DB", error);
        reject(error);
      })
    })
  }

  /**
   * @private
   * @return Promise
   */
  function _openDB() {
    return Utils.$q(function (resolve, reject) {
      try {
        window.sqlitePlugin.openDatabase({
          name: 'personalInfo.db',
          location: 'default',
          androidDatabaseImplementation: 2,
          androidLockWorkaround: 1
        }, function (dbInstance) {
            console.log("OPEN DB SQLite");
            _DB = dbInstance;
            loadDB().then(function () {
              dbIsOpenAndLoaded = true;
              console.log("OPEN AND LOAD DB 2");
              setTimeout(function () {
                (_onDBUsable || angular.noop)(true);
              }, 2000);
              
              resolve();
            }, function (error) {
              console.log("ERROR LOAD DB 2", error);
              reject(error)
            })
          
          //resolve();
        }, function (error) {
          console.error('Open database ERROR: ');
          reject(error)
        })
      } catch (error) {
        console.warn(error.message);
        reject(error);
      }
    })
  }

  /**
   * @param dbResponse
   * @return {Object[]}
   */
  function extractItems(dbResponse) {
    var items = [], i = 0, len = dbResponse.rows.length;

    for (i; i < len; i++) {
      items.push(dbResponse.rows.item(i));
    }

    return items;
  }

  /**
   * @param dbResponse
   * @param rowNumber {number}
   * @return {Object}
   */
  function getItem(dbResponse, rowNumber) {
    return dbResponse.rows.item(rowNumber);
  }

  /**
   * @param dbResponse
   * @param callback {function(Object, number?)}
   */
  function iterate(dbResponse, callback) {
    var i = 0, len = dbResponse.rows.length;

    for (i; i < len; i++)
      callback(dbResponse.rows.item(i), i)
  }

  /**
   * Just for debugging if DB was loaded correctly
   * @private
   * @return Promise {string|*}
   */
  function _showDBTables() {
    console.log("Executing: _showDBTables");
    return Utils.$q(function (resolve, reject) {
      execute("SELECT name FROM sqlite_master WHERE type = 'table'").then(function (resp) {
        var tables = '', i = 0, len = resp.rows.length;

        for (i; i < len; i++) {
          tables += "<br>'" + resp.rows.item(i).name + "'";
        }

        resolve(tables)
      }, function (error) {
        reject(error)
      })
    })
  }

  /**
   * Inserts missing columns in a given table
   * @return {Promise}
   */
  function updateDBSchema() {
    return Utils.$q(function (resolve, reject) {
      execute("PRAGMA table_info('chat_message_list')").then(function (resp) {
        var colNames = [];
        var i;
        for (i =0; i < resp.rows.length; i++) {
          colNames.push(resp.rows.item(i).name);
          //console.log("col: ",resp.rows.item(i));
        }

        var statements = [];
        ['mediaType', 'mediaURL'].forEach(function (colName) {
          if(!colNames.includes(colName))
            statements.push('ALTER TABLE chat_message_list ADD COLUMN '+colName+' TEXT DEFAULT NULL')
        });

        //console.log("Statements ", statements);
        if(statements.length){
          batch(statements).then(function () {
            resolve()
          }, function (error) {
            reject(error)
          })
        }
        else
          resolve()
      }, function (error) {
        reject(error)
      })
    })
  }

  /**
   *
   * @param query
   * @param paramsArray
   * @return {Promise<DBResponseType>}
   */
  function execute(query, paramsArray) {
    return Utils.$q(function (resolve, reject) {
      //if (!_DB) return reject("DB is not available");
      if (!_DB) {
          _openDB().then(function(response){
              execute(query, paramsArray);
          }, function (error) {
              reject("NEW ERROR");
          })
      } else {
          paramsArray = paramsArray || [];
          try {
            _DB.executeSql(query, paramsArray, function (rs) {
              rs.extractItems = function(){
                return extractItems(rs);
              };

              rs.forEachRow = function(cb){
                var i = 0, len = rs.rows.length;

                for (i; i < len; i++) {
                  cb(rs.rows.item(i), i)
                }
              };

              resolve(rs)
            }, function (error) {
              console.error(error);
              reject(error)
            })
          }
          catch (e) {
            reject(e.message);
          }
      }
      

      
    })
  }

  /**
   * @param {Array} sqlStatements
   * @return Promise
   */
  function batch(sqlStatements) {
    return Utils.$q(function (resolve, reject) {
      _DB.sqlBatch(sqlStatements, function (/*nothing*/) {
        resolve()
      }, function (error) {
        reject(error)
      });
    })
  }

  function whenAvailable(fn) {
    if(typeof fn === 'function'){
        if(isUsable()) {
          fn();
        }
        else{
         //_onDBUsable = fn;
         fn();
        }
    }
  }

  return {
    isUsable: isUsable,
    whenAvailable: whenAvailable,
    init: init,
    loadDB: loadDB,
    execute: execute,
    batch: batch,
    extractItems: extractItems,
    getItem: getItem,
    iterate: iterate,
    _openDB: _openDB
  }

}]);

/**
 * @typedef {Object} DBResponseType
 * @property {{length:number, item?:*}} rows
 * @property {number} rowsAffected
 * @property {number} insertId
 * @property {function():*} extractItems
 * @property {function(item, idx?)} forEachRow
 */

/**
 * @typedef {Object} DBRosterRowType
 * @property {number} contactId
 * @property {string} jid
 * @property {string} name
 * @property {string} avatar
 * @property {number} status
 * @property {number} lastMsgId
 */

/**
 * @typedef {Object} DBMessageRowType
 * @property {number} id
 * @property {string} msgId
 * @property {string} text
 * @property {string} mediaType
 * @property {string} mediaURL
 * @property {number} contactId
 * @property {string} sentDate
 * @property {string} receivedDate
 * @property {boolean|'true'|'false'} isMine
 * @property {number} status
 */
