"use strict";

appServices
  .service('notificationsService', ['$rootScope', '$resource', 'ApiEndPoint', 'UserService', 'TagsService', 'AWSServices', 'Utils', 's3',
    function ($rootScope, $resource, ApiEndPoint, UserService, TagsService, AWSServices, Utils, s3) {
      /**
       * @type BadgesType
       */
      var badges = {unreadRegularNotifications: 0, unreadFollowersNotifications: 0};

      var defaultPaginationValues = {start: 0, count: 7};

      var paginator = {
        regular: {
          pagination: {start: defaultPaginationValues.start, count: defaultPaginationValues.count},
          reset: function () {
            paginator.regular.pagination.start = defaultPaginationValues.start;
            paginator.regular.pagination.count = defaultPaginationValues.count;
          }
        },
        followers: {
          pagination: {start: defaultPaginationValues.start, count: defaultPaginationValues.count},
          reset: function () {
            paginator.followers.pagination.start = defaultPaginationValues.start;
            paginator.followers.pagination.count = defaultPaginationValues.count;
          }
        }
      };

      var notifications = {regular: [], followers: []};

      /**
       * @param data {{sequenceId?:(number|string), userInstanceId:(number|string), groupId:(number|string)?}}
       * @return {*}
       */
      function canOpenNotificationWS(data) {
        return $resource(
          //for old versions:
          //ApiEndPoint.url + 'restful/services/SequenceGroupRepository/actions/findNotificationPermit/invoke',
          //for latest version:
          ApiEndPoint.url + 'restful/services/SequenceGroupRepository/actions/findGroupPermit/invoke',
          {}, {
            get: {
              method: 'GET',
              headers: {'Accept': 'application/json;profile=urn:org.apache.isis/v1'}
            }
          }).get({
          "x-isis-querystring": {
            //sequenceId: {value: data.sequenceId},//used with findNotificationPermit
            groupId: {value: (data.groupId || 0).toString()},//<- used with findGroupPermit
            userSubscription: {value: {href: ApiEndPoint.url + "restful/objects/simple.UserSubscription/" + data.userInstanceId}}
          }
        });
      }


      /**
       * function that returns only the pending notifications (everything except followers) count
       * @param userHref {string}
       * @return {Promise}
       */
      function countPendingRegularNotifications(userHref) {
        //console.log("counting Pending Regular Notifications");
        return Utils.$q(function (resolve, reject) {
          $resource(ApiEndPoint.url + 'restful/services/UserNotificationRepository/actions/countPendingNotifications/invoke', {}, {
            get: {
              method: 'GET',
              headers: {'Accept': 'application/json;profile=urn:org.apache.isis/v1'}
            }
          }).get({
            "x-isis-querystring": {userSubscription: {value: {href: userHref}}}
          }).$promise.then(function (data) {
            if (data.result.value) {
              console.log(data.result.value + " unread regular notifications found");
              badges.unreadRegularNotifications = data.result.value;
              $rootScope.$broadcast("notifications.new", {
                unreadRegularNotifications: badges.unreadRegularNotifications,
                unreadFollowersNotifications: badges.unreadFollowersNotifications
              });
              setTimeout(function () {
                $rootScope.$apply();
              }, 10)
            }
            resolve(data.result.value);
          }, function (error) {
            reject(error);
          })
        });
      }

      /**
       * function that returns only the pending followers notifications count
       * @param userHref {string}
       * @returns {Promise}
       */
      function countPendingFollowersNotifications(userHref) {
        //console.log("counting Pending Followers Notifications");
        return Utils.$q(function (resolve, reject) {
          $resource(ApiEndPoint.url + 'restful/services/UserNotificationRepository/actions/countPendingNotificationsFollow/invoke', {}, {
            get: {
              method: 'GET',
              headers: {'Accept': 'application/json;profile=urn:org.apache.isis/v1'}
            }
          }).get({
            "x-isis-querystring": {userSubscription: {value: {href: userHref}}}
          }).$promise.then(function (data) {
            if (data.result.value) {
              badges.unreadFollowersNotifications = data.result.value;
              $rootScope.$broadcast("notifications.new", {
                unreadRegularNotifications: badges.unreadRegularNotifications,
                unreadFollowersNotifications: badges.unreadFollowersNotifications
              });
              setTimeout(function () {
                $rootScope.$apply();
              }, 10)
            }
            resolve(data.result.value);
          }, function (error) {
            reject(error);
          });
        })
      }

      /**
       * @type {{regularNotifications: regularNotifications, followersNotifications: followersNotifications}}
       */
      var updateBadgesCount = {
        /**@param numOfRegularNotif {number}*/
        regularNotifications: function (numOfRegularNotif) {
          if (typeof numOfRegularNotif === 'number') {
            badges.unreadRegularNotifications = numOfRegularNotif;
            $rootScope.$broadcast("notifications.new", {
              unreadRegularNotifications: badges.unreadRegularNotifications,
              unreadFollowersNotifications: badges.unreadFollowersNotifications
            });
            setTimeout(function () {
              $rootScope.$apply()
            }, 10);
          }
          else
            Utils.toast.error("Cant update followers badget count, the argument is not a number");
        },
        /**@param numOfFollowersNotif {number}*/
        followersNotifications: function (numOfFollowersNotif) {
          if (typeof numOfFollowersNotif === 'number') {
            badges.unreadFollowersNotifications = numOfFollowersNotif;
            $rootScope.$broadcast("notifications.new", {
              unreadRegularNotifications: badges.unreadRegularNotifications,
              unreadFollowersNotifications: badges.unreadFollowersNotifications
            });
            setTimeout(function () {
              $rootScope.$apply()
            }, 10);
          }
          else {
            Utils.toast.error("Cant update followers badget count, the argument is not a number");
          }
        }
      };


      /*------------ AWS NOTIFICATIONS --------------*/
      /**
       * @param toUserId {number|string}
       * @return Promise {bolean|*}
       * UPDATED DEEPLINK
       */
       //REVISAR QUE REGRESE TRUE/FALSE QUE ORIGINA PROCESO
      function notifyNewCommentOnSequence(toUserId, postID) {
        //return sendNotificationToUserDevices(
        return sendDeepNotificationToUserDevices(
          toUserId,
          UserService.get_USER_NAME() + " " + Utils.translate('ALERTS.COMMENT_SEQUENCE'),
          'COMMENT',
          postID
        )
      }

      /**
       * @param toUserId {number|string}
       * @return Promise {bolean|*}
       * UPDATED DEEPLINK
       */
      //REVISAR QUE REGRESE TRUE/FALSE WS QUE ORIGINA PROCESO
      function notifyCommentLiked(toUserId, postID) {
        //return sendNotificationToUserDevices(
        return sendDeepNotificationToUserDevices(
          toUserId,
          UserService.get_USER_NAME() + " " + Utils.translate('ALERTS.LIKE_COMMENT'),
          'lIKE',
          postID
        )
      }

      /**
       * @param toUserId {number|string}
       * @return Promise {bolean|*}
       * UPDATED DEEPLINK
       */
       //REVISAR QUE REGRESE TRUE/FALSE WS QUE ORIGINA PROCESO
      function notifySequenceLiked(toUserId, postID) {
        //return sendNotificationToUserDevices(
        return sendDeepNotificationToUserDevices(
          toUserId,
          UserService.get_USER_NAME() + " " + Utils.translate("ALERTS.LIKE_SEQUENCE"),
          'LIKE',
          postID
        )
      }

      /**
       * @param toUserId {number|string}
       * @return Promise {bolean|*}
       * UPDATED DEEPLINK
       */
       //REVISAR QUE REGRESE TRUE/FALSE QUE ORIGINA PROCESO
      function notifyReactToSequence(toUserId, postID) {
        //return sendNotificationToUserDevices(
        return sendDeepNotificationToUserDevices(
          toUserId,
          UserService.get_USER_NAME() + " " + Utils.translate('ALERTS.REACTED_TO_SEQUENCE'),
          'REACT',
          postID
        )
      }

      /**
       * @param promiseTagData {*}
       * @return Promise {boolean|*}
       * UPDATED DEEPLINK
       */
      function notifyTaggedUsersInComment(promiseTagData, postID) {
        return _notifyUsersTaggedInObject(
          promiseTagData,
          UserService.get_USER_NAME() + " " + Utils.translate("ALERTS.TAG_COMMENT"),
          postID
        )
      }

      /**
       * @param promiseTagData {*}
       * @return Promise {boolean|*}
       * UPDATED DEEPLINK
       */
      function notifyTaggedUsersInUpdatedComment(promiseTagData, postID) {
        return _notifyUsersTaggedInObject(
          promiseTagData,
          UserService.get_USER_NAME() + " " + Utils.translate("ALERTS.UPDATE_COMMENT_TAGGED"),
          postID
        )
      }

      /**
       * @param promiseTagData {*}
       * @return Promise {boolean|*}
       * UPDATED DEEPLINK
       */
      function notifyUsersTaggedInSequence(promiseTagData, postID) {
        return _notifyUsersTaggedInObject(
          promiseTagData,
          UserService.get_USER_NAME() + " " + Utils.translate("ALERTS.TAG_SEQUENCE"),
          postID
        )
      }

      /**
       * Helper function
       * @param promiseTagData {*}
       * @param message {string}
       * @return Promise {number|*}
       * UPDATED DEEPLINK
       */
      function _notifyUsersTaggedInObject(promiseTagData, message, postID) {
        return Utils.$q(function (resolve, reject) {
          var userIdsList = Object.keys(promiseTagData);

          var numUsersToNotify = 0;

          //get devices for tags
          var sentNotificationsPromises = [];
          userIdsList.forEach(function (userId) {
            //if the user wants to be notified...
            if (promiseTagData[userId].result.value == true) {
              numUsersToNotify++;
              //sentNotificationsPromises.push(sendNotificationToUserDevices(userId, message))
              sentNotificationsPromises.push(sendDeepNotificationToUserDevices(userId, message, 'TAG', postID))
            }
          });

          Utils.$q.all(sentNotificationsPromises).then(function () {
            resolve(numUsersToNotify)
          }, function (error) {
            console.error("_notifyUsersTaggedInObject", error);
            reject(error)
          });
        })
      }

      /**
       * Helper function
       * @param toUserId {number|string}
       * @param message {string}
       * @return Promise {boolean|*}
       */
      function sendNotificationToUserDevices(toUserId, message) { //, type
        return Utils.$q(function (resolve, reject) {
          console.log("sendNotificationToUserDevices OID:" + toUserId);
          UserService.getDevices(toUserId).$promise.then(function (userTagged) {
            message = (message || "").replace(/\s\s+/g, ' ');
            //for each user device...
            userTagged.result.value.forEach(function (endpointArn) {
              AWSServices.sendNotification(message, endpointArn.title);
              //AWSServices.sendNotificationDeepLink(message, endpointArn.title, true, type);
            });
            resolve(true);
          }, function (error) {
            console.error("User Service get Devices", error);
            reject(error)
          });
        })
      }

      /** New function for DEEPLINK in aws sns
       * Helper function
       * @param toUserId {number|string}
       * @param message {string}
       * @return Promise {boolean|*}
       */
      function sendDeepNotificationToUserDevices(toUserId, message, type, id) { 
        return Utils.$q(function (resolve, reject) {
          console.log("sendNotificationToUserDevices OID:" + toUserId);
          UserService.getDevices(toUserId).$promise.then(function (userTagged) {
            message = (message || "").replace(/\s\s+/g, ' ');
            //for each user device...
            userTagged.result.value.forEach(function (endpointArn) {
              AWSServices.sendNotificationDeepLink(message, endpointArn.title, true, type, id);
            });
            resolve(true);
          }, function (error) {
            console.error("User Service get Devices", error);
            reject(error)
          });
        })
      }
      
      //LuMa Live Streaming 
      function sendDeepNotificationToFollowersDevices(userSubscriptionId, notificationType, message, type, url, postID) {
        return Utils.$q(function(resolve, reject) {
          //First check the new plugin to get if the user is special/follow by everyone or normal
          UserService.findByUserSubscriptionFollow(
            userSubscriptionId
          ).$promise.then( function (response) {
            var isSpecial = response.result.value;
            if(isSpecial) {
              //the user have all members of App as followers
              console.log("SEND TO ALL PUSH");
              AWSServices.sendGlobalNotificationStream(
                message,
                s3.topicARNProduction,
                true,
                type,
                url,
                postID
              );
              resolve(true);
            } else {
              //the user doesn't have all members of App as followers
              console.log("SEND TO VIEWERS PUSH");
              UserService.findFollowersUserDeviceUserSubscriptionId(userSubscriptionId, notificationType).$promise.then(function(response) {
                response.forEach(function(follower) {
                  AWSServices.sendLiveStreamingNotification(message, follower.userDeviceId, true, type, url);
                });
                resolve(true);
              }, function(error) {
                console.log("User Service findFollowersUserDeviceUserSubscriptionId", error);
                reject(error);
              });
            }
          }, function (error) {
            reject(error);
          })
        })
      } 

      /*-------- END OF AWS NOTIFICATIONS -----------*/

      return {
        badges: badges,
        paginator: paginator,
        notifications: notifications,
        countPendingRegularNotifications: countPendingRegularNotifications,
        countPendingFollowersNotifications: countPendingFollowersNotifications,
        updateBadgesCount: updateBadgesCount,
        canOpenNotificationWS: canOpenNotificationWS,
        notifyNewCommentOnSequence: notifyNewCommentOnSequence,
        notifyCommentLiked: notifyCommentLiked,
        notifySequenceLiked: notifySequenceLiked,
        notifyReactToSequence: notifyReactToSequence,
        notifyTaggedUsersInComment: notifyTaggedUsersInComment,
        notifyTaggedUsersInUpdatedComment: notifyTaggedUsersInUpdatedComment,
        notifyUsersTaggedInSequence: notifyUsersTaggedInSequence,
        sendNotificationToUserDevices: sendNotificationToUserDevices,
        sendDeepNotificationToUserDevices: sendDeepNotificationToUserDevices,
        sendDeepNotificationToFollowersDevices: sendDeepNotificationToFollowersDevices //LuMa Live Streaming
      }
    }]);

/**
 * @typedef {Object} BadgesType
 * @property {number} unreadRegularNotifications
 * @property {number} unreadFollowersNotifications
 */

/**
 * @typedef {Function} regularNotifications
 * @param {number} numOfRegularNotif
 */

/**
 * @typedef {Function} followersNotifications
 * @param {number} numOfFollowersNotif
 */
