"use strict";
appServices.service('socketServices', ['chatConstants', function (chatConstants) {
  let socket;
  let ioIsConnected = false;

  /**
   * @param {Object} userData
   * @param {number} userData.userAppId
   * @param {string} userData.userName
   * @param {string?} userData.userAvatar
   * @param {function} onSuccess
   * @param {function} onError
   */
  function ioConnect(userData, onSuccess, onError) {
    if (socket && ioIsConnected) {
      console.log("Socket.io is already connected");
      onSuccess({connected: ioIsConnected, socketId: socket.id});
      return;
    }

    if (!userData.userAppId) {
      onError({
        connected: ioIsConnected,
        message: "socket.io can't be connected because no user id was provided"
      });
      return;
    }

    console.log("Connecting socket.io for user (id: " + userData.userAppId + ') ' + userData.userName + ' ...');

    socket = io.connect(
      chatConstants.DOMAIN,
      {
        transports: ['websocket'],
        query: 'USER_APP_ID=' + userData.userAppId + "&USER_NAME=" + userData.userName + "&USER_AVATAR=" + (userData.userAvatar || ''),
        forceNew: true
      });

    if (socket) {
      initListeners(onSuccess, onError);
    }
    else {
      ioIsConnected = false;
      onError({connected: ioIsConnected, message: "Unknown error, couldn't connect to chat server"});
    }
  }

  function initListeners(onSuccess, onError) {
    socket.on('connecting', function (data) {
      console.log("Socket.io is connecting...", data)
    });

    socket.on('connect', function () {
      ioIsConnected = socket.connected;
      onSuccess({connected: ioIsConnected, socketId: socket.id});
    });

    //Called when user disconnects / server shutdown - AUTOMATICO DEL SERVER
    socket.on('disconnect', function (data) {
      console.log("Socket is disconnect", data);
      ioIsConnected = false;
      onSuccess({connected: ioIsConnected, socketId: null});
    });

    //Called when user reconnects - AUTOMATICO DEL SERVER
    socket.on('reconnect', function (data) {
      console.log("Socket has been reconnected", data);
    });

    //Called when can't get a connection - AUTOMATICO DEL SERVER
    socket.on('reconnect_error', function (data) {
      console.log("Socket can't be reconnected", data);
      onError({connected: ioIsConnected, message: "Socket can't be reconnected"})
    });
  }

  /**
   * @param name
   * @param reference
   */
  function removeListener(name, reference) {
    //console.log("removing listener "+name, typeof reference);
    socket.removeListener(name, reference);
  }

  function isSocketConnected() {
    return ioIsConnected;
  }

  function emit(eventName, data, callback) {
    socket.emit(eventName, data, callback);
  }

  function on(eventName, callbackFn) {
    //console.log("setting socket.on('"+eventName+"')");
    socket.on(eventName, callbackFn);
  }

  return {
    isConnected: ioIsConnected,
    ioConnect: ioConnect,
    removeListener: removeListener,
    emit: emit,
    on: on
  }
}]);
