"use strict";

appServices
.service('TagsService', ['ApiEndPoint', '$resource', 'Utils',
  function (ApiEndPoint, $resource, Utils) {
    /**
     * @param data {{sequenceId:number,commentId:number,notificationType:number}}
     * @return {Promise}
    */
    function getTagFromComments167WS(data) {
      return $resource(
        ApiEndPoint.url + 'restful/services/SequenceTagRepository/actions/findTagSequenceCommentv167/invoke',
        {},
        {
          get: {
            method: 'GET',
            isArray: true,
            headers: {"Accept": "application/json;profile=urn:org.apache.isis/v1;suppress=true"}
          }
        }
      ).get({
        sequenceId: '"' + data.sequenceId + '"',
        notificationType: '"' + data.notificationType + '"',
        commentId: '"' + data.commentId + '"'
      }).$promise;
    }
     
    /**
     * @param data {{objectId:number|string, userId:number|string}}
    */
    function saveTagForCommentWS(data) {
      return $resource(
        ApiEndPoint.url + 'restful/objects/simple.SequenceComments/:objectId/actions/tagComment/invoke',
        {objectId: data.objectId},
        {
          post: {
            method: 'POST',
            headers: {"Accept": "application/json;profile=urn:org.apache.isis/v1"}
          }
        }
      ).save({
        "userSubscription": {
          "value": {
            "href": ApiEndPoint.url + "restful/objects/simple.UserSubscription/" + data.userId
          }
        }
      });
    }
     
    /**
     * @param data {{objectId:number|string, userId:number|string}}
     * @return Promise
    */
    function saveTagForCommentV157WS(data) {
      return $resource(
        ApiEndPoint.url + 'restful/objects/simple.SequenceComments/:objectId/actions/tagCommentv157/invoke',
        {objectId: data.objectId},
        {
          post: {
            method: 'POST',
            headers: {"Accept": "application/json;profile=urn:org.apache.isis/v1"}
          }
        }
      ).save({
        userSubscription: {
          value: {
            href: ApiEndPoint.url + "restful/objects/simple.UserSubscription/" + data.userId
          }
        }
      }).$promise;
    }
     
    /**
     * @param data {{sequenceId:number|string, userId: number|string}}
     * @return {*}
    */
    function linkTagToDescriptionWS(data) {
      return $resource(
        ApiEndPoint.url + 'restful/objects/simple.Sequence/:objectId/actions/tagv157/invoke',
        {objectId: data.sequenceId},
        {
          post: {
            method: 'POST',
            headers: {"Accept": "application/json;profile=urn:org.apache.isis/v1"}
          }
        }
      ).save({
        userSubscription: {
          value: {
            href: ApiEndPoint.url + "restful/objects/simple.UserSubscription/" + data.userId
          }
        }
      });
    }
     
    /**
     * @param data {{sequenceId:number|string, userId:number|string, groupId:(number|string)?}}
     * @return Promise {*}
    */
    function saveTagDescriptionNotificationWS(data) {
      return $resource(
        //for old versions:
        //ApiEndPoint.url + 'restful/objects/simple.Sequence/:objectId/actions/tagNotification/invoke',
        //for latest version:
        ApiEndPoint.url + 'restful/objects/simple.Sequence/:objectId/actions/tagNotificationv191/invoke',
        {objectId: data.sequenceId},
        {
          post: {
            method: 'POST',
            headers: {"Accept": "application/json;profile=urn:org.apache.isis/v1"}
          }
        }
      ).save({
        userSubscription: {
          value: {
            href: ApiEndPoint.url + "restful/objects/simple.UserSubscription/" + data.userId
          }
        },
        groupId: {value: (data.groupId || 0).toString()}//<--to use with tagNotificationv191
      }).$promise;
    }
     
    /**
     * @param data {{objectId:number|string, userId:number|string, groupId:(number|string)?}}
     * @return Promise
    */
    function saveTagCommentNotificationWS(data) {
      return $resource(
        //for old versions:
        //ApiEndPoint.url + 'restful/objects/simple.SequenceComments/:objectId/actions/tagCommentNotification/invoke',
        //for latest version:
        ApiEndPoint.url + 'restful/objects/simple.SequenceComments/:objectId/actions/tagCommentNotificationv191/invoke',
        {objectId: data.objectId},
        {
          post: {
            method: 'POST',
            headers: {"Accept": "application/json;profile=urn:org.apache.isis/v1"}
          }
        }
      ).save({
        userSubscription: {
          value: {
            href: ApiEndPoint.url + "restful/objects/simple.UserSubscription/" + data.userId
          }
        },
        groupId: {value: (data.groupId || 0).toString()}//<--to use with tagNotificationv191
      }).$promise;
    }
     
    /**
     * @param userId {number|string}
     * @return Promise
    */
    function deleteTaggedUserWS(userId) {
      return $resource(
        ApiEndPoint.url + 'restful/objects/simple.SequenceTag/:objectId/actions/delete/invoke',
        {objectId: userId},
        {
          post: {
            method: 'POST',
            headers: {"Accept": "application/json;profile=urn:org.apache.isis/v1"}
          }
        }
      ).save().$promise;
    }
     
    /**
     * @param data {{term:string}}
     * @return Promise {Array|*}
    */
    function searchHashtagsWS(data) {
      return Utils.$q(function (resolve, reject) {
        $resource(
          ApiEndPoint.url + 'restful/services/HashTagRepository/actions/findHashTagStart/invoke',
          {},
          {
            get: {
              method: 'GET',
              isArray: true,
              headers: {"Accept": "application/json;profile=urn:org.apache.isis/v1;suppress=true"}
            }
          }
        ).get({hashTag: data.term}).$promise.then(function (response) {
          Utils.removeLastItemIfHasNoAttribute(response, 'hashTag');
          var hashList = [];
          response.forEach(function (hash) {
            hashList.push({
              id: Utils.getLastUrlComponent(hash.hashTagObj.href),
              hash: hash.hashTag
            });
          });
          resolve(hashList);
        }, function (error) {
          console.error("searchHashtagsWS", error);
          reject(error)
        })
      })
    }
     
    /**
     * @param tagId {number|string}
     * @return {*}
    */
    function removeHastagFromDBWS(tagId) {
      return $resource(
        ApiEndPoint.url + 'restful/objects/simple.SequenceHashTag/:objectId/actions/delete/invoke', {
          objectId: tagId
        },
        {
          post: {
            method: 'POST',
            headers: {"Accept": "application/json;profile=urn:org.apache.isis/v1"}
          }
        }
      ).save();
    }
     
    /**
     * @param data {{hTagId:number|string, amount?:number}}
     * @return {*}
     */
    function updateHashTagCountWS(data) {
      return $resource(
        ApiEndPoint.url + 'restful/objects/simple.HashTag/:objectId/actions/hashTagCount/invoke', {
          objectId: data.hTagId
        },
        {
          post: {
            method: 'POST',
            headers: {"Accept": "application/json;profile=urn:org.apache.isis/v1;"}
          }
        }
      ).save({
        int: {value: data.amount || 1}
      })
    }
     
    /**
     * @param tagName {string}
     * @return {*}
    */
    function createHashTagWS(tagName) {
      return $resource(
        ApiEndPoint.url + 'restful/services/HashTagRepository/actions/addHashTag/invoke',
        {},
        {
          post: {
            method: 'POST',
            headers: {"Accept": "application/json;profile=urn:org.apache.isis/v1;"}
          }
        }
      ).save({hashtag: {value: tagName}});
    }
     
    /**
     * @param data {{sequenceId: number|string, hashTagId:number|string}}
     * @return {*}
    */
    function linkHashtoSequenceWS(data) {
      return $resource(
        ApiEndPoint.url + 'restful/services/SequenceHashTagRepository/actions/addSequenceHashTag/invoke', {},
        {
          post: {
            method: 'POST',
            headers: {"Accept": "application/json;profile=urn:org.apache.isis/v1;"}
          }
        }
      ).save({
        sequence: {
          value: {
            href: ApiEndPoint.url + "restful/objects/simple.Sequence/" + data.sequenceId
          }
        },
        hashTag: {
          value: {
            href: ApiEndPoint.url + "restful/objects/simple.HashTag/" + data.hashTagId.id
          }
        }
      })
    }
     
    /**
     *
     * @param textList {{hash: string, sequenceHashTag: {href: (null)}, id: (null)}[]}
     * @return {Promise[]}
    */
    function createHashtags(textList) {
      var promises = [];
      textList.forEach(/**@type {{hash: string, sequenceHashTag: {href: (null)}, id: number}}*/function (tag) {
        console.log("Creating hashtag: " + tag.hash);
        promises.push(createHashTagWS(tag.hash).$promise);
      });
      return promises;
    }
     
    /**
     *
     * @param data {{tagList:Array, sequenceInstanceId:(number|string)}}
     * @return {{Promise}}
    */
    function linkTagsToPostDescription(data) {
      var saveTagsPromises = {};
      data.tagList.forEach(function (tag) {
        var userId = tag.id || Utils.getLastUrlComponent(tag.userSubscription.href);
        console.log("linking user tagged to sequence: ", tag);
        saveTagsPromises[userId] = linkTagToDescriptionWS({
          sequenceId: data.sequenceInstanceId,
          userId: userId
        }).$promise;
      });
      return saveTagsPromises;
    }
     
    /**
     *
     * @param tagsList Array
     * @return {{Promise}}
    */
    function unlinkTagsFromPostDescription(tagsList) {
      var deleteUserTaggedPromises = {};
      tagsList.forEach(function (tag) {
        if (tag.sequenceTag) {
          var sequenceTagId = Utils.getLastUrlComponent(tag.sequenceTag.href);
          console.log("Unlinking tag", tag);
          deleteUserTaggedPromises[sequenceTagId] = deleteTaggedUserWS(sequenceTagId);
        } else if (tag.id) {
          //new tag with no proper href to delete
          console.log("can't unlink new tag", tag);
        }
        else
          console.log("can't unlink tag", tag);
        });
      return deleteUserTaggedPromises;
    }
     
    /**
     *
     * @param data {{hashtagIdList: Array, sequenceInstanceId:(number|string)}}
     * @return {Promise[]}
    */
    function linkHashtagToPostDescription(data) {
      var linkHashToPostPromises = [];
      data.hashtagIdList.forEach(function (hashtagId) {
        if(hashtagId) {
          linkHashToPostPromises.push(addSequenceHashTagv200({
            sequenceId: data.sequenceInstanceId,
            hashTagId: hashtagId.id,
            notificationType: data.notificationType,
            commentID: data.commentID
          }).$promise
          )
          /*linkHashToPostPromises.push(linkHashtoSequenceWS({
              sequenceId: data.sequenceInstanceId,
              hashTagId: hashtagId
            }).$promise
          );*/
        } else {
          console.log("old hashtag can't be added");
        }
      });
      return linkHashToPostPromises;
    }
     
    /**
     *
     * @param hashtagList Array
     * @return {Promise[]}
    */
    function unlinkHashtagFromPostDescription(hashtagList) {
      var promises = [];
      hashtagList.forEach(/**@type {{hash: string, sequenceHashTag: {href: string}}}*/function (item) {
        if(item.sequenceHashTag)
          promises.push(removeHastagFromDBWS(Utils.getLastUrlComponent(item.sequenceHashTag.href)).$promise)
        else if (item.id)
          //new tag with no proper href to delete
          console.log("can't unlink new hashtag", item);
        else
          console.log("can't unlink hash", item);
      });
      return promises;
    }
     
    /**
     *
     * @param hashtagList
     * @return {{Promise}}
    */
    function increaseHashtagsCount(hashtagList) {
      var promises = {};
      hashtagList.forEach(/**@type {{hash: string, id:number}}}*/function (item) {
        promises[item.id] = updateHashTagCountWS({hTagId: item.id}).$promise
      });
      return promises;
    }
     
    /**
     * @param text {string}
     * @param tagsList {Array}
     * @return {Array}
    */
    function removeTagsNotFoundInText(text, tagsList) {
      tagsList = tagsList || [];
      var tagsKept = tagsList.filter(function (tag) {
        var nameToCheck = "@" + tag.name.replace(/\s\s+/g, ' ').replace(/\s+$/g, '');
        if (text.indexOf(nameToCheck) > -1)  return tag;
      });
      return tagsKept;
    }

    /**
     *
     * @param tagsList {Array}
    */
    function removeDuplicatedTags(tagsList) {
      var ids = [];
      var result = [];
      tagsList.forEach(function (tag) {
        var id = tag.id || +Utils.getLastUrlComponent(tag.userSubscription.href);                
        if(!ids.includes(+id)) {
          ids.push(+id);
          result.push(tag);
        }
      });
      return result;
    }
     
    /**
     * @param text {string}
     * @param listToClean {{hash:string}[]}
     * @return {Array}
    */
    function removeHashTagsNotFoundInText(text, listToClean) {
      /**@type {{hash: string, sequenceHashTag: {href: null}, id: null}[]}*/
      var tagsInText = extractHashTagsFromText(text);
      return listToClean.filter(function (item) {
        var found = tagsInText.some(/**@type {{hash: string, sequenceHashTag: {href: null}, id: null}}*/function (tag) {
          return item.hash.toUpperCase() === tag.hash.toUpperCase()
        });
        if (found) return item;
      });
    }

    /**
     *
     *
    **/
    function addUncheckedTags(text, listToCheck) {
      return new Promise( resolve => { //LuMa HashTags in comments
        var tagsAlready = extractHashTagsFromText(text);
        var listToReturn = [];
        var searchPromises = [];
        var index = 0;
        tagsAlready.forEach( function (optimeTags) {
          var foundIt = false;
          for(var i=0; i<listToCheck.length; i++) {
            if(optimeTags.hash.toUpperCase() === listToCheck[i].hash.toUpperCase()) {
              foundIt = true;
              listToReturn.push(listToCheck[i]);
              break;
            }
          }
          if(!foundIt) {
            searchPromises[index] = searchHashtagsWS({
              term: optimeTags.hash
            }).then( function (hashList) {
              if(hashList.length > 0) {
                /*LuMa HashTag in comments*/ 
                var foundInSearch = false;
                for(var i = 0; i < hashList.length; i++) {
                  var found = hashList[i];
                  if(found.hash.toUpperCase() === optimeTags.hash.toUpperCase()) {
                    found.new = false;
                    listToReturn.push(found);
                    foundInSearch = true;
                    break;
                  }
                }
                if(!foundInSearch) {
                  var item = {
                    hash: optimeTags.hash,
                    id: null,
                    new: true
                  }
                  listToReturn.push(item);
                }
              } else {
                var item = {
                  hash: optimeTags.hash,
                  id: null,
                  new: true
                }
                listToReturn.push(item);
              }
            });
            index ++;
          }
        });
        listToReturn.forEach(function(hashTag) {
          if(hashTag.id == null) {
            searchPromises[index] = searchHashtagsWS({
              term: hashTag.hash
            }).then( function (hashList) {
              if(hashList.length > 0) {
                for(var i = 0; i < hashList.length; i++) {
                  var found = hashList[i];
                  if(found.hash.toUpperCase() === hashTag.hash.toUpperCase()) {
                    hashTag.id = found.id;
                    break;
                  }
                }
              }
            });
            index ++;
          }
        });
        Utils.$q.all(searchPromises).then(function() {
          resolve(listToReturn);
        });
      })
    }

    /**
     * Removes the duplicated hashtags and retuns an array of hashtags whitout '#'
     * @param text {string}
     * @return {{hash:string, sequenceHashTag:{href:null}, id: null}[]}
    */
    function extractHashTagsFromText(text) {
      text = text || '';
      var hashTagsFromText = text.match(/\#{1}\S+/g) || []; 
      return hashTagsFromText.sort().filter(function (item, pos, ary) {
        return !pos || item != ary[pos - 1];
      }).map(function (item) {
        return {hash: item.substring(1), sequenceHashTag: {href: null}, id: null}
      }).filter(function (item) {//the items maybe still have a #, don't return those items
        return item.hash.indexOf('#') === -1
      })
    }
     
    /**
     * @param list1 {{hash:string, sequenceHashTag:{href:string|null}, id?:(number|null)}[]}
     * @param list2 {{hash:string, sequenceHashTag:{href:string|null}, id?:(number|null)}[]}
     * @return {{hash:string, sequenceHashTag?:{href:string|null}, id?: (number|null)}[]}
    */
    function getHTagsFromList1_NOT_IN_List2(list1, list2) {
      return list1.filter(function (item) {
        var found = list2.some(function (tag) {
          return item.hash === tag.hash
        });
        if (!found) return item
      })
    }
     
    /**
     * @param list1 {{hash:string, sequenceHashTag:{href:string|null}, id?:(number|null)}[]}
     * @param list2 {{hash:string, sequenceHashTag:{href:string|null}, id?:(number|null)}[]}
     * @return {{hash:string, sequenceHashTag:{href:string|null}, id?: (number|null)}[]}
    */
    function getCommonHTagsInLists(list1, list2) {
      return list1.filter(function (item) {
        var found = list2.some(function (tag) {
          return item.hash === tag.hash
        });
        if (found) return item;
      })
    }
     
    function addHashNotFoundInText(text, hashList) {
      /*if(hashList.length>0){
      var parts = text.match(/\#{1}\S+/g);
      for(var i = 0; i < parts.length; i++){
      var hashClean = parts[i].split('#');
      var promiseHash;
      for (var j = 0; j < $scope.data.hashList.length; j++){
      }
      }
      }else{
      }*/
    }
     
    /**
     * @param text {string}
     * @param taggedUsers {Object[]}
    */
    function getTextWithFormatedTags(text, taggedUsers) {
      taggedUsers = taggedUsers || [];
     
      var positionTag = 0;
      var taggedUserId = 0;
      var fixedName;
     
      taggedUsers.forEach(function (tag) {
        /* probar si este codigo es equivalente (si lo es, eliminar el actualmente usado)
         * var fixedHtml = description.replace(/^\[{1}@{1}/g, '<span class="mentionTag">@');
         * fixedHtml = fixedHtml.replace(/(\s{1})\[{1}@{1}/g, ' <span class="mentionTag">@');
         * return fixedHtml.replace(/\]{1}/g, '</span>&nbsp;');
         */
        fixedName = '@' + tag.name.replace(/\s\s+/g, ' ').replace(/\s+$/g, '');
        //text = text.replace(/\s\s+/g, ' ').replace(/\s+$/g, '');
        var fixedNameUpper = fixedName.toUpperCase();
        var textUpper = text.toUpperCase();
        if (textUpper.indexOf(fixedNameUpper) !== -1) {
          positionTag = textUpper.indexOf(fixedNameUpper);
          taggedUserId = tag.id || Utils.getLastUrlComponent(tag.userSubscription.href);
          text = text.substr(0, positionTag) +
                '<span class="mentionTag" id="' + taggedUserId + '">' + fixedName +
                '</span>&nbsp;' + text.substr(positionTag + fixedName.length);
        }
      });
     if(text != "")
        return text.replace(/(^|\s{1})\[{1}/g, '').replace(/(\s{1})\]{1}/g, '');
      else
        return ""
    }
                                     
    /**
     * @param text {string}
     * @param hashList {{hash:string}[]}
    */
    function getTextWithFormatedHashes(text, hashList) {
      //console.log("getTextWithFormatedHashes", text, hashList);
      hashList = hashList || [];
                                     
      Utils.removeLastItemIfHasNoAttribute(hashList, 'hash');
                                     
      if (!hashList.length)
        return text;
                                     
      var newText = text;
      var pivotInsert = 0;
      var fixHash = "";
      var positionHashTag = 0;
      var calcEnd = 0;
      var fixedHashArray = [];
                                     
      for (var a = 0; a < hashList.length; a++) {
        fixedHashArray[a] = '#' + hashList[a].hash;
        //console.log("Hash found: " + fixedHashArray[a])
      }
                                     
      fixedHashArray.sort(function (a, b) {
        return b.length - a.length;
      });

      //clean the array from repeated hashes
      var dups = [];
      var arr = fixedHashArray.filter(function (data) {
        if(dups.indexOf(data) == -1) {
          dups.push(data);
          return true
        }
        return false;
      });

      fixedHashArray = arr;
                                     
      for (var b = 0; b < fixedHashArray.length; b++) {
        fixHash = fixedHashArray[b];
        var fixHashUpper = fixHash.toUpperCase();
        var newTextUpper = newText.toUpperCase();
        if (newTextUpper.indexOf(fixHashUpper) !== -1) { 
          positionHashTag = newTextUpper.indexOf(fixHashUpper, pivotInsert);
          if (positionHashTag === 0) {
            newText = newText.substr(0, positionHashTag) + '<span class="mentionTag" name="' + fixHash + '">' + fixHash + '</span>&nbsp;' + newText.substr(positionHashTag + fixHash.length); //LuMa HashTags in comments
            pivotInsert = pivotInsert + (positionHashTag - pivotInsert) + 31 + fixHash + 2 + fixHash.length + 13; //LuMa HashTags in comments
          }
          else {
            calcEnd = positionHashTag + fixHash.length;
            if (calcEnd === newText.length) {
              newText = newText.substr(0, positionHashTag) + '<span class="mentionTag" name="' + fixHash + '">' + fixHash + '</span>&nbsp;' + newText.substr(positionHashTag + fixHash.length); //LuMa HashTags in comments
              pivotInsert = 0;
            }
            else {
              positionHashTag = newTextUpper.indexOf(" " + fixHashUpper + " ");
              if (positionHashTag === -1) {
                positionHashTag = newTextUpper.indexOf(" " + fixHashUpper);
              }
              if (positionHashTag === -1) {
                positionHashTag = newTextUpper.indexOf(fixHashUpper + " ");
                positionHashTag = positionHashTag - 1;
              }
              if (positionHashTag < 0) {
                var charComp = '';
                for (var i = 0; i < newText.length; i++) {
                  if (newText.charCodeAt(i) === 10) {
                    charComp = newText[i];
                  }
                }
                positionHashTag = newTextUpper.indexOf(charComp + fixHashUpper);
              }
              positionHashTag = positionHashTag + 1;
              newText = newText.substr(0, positionHashTag) + '<span class="mentionTag" name="' + fixHash + '">' + fixHash + '</span>&nbsp;' + newText.substr(positionHashTag + fixHash.length); //LuMa HashTags in comments
            }
          }
        }
      }                               
      return newText
    }

    //LuMa HashTag in comments
    function addSequenceHashTagv200(data) {
      return $resource(ApiEndPoint.url + 'restful/services/SequenceHashTagRepository/actions/addSequenceHashTagv200/invoke',{},{
        post:{
          method: 'POST',
          headers: {"Accept": "application/json;profile=urn:org.apache.isis/v1;"}
        }
      }).save({
        sequence: {
          value: {href: ApiEndPoint.url + 'restful/objects/simple.Sequence/' + data.sequenceId}
        },
        hashTag: {
          value: {href: ApiEndPoint.url + "restful/objects/simple.HashTag/" + data.hashTagId}
        },
        notificationType: {value: '' + data.notificationType},
        commentID: {value: '' + data.commentID}
      });
    }

    //LuMa HashTag in comments
    function findSequenceHashTagComment(data) {
      return $resource(
          ApiEndPoint.url + 'restful/services/SequenceHashTagRepository/actions/findSequenceHashTagComment/invoke', {}, {
            get: {
              method: 'GET',
              isArray: true,
              headers: {"Accept": "application/json;profile=urn:org.apache.isis/v1;"}
            }
          }).get({
            sequenceId: '"' + data.sequenceId + '"',
            notificationType: '"' + data.notificationType + '"' || "7",
            commentId: '"' + data.commentId + '"' || "0"
          }).$promise;
    }
                                     
    return {
      getTagFromComments167WS: getTagFromComments167WS,
      saveTagForCommentWS: saveTagForCommentWS,
      saveTagForCommentV157WS: saveTagForCommentV157WS,
      linkTagToDescriptionWS: linkTagToDescriptionWS,
      linkHashtoSequenceWS: linkHashtoSequenceWS,
      saveTagDescriptionNotificationWS: saveTagDescriptionNotificationWS,
      saveTagCommentNotificationWS: saveTagCommentNotificationWS,
      deleteTaggedUserWS: deleteTaggedUserWS,
      searchHashtagsWS: searchHashtagsWS,
      removeHastagFromDBWS: removeHastagFromDBWS,
      updateHashTagCountWS: updateHashTagCountWS,
      createHashTagWS: createHashTagWS,
     
      createHashtags: createHashtags,
      linkTagsToPostDescription: linkTagsToPostDescription,
      unlinkTagsFromPostDescription: unlinkTagsFromPostDescription,
      linkHashtagToPostDescription: linkHashtagToPostDescription,
      unlinkHashtagFromPostDescription: unlinkHashtagFromPostDescription,
      increaseHashtagsCount: increaseHashtagsCount,
      extractHashTagsFromText: extractHashTagsFromText,
      getHTagsFromList1_NOT_IN_List2: getHTagsFromList1_NOT_IN_List2,
      getCommonHTagsInLists: getCommonHTagsInLists,
      removeTagsNotFoundInText: removeTagsNotFoundInText,
      removeHashTagsNotFoundInText: removeHashTagsNotFoundInText,
      addUncheckedTags: addUncheckedTags,
      getTextWithFormatedTags: getTextWithFormatedTags,
      getTextWithFormatedHashes: getTextWithFormatedHashes,
      removeDuplicatedTags: removeDuplicatedTags,
      addSequenceHashTagv200: addSequenceHashTagv200, //LuMa HashTags in comments
      findSequenceHashTagComment: findSequenceHashTagComment //LuMa HashTags in comments
    }
}]);
