"use strict";
appControllers
  .factory('timelinesService', function () {

    var eventsAttached = false;

    /**
     * @type {{
     *  events: {
     *    timeline: {refresh:  function(*)},
     *    sequence: {onCreate:  function(*), onDelete:  function(*), onTitleEdited:  function(*), onContentEdited:  function(*), onCountersUpdated:  function(*), onShareableUpdated: function(*)},
     *    comment: {onCreate:  function(*), onUpdate:  function(*), onDelete:  function(*), onEnabledUpdated:  function(*), onLikeUpdated:  function(*)},
     *    advertisement: {onSettingsUpdated:  function(*)}
     *  },
     *  ctrlCreated: boolean,
     *  isFirstTime: boolean
     * }}
     */
    var defaultTimeline = {
      events: {
        timeline: {
          refresh: null
        },
        sequence: {
          onCreate: null,
          onDelete: null,
          onTitleEdited: null,
          onContentEdited: null,
          onCountersUpdated: null,
          onShareableUpdated: null
        },
        comment: {
          onCreate: null,
          onUpdate: null,
          onDelete: null,
          onEnabledUpdated: null,
          onLikeUpdated: null
        },
        advertisement:{
          onSettingsUpdated:null
        }
      },
      ctrlCreated: false,
      isFirstTime: true
    };

    /**
     * @type {{
     *   events: {
     *     timeline: {refresh: function(*)},
     *     sequence: {onCreate: function(*), onDelete: function(*), onTitleEdited: function(*), onContentEdited: function(*), onCountersUpdated: function(*), onShareableUpdated: function(*)},
     *     comment: {onCreate: function(*), onUpdate: function(*), onDelete: function(*), onEnabledUpdated: function(*), onLikeUpdated: function(*)}
     *   },
     *   ctrlCreated: boolean, groupId: null,
     *   isFirstTime: boolean,
     *   pinnedSequenceFetched: boolean
     * }}
     */
    var groupTimeline = {
      events: {
        timeline: {
          refresh: null
        },
        sequence: {
          onCreate: null,
          onDelete: null,
          onTitleEdited: null,
          onContentEdited: null,
          onCountersUpdated: null,
          onShareableUpdated: null
        },
        comment: {
          onCreate: null,
          onUpdate: null,
          onDelete: null,
          onEnabledUpdated: null,
          onLikeUpdated: null
        }
      },
      ctrlCreated: false,
      groupId: null,
      isFirstTime: true,
      pinnedSequenceFetched: false
    };
    /**
     *
     * @type {{
     *  events: {
     *    sequence: {
     *      onCreate: function(*),
     *      onDelete: function(*),
     *      onTitleEdited: function(*),
     *      onContentEdited: function(*),
     *      onCountersUpdated: function(*),
     *      onShareableUpdated: function(*)
     *    },
     *    comment: {
     *      onCreate: function(*),
     *      onUpdate: function(*),
     *      onDelete: function(*),
     *      onEnabledUpdated: function(*),
     *      onLikeUpdated: function(*)
     *    }
     *  }
     * }}
     */
    var myProfileTimeline = {
      events:{
        sequence: {
          onCreate: null,
          onDelete: null,
          onTitleEdited: null,
          onContentEdited: null,
          onCountersUpdated: null,
          onShareableUpdated: null
        },
        comment: {
          onCreate: null,
          onUpdate: null,
          onDelete: null,
          onEnabledUpdated: null,
          onLikeUpdated: null
        }
      }
    }

    /**
     * @type {{
     *   events: {
     *     timeline: {onRefresh: function(*)},
     *     sequence: {onCreate: function(*), onDelete: function(*)},
     *     comment: {
     *       onCreate: function(*),
     *       onUpdate: function(*),
     *       onDelete: function(*),
     *       onLikeCounterUpdated: function(*)
     *     }
     *   }
     * }}
     */
    var hashTimeline = {
      events: {
        timeline: {
          onRefresh: null
        },
        sequence: {
          onCreate: null,
          onDelete: null
        },
        comment: {
          onCreate: null,
          onUpdate: null,
          onDelete: null,
          onLikeCounterUpdated: null
        }
      }
    };


    return {
      defaultTimeline: defaultTimeline,
      groupTimeline: groupTimeline,
      myProfileTimeline:myProfileTimeline,
      hashTimeline: hashTimeline,
      eventsAttached: eventsAttached
    }
  });
