"use strict";
appServices
  .factory('ChatFactory2', ['$rootScope', 'socketServices', 'DBServices', 'UserService', 'ImageService3', 'notificationsService', 'Utils', function ($rootScope, socketService, DBServices, UserService, ImageService3, notificationsService, Utils) {

    var mySocket = socketService;
    var myDB = DBServices;
    var _myId = null;

    /**
     *
     * @type {Array<{id:number, isOnline:boolean}>}
     * @private
     */
    var _contactRooms = [];

    var socketId;
    var chatIsUsable = false;

    var listenersToRemove = [];

    var _myCurrentRoomId = 0;

    var _totalUnreadMsgs = 0;


    /**
     App: saves msg with status: MSG_OUTGOING
     Server: receives msg and emits to App ack: MSG_PROCESSING
     App: changes msg status to: MSG_PROCESSING
     Server: finds out whether peer is or not online
     Server: if peer is NOT found, saves msg in server DB and replies back with status: MSG_ARCHIVED
     Server: if peer IS online emits status: MSG_DELIVERING
     App: if server ack == MSG_ARCHIVED sends push notification and changes msg status to: MSG_ARCHIVED
     App: if server ack == MSG_DELIVERING changes msg status to MSG_DELIVERING
     Server: emits (sends) msg to peer
     Peer: receives and saves msg with status MSG_INCOMING_SAVED
     Peer: if received in other room emits ack to server {read:false}
     Peer: if received in same room emits ack to server {read:true}
     Server: receives peer ack and emits it back to Sender
     App: receives peer acknowledge and updates msg status: (read? MSG_OUTGOING_READ : MSG_OUTGOING_SAVED)
     */

    var STATUS = {
      MSG_OUTGOING: 1,//FOR SENDER: when I try to send the message to server
      MSG_ARCHIVED: 2,//FOR SENDER: when server processed msg and saved due to receiver is offline
      MSG_DELIVERING: 3,//FOR SENDER: when server is sending msg to receiver
      MSG_RECEIVER_SAVED: 4,//FOR SENDER: when receiver got the msg but wasn't in same room
      MSG_RECEIVER_READ: 5//FOR SENDER: when receiver got the msg and was in same room
    };

    var MEDIA_TYPE = {
      IMAGE: 'image',
      AUDIO: 'audio',
      VIDEO: 'video',
      OTHER: 'other'
    };


    /**
     *
     * @param {Object} data
     * @param {number} data.toContactId
     * @param {string} data.text
     * @param {number?} data.fileId
     * @param {Object?} data.extraData
     * @constructor
     */
    function PrivateMessageToSend(data) {
      var d = new Date();

      this.uniqueId = _myId + ':' + d.getTime();
      this.contactId = +data.toContactId;
      this.groupId = null;
      this.text = data.text;
      this.fileId = data.fileId || null;
      this.extraData = data.extraData || null;
      this.createdAt = d.toISOString();
      this.processedAt = null;
      this.receivedAt = null;
      this.isMine = 1;//0=false,1=true
      this.status = STATUS.MSG_OUTGOING
    }

    /**
     *
     * @param {Object} data
     * @param {number} data.uniqueId
     * @param {number} data.senderAppId
     * @param {string} data.text
     * @param {number} data.fileId
     * @param {Object} data.extraData
     * @param {string} data.createdAt
     * @param {string} data.processedAt
     * @constructor
     */
    function PrivateMessageReceived(fromUserAppId, data) {
      var d = new Date().toISOString();

      //console.log("saving msg to current room = " + (toRoomId === _myCurrentRoomId));

      this.uniqueId = data.uniqueId;
      this.contactId = +data.senderAppId;
      this.groupId = null;
      this.text = data.text;
      this.fileId = data.fileId || null;
      this.extraData = data.extraData || null;
      this.createdAt = data.createdAt;
      this.processedAt = data.processedAt;
      this.receivedAt = d;
      this.isMine = 0;//{0=false,1=true}
      this.status = _myCurrentRoomId === fromUserAppId ? STATUS.MSG_RECEIVER_READ : STATUS.MSG_RECEIVER_SAVED
    }

    /**
     * @param contacAppId {number}
     * @return {number}
     */
    function setChatWithUserId(contacAppId) {
      return _myCurrentRoomId = contacAppId;
    }


    //------------------------ SOCKET HELPERS ------------------------
    let
      _onPeerConnChange,
      _onPeerLeftRoom,
      _onMsgACKByPeer,
      _onMsgArchived;


    /**
     *
     * @type {{isConnected: (function(): boolean), connect: (function(boolean, number, string, string=)), getPendingMsgs: (function(number): Promise<number>), cleanPendingMsgsRcvd: (function(*=): Promise<number>), joinToContactRoom: (function(*=): Promise<any>), createContactRoom: (function(number, boolean, boolean=): boolean), joinToAllContactsRooms: (function(): Promise<any>), joinRoom: (function(number): Promise<{isPeerOnLine: boolean, roomId: string, isInSameRoom: boolean}>), leavePrivateRoom: (function(number): Promise<any>), send: (function(PrivateMessageToSend): Promise<any>), setOnPeerConnChange: setOnPeerConnChange, setOnPeerLeftRoom: setOnPeerLeftRoom, setOnMsgACKByPeer: setOnMsgACKByPeer, setOnMsgArchived: setOnMsgArchived, getHistoricMessages: (function(): Promise<any>)}}
     */
    var socket = {
      isConnected: function () {
          console.log("CHAT ES USABLE");
        return chatIsUsable;
      },
      /**
       * Waits for DB to be initialized (if not so)
       * Tries to connect socket
       * whenever the socket connection state changes, the success callback is fired (and this fn will broadcast such change)
       * when the connection (and reconnection) attemp fails, the error callback is fired
       * @param isSameUser {boolean}
       * @param myId {number}
       * @param name {string}
       * @param avatar {string?}
       * @return {Promise}
       */
      connect: function (isSameUser, myId, name, avatar) {
        console.log("chat es usable ver 2")
        return Utils.$q(function (resolve, reject) {
        console.log("chat es usable ver 3")
          myDB.whenAvailable(function () {
              console.log("chat es usable ver 4")
            //util.loadAllMsgInLS();
            //util.loadAllContactsInLS();

            mySocket.ioConnect({userAppId: myId, userName: name, userAvatar: avatar},
              /**@param resp {{connected: boolean, socketId: string}}*/
              function (resp) {
                chatIsUsable = resp.connected;

                if (chatIsUsable) {
                  _myId = myId;

                  socketId = resp.socketId;
                  _myCurrentRoomId = _myId;
                  _setSocketListeners();

                  //just give time to update things...
                  Utils.$timeout(function () {
                    if (isSameUser) {
                      socket.joinToAllContactsRooms().then(function () {
                        //Retrieve the messages not delivered... to me
                        _getPendingMsgs(_myId).then(function (total) {
                          //console.log("----- total pending messages",total);
                          if (total) {
                            DB.countAllUnreadMessages().then().finally(function () {
                              Utils.toast.getCtrl().info(
                                Utils.translate("PRIVATEMESSAGE.NEW_MSGS_ALERT"),//texto
                                Utils.translate("GLOBAL.HELLO"),//titulo
                                {
                                  iconClass: 'toast-info-coral',//custom class (see style.css)
                                }
                              );
                            });
                          }
                        }, function (/*error*/) {
                        });
                      }, function (errors) {
                        console.error("joinToAllContactsRooms", errors)
                      }).finally(function () {
                        $rootScope.$broadcast("socketIO.connection", chatIsUsable);
                      });
                    }
                    else
                      $rootScope.$broadcast("socketIO.connection", chatIsUsable);
                  }, 1000)
                }
                else {
                  console.log("removing socket listeners due to disconection...");
                  listenersToRemove.forEach(function (listener) {
                    _rmEventListener(listener.evtName, listener.ref);
                  });
                  listenersToRemove = [];

                  _contactRooms.forEach(function (contact) {
                    contact.isOnline = false;
                  });

                  $rootScope.$broadcast("socketIO.connection", chatIsUsable);
                }

                resolve(resp);
              }, function (connError) {
                console.error(connError.message);
                reject(connError);
              });
          });
        })
      },

      getPendingMsgs: _getPendingMsgs,

      cleanPendingMsgsRcvd: _cleanPendingMsgsRcvd,

      /**
       * ONLY FOR RECEIVING NOTIFICATIONS
       * Joins to default contact room, must be called when creating a contact
       * @param contactId
       * @return {Promise<any>}
       */
      joinToContactRoom: function (contactId) {
        return _emit('room:join', contactId).then(function (isOnline) {
          console.log("*** joined to contact's room: %s, contact is online: %s", contactId, isOnline);

          if (contactId !== _myId)
            _contactRooms.push({id: contactId, isOnline: isOnline});

          return Utils.$q.resolve(isOnline);
        }, function (error) {
          console.error("---------- can't join to room %s", _myId, error);
          return Utils.$q.reject(error);
        });
      },

      /**
       * @param contactId {number}
       * @param isOnline {boolean}
       * @param skipCheck {boolean?} - Don't check for existence
       * @return {Boolean}
       */
      createContactRoom: function (contactId, isOnline, skipCheck) {
        var exists = false;
        if (!skipCheck) {
          exists = _contactRooms.some(function (contact) {
            return contact.id === contactId;
          });
        }


        if (!exists) {
          //console.log("New contact (%d) detected, added to _contactRooms!", contactId);
          _contactRooms.push({id: contactId, isOnline: isOnline});
        }

        return exists;
      },

      /**
       * ONLY FOR RECEIVING NOTIFICATIONS
       * Joins to all my contacts rooms to receive their notifications
       * @return {Promise<any>}
       */
      joinToAllContactsRooms: function () {
        return myDB.execute("SELECT id FROM chat_contact_list").then(function (dbResp) {
          _contactRooms = [];//clean the rooms
          var promises = {};
          dbResp.forEachRow(function (item) {
            //console.log("Joining to contact room %s", item.id);
            promises[item.id] = _emit('room:join', item.id).then(function (isOnline) {
              //console.log("-------- joined to contact's room: %s, contact is online: %s", item.id, isOnline);
              _contactRooms.push({id: item.id, isOnline: isOnline});
              return Utils.$q.resolve()
            }, function (error) {
              console.error("---------- can't join to room %s", item.id, error);
              return Utils.$q.reject();
            })
          });
          return Utils.$q.all(promises);
        }, function (dbError) {
            console.error("load contacts in LS", dbError);
            /*return myDB._openDB().then(function () {
              return Utils.$q.resolve(true);
            }, function (error) {
              console.error("Can't recreate Chat tables", error);
              return Utils.$q.reject(error);
            })*/
        })
      },

      /**
       * ONLY FOR CHATTING
       * Joins to a contact room, must be called when entering to the contact's room
       * @param peerAppId {number}
       * @return {Promise<{isPeerOnLine: boolean, roomId: string, isInSameRoom:boolean}>}
       */
      joinRoom: function (peerAppId) {
        return _emit('room.private:join', {peerAppId: peerAppId}).then(
          /**
           * @param joinResp {{isPeerOnLine: boolean, roomId: number, isInSameRoom:boolean}}
           * @return {Promise<{isPeerOnLine: boolean, roomId: number, isInSameRoom:boolean}>}
           */
          function (joinResp) {
            _myCurrentRoomId = joinResp.roomId;
            return Utils.$q.resolve(joinResp)
          }, function (error) {
            return Utils.$q.reject(error)
          });
      },


      /**
       * @param roomId {number}
       * @return {Promise<any>}
       */
      leavePrivateRoom: function (roomId) {
        return _emit('room.private:leave', roomId).then(function (oldRoomId) {
          _myCurrentRoomId = _myId;
          return Utils.$q.resolve(oldRoomId)
        }, function (error) {
          return Utils.$q.reject(error)
        });
      },

      /**
       *
       * @param {PrivateMessageToSend} data
       * @return {Promise<any>}
       */
      send: function (data) {
        return _DB_insertMsg({
          uniqueId: data.uniqueId,
          contactId: data.contactId,
          groupId: data.groupId,
          text: data.text,
          fileId: data.fileId,
          extraData: data.extraData,
          createdAt: data.createdAt,
          processedAt: data.processedAt,
          receivedAt: data.receivedAt,
          isMine: data.isMine,
          status: data.status
        }).then(function (insertedId) {
          //console.log("The message was inserted with id", insertedId);
          //Remove unnecessary data for socket
          /*delete data.fileId;
          delete data.isMine;
          delete data.status;
          delete data.processedAt;
          delete data.receivedAt;*/

          //remove undesired data
          var msgToSend = {
            uniqueId: data.uniqueId,
            groupId: data.groupId,
            contactId: data.contactId,
            text: data.text,
            extraData: data.extraData,
            createdAt: data.createdAt
          };

          //add file data if needed
          if (data.fileId) {
            msgToSend.fileType = data.fileType;
            msgToSend.fileSrc = data.fileSrc;
            msgToSend.fileMimeType = data.fileMimeType;
          }


          return _emit('message.private:send', msgToSend).then(function (resp) {
            //console.log("(send:CB) Updating msg status to: ", resp);
            _DB_updateSentMsgById(insertedId, resp).then(function (rowsAffected) {
              if (rowsAffected)
                console.log("The msg inserted with id %d was updated to: ", insertedId, resp);
              else
                console.error("The msg inserted with id %d couldn't be updated to", insertedId, resp);
            }, function (error) {
              console.error("msg updated", error)
            });
            return Utils.$q.resolve(resp)
          }, function (error) {
            return Utils.$q.reject(error)
          })
        }, function (dbError) {
          return Utils.$q.reject(dbError)
        })
      },

      /**
       *
       * @param fn
       * @return {*}
       */
      setOnPeerConnChange: function (fn) {
        if (typeof fn !== 'function') {
          console.warn('setOnPeerConnChange: the parameter is not a function');
          return _onPeerConnChange = angular.noop;
        }

        _onPeerConnChange = fn;

        return function () {
          _rmEventListener('peer.connection:change', fn);
        }
      },

      /**
       *
       * @param fn
       * @return {*}
       */
      setOnPeerLeftRoom: function (fn) {
        if (typeof fn !== 'function') {
          console.warn('setOnPeerLeftRoom: the parameter is not a function');
          return _onPeerLeftRoom = angular.noop;
        }

        _onPeerLeftRoom = fn;

        return function () {
          _rmEventListener('room.private:userLeft', fn);
        }
      },

      /**
       *
       * @param fn
       * @return {*}
       */
      setOnMsgACKByPeer: function (fn) {
        if (typeof fn !== 'function') {
          console.warn('setOnMsgReceivedByPeer: the parameter is not a function');
          return _onMsgACKByPeer = angular.noop;
        }

        _onMsgACKByPeer = fn;

        return function () {
          _rmEventListener('message.private:peerACK', fn);
        }
      },

      setOnMsgArchived: function (fn) {
        if (typeof fn !== 'function') {
          console.warn('setOnMsgArchived: the parameter is not a function');
          return _onMsgArchived = angular.noop;
        }

        _onMsgArchived = fn;

        return function () {
          _rmEventListener('message.private:archived', fn);
        }
      },

      /**
       *
       * @return {Promise<any>}
       */
      getHistoricMessages: function () {
        //$rootScope.$broadcast('message.historic:loading');

        return _emit("message.private:getHistoric", _myId).then(function (messageList) {
          //console.log("----- getHistoricMessages: Your historic messages are", messageList);
          if (!messageList || !messageList.length) {
            $rootScope.$broadcast('message.historic:loaded', 0);
            return Utils.$q.resolve(0);
          }

          var insertFilePromises = {};
          var insertPromises = [];

          return _pendingMsgSaveOrUpdateContact(messageList, true/*for historic*/).then(function () {
            messageList.forEach(function (msg) {
              msg.contactId = msg.senderAppId !== _myId ? msg.senderAppId : msg.receiverAppId;
              msg.fileId = null;
              msg.isMine = msg.senderAppId === _myId ? 1 : 0;

              msg.status = STATUS.MSG_RECEIVER_READ;//_myCurrentRoomId === msg.contactId ? STATUS.MSG_RECEIVER_READ : STATUS.MSG_RECEIVER_SAVED;

              if (msg.fileSrc) {
                var ext = msg.fileSrc.split('.');
                ext = ext.length > 1 ? ext.pop() : '';

                insertFilePromises[msg.contactId] = DB.insertNewFile({
                  fileType: msg.fileType,
                  fileMimeType: msg.fileMimeType,
                  fileExtension: ext,
                  fileSrc: msg.fileSrc
                }).then(function (id) {
                  msg.fileId = id;
                  insertPromises.push(__insertMsgOnDB(msg));

                  return Utils.$q.resolve(id);
                }, function (error) {
                  console.error("getHistoricMessages::insertNewFile", error);
                  insertPromises.push(__insertMsgOnDB(msg));

                  return Utils.$q.reject(error);
                });
              }
              else {
                insertPromises.push(__insertMsgOnDB(msg));
              }
            });


            return Utils.$q.all(insertFilePromises).then(function (fileResponses) {
              return Utils.$q.all(insertPromises).then(function (responses) {
                //console.log("----- insert responses", responses);
                var totalFetched = responses.length;

                _emit('message.pending:clear', _myId).then(function (resp) {
                  console.log("getHistoricMessages:message.pending:clear OK", resp)
                }, function (error) {
                  console.error("getHistoricMessages: message.pending:clear error", error)
                });

                util.loadAllMsgInLS();
                util.loadAllContactsInLS();

                $rootScope.$broadcast('message.historic:loaded', totalFetched);

                return Utils.$q.resolve(totalFetched);
              }, function (errors) {
                $rootScope.$broadcast('message.historic:loaded', 0);
                return Utils.$q.reject(errors);
              })
            }, function (errors) {
              $rootScope.$broadcast('message.historic:loaded', 0);
              return Utils.$q.reject(errors);
            });
          }, function (errors) {
            $rootScope.$broadcast('message.historic:loaded', 0);
            return Utils.$q.reject(errors);
          });

          function __insertMsgOnDB(msg) {
            return _DB_insertMsg(msg).then(function (insertedId) {
              return Utils.$q.resolve(insertedId)
            }, function (error) {
              console.error("Couldn't insert the pending messages", error);
              return Utils.$q.reject(error)
            })
          }
        }, function (error) {
          $rootScope.$broadcast('message.historic:loaded', 0);
          console.error("Couldn't get socket::DB.getHistoric()", error);
          return Utils.$q.reject(error);
        });
      }
    };

    function _setSocketListeners() {
      console.log("setting socket listeners...");
      mySocket.on('peer.connection:change', _OnPeerConnChange);
      listenersToRemove.push({evtName: 'peer.connection:change', ref: _OnPeerConnChange});

      mySocket.on('room.private:userLeft', _OnPeerLeftRoom);
      listenersToRemove.push({evtName: 'room.private:userLeft', ref: _OnPeerLeftRoom});

      mySocket.on('message.private:new', _OnMsgPrivateNew);
      listenersToRemove.push({evtName: 'message.private:new', ref: _OnMsgPrivateNew});

      mySocket.on('message.private:archived', _OnMsgPrivateArchived);
      listenersToRemove.push({evtName: 'message.private:archived', ref: _OnMsgPrivateArchived});

      mySocket.on('message.private:receiver-acknowledged', _OnMsgReceiverAcknowledged);
      listenersToRemove.push({evtName: 'message.private:receiver-acknowledged', ref: _OnMsgReceiverAcknowledged});
    }

    /**
     *
     * @param data {{userAppId:number, isOnline:boolean}}
     * @private
     */
    function _OnPeerConnChange(data) {
      //console.log("------- FACTORY: The user %d connection is online: %s", data.userAppId, data.isOnline);
      _contactRooms.some(function (contact) {
        if (contact.id === data.userAppId) {
          contact.isOnline = data.isOnline;
          return true;
        }
      });

      (_onPeerConnChange || angular.noop)(data);
    }

    function _OnPeerLeftRoom(data) {
      console.log("The user has left the room", data);
      (_onPeerLeftRoom || angular.noop)(data);
    }

    /**
     * @param ioResp {{msg: *, fromUserAppId: number, senderName: string}}
     * @private
     */
    function _OnMsgPrivateNew(ioResp) {
      console.log("----- You have a new message!", ioResp);
      //console.log(ioResp.fromUserAppId, _myCurrentRoomId);

      if (ioResp.msg.fileSrc) {
        var ext = ioResp.msg.fileSrc.split('.');
        ext = ext.length > 1 ? ext.pop() : '';

        DB.insertNewFile({
          fileType: ioResp.msg.fileType,
          fileMimeType: ioResp.msg.fileMimeType,
          fileExtension: ext,
          fileSrc: ioResp.msg.fileSrc
        }).then(function (id) {
          var fileInfo = {
            id: id,
            fileType: ioResp.msg.fileType,
            fileSrc: ioResp.msg.fileSrc
          };
          _finalizeInsertNewMsg(ioResp, fileInfo);
        }, function (error) {
          console.error("_OnMsgPrivateNew::insertNewFile", error);
          _finalizeInsertNewMsg(ioResp);
        });
      }
      else
        _finalizeInsertNewMsg(ioResp);
    }

    /**
     *
     * @param ioResp {*}
     * @param fileInfo {{id:number, fileType:string, fileSrc:string}?}
     * @private
     */
    function _finalizeInsertNewMsg(ioResp, fileInfo) {
      fileInfo = fileInfo || {};

      /**@type {PrivateMessageReceived}*/
      var appMsg = new PrivateMessageReceived(ioResp.fromUserAppId, ioResp.msg);

      appMsg.fileId = fileInfo.id;

      //insert Attributes to show in template
      appMsg.fileType = fileInfo.fileType;
      appMsg.fileSrc = fileInfo.fileSrc;

      _DB_insertMsg(appMsg).then(function (/*insertedId*/) {
        if (ioResp.fromUserAppId !== _myCurrentRoomId) {
          Utils.toast.info(ioResp.senderName + Utils.translate('PRIVATEMESSAGE.APPALERT_NEWMSG'), 5000, "top");
          $rootScope.$broadcast('chat.messages.unread', {total: ++_totalUnreadMsgs});
        }

        var exists = _contactRooms.some(function (contact) {
          return contact.id === ioResp.fromUserAppId;
        });

        var forThisRoom = ioResp.fromUserAppId === _myCurrentRoomId ? true : false;

        if (!exists) {
          DB.insertNewContact({
            id: ioResp.fromUserAppId,
            name: ioResp.senderName,
            avatar: '',
            bio: ''
          }).then(function () {
            socket.createContactRoom(ioResp.fromUserAppId, true);

            Utils.$timeout(function () {
              _ackReceivedMessage(ioResp.fromUserAppId, appMsg, forThisRoom);
            }, 1000);
          }, function (error) {
            console.error("_OnMsgPrivateNew::insertNewContact", error);
          });
        }
        else {
          Utils.$timeout(function () {
            _ackReceivedMessage(ioResp.fromUserAppId, appMsg, forThisRoom);
          }, 1000);
        }
      }, function (error) {
        console.error("_OnMsgPrivateNew::_DB_insertMsg", error)
      });
    }

    function _ackReceivedMessage(fromUserAppId, appMsg, ftRoom) {
      _emit('message.private:receiver-acknowledge', {
        senderAppId: appMsg.contactId,
        fromUserAppId: fromUserAppId,
        status: appMsg.status,
        processedAt: appMsg.processedAt,
        receivedAt: appMsg.receivedAt,
        uniqueId: appMsg.uniqueId
      }).then(function (resp) {
        console.log("message.private:receiver-acknowledge has been ack'ed by server", resp)
      }, function (error) {
        console.log("message.private:receiver-acknowledge couldn't be ack'ed by server", error)
      });

      if(ftRoom)
        $rootScope.$broadcast('message.private:new', {msg: appMsg});
    }

    /**
     *
     * @param data {{uniqueId: number, processedAt:string}}
     * @private
     */
    function _OnMsgPrivateArchived(data) {
      console.log("***** The message timed out", data);
      myDB.execute(
        "UPDATE chat_message_list SET status = ?, processedAt = ? WHERE uniqueId = ?",
        [STATUS.MSG_ARCHIVED, data.processedAt, data.uniqueId]
      ).then(function (affectedRows) {
        if (affectedRows)
          (_onMsgArchived || angular.noop)(data)
      }, function (dbError) {
        console.error("_OnMsgPrivateArchived", dbError);
      })
    }

    /**
     *
     * @param ackObj {{fromUserAppId: number, status: number, processedAt:string, receivedAt:string, uniqueId:number}}
     * @private
     */
    function _OnMsgReceiverAcknowledged(ackObj) {
      //console.log("--> _OnMsgReceiverAcknowledged", ackObj);
      myDB.execute(
        'UPDATE chat_message_list SET status = ?, processedAt = ? , receivedAt = ? WHERE uniqueId = ?',
        [ackObj.status, ackObj.processedAt, ackObj.receivedAt, ackObj.uniqueId]
      ).then(function (dbResp) {
        if (dbResp.rowsAffected)
          console.log("---Your sent message status was updated to %d", ackObj.status);
        else
          console.warn("--- The sent (ack'ed) msg wasn't updated (it may be not found)")
      }, function (dbError) {
        console.error("Couldn't update the sent (ack'ed) msg", dbError);
      });

      (_onMsgACKByPeer || angular.noop)(ackObj);
    }

    /**
     *
     * @param {number} myId
     * @return {Promise<number>} - total
     * @private
     */
    function _getPendingMsgs(myId) {
      return _emit('message.private:getPending', myId).then(function (messageList) {
        //console.log("------ message.private:getPending", messageList);
        if (!messageList || !messageList.length) return Utils.$q.resolve(0);

        var insertFilePromises = {};
        var insertPromises = [];
        var timestamp = new Date().toISOString();

        return _pendingMsgSaveOrUpdateContact(messageList).then(function () {
          messageList.forEach(function (msg) {
            msg.contactId = msg.senderAppId;
            msg.fileId = null;
            msg.receivedAt = timestamp;
            msg.isMine = 0;

            msg.status = _myCurrentRoomId === msg.senderAppId ? STATUS.MSG_RECEIVER_READ : STATUS.MSG_RECEIVER_SAVED;

            if (msg.fileSrc) {
              var ext = msg.fileSrc.split('.');
              ext = ext.length > 1 ? ext.pop() : '';

              insertFilePromises[msg.contactId] = DB.insertNewFile({
                fileType: msg.fileType,
                fileMimeType: msg.fileMimeType,
                fileExtension: ext,
                fileSrc: msg.fileSrc
              }).then(function (id) {
                msg.fileId = id;
                insertPromises.push(__insertMsgOnDB(msg));

                return Utils.$q.resolve(id);
              }, function (error) {
                console.error("_OnMsgPrivateNew::insertNewFile", error);
                insertPromises.push(__insertMsgOnDB(msg));

                return Utils.$q.reject(error);
              });
            }
            else {
              insertPromises.push(__insertMsgOnDB(msg));
            }
          });


          return Utils.$q.all(insertFilePromises).then(function (fileResponses) {
            return Utils.$q.all(insertPromises).then(function (responses) {
              //console.log("----- insert responses", responses);
              var totalFetched = responses.length;

              _emit('message.pending:saved', {receiverAppId: myId, receivedAt: timestamp}).then(function (resp) {
                console.log("message.pending:saved has been ack'ed by server: affectedRows", resp)
              }, function (error) {
                console.log("message.pending:saved couldn't be ack'ed by server", error)
              });

              $rootScope.$broadcast('message.pending:loaded', totalFetched);

              return Utils.$q.resolve(totalFetched);
            }, function (errors) {
              return Utils.$q.reject(errors);
            })
          }, function (errors) {
            return Utils.$q.reject(errors);
          })
        }, function (errors) {
          return Utils.$q.reject(errors);
        });

        function __insertMsgOnDB(msg) {
          return _DB_insertMsg(msg).then(function (insertedId) {
            return Utils.$q.resolve(insertedId)
          }, function (error) {
            console.error("Couldn't insert the pending messages", error);
            return Utils.$q.reject(error)
          })
        }
      }, function (error) {
        console.error("Couldn't get your pending messages", error);
        return Utils.$q.reject(error)
      })
    }

    /**
     *
     * @param messageList {Array<any>}
     * @param forHistoric {boolean?}
     * @return {Promise<any>}
     * @private
     */
    function _pendingMsgSaveOrUpdateContact(messageList, forHistoric) {
      //console.log("messageList", messageList);
      var usersInfoPromises = {};//a set of no-repeated contacts

      messageList.forEach(function (msg) {
        //if it's for historic the id comes in msg.contactId
        var contactId = forHistoric ? (msg.senderAppId !== _myId ? msg.senderAppId : msg.receiverAppId) : msg.senderAppId;
        //console.log(contactId);
        if (!usersInfoPromises[contactId]) {
          //console.log("inserting new contact: %d", contactId);
          usersInfoPromises[contactId] = UserService.getUserSubscriptionByinstanceIdProfileData(contactId).$promise;
        }
      });

      return Utils.$q.all(usersInfoPromises).then(function (resolvedResps) {
        var insertOrUpdatePromises = [];

        Object.keys(resolvedResps).forEach(function (userId) {
          var appUserId = +userId;//needs to be a number
          var exists = _contactRooms.some(function (contact) {
            return contact.id === appUserId;
          });

          if (exists) {
            insertOrUpdatePromises.push(
              DB.updateContactInfo({
                id: appUserId,
                name: resolvedResps[appUserId][0].name,
                avatar: resolvedResps[appUserId][0].profilePictureUrl,
                bio: ''//resolvedResps[appUserId][0].????
              }).then(function (affectedRows) {
                return Utils.$q.resolve({updated: affectedRows > 0})
              }, function (error) {
                console.error("couldn't UPDATE new contact @ _pendingMsgSaveOrUpdateContact", error);
                return Utils.$q.reject(error);
              })
            )
          }
          else {
            insertOrUpdatePromises.push(
              DB.insertNewContact({
                id: appUserId,
                name: resolvedResps[appUserId][0].name,
                avatar: resolvedResps[appUserId][0].profilePictureUrl,
                bio: ''//resolvedResps[appUserId][0].????
              }).then(function (id) {
                return socket.joinToContactRoom(appUserId).then(function (isOnline) {
                  _contactRooms.push({id: appUserId, isOnline: isOnline});
                  return Utils.$q.resolve({id: id})
                }, function (/*error*/) {
                  _contactRooms.push({id: appUserId, isOnline: false});
                  return Utils.$q.resolve({id: id})
                });
              }, function (error) {
                console.error("couldn't INSERT new contact @ _pendingMsgSaveOrUpdateContact", error);
                return Utils.$q.reject(error);
              })
            )
          }
        });

        return Utils.$q.all(insertOrUpdatePromises);
      }, function (errors) {
        console.error("Couldn't get the Contact's info @ _pendingMsgSaveOrUpdateContact", errors);
        return Utils.$q.reject(errors);
      });

    }

    /**
     * Cleans all pending messages marked as received (with receivedAt date) by peer
     * @param myId
     * @return {Promise<number>} - total msgs deleted
     * @private
     */
    function _cleanPendingMsgsRcvd(myId) {
      return _emit('message.pending:cleanReceived', myId).then(function (deliveredMsgList) {
        //console.log("deliveredMsgList", deliveredMsgList);
        var statements = [];
        deliveredMsgList.forEach(function (deliveredMsg) {
          statements.push([
            "UPDATE chat_message_list SET status = ?, receivedAt = ? WHERE uniqueId = ?",
            [STATUS.MSG_RECEIVER_READ, deliveredMsg.receivedAt, deliveredMsg.uniqueId]
          ])
        });

        var totalStatements = statements.length;
        if (totalStatements) {
          return myDB.batch(statements).then(function (/*nothing*/) {
            console.log("------- %s pending delivered messages were confirmed successfully", totalStatements);
            return Utils.$q.resolve(totalStatements);
          }, function (dbError) {
            console.error("Couldn't update the pending delivered messages", dbError);
            return Utils.$q.reject(dbError);
          })
        }
        else
          return Utils.$q.resolve(0);
      }, function (error) {
        console.error("emit message.pending:cleanReceived", error);
        return Utils.$q.reject(error);
      })
    }

    /**
     *
     * @param evtName
     * @param fncReference
     * @private
     */
    function _rmEventListener(evtName, fncReference) {
      if (typeof fncReference === 'function') {
        //console.log("Unsubscribing from evt: " + evtName);
        mySocket.removeListener(evtName, fncReference);
      }
    }


    /**
     * @param eventName
     * @param dataToSend
     * @return {Promise}
     */
    function _emit(eventName, dataToSend) {
      return Utils.$q(function (resolve, reject) {
        mySocket.emit(eventName, dataToSend, function (response) {
          response.success ? resolve(response.data) : reject(response.data);
        });
      })
    }

    //======================= END SOCKET HELPERS =======================


    //-------------------------- DB HELPERS --------------------------
    /**
     *
     *
     * @type {{getAllContacts: (function(): Promise<Array<any>>), getInboxContacts: (function(): Promise<any>), getCurrentGroups: (function(): Promise<any>), getCurrentRooms: (function(): Promise<any>), getInboxContact: (function(number): Promise<any>), countAllUnreadMessages: (function(): Promise<number>), insertNewContact: (function({id: number, name: string, avatar: string, bio: string}): Promise<number>), updateContactInfo: (function({id: number, name: string, avatar: string, bio: string}): Promise<number>), insertNewFile: (function({fileType: string, fileMimeType: string, fileExtension: string, fileSrc: string}): Promise<number>), getTotalMsgsFromContac: (function(*): Promise<number>), getAllContactMsgs: (function(*, number=, number=): Promise<any[]>), cleanDB: (function(): Promise<any>)}}
     */
    var DB = {
      /**
       *
       * @return {Promise<Array<any>>}
       */
      getAllContacts: function () {
        return myDB.execute(
          "SELECT * FROM chat_contact_list WHERE status = 1"
        ).then(function (dbResp) {
          return Utils.$q.resolve(dbResp.extractItems())
        }, function (dbError) {
          console.error("", dbError);
          return Utils.$q.reject(dbError);
        });
      },
      /**
       *
       * @return {Promise<any>}
       */
      getInboxContacts: function () {
        return myDB.execute(
          "SELECT id, name, avatar, description FROM chat_contact_list WHERE status = 1"
        ).then(function (dbResp) {
          var items = [];
          var lastMsgStatements = [];
          var unreadMsgStatements = [];
          dbResp.extractItems().forEach(function (item) {
            var isOnline = false;
            _contactRooms.some(function (contact) {
              if (contact.id === item.id) {
                isOnline = contact.isOnline;
                return true;
              }
            });
            items.push({
              socket: {isOnline: isOnline},
              user: {
                id: item.id,
                name: item.name,
                avatar: item.avatar,
                description: item.description
              },
              message: {}
            });

            //because the records are inserted unsorted (not as the were retrieved in getHistoric function)
            //We need to sort them first by processedAt (desc) and choose the older one
            lastMsgStatements.push([
              "SELECT id, uniqueId, text, fileId, isMine, createdAt, processedAt, receivedAt FROM chat_message_list WHERE id = " +
              "(SELECT id FROM chat_message_list WHERE contactId = ? ORDER BY processedAt DESC LIMIT 1)", [item.id]
            ]);

            unreadMsgStatements.push([
              "SELECT COUNT(id) AS total FROM chat_message_list WHERE isMine = 0 AND contactId = ? AND status = ?",
              [item.id, STATUS.MSG_RECEIVER_SAVED]
            ]);
          });

          if (!lastMsgStatements.length) return Utils.$q.resolve(items);

          var msgPromises = [];
          lastMsgStatements.forEach(function (statement) {
            //console.log(statement[0]);
            msgPromises.push(myDB.execute(statement[0], statement[1]));
          });

          var unreadMsgpromises = [];
          unreadMsgStatements.forEach(function (statement) {
            unreadMsgpromises.push(myDB.execute(statement[0], statement[1]));
          });

          return Utils.$q.all({
            msg: Utils.$q.all(msgPromises),
            unread: Utils.$q.all(unreadMsgpromises)
          }).then(function (allResponses) {
            _totalUnreadMsgs = 0;
            allResponses.msg.forEach(function (dbResp, idx) {
              if (dbResp.rows.length) {
                items[idx].message = {
                  id: dbResp.rows.item(0).id,
                  uniqueId: dbResp.rows.item(0).uniqueId,
                  text: dbResp.rows.item(0).text,
                  fileId: dbResp.rows.item(0).fileId,
                  createdAt: dbResp.rows.item(0).createdAt,
                  processedAt: dbResp.rows.item(0).processedAt,
                  receivedAt: dbResp.rows.item(0).receivedAt,
                  date: (dbResp.rows.item(0).isMine ? dbResp.rows.item(0).createdAt : dbResp.rows.item(0).receivedAt),
                  numUnreadMsgs: allResponses.unread[idx].rows.item(0).total
                };

                _totalUnreadMsgs += items[idx].message.numUnreadMsgs;
              }
            });

            $rootScope.$broadcast('chat.messages.unread', {total: _totalUnreadMsgs});

            //console.log("items", items);
            return Utils.$q.resolve(items)
          }, function (dbErrors) {
            console.error("dbErrors", dbErrors);
            return Utils.$q.reject(dbErrors);
          });
        }, function (dbError) {
          return Utils.$q.reject(dbError)
        })
      },

      /**
       *
       * @return {Promise<any>}
       */
      getCurrentGroups: function () {
        return myDB.execute(
          "SELECT id, name, avatar FROM chat_group_list WHERE status = 1"
        ).then(function (dbResp) {
          var items = dbResp.extractItems();
          console.log("getCurrentGroups", items);
          return Utils.$q.resolve(items)
        }, function (dbError) {
          return Utils.$q.reject(dbError)
        })
      },

      /**
       *
       * @return {Promise<any>}
       */
      getCurrentRooms: function () {
        return myDB.execute(
          "SELECT id, name, avatar, description FROM chat_contact_list WHERE status = 1"
        ).then(function (dbResp) {
          var items = dbResp.extractItems();
          //console.log("getCurrentRooms", items);
          return Utils.$q.resolve(items)
        }, function (dbError) {
          return Utils.$q.reject(dbError)
        })
      },

      /**
       *
       * @param contactId {number}
       * @return {Promise<any>}
       */
      getInboxContact: function (contactId) {
        return myDB.execute(
          "SELECT id, name, avatar, description FROM chat_contact_list WHERE id = ? AND status = 1",
          [contactId]
        ).then(function (dbResp) {
          var contact = {};
          dbResp.extractItems().forEach(function (item) {
            var isOnline = false;
            _contactRooms.some(function (contact) {
              if (contact.id === item.id) {
                isOnline = contact.isOnline;
                return true;
              }
            });
            contact.socket = {isOnline: isOnline};
            contact.user = {
              id: item.id,
              name: item.name,
              avatar: item.avatar,
              description: item.description
            };
            contact.message = {};
          });

          return Utils.$q.resolve(contact);
        }, function (dbError) {
          return Utils.$q.reject(dbError)
        })
      },

      /**
       *
       * @return {Promise<number>}
       */
      countAllUnreadMessages: function () {
        return myDB.execute("SELECT id FROM chat_contact_list WHERE status = 1").then(function (dbResp) {
          /**@type {Array<Promise<DBResponseType>>}*/
          var unreadMsgpromises = [];
          dbResp.forEachRow(function (item) {
            unreadMsgpromises.push(
              myDB.execute(
                "SELECT COUNT(id) AS total FROM chat_message_list WHERE isMine = 0 AND contactId = ? AND status = ?",
                [item.id, STATUS.MSG_RECEIVER_SAVED]
              )
            );
          });

          return Utils.$q.all(unreadMsgpromises).then(function (allResponses) {
            //console.log("countUnreadMsgs: allResponses", allResponses);
            _totalUnreadMsgs = 0;

            allResponses.forEach(function (resp) {
              resp.forEachRow(function (item) {
                //console.log(item);
                _totalUnreadMsgs += item.total;
              });
            });

            $rootScope.$broadcast('chat.messages.unread', {total: _totalUnreadMsgs});

            return Utils.$q.resolve(_totalUnreadMsgs)
          }, function (errors) {
            return Utils.$q.reject(errors)
          })
        })
      },

      /**
       * @param data {{id: number, name: string, avatar: string, bio:string}}
       * @return {Promise<number>}
       */
      insertNewContact: function (data) {
        return myDB.execute(
          "INSERT INTO chat_contact_list (id, name, avatar, description, status) VALUES (?, ?, ?, ?, ?)",
          [data.id, data.name, data.avatar, data.bio, 1]
        ).then(function (dbResp) {
          return Utils.$q.resolve(dbResp.insertId)
        }, function (dbError) {
          return Utils.$q.reject(dbError)
        })
      },

      /**
       * @param data {{id: number, name: string, avatar: string, bio:string}}
       * @return {Promise<number>} - affected rows
       */
      updateContactInfo: function (data) {
        return myDB.execute(
          "UPDATE chat_contact_list set name = ?, avatar = ?, description = ? WHERE id = ?",
          [data.name, data.avatar, data.bio, data.id]
        ).then(function (dbResp) {
          return Utils.$q.resolve(dbResp.rowsAffected)
        }, function (dbError) {
          return Utils.$q.reject(dbError)
        })
      },

      /**
       *
       * @param data  {{fileType: string, fileMimeType: string, fileExtension: string, fileSrc: string}}
       * @return {Promise<number>}
       */
      insertNewFile: function (data) {
        return myDB.execute(
          "INSERT INTO chat_file_list (type, mimeType, extension, src) VALUES (?,?,?,?)",
          [data.fileType, data.fileMimeType, data.fileExtension, data.fileSrc]
        ).then(function (dbResp) {
          return Utils.$q.resolve(dbResp.insertId);
        }, function (dbError) {
          console.error("", dbError);
          return Utils.$q.reject(dbError);
        })
      },

      /**
       *
       * @param contactId
       * @return {Promise<number>}
       */
      getTotalMsgsFromContac: function (contactId) {
        return myDB.execute(
          "SELECT COUNT(id) AS total FROM chat_message_list WHERE contactId = ?", [contactId]
        ).then(function (dbResp) {
          return Utils.$q.resolve(dbResp.rows.item(0).total)
        }, function (dbError) {
          console.error("get TotalMsgsFromContac", dbError);
          return Utils.$q.reject(dbError)
        })
      },

      /**
       *
       * @param contactId
       * @param start {number?}
       * @param count {number?}
       * @return {Promise<[any]>}
       */
      getAllContactMsgs: function (contactId, start, count) {
        return myDB.execute(
          "SELECT * FROM chat_message_list WHERE contactId = ? ORDER BY createdAt LIMIT ?, ?",
          [contactId, start || 0, count || -1]
        ).then(function (dbResp) {
          var msglistLen = dbResp.rows.length;
          var msgs = [];

          if (!msglistLen) return Utils.$q.resolve(msgs);

          var arrLen = 0;

          var unreadMsgIds = [];

          var filePromises = {};
          dbResp.forEachRow(function (item, idx) {
            //console.log(idx, item);

            var dDiv = '';
            if (idx === 0)
              dDiv = new Date().toISOString().slice(0, 10);
            else if (item.createdAt.slice(0, 10) !== msgs[arrLen - 1].msg.createdAt.slice(0, 10)) {
              dDiv = item.createdAt.slice(0, 10)
            }

            arrLen = msgs.push({
              dividerDate: dDiv,
              msg: item,
              mediaProgress: 100
            });

            if (!item.fileId) {
              if (!item.isMine && item.status === STATUS.MSG_RECEIVER_SAVED)
                unreadMsgIds.push(item.id);

              return;
            }

            //console.log("Qry file id: ", item.fileId);
            filePromises[item.id] = myDB.execute(
              "SELECT * FROM chat_file_list WHERE id = ?", [item.fileId]
            ).then(function (dbResp) {
              //console.log(dbResp);
              //insert new attributes to show in chat template
              item.fileSrc = dbResp.rows.item(0).src;
              item.fileType = dbResp.rows.item(0).type;

              if (!item.isMine && item.status === STATUS.MSG_RECEIVER_SAVED)
                unreadMsgIds.push(item.id)
            }, function (dbError) {
              console.error("Couldn't get the file info", dbError)
            })
          });

          return Utils.$q.all(filePromises).then(function () {
            _DB_updateUnreadMsgs(unreadMsgIds).then();
            return Utils.$q.resolve(msgs);
          }, function (errors) {
            return Utils.$q.reject(errors);
          })
        }, function (dbError) {
          return Utils.$q.reject(dbError)
        })
      },

      cleanDB: function () {
        var statements = [
          ["DROP TABLE chat_contact_list"],
          ["DROP TABLE chat_message_list"],
          ["DROP TABLE chat_group_list"],
          ["DROP TABLE chat_group_contact_rel"],
          ["DROP TABLE chat_file_list"],
        ];

        _totalUnreadMsgs = 0;
        _myCurrentRoomId = 0;

        return myDB.batch(statements).then(function () {
          return myDB.loadDB().then(function () {
            return Utils.$q.resolve(true);
          }, function (error) {
            console.error("Can't recreate Chat tables", error);
            return Utils.$q.reject(error);
          })
        }, function (error) {
          console.error("Clean db before logout", error);
          return Utils.$q.reject(errors);
        });
      }
    };

    /**
     * Inserts an incoming/outgoing message
     * @param {Object} data
     * @param {number} data.uniqueId,
     * @param {number} data.contactId
     * @param {number} data.groupId
     * @param {string} data.text
     * @param {number} data.fileId
     * @param {string} data.extraData
     * @param {string} data.createdAt
     * @param {string} data.processedAt
     * @param {string} data.receivedAt
     * @param {number} data.isMine
     * @param {number} data.status
     * @return {Promise<number>} - the inserted id
     * @private
     */
    function _DB_insertMsg(data) {
      //console.log("inserting msg", data);
      return myDB.execute(
        "INSERT INTO chat_message_list " +
        "(uniqueId, contactId, groupId, text, fileId, extraData, createdAt, processedAt, receivedAt, isMine, status) " +
        "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)",
        [data.uniqueId, data.contactId, data.groupId, data.text, data.fileId, data.extraData, data.createdAt, data.processedAt, data.receivedAt, data.isMine, data.status]
      ).then(function (dbResp) {
        return Utils.$q.resolve(dbResp.insertId)
      }, function (dbError) {
        return Utils.$q.reject(dbError)
      })
    }

    /**
     *
     * @param msgId {number}
     * @param ioResp {{status:STATUS, uniqueId: number, processedAt:string}}
     * @return {Promise<number>} - affected rows
     * @private
     */
    function _DB_updateSentMsgById(msgId, ioResp) {
      return myDB.execute(
        "UPDATE chat_message_list SET status = ?, uniqueId = ?, processedAt = ? WHERE id = ? AND status < ?",
        [ioResp.status, ioResp.uniqueId, ioResp.processedAt, msgId, ioResp.status]
      ).then(function (dbResp) {
        return Utils.$q.resolve(dbResp.rowsAffected);
      }, function (dbError) {
        return Utils.$q.reject(dbError)
      })
    }

    /**
     *
     * @param {Array<number>} msgIdList
     * @return {Promise<number>}
     * @private
     */
    function _DB_updateUnreadMsgs(msgIdList) {
      //console.log("Unread msg ids to be updated", msgIdList);
      if (!msgIdList.length) return Utils.$q.resolve(0);

      var updateStatements = [];
      msgIdList.forEach(function (id) {
        updateStatements.push([
          "UPDATE chat_message_list SET status = ? WHERE id = ?", [STATUS.MSG_RECEIVER_READ, id]
        ])
      });

      return myDB.batch(updateStatements).then(function () {
        //console.log("All unread messages were updated!");
        _totalUnreadMsgs -= updateStatements.length;

        $rootScope.$broadcast('chat.messages.unread', {total: _totalUnreadMsgs});

        return Utils.$q.resolve(updateStatements.length)
      }, function (errors) {
        console.error("_updateUnreadMsgs", errors);
        return Utils.$q.reject(errors)
      })
    }

    //======================== END DB HELPERS ========================


    //-------------------------- util HELPERS --------------------------
    /**
     *
     * @type {{loadAllMsgInLS: loadAllMsgInLS, loadAllContactsInLS: loadAllContactsInLS, generateAnchors: (function(string): string), extractLinks: (function(*=): Array), generateLinkPreview: (function(*): *), selectImageForMsg: (function()), uploadPrivateMsgImage: (function({imgURI: string, fromId: number, toId: number, msgId: number}): *), getDivider: (function(number, string, string): string), sendPushNotification: (function(*=): Promise)}}
     */
    var util = {
      loadAllMsgInLS: function () {
        console.log("loading msgs to LS...");
        myDB.execute("SELECT * FROM chat_message_list").then(function (dbResp) {
          //console.log("loadAllMsgInLS dbResp", dbResp);
          localStorage.setItem("chatMessages", JSON.stringify(dbResp.extractItems()));
        }, function (dbError) {
          console.error("load msg in LS", dbError)
        })
      },

      loadAllContactsInLS: function () {
        console.log("loading contacts to LS...");
        myDB.execute("SELECT * FROM chat_contact_list").then(function (dbResp) {
          //console.log("load AllContactsInLS dbResp", dbResp);
          localStorage.setItem("chatContacts", JSON.stringify(dbResp.extractItems()));
        }, function (dbError) {
          console.error("load contacts in LS", dbError)
        })
      },

      /**
       * Detects whether the link starts from www, http, https, ftp, file and if so
       * then converts the text to Clickable HTML link (<a href="{text}">{text}</a>)
       * @param text {string}
       * @returns {string}
       */
      generateAnchors: function (text) {
        if(!text) return '';

        var exp = /(\b(https?|ftp|file):\/\/[-A-Z0-9+&@#\/%?=~_|!:,.;]*[-A-Z0-9+&@#\/%=~_|])/ig;
        var anchor = text.replace(exp, function (str) {
          str = str.toLowerCase().trim();
          return '<a onclick="angular.element(this).scope().openInappBrowser(\'' + str + '\')">' + str + '</a>'
        });
        var exp2 = /(^|[^\/])(www\.[\S]+(\b|$))/gim;
        return anchor.replace(exp2, function (str) {
          str = str.toLowerCase().trim();
          return ' <a onclick="angular.element(this).scope().openInappBrowser(\'http://' + str + '\')">' + str + '</a>';
        });
      },

      extractLinks: function (text) {
        if(!text) return '';

        var exp = /(\b(https?|ftp|file):\/\/[-A-Z0-9+&@#\/%?=~_|!:,.;]*[-A-Z0-9+&@#\/%=~_|])/ig;
        var exp2 = /(^|[^\/])(www\.[\S]+(\b|$))/gim;

        var result1;
        var result2;
        var results = [];
        while (result1 = exp.exec(text)) {
          results.push({index: result1.index, url: result1[0]});
        }

        while (result2 = exp2.exec(text)) {
          results.push({index: result2.index, url: result2[0]});
        }

        return results;
      },

      generateLinkPreview: function (link) {
        return $resource(
          'http://api.linkpreview.net/?key=5b8da24beb4211b308c0bf9f1e621437a73c326840a20&q=' + link,
          {},
          {
            get: {
              method: 'GET',
              //isArray: true,
              headers: {"Accept": "application/json"}
            }
          }
        ).get();
      },

      /**
       * @return {Promise}
       */
      selectImageForMsg: function () {
        return Utils.$q(function (resolve, reject) {
          ImageService3.showImageSelectionOptions().then(function (index) {
            if (index === -1) return resolve();//<- cancelled/closed the options menu

            ImageService3.openCameraOrAlbumOrGallery({sourceType: index}).then(function (imageURI) {
              resolve(imageURI)
            }, function (error) {
              if (['has no access to assets', 'No Image Selected'].indexOf(error) === -1) {
                console.error("openCameraOrLibrary(" + index + ")", error);
                reject(error)
              }
              else
                resolve();//<- cancelled/closed the image picker
            });
          });
        })
      },

      /**
       * @param data {{imgURI: string, fromId: number, toId:number, msgId: number}}
       * @return {Promise<any>}
       */
      uploadPrivateMsgImage: function (data) {
        var ext = data.imgURI.split('.');
        ext = ext.length > 1 ? ext.pop() : 'jpg';

        return ImageService3.convertUriToImage64(data.imgURI).then(function (imageB64) {
          var photoKey = '/privateChat/from/' + data.fromId + '/to/' + data.toId + '/image/' + data.msgId + '.' + ext;
          return ImageService3.uploadImageToS3(photoKey, imageB64).then(function (uploadS3Response) {
            return Utils.$q.resolve(uploadS3Response)
          }, function (uploadS3Error) {
            console.error(uploadS3Error);
            return Utils.$q.reject(uploadS3Error);
          });
        }, function (error) {
          console.error("convertUriToImage64", error);
          return Utils.$q.reject(error);
        });
      },

      /**
       *
       * @param idx {number}
       * @param currDate {string}
       * @param prevDate {string}
       * @return {string}
       */
      getDivider: function (idx, currDate, prevDate) {
        var dDiv = '';
        if (idx === 0)
          dDiv = new Date().toISOString().slice(0, 10);
        else if (currDate.slice(0, 10) !== prevDate.slice(0, 10)) {
          dDiv = currDate.slice(0, 10)
        }

        return dDiv;
      },

      /**
       *
       * @param toUserId
       * @return {Promise<any>}
       */
      sendPushNotification: function (toUserId) {
        var msg = UserService.get_USER_NAME() + Utils.translate('PRIVATEMESSAGE.APPALERT_NEWMSG');
        return notificationsService.sendNotificationToUserDevices(toUserId, msg);
      }
    };

    //======================== END util HELPERS ========================


    function getContactConnStates() {
      return _contactRooms;
    }

    /**
     * @return {number}
     */
    function getTotalUnreadMsgs() {
      //console.log("Getting total messages (%d)", _totalUnreadMsgs);
      return _totalUnreadMsgs;
    }

    /**
     *
     * @return {Promise}
     */
    function logout() {
      return _emit('logout', null);
    }

    return {
      socket: socket,
      DB: DB,
      STATUS: STATUS,
      MEDIA_TYPE: MEDIA_TYPE,
      util: util,

      getContactConnStates: getContactConnStates,
      getTotalUnreadMsgs: getTotalUnreadMsgs,
      setChatWithUserId: setChatWithUserId,
      PrivateMessageToSend: PrivateMessageToSend,
      PrivateMessageReceived: PrivateMessageReceived,
      logout: logout
    }
  }]);


/**
 * @typedef {Object} MessageToPersist
 * @property MessageToPersist.id {null}
 * @property MessageToPersist.uniqueId: {null}
 * @property MessageToPersist.contactId: {number}
 * @property MessageToPersist.groupId: {number}
 * @property MessageToPersist.text: {string}
 * @property MessageToPersist.fileId: {number}
 * @property MessageToPersist.extraData: {Object}
 * @property MessageToPersist.createdAt: {string}
 * @property MessageToPersist.processedAt: {null}
 * @property MessageToPersist.receivedAt: {null}
 * @property MessageToPersist.isMine: {number}
 * @property MessageToPersist.status: {number}
 */

