"use strict";
appServices
  .factory('CommentFactory', ['$rootScope', 'UserService', 'cqnzService', 'TagsService', 'permissionValidationService', 'notificationsService', 's3', 'ImageService3', 'FileService3', 'Utils', 'AWSServices', 'FileService2',
    function ($rootScope, UserService, cqnzService, TagsService, permissionValidationService, notificationsService, s3, ImageService3, FileService3, Utils, AWSServices, FileService2) {

      /**
       * @param data {{
       *   comment: (CommentRecordType|NewCommentResponseType|*),
       *   sequenceMetadata: SequenceMetadataType,
       *   sequenceOwnerMetadata: SequenceOwnerMetadataType
       * }}
       * @return {CommentStructureType}
       */
      function getCommentBasicStructure(data) {
        //console.log("getCommentBasicStructure", data);
        /**@type {CommentStructureType}*/
        var commentBasicStructure = {
          //Metadata de la secuencia a la que este comentario pertenece
          sequenceMetadata: data.sequenceMetadata,

          //Metadata del dueño de la secuencia
          sequenceOwnerMetadata: data.sequenceOwnerMetadata,

          metadata: {},
          ownerMetadata: {},

          belongsMe: UserService.get_USERSUBSCRIPTIONINSTANCE_ID() === +Utils.getLastUrlComponent(data.comment.userSubscription.href),
          canModify: !!(data.comment.canModify || false),
          canEdit: !!(data.comment.canEdit || false),
          canDelete: !!(data.comment.canDelete || false),
          groupId: data.comment.groupId || 0,
          isReply: (data.comment.commentType || 'C').toUpperCase() === 'R',
          isUpdating: data.comment.isUpdating || false,
          isReplying: data.comment.isReplying || false,

          /*todo: replacement for future releases
           description:{
           formated:"",
           editable:"",
           backup:""
           }*/
          description: cqnzService.getTextWithAnchors(data.comment.comment || data.comment.editableDescription) + "",
          editableDescription: (data.comment.comment || data.comment.editableDescription) + "",
          backupDescription: (data.comment.comment || data.comment.backupDescription) + "",
          imageUrl: data.comment.url || '',
          responses: [],

          /*todo: replacement for future releases
           like:{
           isActive:null,
           counter:null
           }*/
          likeActive: !!(data.comment.likeActive || false),
          likeCounter: +(data.comment.likeCounter || data.comment.likeCount),

          tagsList: null,//need to be set by the invoker
          //LuMa HashTags in comment
          hashTagsList: null,

          //Methods
          /**
           * @param form {messageComposerFormType}
           * @return {Promise}
           */
          updateMe: function (form) {
            return updateComment(this, form);
          },
          /**
           * @param form {messageComposerFormType}
           * @return {Promise}
           */
          addReply: function (form) {
            return replyToComment(this, form);
          },
          /**
           * @param sendBroadcast {boolean?}
           * @return {Promise}
           */
          deleteMe: function (sendBroadcast) {
            if (!this.responses.length)
              return deleteSingleCommentOrReply(this, sendBroadcast);
            else
              return deleteCommentAndReplies(this, sendBroadcast);
          },//fixme: define function
          /**
           * @return {Promise}
           */
          likeMe: function () {
            return likeToComment(this)
          }
        };

        //Metadata del comentario
        if (data.comment.metadata)
          commentBasicStructure.metadata = data.comment.metadata;
        else
          commentBasicStructure.metadata = {
            parentCommentId: null,
            id: +data.comment.commentId,
            instanceId: +Utils.getLastUrlComponent(data.comment.sequenceComments.href),
            href: data.comment.sequenceComments.href
          };
        commentBasicStructure.metadata.parentCommentId = data.comment.parentCommentId;

        //metadata del dueño del comentario
        if (data.comment.ownerMetadata)
          commentBasicStructure.ownerMetadata = data.comment.ownerMetadata;
        else
          commentBasicStructure.ownerMetadata = {
            id: +Utils.getLastUrlComponent(data.comment.userSubscription.href),
            name: data.comment.name || 'unnamed user',
            avatar: Utils.getUserImgUrl(data.comment),
            href: data.comment.userSubscription.href
          };

        return commentBasicStructure;
      }

      /**
       * @param data  {{
                             *   comment:*,
                             *   sequenceMetadata: SequenceMetadataType,
                             *   sequenceOwnerMetadata: SequenceOwnerMetadataType
                             * }}
       * @return {CommentStructureType}
       */
      function buildCommentObject(data) {
        //console.log("createCommentObject data",data);

        /**@type {CommentStructureType}*/
        var commentStruct = getCommentBasicStructure({
          comment: data.comment,
          sequenceMetadata: data.sequenceMetadata,
          sequenceOwnerMetadata: data.sequenceOwnerMetadata
        });

        var canDelete = false;
        if (commentStruct.sequenceMetadata.belongsMe)//is my sequence??
          canDelete = permissionValidationService.canDeleteCommentInGralTimeline() || permissionValidationService.canDeleteCommentInGroupTimeline();
        else
          canDelete = commentStruct.belongsMe && (permissionValidationService.canDeleteCommentInGralTimeline() || permissionValidationService.canDeleteCommentInGroupTimeline());

        var canEdit = commentStruct.belongsMe && (permissionValidationService.canUpdateCommentInGralTimeline() || permissionValidationService.canDeleteCommentInGroupTimeline());

        if (permissionValidationService.amISuperAdmin())
          canDelete = canEdit = true;


        commentStruct.canModify = canEdit || canDelete;
        commentStruct.canEdit = canEdit;
        commentStruct.canDelete = canDelete;

        cqnzService.getSmileysCountCommentsWS({
          userId: UserService.get_USERSUBSCRIPTIONINSTANCE_ID(),
          commentId: commentStruct.metadata.instanceId
        }).then(function (chkLikeActive) {
          commentStruct.likeActive = chkLikeActive.result.value;
        }, function (error) {
          console.error("findSmileysCountCommentsWS", error);
        });

        //console.log("commentStruct",commentStruct)
        TagsService.getTagFromComments167WS({
          sequenceId: commentStruct.sequenceMetadata.id,
          commentId: commentStruct.metadata.id,
          notificationType: 8
        }).then(function (commentTagsList) {
          commentStruct.description = TagsService.getTextWithFormatedTags(
            commentStruct.description,
            commentTagsList
          );

          commentStruct.tagsList = [];
          commentTagsList.forEach(function (tag) {
            commentStruct.tagsList.push(tag)
          });

          //LuMa HashTag in comments
          TagsService.findSequenceHashTagComment({
            sequenceId: commentStruct.sequenceMetadata.id,
            notificationType: 8,
            commentId: commentStruct.metadata.id
          }).then(function(commentHashTagList) {
            Utils.removeLastItemIfHasNoAttribute(commentHashTagList, 'hash');
            commentStruct.description = TagsService.getTextWithFormatedHashes(
              commentStruct.description,
              commentHashTagList
            );
            commentStruct.hashTagsList = commentHashTagList;
          }, function (errorCommentHashTag) {
            console.log("findSequenceHashTagComment ", errorCommentHashTag);
          });

        }, function (error) {
          console.error("getTagFromComments167WS", error);
        });

        //console.log("commentStruct", commentStruct);

        return commentStruct;
      }

      /**
       * @param params {{
       *   sequenceMetadata:SequenceMetadataType,
       *   sequenceOwnerMetadata:SequenceOwnerMetadataType,
       *   commentId:number|string
       *   description:string,
       *   tagsList:Array,
       *   imageData:(string|null)
       *   groupId?:number|string,
       *   isReply:boolean,
       *   isReplyToReply: boolean
       * }}
       * @return Promise {CommentStructureType|*}
       */
      function createComment(params) {
        //console.log("params", params);
        var deferred = Utils.$q.defer();

        var qry, qryParams;

        if (params.isReply) {
          qryParams = {
            sequenceId: params.sequenceMetadata.instanceId,
            commentId: params.commentId,
            userId: UserService.get_USERSUBSCRIPTIONINSTANCE_ID(),
            comment: params.description,
            groupId: params.groupId
          };
          console.log("creando respuesta con params", qryParams);
          qry = cqnzService.addSequenceCommentReplyWS(qryParams)
        }
        else {
          qryParams = {
            sequenceId: params.sequenceMetadata.instanceId,
            userId: UserService.get_USERSUBSCRIPTIONINSTANCE_ID(),
            comment: params.description,
            groupId: params.groupId
          };
          console.log("creando comentario con params", qryParams);
          qry = cqnzService.addSequenceCommentv1711WS(qryParams)
        }

        qry.then(/**@param theNewComment {NewCommentResponseType}*/function (theNewComment) {
          console.log("new comment", theNewComment);

          //this attributes aren't available when creating a new comment
          //only when creating a commnt's response
          theNewComment.userSubscription = theNewComment.userSubscription || theNewComment.userId;
          theNewComment.name = theNewComment.name || UserService.get_USER_NAME();


          /**
           * @type {CommentStructureType}
           */
          var commentStruct = getCommentBasicStructure({
            comment: theNewComment,
            sequenceMetadata: params.sequenceMetadata,
            sequenceOwnerMetadata: params.sequenceOwnerMetadata
          });
          commentStruct.metadata.parentCommentId = params.commentId;

          commentStruct.ownerMetadata.avatar = UserService.get_USER_PHOTO();

          var canDelete = permissionValidationService.canDeleteCommentInGralTimeline();

          var canEdit = permissionValidationService.canUpdateCommentInGralTimeline();

          if (permissionValidationService.amISuperAdmin()) {
            canDelete = canEdit = true;
          }

          commentStruct.canModify = canEdit || canDelete;
          commentStruct.canEdit = canEdit;
          commentStruct.canDelete = canDelete;
          params.tagsList = TagsService.removeDuplicatedTags(params.tagsList);
          console.log('try param', params.tagsList);
          commentStruct.tagsList = params.tagsList;


          var saveTagsPromises = {};
          params.tagsList.forEach(function (tag) {
            var userId = tag.id || Utils.getLastUrlComponent(tag.userSubscription.href);
            //console.log("saving user tagged: ", tag);
            saveTagsPromises[userId] = TagsService.saveTagForCommentV157WS({
              objectId: commentStruct.metadata.instanceId,
              userId: userId
            });
          });

          //After add tags to the sequence and make notification in App
          Utils.$q.all(saveTagsPromises).then(function (promiseTagData) {
            //console.log("saveTagsPromises", promiseTagData);
            //console.log("commentStruct.tagsList", commentStruct.tagsList);

            //maybe we need the saved id to delete the tag
            //(if user wants to update the comment right before reloading)
            Object.keys(promiseTagData).forEach(function (userId) {
              commentStruct.tagsList.some(function (tag) {
                if (tag.id == userId || (tag.userSubscription && Utils.getLastUrlComponent(tag.userSubscription.href) == userId)) {
                  tag.sequenceTag = promiseTagData[userId].result.members.sequenceTag.value;
                  return true;
                }
              })
            });

            commentStruct.description = TagsService.getTextWithFormatedTags(
              commentStruct.editableDescription,
              commentStruct.tagsList
            );

            //LuMa HashTags in comments
            if(params.hashTagsList){
              var addSeqHashTagPromises = {};
              var updateCountPromises = {};
              var createHashTagPromises = {};
              var index = 0;
              params.hashTagsList.forEach(function (hashTag) {
                var id = hashTag.id;
                if(id != null) {
                  //Add to Sequence
                  addSeqHashTagPromises[id] = TagsService.addSequenceHashTagv200({
                    sequenceId: params.sequenceMetadata.instanceId,
                    hashTagId: id,
                    notificationType: 8,
                    commentID: theNewComment.commentId
                  }).$promise;

                  //Update count
                  updateCountPromises[id] = TagsService.updateHashTagCountWS({
                    hTagId: id, 
                    amount: 1
                  }).$promise;
                }
                else {
                  createHashTagPromises[index] = TagsService.createHashTagWS(hashTag.hash).$promise.then(function (newHashTag) {
                    hashTag.id = newHashTag.result.instanceId;
                    hashTag.new = false;

                    id = newHashTag.result.instanceId;
                    addSeqHashTagPromises[id] = TagsService.addSequenceHashTagv200({
                      sequenceId: params.sequenceMetadata.instanceId,
                      hashTagId: id,
                      notificationType: 8,
                      commentID: theNewComment.commentId
                    }).$promise;
                  }, function(createHashTagError) {
                    console.log('createHashTagError', createHashTagError);
                  });
                  index++;
                }
              });

              Utils.$q.all(createHashTagPromises).then(function(createResponse) {
                Utils.$q.all({
                  addSeqHashTagPromises: addSeqHashTagPromises,
                  updateCountPromises: updateCountPromises
                }).then(function(responses) {
                  commentStruct.description = TagsService.getTextWithFormatedHashes(commentStruct.description, params.hashTagsList);
                  TagsService.findSequenceHashTagComment({
                    sequenceId: params.sequenceMetadata.instanceId,
                    notificationType: 8,
                    commentId: theNewComment.commentId
                  }).then(function(hashTags) {
                    Utils.removeLastItemIfHasNoAttribute(hashTags, 'hash');
                    commentStruct.hashTagsList = hashTags;
                  }, function(findHashTagsError) {
                    console.log("findSequenceHashTagComment ", findHashTagsError);
                  });                  
                }, function(addAndUpdateError) {
                  console.log('Error adding/updatingCount HashTag', addAndUpdateError)
                });
              });
            }

            if (FileService2.images().length) {
              uploadImageForComment(commentStruct).then(function (s3ImageUrl) {
                commentStruct.imageUrl = s3ImageUrl;
                $rootScope.$broadcast('sequence.comment.created', {comment: commentStruct});
                deferred.resolve(commentStruct);

                console.log("Comment created", commentStruct);
              }, function (error) {
                commentStruct.imageUrl = '';
                $rootScope.$broadcast('sequence.comment.created', {comment: commentStruct});

                Utils.toast.warning(Utils.translate("COMMENT.DIALOG.IMAGE_UPLOAD_ERROR"));
                console.error("uploadCommentImage", error);

                deferred.resolve(commentStruct);
              }).finally(function () {
                FileService2.cleanImages();
              })
            }
            else {
              $rootScope.$broadcast('sequence.comment.created', {comment: commentStruct});
              deferred.resolve(commentStruct);
            }


            var saveTagsCommentNotifsPromises = {};
            Object.keys(promiseTagData).forEach(function (userId) {
              saveTagsCommentNotifsPromises[userId] = TagsService.saveTagCommentNotificationWS({
                objectId: commentStruct.metadata.instanceId,
                userId: userId,
                groupId: params.groupId
              })
            });

            //notify to the tagged users in comment
            Utils.$q.all(saveTagsCommentNotifsPromises).then(function (promiseTagCommentNotifData) {
              //console.log("promiseTagCommentNotifData", promiseTagCommentNotifData);
              notificationsService.notifyTaggedUsersInComment(
                promiseTagCommentNotifData,
                params.sequenceMetadata.id
              ).then(function (totalUsers) {
                if (totalUsers) console.log("Notification was sent to " + totalUsers + " new tagged user");
              }, function (error) {
                console.error("Couldn't sent notification(s) to new tagged user", error);
              });
            }, function (error) {
              console.error("all: saveTagCommentNotificationWS", error)
            });
          }, function (error) {
            console.error('saveTagsPromises', error);
            deferred.reject(error)
          });

          //notify to the comment's owner
          cqnzService.sendCommentUserNotificationWS().get({
            objectId: params.sequenceMetadata.instanceId
          }).$promise.then(function (userNotify) {
            //if user wants to be notified...
            if (userNotify.result.value && params.sequenceOwnerMetadata.id != UserService.get_USERSUBSCRIPTIONINSTANCE_ID()) {
              notificationsService.notifyNewCommentOnSequence(
                params.sequenceOwnerMetadata.id,
                params.sequenceMetadata.id
                ).then(function () {
                //console.log("Notification of new comment sent")
              }, function (error) {
                console.error("Notification of new comment couldn't be sent", error)
              });

              
            }
            if(params.sequenceOwnerMetadata.id != UserService.get_USERSUBSCRIPTIONINSTANCE_ID()) {
              //notificar que han comentado en un post donde han comentado
              //avisar que han dado like a post que han comentado
              var wsData = {
                sequenceId: params.sequenceMetadata.id,
                userSubscriptionId: UserService.get_USERSUBSCRIPTION_ID(),
                groupId: params.groupId === null ? 0 : params.groupId,
                notificationType: 18 //NEED TYPE 18
              }
              //NEED NEW WS WITH TYPE
              UserService.getDevicesUsersCommentAPostWithType(wsData).then( function (dataWS) {
                Utils.removeLastItemIfHasNoAttribute(dataWS, 'userDeviceId');
                var message = UserService.get_USER_NAME() + Utils.translate('ALERTS.COMMENTED_POST_COMMENTED');
                message = (message || "").replace(/\s\s+/g, ' ');
                dataWS.forEach( function(dataDevices) {

                  AWSServices.sendNotifaboutPostCommented(
                    message, 
                    dataDevices.userDeviceId,
                    params.groupId === null ? 0 : params.groupId,
                    params.sequenceMetadata.id
                  )

                })
              })
            }
          }, function (error) {
            console.error("sendCommentUserNotificationWS", error);
          });
        }, function (error) {
          console.error("addSequenceComment", error);
          deferred.reject(error)
        });

        return deferred.promise;
      }

      /**
       * @param comment {CommentStructureType}
       * @param form {messageComposerFormType}
       * @return Promise {CommentStructureType|*}
       */
      function updateComment(comment, form) {
        return Utils.$q(function (resolve, reject) {
          //No changes in comment, nothing to do
          if (!form.description ||
            (comment.backupDescription === form.description)) return resolve(comment);

          comment.isUpdating = true;
          updateComentDescription(comment, {
            description: form.description,
            newTagsList: form.savedTags,
            newHashTagsList: form.savedHashes
          }).then(function (theEditedComment) {
            console.log("theEditedComment", theEditedComment);
            $rootScope.$broadcast('sequence.comment.edited', {comment: theEditedComment});
            resolve(theEditedComment);
          }, function (errors) {
            reject(errors);
          }).finally(function () {
            comment.isUpdating = false;
          });
        })
      }

      /**
       * @param comment {CommentStructureType}
       * @param form {messageComposerFormType}
       * @return Promise {CommentStructureType|*}
       */
      function replyToComment(comment, form) {
        //console.log("form", form)
        return Utils.$q(function (resolve, reject) {
          if (!form.description ||
            (comment.backupDescription === form.description && comment.imageUrl === form.imagePath)) return resolve(comment);

          var isReplyToReply = comment.isReply;
          var theCommentId = isReplyToReply ? comment.metadata.parentCommentId : comment.metadata.id;

          createComment({
            sequenceMetadata: comment.sequenceMetadata,
            sequenceOwnerMetadata: comment.sequenceOwnerMetadata,
            commentId: theCommentId,
            description: form.description,
            tagsList: form.savedTags,
            hashTagsList: form.savedHashes, //LuMa HashTags in comments
            imageData: form.imagePath,
            groupId: comment.groupId || 0,
            isReply: true,
            isReplyToReply: isReplyToReply
          }).then(/**@param newComment {CommentStructureType}*/function (newComment) {
            console.log("replyToComment", newComment);
            $rootScope.$broadcast("reply:new");
            resolve(newComment);
          }, function (error) {
            console.error("replyToComment->createComment", error);
            reject(error);
          })
        })
      }

      /**
       * @param comment {CommentStructureType}
       * @param data {{description:string, newTagsList:*}}
       * @return {Promise}
       */
      function updateComentDescription(comment, data) {
        console.log("Updating comment's description...");
        var deferred = Utils.$q.defer();

        cqnzService.updateCommentDescriptionWS({
          commentId: comment.metadata.instanceId,
          comment: data.description
        }).then(function (/*updateResponse*/) {
          comment.editableDescription = data.description;
          comment.backupDescription = data.description;

          var allTagsAreNew = !comment.tagsList.length;

          //Remove all the users previously tagged in the text
          var deleteUserTaggedPromises = {};
          comment.tagsList.forEach(function (tag) {
            if (tag.sequenceTag) {
              var sequenceTagId = Utils.getLastUrlComponent(tag.sequenceTag.href);
              console.log("Deleting tag", tag);
              deleteUserTaggedPromises[sequenceTagId] = TagsService.deleteTaggedUserWS(sequenceTagId);
            }
            else
              console.log("can't delete tag", tag)
          });

          //after all the olds tags were deleted
          Utils.$q.all(deleteUserTaggedPromises).then(function () {
            //Remove unused tagged users from description
            /*comment.tagsList = TagsService.removeTagsNotFoundInText(
             comment.editableDescription,
             comment.tagsList
             );*/

            //join old tags with the new ones
            //data.newTagsList.push.apply(data.newTagsList, comment.tagsList);

            //beautify the new text
            data.newTagsList = TagsService.removeDuplicatedTags(data.newTagsList);
            comment.description = TagsService.getTextWithFormatedTags(
              comment.editableDescription,
              data.newTagsList
            );

            //LuMa HashTags in comment
            var deleteHashTagsPromises = {};
            if(comment.hashTagsList) {
              //Delete current hashtags
              comment.hashTagsList.forEach(function(hashTag) {
                if(hashTag.sequenceHashTag && hashTag.sequenceHashTag.href) {
                  console.log('HashTag to delete: ', hashTag);
                  var id = Utils.getLastUrlComponent(hashTag.sequenceHashTag.href);
                  deleteHashTagsPromises[id] = TagsService.removeHastagFromDBWS(id).$promise;
                }
              });
            }

            Utils.$q.all(deleteHashTagsPromises).then(function(response) {
              if(data.newHashTagsList){
                var addSeqHashTagPromises = {};
                var updateCountPromises = {};
                var createHashTagPromises = {};
                var index = 0;

                data.newHashTagsList.forEach(function (hashTag) {
                  var id = hashTag.id;
                  if(id != null) {
                    //Add to Sequence
                    addSeqHashTagPromises[id] = TagsService.addSequenceHashTagv200({
                      sequenceId: comment.sequenceMetadata.instanceId,
                      hashTagId: id,
                      notificationType: 8,
                      commentID: comment.metadata.id
                    }).$promise;

                    //Update count
                    updateCountPromises[id] = TagsService.updateHashTagCountWS({
                      hTagId: id, 
                      amount: 1
                    }).$promise;
                  }
                  else {
                    createHashTagPromises[index] = TagsService.createHashTagWS(hashTag.hash).$promise.then(function (newHashTag) {
                      hashTag.id = newHashTag.result.instanceId;
                      hashTag.new = false;

                      id = newHashTag.result.instanceId;
                      addSeqHashTagPromises[id] = TagsService.addSequenceHashTagv200({
                        sequenceId: comment.sequenceMetadata.instanceId,
                        hashTagId: id,
                        notificationType: 8,
                        commentID: comment.metadata.id
                      }).$promise;
                    }, function(createHashTagError) {
                      console.log('createHashTagError', createHashTagError);
                    });
                    index++;
                  }
                });

                Utils.$q.all(createHashTagPromises).then(function(createResponse) {
                  Utils.$q.all({
                    addSeqHashTagPromises: addSeqHashTagPromises,
                    updateCountPromises: updateCountPromises
                  }).then(function(responses) {
                    TagsService.findSequenceHashTagComment({
                      sequenceId: comment.sequenceMetadata.instanceId,
                      notificationType: 8,
                      commentId: comment.metadata.id
                    }).then(function(hashTags) {
                      Utils.removeLastItemIfHasNoAttribute(hashTags, 'hash');
                      comment.hashTagsList = hashTags;
                      comment.description = TagsService.getTextWithFormatedHashes(comment.description, comment.hashTagsList);
                    }, function(findHashTagsError) {
                      console.log('findSequenceHashTagComment', findHashTagsError);
                    });
                  }, function(addAndUpdateError) {
                    console.log('Error adding/updatingCount HashTag', addAndUpdateError);
                  });
                });
              }
            }, function(deleteHashTagsError) {
              console.log('Delete HashTags Error', deleteHashTagsError);
            });

            //Add tags to the sequence
            var saveTagsPromises = {};
            data.newTagsList.forEach(function (tag) {
              var userId = tag.id || Utils.getLastUrlComponent(tag.userSubscription.href);
              saveTagsPromises[userId] = TagsService.saveTagForCommentV157WS({
                objectId: comment.metadata.instanceId,
                userId: userId
              });
            });


            Utils.$q.all(saveTagsPromises).then(function (promiseTagData) {
              //maybe we need the saved id for delete the tag (if user wants to update the comment before reloading)
              Object.keys(promiseTagData).forEach(function (userId) {
                data.newTagsList.some(function (tag) {
                  if (tag.id || (tag.userSubscription && Utils.getLastUrlComponent(tag.userSubscription.href) == userId)) {
                    tag.sequenceTag = promiseTagData[userId].result.members.sequenceTag.value;
                    return true;
                  }
                })
              });

              comment.tagsList = data.newTagsList;

              deferred.resolve(comment);

              var notificationsSender = allTagsAreNew ?
                notificationsService.notifyTaggedUsersInComment :
                notificationsService.notifyTaggedUsersInUpdatedComment;

              var saveTagsCommentNotifsPromises = {};
              Object.keys(promiseTagData).forEach(function (userId) {
                saveTagsCommentNotifsPromises[userId] = TagsService.saveTagCommentNotificationWS({
                  objectId: comment.metadata.instanceId,
                  userId: userId,
                  groupId: params.groupId
                })
              });

              Utils.$q.all(saveTagsCommentNotifsPromises).then(function (promiseTagCommentNotifData) {
                //console.log("promiseTagCommentNotifData", promiseTagCommentNotifData);
                notificationsSender(
                  promiseTagCommentNotifData,
                  comment.sequenceMetadata.id
                ).then(function (totalUsers) {
                  if (totalUsers)
                    console.log("Notification sent to " + totalUsers + ' ' + (allTagsAreNew ? 'new ' : '') + "tagged user")
                }, function (error) {
                  console.error('notificationsSender', error);
                });
              }, function (error) {
                console.error("saveTagsCommentNotifsPromises", error);
              });
            }, function (error) {
              comment.isUpdating = false;
              console.error('saveTagsPromises', error);
              Utils.toast.error(Utils.translate('SEQUENCE.DIALOG.EDIT_POST_COMMENT_ERROR'));
              deferred.reject(error);
            })
          }, function (error) {
            comment.isUpdating = false;
            console.error('deleteUserTaggedPromises', error);
            Utils.toast.error(Utils.translate('SEQUENCE.DIALOG.EDIT_POST_COMMENT_ERROR'));
            deferred.reject(error);
          });
        }, function (updateError) {
          comment.isUpdating = false;
          console.error("changeCommentWS", updateError);
          Utils.toast.error(Utils.translate('SEQUENCE.DIALOG.EDIT_POST_COMMENT_ERROR'));
          deferred.reject(updateError);
        });

        return deferred.promise;
      }

      /**
       * @param comment {CommentStructureType}
       * @return {Promise}
       */
      function uploadImageForComment(comment) {
        console.log("uploadImageForComment metadata: ", comment.metadata);

        return Utils.$q(function (resolve, reject) {
          var photoKey = _getCommentOrReplyPhotoKey({
            objectInstanceId: comment.metadata.instanceId,
            isReply: comment.isReply
          });

          uploadImageToS3({
            photoKey: photoKey
          }).then(function (s3Response) {
            console.log("image uploaded, Key: " + photoKey);
            console.log("image uploaded, Location: " + s3Response.Location);
            updateCommentImageDBRecord(comment.metadata.instanceId, s3Response.Location).then(function (result) {
              resolve(s3Response.Location);
            }, function (updateDBError) {
              console.error("uploadImageForComment->updateCommentImageDBRecord", updateDBError);
              cqnzService.deleteS3Objects([{Key: photoKey}]).then(function () {
              }, function (deleteError) {
                console.error("uploadImageForComment->deleteS3Objects", deleteError);
              }).finally(function () {
                reject(updateDBError);
              });
            })
          }, function (error) {
            console.error("uploadImageForComment->uploadImageToS3", error);
            reject(error);
          });
        })
      }

      /**
       * Uploads a file to s3
       * @param data {{photoKey: string}}
       * @returns Promise {*}
       */
      function uploadImageToS3(data) {
        return Utils.$q(function (resolve, reject) {
          ImageService3.convertUriToImage64(FileService2.images()[0]).then(function (imageB64Data) {
            var s3Service = new AWS.S3({
              apiVersion: '2006-03-01',
              params: {Bucket: s3.bucket}
            });

            //photo to blob
            var file = Utils.dataURLtoBlob(imageB64Data);

            console.log("Uploading image Key: " + data.photoKey);

            console.log("file.type", file.type);

            s3Service.upload({
              Key: data.photoKey,
              Body: file,
              ACL: 'public-read',
              ContentType: file.type
            }, function (uploadError, uploadResponse) {
              console.log("uploadResponse", uploadResponse);
              if (uploadError && !uploadResponse) return reject(uploadError);
              resolve(uploadResponse);
            });
          }, function (error) {
            reject(error)
          })
        });
      }

      //ASK AWS TO RETRY UPLOAD UPDATING TIMESTAMP
      AWS.events.on('retry', function(response) {  
        if (response.error.name === 'RequestTimeTooSkewed') {
          console.error('User time is not correct. Handling error!');
          console.log('AWS systemClockOffset:', AWS.config.systemClockOffset);
          var serverTime = Date.parse(response.httpResponse.headers['date']);
          var timeNow = new Date().getTime();
          console.log('AWS timestamp:', new Date(serverTime));
          console.log('Browser timestamp:', new Date(timeNow));
          AWS.config.systemClockOffset = Math.abs(timeNow - serverTime);
          response.error.retryable = true;
          console.log('Setting systemClockOffset to:', AWS.config.systemClockOffset);
          console.log('Retrying uploading to S3 once more...');
        }
      });

      /**
       *
       * @param commentInstanceId
       * @param imageUrl
       * @return {Promise}
       */
      function updateCommentImageDBRecord(commentInstanceId, imageUrl) {
        return Utils.$q(function (resolve, reject) {
          cqnzService.updateCommentImageWS({
            commentId: commentInstanceId,
            commentImageUrl: imageUrl
          }).$promise.then(function (updateImageResp) {
            console.log("comment image updated correctly", updateImageResp);
            resolve(updateImageResp.result.value);
          }, function (updateError) {
            console.error("Couldn't update image url", updateError);
            reject(updateError);
          })
        })
      }

      /**
       * @param comment {CommentStructureType}
       * @param sendBroadcast {boolean?}
       * @return {Promise}
       */
      function deleteSingleCommentOrReply(comment, sendBroadcast) {
        return Utils.$q(function (resolve, reject) {
          var photoKey = _getCommentOrReplyPhotoKey({
            objectInstanceId: comment.metadata.instanceId,
            isReply: comment.isReply
          });

          console.log("removing s3 objects...");
          cqnzService.deleteS3Objects([{Key: photoKey}]).then(function () {
            console.log("s3 objects removed successfully!");
            console.log("deleting comment...");
            cqnzService.deleteCommentDBRecordWS(comment.metadata.instanceId).$promise.then(function (/*data*/) {
              console.log("Comment deleted successfully!");
              if (sendBroadcast)
                $rootScope.$broadcast('sequence.comment.deleted', {comment: comment, childrenDeleted: 0});

              resolve(1);
            }, function (s3error) {
              console.error("deleteS3Objects", s3error);
              console.log("Setting empty string to imageUrl path...");
              updateCommentImageDBRecord(comment.metadata.instanceId, '').then(function (result) {
                console.log("image path updated successfully!");
                reject(s3error);
              }, function (error1) {
                console.error("deleteS3Objects->updateCommentImageDBRecord", error1);
                reject(s3error);
              }).finally(function () {
                Utils.toast.warning(Utils.translate('ERROR.UNKNOWN'));
                console.log("deleteS3Objects", s3error);
              })
            });
          }, function (error) {
            Utils.toast.warning(Utils.translate("COMMENT.DIALOG.DELETE_IMAGE_ERROR"));
            console.error("deleteS3Objects", error);
            reject(error);
          })
        })
      }

      /**
       * @param comment {CommentStructureType}
       * @param sendBroadcast {boolean?}
       * @return {Promise}
       */
      function deleteCommentAndReplies(comment, sendBroadcast) {
        return Utils.$q(function (resolve, reject) {
          var qryParams = {
            sequenceId: comment.sequenceMetadata.id,
            commentId: comment.metadata.id,
            start: 0,
            count: 1000
          };
          console.log("Obteniendo respuestas a comentarios con params", qryParams);

          /*return cqnzService.deleteCommentDBRecordWS(comment.metadata.instanceId).$promise.then(function () {
           resolve(true);
           }, function (error) {
           console.error("deleteCommentDBRecordWS(" + comment.metadata.instanceId + ")", error);
           reject(error);
           });//<-deleteCommentDBRecordWS()*/

          //obtener el registro de las respuestas
          cqnzService.findTheRepliesOfACommentWS(qryParams).$promise.then(/**@param commentReplies {CommentRecordType[]}*/function (commentReplies) {
            Utils.removeLastItemIfHasNoAttribute(commentReplies, 'commentId');
            console.log("Replies found", commentReplies);

            //por cada respuesta eliminar su imagen
            var photoKeys = [];
            commentReplies.forEach(/**@param reply {CommentRecordType}*/function (reply) {
              if (reply.url) {
                console.log("image to delete: " + reply.url);
              }

              var photoKey = _getCommentOrReplyPhotoKey({
                objectInstanceId: Utils.getLastUrlComponent(reply.sequenceComments.href),
                isReply: true
              });

              photoKeys.push({Key: photoKey});
            });//<- forEach reply

            console.log("Eliminando imagenes de las respuestas");
            cqnzService.deleteS3Objects(photoKeys).then(function (value) {
              console.log("All replies pics were deleted");
            }, function (error) {
              console.error("delete replies pics", error);
            }).finally(function () {
              console.log("eliminando imagen del comentario padre: " + comment.imageUrl);
              var photoKey = _getCommentOrReplyPhotoKey({
                objectInstanceId: comment.metadata.instanceId,
                isReply: false
              });

              cqnzService.deleteS3Objects([{Key: photoKey}]).then(function (value) {
                console.log("parent comment deleteS3Objects OK");
              }, function (error) {
                console.error("parent comment deleteS3Objects", error);
              }).finally(function () {
                console.log("eliminando registro del comentario padre");
                cqnzService.deleteCommentDBRecordWS(comment.metadata.instanceId).$promise.then(function () {
                  if (sendBroadcast)
                    $rootScope.$broadcast("sequence.comment.deleted", {
                      comment: comment,
                      childrenDeleted: commentReplies.length
                    });
                  resolve(commentReplies.length + 1);
                }, function (error) {
                  console.error("deleteCommentDBRecordWS(" + comment.metadata.instanceId + ")", error);
                  reject(error);
                })//<-deleteCommentDBRecordWS()
              });//<-deleteS3Objects.finally()
            })//<- deleteS3Objects().finally()
          }, function (error) {
            console.error("findTheAnswersOfACommentWS", error);
            Utils.toast.error(Utils.translate("ERROR.UNKNOWN"));
            reject(error);
          })//<-findTheAnswersOfACommentWS()
        })
      }

      /**
       * @param data {{objectInstanceId:number|string, isReply:boolean}}
       * @return {string}
       * @private
       */
      function _getCommentOrReplyPhotoKey(data) {
        var commentInstanceIdHash = Utils.md5.createHash(data.objectInstanceId.toString());
        return s3.prefix + '/comments/commentSequence/' + commentInstanceIdHash +
          (data.isReply ? '/reply/' : '/') + 'foto.jpg';
      }

      /**
       * @param comment {*}
       * @return Promise {CommentStructureType|*}
       */
      function likeToComment(comment) {
        return Utils.$q(function (resolve, reject) {
          var newActiveLike = !comment.likeActive;
          comment.likeCounter += newActiveLike ? 1 : -1;
          comment.likeActive = newActiveLike;

          if(comment.groupId) {
            cqnzService.setLikeToCommentGroupWS({
              commentId: comment.metadata.instanceId,
              userId: UserService.get_USERSUBSCRIPTIONINSTANCE_ID(),
              groupId: comment.groupId+""
            }).then(function (response) {
              resolve(comment);

              $rootScope.$broadcast('sequence.comment.like.updated', {
                comment: comment,
                likeCounter: comment.likeCounter,
                likeActive: comment.likeActive
              });

              //Send push notification
              if ( response.result.value && comment.ownerMetadata.id != UserService.get_USERSUBSCRIPTIONINSTANCE_ID() && newActiveLike ) {
                notificationsService.notifyCommentLiked(
                  comment.ownerMetadata.id,
                  comment.sequenceMetadata.id)
                
              }
            }, function (error) {
              console.error("setLikeToSequence error:", error);
              //rollback 'like' to the original state...
              comment.likeCounter += newActiveLike ? -1 : 1;
              comment.likeActive = !newActiveLike;
              Utils.toast.warning(Utils.translate('SEQUENCE.DIALOG.LIKE_COMMENT_ERROR'));
              reject(error);
            });
          } else {
            cqnzService.setLikeToCommentWS({
              commentId: comment.metadata.instanceId,
              userId: UserService.get_USERSUBSCRIPTIONINSTANCE_ID()
            }).then(function (response) {
              resolve(comment);

              $rootScope.$broadcast('sequence.comment.like.updated', {
                comment: comment,
                likeCounter: comment.likeCounter,
                likeActive: comment.likeActive
              });

              //Send push notification
              if ( response.result.value && comment.ownerMetadata.id != UserService.get_USERSUBSCRIPTIONINSTANCE_ID() && newActiveLike ) {
                notificationsService.notifyCommentLiked(
                  comment.ownerMetadata.id,
                  comment.sequenceMetadata.id)
                
              }
            }, function (error) {
              console.error("setLikeToSequence error:", error);
              //rollback 'like' to the original state...
              comment.likeCounter += newActiveLike ? -1 : 1;
              comment.likeActive = !newActiveLike;
              Utils.toast.warning(Utils.translate('SEQUENCE.DIALOG.LIKE_COMMENT_ERROR'));
              reject(error);
            });
          }
        })
      }

      /**
       * @param data {{
                             *   commentId: number|string,
                             *   sequenceMetadata: SequenceMetadataType,
                             *   sequenceOwnerMetadata: SequenceOwnerMetadataType,
                             *   start?:number,
                             *   count?:number
                             * }}
       * @return {Promise}
       */
      function getCommentReplies(data) {
        //console.log("getCommentReplies", data);
        return Utils.$q(function (resolve, reject) {
          var qryParams = {
            sequenceId: data.sequenceMetadata.id,
            commentId: data.commentId,
            start: data.start || 0,
            count: data.count || 10
          };
          //console.log("Obteniendo respuestas a comentarios con params", qryParams);
          cqnzService.findTheRepliesOfACommentWS(qryParams).$promise.then(/**@param commentReplies {CommentRecordType[]}*/function (commentReplies) {
            Utils.removeLastItemIfHasNoAttribute(commentReplies, 'commentId');
            //console.log("Replies found", commentReplies);

            var childComments = [];

            commentReplies.forEach(function (reply) {
              reply.commentType = 'R';
              reply.parentCommentId = data.commentId;
              var childComment = buildCommentObject({
                comment: reply,
                sequenceMetadata: data.sequenceMetadata,
                sequenceOwnerMetadata: data.sequenceOwnerMetadata
              });

              //console.log("childComment", childComment);
              childComments.push(childComment);
            });

            resolve(childComments);
          }, function (error) {
            console.error("findTheAnswersOfAComment", error);
            reject(error);
          });//fetch child comments onError()
        })
      }

      return {
        buildCommentObject: buildCommentObject,
        createComment: createComment,
        getCommentReplies: getCommentReplies
      }
    }

  ]);

/**
 * @typedef {Object} CommentMetadataType
 * @property {number|null} parentCommentId
 * @property {number} id
 * @property {number} instanceId
 * @property {string} href
 */

/**
 * @typedef {Object} CommentOwnerMetadataType
 * @property {number} id
 * @property {string} name
 * @property {string} avatar
 * @property {string} href
 */


/**
 * @typedef {Object} CommentStructureType
 * @property {SequenceMetadataType} sequenceMetadata
 * @property {SequenceOwnerMetadataType} sequenceOwnerMetadata
 * @property {CommentMetadataType} metadata
 * @property {CommentOwnerMetadataType} ownerMetadata
 * @property {boolean} belongsMe
 * @property {boolean} canModify
 * @property {boolean} canEdit
 * @property {boolean} canDelete
 * @property {number} groupId
 * @property {boolean} isReply
 * @property {boolean} isUpdating
 * @property {boolean} isReplying
 * @property {string} description
 * @property {string} editableDescription
 * @property {string} backupDescription
 * @property {string} imageUrl
 * @property {Array} responses
 * @property {boolean} likeActive
 * @property {number} likeCounter
 * @property {(null|Array)} tagsList
 * @property {(function(messageComposerFormType): Promise)} updateMe
 * @property {(function(messageComposerFormType): Promise)} addReply
 * @property {(function(boolean): Promise)} deleteMe
 * @property {(function(): Promise)} likeMe
 */

/**
 * @typedef {Object} CommentRecordType - The comment/reply item obtained from WS
 * @property {number} answerCount
 * @property {string} comment
 * @property {number} commentId
 * @property {number} groupId
 * @property {number} likeCount
 * @property {string} name - the username
 * @property {(string|null)} profilePicture
 * @property {(string|null)} profilePictureUrl
 * @property {{href:string}} sequenceComments
 * @property {(string|null)} url - the comment's image
 * @property {{href:string}} userSubscription
 */

/**
 * @typedef {Object} NewCommentResponseType - Structure response when adding a new comment/reply
 * @property {number} answerCommentId
 * @property {number} answerCount
 * @property {string} comment
 * @property {number} commentId
 * @property {string} commentType - 'C'|'R'
 * @property {string} creationTime
 * @property {number} groupId
 * @property {number} likeCount
 * @property {string} name - the username
 * @property {{href:string}} sequence
 * @property {{href:string}} sequenceComments
 * @property {(string|null)} url - the comment's image
 * @property {{href:string}} userId
 * @property {{href:string}} userSubscription
 */
