"use strict";
appServices
  .factory('GroupFactory', ['$rootScope', 'UserService', 'GroupsService', 'Utils', function ($rootScope, UserService, GroupsService, Utils) {

    var
      availableGroupsList = [],
      pendigGroupsList = [],
      totalAvailableGroups = 0,
      currentGroupIndex = 0, //0 = default timeline, > 0 = some group.
      currentGroupTab = 0;//0 = my groups, 1 = other available groups

    function getCurrentAvailableUserGroupsList() {
      return availableGroupsList;
    }

    function setCurrentAvailableUserGroupsList(groupsList) {
      availableGroupsList = groupsList;
    }

    function getCurrentTotalAvailableGroups() {
      return totalAvailableGroups;
    }

    function getPendingUserGroupsList() {
      return pendigGroupsList;
    }

    function setCurrentGroupIndex(value) {
      currentGroupIndex = value || 0;
      window.localStorage.setItem('CurrentGroup', value || 0);
    }

    function getCurrentGroupIndex() {
      return currentGroupIndex;
    }

    function setCurrentGroupTab(value) {
      return currentGroupTab = value || 0;
    }

    function getCurrentGroupTab() {
      return currentGroupTab;
    }

    /**
     * @param {Object} data
     * @param {string} data.userSubscriptionHref
     * @param {number?} data.start - default 0
     * @param {number?} data.count - default 0 (all)
     * @return {Promise} - [{*}] | [] | *
     */
    function loadAvailableUserGroups(data) {
      return Utils.$q(function (resolve, reject) {
        GroupsService.getUserGroupsWS({
          userSubscriptionHref: data.userSubscriptionHref,
          start: data.start || 0,
          count: data.count || 0
        }).then(function (groups) {
          Utils.removeLastItemIfHasNoAttribute(groups, 'group');
          groups.forEach(function (res) {
            res.href = res.group.href;
            res.hrefId = Utils.getLastUrlComponent(res.group.href);
            res.id = res.groupId;
            delete res.groupId;
            res.image = res.url || 'img/photoFrame.svg';
            delete res.url;
            res.backgroundImage = res.backgroundUrl || 'img/postBg.png';
            delete res.backgroundUrl;
            res.imMember = true;
            res.name = res.groupName;
            delete res.groupName;
            res.isPreview = false;
            res.privacyTranslated = Utils.translate('GROUPS.MAIN_PAGE.' + res.privacy.toUpperCase());
          });

          availableGroupsList = groups;

          window.localStorage.setItem("totalGroups", availableGroupsList.length.toString());
          totalAvailableGroups = availableGroupsList.length;

          //console.log("--- (" + totalAvailableGroups + ") Available user groups loaded!!");
          resolve(availableGroupsList)
        }, function (error) {
          console.error("getUserGroups", error);
          reject(error)
        });
      })
    }

    /**
     * Gets the groups where the user has been banned/locked from
     * @param userSubscriptionHref {string}
     * @return {Promise}
     */
    function loadPendingUserGroups(userSubscriptionHref) {
      return Utils.$q(function (resolve, reject) {
        GroupsService.getUserJoinRequests(userSubscriptionHref).then(function (pendingGroupsList) {
          Utils.removeLastItemIfHasNoAttribute(pendingGroupsList, 'groupId');

          var groupList = [];
          pendingGroupsList.forEach(function (res) {
            groupList.push({
              id: res.groupId,
              href: res.group.href,
              hrefId: Utils.getLastUrlComponent(res.group.href),
              groupMemberHrefId: Utils.getLastUrlComponent(res.groupMember.href),
              name: res.groupName,
              description: res.description,
              image: res.url || 'img/photoFrame.svg',
              backgroundImage: res.backgroundUrl || 'img/postBg.png',
              privacy: res.privacy,
              privacyTranslated: Utils.translate('GROUPS.MAIN_PAGE.' + res.privacy.toUpperCase())
            });
          });

          pendigGroupsList = groupList;
          resolve(groupList)
        }, function (joinError) {
          console.error(joinError);
          reject(joinError);
        })
      })
    }

    /**
     * @param {Object} data
     * @param {string} data.userSubscriptionHref
     * @param {number} data.start - default 0
     * @param {number} data.count - default 0 (all)
     */
    function loadOtherGroups(data) {
      return Utils.$q(function (resolve, reject) {
        GroupsService.getOtherGroupsWS({
          userSubscriptionHref: data.userSubscriptionHref,
          start: data.start || 0,
          count: data.count || 0
        }).then(function (otherGroups) {
          Utils.removeLastItemIfHasNoAttribute(otherGroups, 'group');
          otherGroups.forEach(function (res) {
            res.id = res.groupId;
            delete res.groupId;
            res.hrefId = Utils.getLastUrlComponent(res.group.href);
            res.href = res.group.href;
            res.name = res.groupName;
            delete res.groupName;
            res.image = res.url || 'img/photoFrame.svg';
            delete res.url;
            res.backgroundImage = res.backgroundUrl || 'img/postBg.png';
            delete res.backgroundUrl;
            res.privacyTranslated = Utils.translate('GROUPS.MAIN_PAGE.' + res.privacy.toUpperCase());
            res.isPreview = true;//because you can't see other groups info
            res.imMember = false;
          });
          console.log("--- (" + otherGroups.length + ") other groups loaded");
          resolve(otherGroups);
        }, function (getOtherGroupsError) {
          console.error("getOtherGroupsError", getOtherGroupsError);
          reject(getOtherGroupsError)
        });
      })
    }

    /**
     * @param {Object} data
     * @param {number} data.groupId
     * @param {number?} data.start - default 0
     * @param {number?} data.count - default 0 (all)
     * @return {Promise} - GroupMemberType[]
     */
    function getGroupMembers(data) {
      return Utils.$q(function (resolve, reject) {
        GroupsService.getGroupMembersWS({
          groupId: data.groupId,
          start: data.start || 0,
          count: data.count || 0
        }).then(/**@param {GroupMemberWSType[]} membersList*/function (membersList) {
          Utils.removeLastItemIfHasNoAttribute(membersList, 'userSubscription');
          membersList.forEach(function (member) {
            member.avatar = Utils.getUserImgUrl(member);
            member.groupMemberHrefId = Utils.getLastUrlComponent(member.groupMember.href);
            member.roleFlag = calcUserRoleFlag(member);
          });
          resolve(membersList);
        }, function (error) {
          console.error(error);
          reject(error)
        });
      })
    }

    function getGroupSpecificMembers(data) {
      return Utils.$q(function (resolve, reject) {
        GroupsService.getSpecificGroupMembersWS({
          groupId: data.groupId,
          start: data.start || 0,
          count: data.count || 0,
          parSearch: data.txtSearch || ''
        }).then(/**@param {GroupMemberWSType[]} membersList*/function (membersList) {
          Utils.removeLastItemIfHasNoAttribute(membersList, 'userSubscription');
          membersList.forEach(function (member) {
            member.avatar = Utils.getUserImgUrl(member);
            member.groupMemberHrefId = Utils.getLastUrlComponent(member.groupMember.href);
            member.roleFlag = calcUserRoleFlag(member);
          });
          resolve(membersList);
        }, function (error) {
          console.error(error);
          reject(error)
        });
      })
    }


    /**
     * Sets the user's role flag
     * User's role Chart
     * [0] = available state flag
     * [1] = current state flag
     * [-] = unavailable state flag (disabled/hidden)
     *
     *   8  |    4    |  2   |  1   |
     * ADMIN|MODERATOR|LOCKED|BANNED| State |Description
     *   0  |    0    |   0  |  0   | =  0  |Regular user
     *   -  |    -    |   -  |  1   | =  1  |Regular user banned *(don't show this user)
     *   -  |    -    |   1  |  -   | =  2  |Regular user locked
     *   -  |    1    |   -  |  -   | =  4  |Moderator user
     *   1  |    -    |   -  |  -   | =  8  |Admin user
     * @param user
     * @returns {number}
     */
    function calcUserRoleFlag(user) {
      return (!!user.administrator) * 8 +
        (!!user.moderator) * 4 +
        user.locked * 2 +
        user.banned;
    }

    /**
     * @returns {Promise}
     */
    function showGroupsListModal(groupList, opts) {
      opts = opts || {};
      return Utils.modal.show({
        templateUrl: 'templates/Timelines/timeline-shareCqnz-modal.html',
        locals: {
          groupList: groupList,
          headerText: opts.headerText,
          okButtonText: opts.okButtonText,
          mainMessage: opts.mainMessage,
          selectSingleMode: opts.selectSingleMode
        },
        controller: function ($scope, $mdDialog, locals) {
          $scope.locals = locals;
          $scope.ok = function () {
            $mdDialog.hide($scope.selected)
          };
          $scope.cancel = function () {
            $mdDialog.cancel()
          };

          $scope.selected = [];//<- the selected groups

          $scope.isIndeterminate = function () {
            return (
              $scope.selected.length !== 0 &&
              $scope.selected.length !== $scope.locals.groupList.length
            );
          };
          $scope.exists = function (group) {
            return $scope.selected.indexOf(group) > -1;
          };
          $scope.toggle = function (group) {
            var idx = $scope.selected.indexOf(group);
            idx > -1 ? $scope.selected.splice(idx, 1) : $scope.selected.push(group);
          };
          $scope.toggleAll = function () {
            if ($scope.selected.length === $scope.locals.groupList.length) {
              $scope.selected = [];
            }
            else if ($scope.selected.length === 0 || $scope.selected.length > 0) {
              $scope.selected = $scope.locals.groupList.slice(0);
            }
          };
          $scope.selectRadio = function (group) {
            $scope.selected[0] = group;
          }
        }
      })
    }


    function findSequenceGroups(sequenceId, userHref) {
      return GroupsService.findSequenceGroupsWS(sequenceId, userHref);
    }

    return {
      getCurrentAvailableUserGroupsList: getCurrentAvailableUserGroupsList,
      setCurrentAvailableUserGroupsList: setCurrentAvailableUserGroupsList,
      getCurrentTotalAvailableGroups: getCurrentTotalAvailableGroups,
      getPendingUserGroupsList: getPendingUserGroupsList,
      setCurrentGroupIndex: setCurrentGroupIndex,
      getCurrentGroupIndex: getCurrentGroupIndex,
      setCurrentGroupTab: setCurrentGroupTab,
      getCurrentGroupTab: getCurrentGroupTab,
      loadAvailableUserGroups: loadAvailableUserGroups,
      loadPendingUserGroups: loadPendingUserGroups,
      loadOtherGroups: loadOtherGroups,
      showGroupsListModal: showGroupsListModal,
      findSequenceGroups: findSequenceGroups,
      getGroupMembers: getGroupMembers,
      getGroupSpecificMembers: getGroupSpecificMembers,
      calcUserRoleFlag: calcUserRoleFlag
    }
  }]);

/**
 * @typedef {Object} GroupMemberType
 * @property {boolean} administrator
 * @property {string} avatar
 * @property {boolean} banned
 * @property {{href:string}} groupMember
 * @property {number} groupMemberHrefId
 * @property {string} groupName
 * @property {boolean} locked
 * @property {boolean} moderator
 * @property {(string|null)} profilePictureBackgroundUrl
 * @property {(string|null)} profilePictureUrl
 * @property {number} roleFlag
 * @property {string} userBio
 * @property {{href:string}} userGroupLocked - only if user has been locked
 * @property {{href:string}} userSubscription
 */
