"use strict";
appServices
  .factory('sessionFactory', ['AuthService', 'UserService', 'DBServices', 'changeProfilePictureService', 'AWSServices', 'AppAdminRolesService', 'Utils',
    function (AuthService, UserService, DBServices, changeProfilePictureService, AWSServices, AppAdminRolesService, Utils) {
      /**
       * @return {{ACTIVE: string, CANCELLED: string, BANNED: string, BLOCKED: string, DELETED: string}}
       */
      function getStatusList() {
        return {
          ACTIVE: 'Active',
          CANCELLED: 'Canceled',
          BANNED: 'Banned',
          BLOCKED: 'Blocked',
          DELETED: 'Deleted'
        };
      }

      /**
       * @param username {string}
       * @param password {string}
       * @return {Promise}
       */
      function loginWithPublicUser(username, password) {
        return AuthService.login(username.toUpperCase(), password);
      }

      /**
       * @param username
       * @return {Promise}
       */
      function getUserSubscriptionByUsername(username) {
        return Utils.$q(function (resolve, reject) {
          UserService.findUserSubscriptionByUserName().get({userName: username}).$promise.then(function (resp) {
            resolve(resp[0])
          }, function (error) {
            reject(error)
          })
        })
      }

      /**
       * @param username
       * @return {Promise}
       */
      function getAccountStatusByUserName(username) {
        return Utils.$q(function (resolve, reject) {
          getUserSubscriptionByUsername(username).then(function (response) {
            console.log("findUserSubscriptionByUserName", response);
            var status = response.accountStatus;
            var STATUS = getStatusList();
            switch (status) {
              case STATUS.ACTIVE:
                status = STATUS.ACTIVE;
                break;
              case STATUS.CANCELLED:
                status = STATUS.CANCELLED;
                break;
              case STATUS.BANNED:
                status = STATUS.BANNED;
                break;
              case STATUS.BLOCKED:
                status = STATUS.BLOCKED;
                break;
            }

            resolve(response.name ? status : STATUS.DELETED);
          }, function (error) {
            console.error("getAccountStatusByUserName", error);
            reject(error);
          });
        })
      }

      function loadProfileImages(userHrefId) {
        //getBackgroundPicture
        changeProfilePictureService.getBackgroundPictureFromId({
          userId: userHrefId
        }).then(function success(response) {
          console.log("get");
          if (response.result)
            window.localStorage.setItem("bkgMyProfile", response.result.value)
        }, function (response) {
          console.log("getBackgroundPictureFromId", response);
        });

        //Request for largest profile photo
        UserService.findUserProfilePictureLargeById({
          objectId: userHrefId
        }).get().$promise.then(function success(result) {
          //If the user doesn't have a large profile photo we use the photo in the subscription
          //This must be temporal and needs to be deprecated when all users have large photo
          if (result[0].profilePicture && result[0].profilePicture !== null) {
            window.localStorage.setItem("photoProfileBlob", 'data:image/jpeg;base64,' + result[0].profilePicture.split(':')[2]);
          }
          if (result[0].profilePictureUrl && result[0].profilePictureUrl !== null) {
            window.localStorage.setItem("photoProfileBlob", result[0].profilePictureUrl);
          }
        }, function (error) {
          console.error("Error findUserProfilePictureLargeById", error);
        });
      }

      /**
       * @return {Promise}
       */
      function checkDB() {
        return Utils.$q(function (resolve, reject) {
          //Check if the user still have their logged count stored in the device
          DBServices.execute(
            "SELECT * FROM userLogCount WHERE name = ?",
            [window.localStorage.getItem("name")]
          ).then(function (resultSet) {
            if (resultSet.rows.length > 0) {
              //Check if user still have tutorial data stored
              DBServices.execute("SELECT * FROM tutorials").then(function (resultSetTwo) {
                if (resultSetTwo.rows.length > 0) {
                  resolve(true);
                } else {
                  //send populate tutorials
                  resolve(setDefaultsDataBase(1));
                }
              }, function (error) {
                console.error('Execute SELECT tutorials error out', error);
                reject(error)
              })
            } else {
              //send populate log count table
              resolve(setDefaultsDataBase(2));
            }
          }, function (error) {
            console.error('Execute SELECT logCount error in', error);
            resolve(setDefaultsDataBase(4));
          });
        })
      }

      /**
       * @param roleHrefId {number}
       * @returns {Promise}
       */
      function getRolePermissionAndSetUserData(roleHrefId) {
        return Utils.$q(function (resolve, reject) {
          if (!roleHrefId) roleHrefId = 2;//<--regular user
          var currentRoleId = window.localStorage.getItem('userRoleId');

          if (currentRoleId !== roleHrefId) {
            window.localStorage.setItem('userRoleId', roleHrefId.toString());
            //console.log("New role detected");
          }

          AppAdminRolesService.getRolePermissions(roleHrefId).then(function (resources) {
            //console.log("Updating role/permissions...");
            //console.log("Permissions",resources);

            Utils.removeLastItemIfHasNoAttribute(resources, 'codeName');

            /**@type {string[]}*/
            var PERMISSIONS = [];
            resources.forEach(function (permission) {
              PERMISSIONS.push(permission.codeName);
            });

            //console.log("PERMISSIONS",PERMISSIONS);
            UserService.set_USER_PERMISSIONS(PERMISSIONS);

            resolve(true)
          }, function (error) {
            console.error("getRolePermissions", error);
            reject(error)
          })
        })
      }

      function getAndSetAWSToken() {
        if (window.localStorage.getItem('AWSToken') && (window.localStorage.getItem('AWSToken') != '(null)')) {
          AWSServices.awsInit();
        } else {
          var isIOS = ionic.Platform.isIOS();
          if (isIOS === true) {
              window.FCMPlugin.hasPermission(function(response) {
                  console.log("RES:", response);
                  if(response) {
                      if (window.ApnsToken) {
                        window.ApnsToken.getToken(function (token) {
                          window.localStorage.setItem('AWSToken', token);
                          AWSServices.awsInit();
                        }, function (error) {
                          console.error("can't get ios token", error);
                        })
                      }
                      /*window.FCMPlugin.getToken(function(token) {
                        window.localStorage.setItem('AWSToken', token);
                          AWSServices.awsInit();
                      }, function(error) {
                        console.error("Couldn't get FCM token because: ", error);
                      });*/
                  }
              }, function (error) {
                  console.error("ERR: ", error);
              })
          } else {
            if (window.FCMPlugin) {
              window.FCMPlugin.getToken(function (token) {
                window.localStorage.setItem('AWSToken', token);
                AWSServices.awsInit();
              }, function (response) {
                console.error("couldn't get FCM token because: ", response);
              })
            }
          }
        }
      }

      /**
       * @param username
       * @return {Promise}
       */
      function findUserByUserName(username) {
        return Utils.$q(function (resolve, reject) {
          UserService.findUserByUserName().get({userName: username}, function (response) {
            resolve(response[0])
          }, function (error) {
            console.error("Error findUserByUserName", error);
            reject(error)
          });
        });
      }

      /**
       * @param option {number}
       * @return {Promise}
       */
      function setDefaultsDataBase(option) {
        return Utils.$q(function (resolve, reject) {
          switch (option) {
            case 1://case to set tutorial in default
              var statusJson1 = JSON.stringify({first: true, second: true});
              var statusJson2 = JSON.stringify({first: true});
              DBServices.batch([
                ["INSERT INTO tutorials (name, visited) VALUES (?,?)", ["discovery", statusJson1]],
                ["INSERT INTO tutorials (name, visited) VALUES (?,?)", ["upload", statusJson1]],
                ["INSERT INTO tutorials (name, visited) VALUES (?,?)", ["favorites", statusJson1]],
                ["INSERT INTO tutorials (name, visited) VALUES (?,?)", ["timelineOrganize", statusJson2]]
              ]).then(function () {
                resolve(setDefaultsDataBase(4));
              }, function (error) {
                console.error('Transaction ERROR:', error);
                resolve(setDefaultsDataBase(4));
              });
              break;
            case 2://case to set log count in default
              DBServices.execute(
                "INSERT INTO userLogCount (name, count) VALUES (?,?)",
                [window.localStorage.getItem("name"), 1]
              ).then(function () {
                resolve(setDefaultsDataBase(3));
              }, function (error) {
                console.error("Insert log Count error", error);
                resolve(setDefaultsDataBase(4));
              });
              break;
            case 3://after case 2 we check if table for tutorial have any data
              DBServices.execute("SELECT * FROM tutorials").then(function (resultSet) {
                resolve(resultSet.rows.length > 0 ? setDefaultsDataBase(4) : setDefaultsDataBase(1));
              }, function (error) {
                console.error("SELECT tutorials error:", error);
                resolve(setDefaultsDataBase(4));
              });
              break;
            case 4:
              resolve(true);
              break;
            default:
              reject("option " + option + " is invalid");
          }
        })
      }

      return {
        loginWithPublicUser: loginWithPublicUser,
        getUserSubscriptionByUsername: getUserSubscriptionByUsername,
        getAccountStatusByUserName: getAccountStatusByUserName,
        getRolePermissionAndSetUserData: getRolePermissionAndSetUserData,
        findUserByUserName: findUserByUserName,
        loadProfileImages: loadProfileImages,
        getAndSetAWSToken: getAndSetAWSToken,
        checkDB: checkDB,
        getStatusList: getStatusList
      }
    }]);
