"use strict";
appServices
  .factory('SequenceFactory', ['$rootScope', 'CommentFactory', 'cqnzService', 'UserService', 'permissionValidationService', 'notificationsService', 'TagsService', 's3', 'md5', 'Utils', '$sce', '$filter', 'AWSServices', 'Push_Notif_Variables', '$window',
    function ($rootScope, CommentFactory, cqnzService, UserService, permissionValidationService, notificationsService, TagsService, s3, md5, Utils, $sce, $filter, AWSServices, Push_Notif_Variables, $window) {

      /**
       * @param data {*}
       * @param config {{repliesStart: number?, repliesCount:number?}?}
       * @return {SequenceType}
       */
      function buildSequenceObject(data, config) {
        config = config || {};
        /**@type {SequenceMetadataType}*/
        var sequenceMetadata = {
          id: +data.basicData.sequenceId,
          instanceId: +Utils.getLastUrlComponent(data.basicData.sequence.href),
          href: data.basicData.sequence.href + "",
          dateTime: data.basicData.creationTime,
          finalDate: data.basicData.finalDate,
          lifetime: data.basicData.lifetime,
          isPinned: data.basicData.pinned,
          isAdvertisement: data.basicData.isAdvertisement,
          mustBeDeleted: !(!data.basicData.mustBeDeleted),
          belongsMe: (UserService.get_USERSUBSCRIPTIONINSTANCE_ID()) === (+data.sequenceOwnerId),
          reviewStatus: data.basicData.status? data.basicData.status : null,
          groupName: data.basicData.groupName? data.basicData.groupName : null,
          groupId: data.basicData.groupId? data.basicData.groupId : null
        };

        /**@type {SequenceOwnerMetadataType}*/
        var sequenceOwnerMetadata = {
          id: +data.sequenceOwnerId,
          //instanceId:pendiente...
          name: data.basicData.name + "",
          avatar: Utils.getUserImgUrl(data.basicData),
          instID: data.basicData.userSubscriptionId.title.replace(',','')
        };

        //console.log("configureSequence",data.comments)
        data.comments = data.comments || [];
        Utils.removeLastItemIfHasNoAttribute(data.comments, 'sequenceComments');

        /**@type {CommentStructureType[]}*/
        var commentsList = [];

        //console.log("buildSequenceObject: comments", data.comments);
        data.comments.forEach(/**@param comment {CommentRecordType}*/function (comment) {
          comment.groupId = comment.groupId || data.basicData.groupId || 0;
          var parentComment = CommentFactory.buildCommentObject({
            comment: comment,
            sequenceMetadata: sequenceMetadata,
            sequenceOwnerMetadata: sequenceOwnerMetadata
          });

          if (comment.answerCount) {
            CommentFactory.getCommentReplies({
              commentId: comment.commentId,
              sequenceMetadata: sequenceMetadata,
              sequenceOwnerMetadata: sequenceOwnerMetadata,
              start: config.repliesStart !== undefined ? config.repliesStart : 0,
              count: config.repliesCount !== undefined ? config.repliesCount : 1
            }).then(function (childComments) {
              parentComment.responses = childComments;
              //console.log("Parent comment", parentComment);
              commentsList.push(parentComment);
            }, function (error) {
              parentComment.responses = [];
              //console.log("Parent comment", parentComment);
              commentsList.push(parentComment);
            });//<-- getChildComments()
          }
          else
            commentsList.push(parentComment);
        });

        var showExtraText = false;
        var textForShowView = "";
        var showExtraText01 = false;
        //DESCRIPTION ZONE FOR TAG HASH AND URL
          if(window.location.hash != "#/showNotification") {
            textForShowView = $filter('limitTo')( (cqnzService.getTextWithAnchors(
              TagsService.getTextWithFormatedHashes(
                TagsService.getTextWithFormatedTags(data.basicData.description, data.tagsList || []),
                data.hashesList || []
              )
            )), 500, 0 );
            if(textForShowView.length > 499) {
              showExtraText01 = true;
              textForShowView = textForShowView + "...";
            }
          } else {
            textForShowView = cqnzService.getTextWithAnchors(
              TagsService.getTextWithFormatedHashes(
                TagsService.getTextWithFormatedTags(data.basicData.description, data.tagsList || []),
                data.hashesList || []
              )
            );
            showExtraText01 = false;
          }
        //END TITLE ZONE FOR TAG HASH AND URL
        //new code for title
          if(data.basicData.title === undefined || data.basicData.title === null) {
            data.basicData.title = '';
          }
          var  titleForShowView = "";
          var showExtraText02 = false;
        //TITLE  ZONE FOR TAG HASH AND URL
          if(window.location.hash != "#/showNotification") {
            //titleForShowView = $filter('limitTo')( (data.basicData.title), 500, 0 );
            titleForShowView = $filter('limitTo')( (cqnzService.getTextWithAnchors(
              TagsService.getTextWithFormatedHashes(
                TagsService.getTextWithFormatedTags(data.basicData.title, data.tagsList || []),
                data.hashesList || [] 
              )
            )), 500, 0);
            if(titleForShowView.length > 499) {
              showExtraText02 = true;
              titleForShowView = titleForShowView + "...";
            }
          } else {
            //titleForShowView = data.basicData.title;
            titleForShowView = cqnzService.getTextWithAnchors(
              TagsService.getTextWithFormatedHashes(
                TagsService.getTextWithFormatedTags(data.basicData.title, data.tagsList || []),
                data.hashesList || []
              )
            );
            showExtraText02 = false;
          }
        //DESCRIPTION ZONE FOR TAG HASH AND URL
        if(showExtraText01 || showExtraText02) {
          showExtraText = true;
        } else {
          showExtraText = false;
        }

        var haveUrl = false;
        if(data.basicData.url === undefined || data.basicData.url === null || data.basicData.url === '') {
          haveUrl = false;
        } else {
          haveUrl = true;
        }
        var inScheduledPage = false;
        if(window.location.hash === "#/scheduledPosts") {
          inScheduledPage = true;
        }
        if(window.location.hash === "#/scheduledPosts"){
          data.groupsOwned = data.groupsOwned;
        } else {
          data.groupsOwned = [];
        }

        //NEW ZONE FOR GET IF POST RECOMMENDED
          if(!data.basicData.recommendedPost){
            data.basicData.recommendedPost = {
              id: null,
              name: null,
              is: null,
              signalText: null
            }
          }
        //NEW ZONE FOR GET IF POST RECOMMENDED
        //
          var vUPost = false;
          if(data.basicData.verifyUser)
            vUPost = data.basicData.verifyUser;
        //
        var sequenceObj = {
          advertisement: {
            header: {
              color: data.basicData.headerFontColor + "",
              bgColor: data.basicData.headerBgColor + ""
            },
            counters: {
              views: data.viewsCounter,
              clicks: data.clicksCounter,
              URLClicks: data.URLClicksCounter
            }
          },
          comments: {
            enabled: !!(data.basicData.commentEnable && permissionValidationService.canCreateCommentInGralTimeline()),
            counter: +data.basicData.sequenceCommentCount,
            items: commentsList
          },
          content: {
            type: +(data.basicData.sequenceType || cqnzService.getCqnzType(data.basicData.templateId)),
            templateId: data.basicData.templateId + "",
            templateIdBackup: data.basicData.templateId + "",
            items: data.sequenceItems,
            videoPlayedCounter: +data.basicData.playVideoCount
          },
          description: {
            title: textForShowView,
            titleEditable: data.basicData.description + "",
            titleBackup: data.basicData.description + "",
            hashesList: data.hashesList || [],
            tagsList: data.tagsList || [],
            isUpdating: false,
            showExtraText: showExtraText
          },
          title: { //new add
            title: titleForShowView,
            titleEditable: data.basicData.title + "",
            titleBackup: data.basicData.title + "",
            hashesList: data.hashesList || [],
            tagsList: data.tagsList || [],
            isUpdating: false
          },
          url: { //new add
            have: haveUrl,
            url: data.basicData.url,
            urlBackup: data.basicData.url,
            isUpdating: false
          },
          metadata: sequenceMetadata,
          ownerMetadata: sequenceOwnerMetadata,
          like: {//likes of the post
            active: data.likeActive,//fixme: change it for isActive
            //enabled: permissionValidationService.canLikeCqnzInGralTimeline() || permissionValidationService.canLikeCqnzInGroupTimeline()
            //botonState: window.Ionic.WebView.convertFileSrc('../../img/icons/sun2.svg'),
            counter: +data.basicData.likeCount
          },
          react: {
            counter: +data.basicData.reactCount,
            enabled: data.followEachOther
          },
          share: {
            isSharing: false,
            isShared: !(!data.basicData.isShared),
            enabled: !(!data.basicData.shareable),
            counter: +data.basicData.shareCount
          },
          scheluded: {
            time: data.basicData.initialTime || null,
            date: data.basicData.initialDate || null,
            inScheduledPage: inScheduledPage
          },
          waitingGroups: {
            have: data.isLoadingGroups,
            list: data.groupsOwned
          },
          city: {
            real: data.basicData.city ? data.basicData.city : null,
            editable: data.basicData.city ? data.basicData.city : null,
            backUp: data.basicData.city ? data.basicData.city : null
          },
          Device: {
            Android: ionic.Platform.isAndroid(),
            Ios: ionic.Platform.isIOS(),
            Pad: ionic.Platform.isIPad() ? true: (ionic.Platform.platform() === 'macintel'? true : false)
          },
          suggestedPost: {
            show: data.basicData.suggestedPost,
            isFriend: 3, // 3 -> no friends(default) 2-> pending 1-> friends
            isActive: true,
            msgShow: Utils.translate('GLOBAL.FOLLOW'),
            friendInstanceID: null,
            friendID: null,
            waiting: false,
            signalText: Utils.translate('TIMELINE.SUGGESTED_POST')
          },
          recommendedPost: {
            id: data.basicData.recommendedPost.id,
            name: data.basicData.recommendedPost.name,
            is: data.basicData.recommendedPost.is,
            signalText: 'RECOMMENDED'
          },
          verifiedUserPost: {
            active: vUPost
          },
          //QRCode:{
          //  type: data.basicData.promotion
            // no sé que atributos quierás meter aqui
            //por ejemplo type: data.basicData.QRCode.type (<-- esto se debe setear desde el controlador de la secuencia)
          //},

          reportMe: function () {
            setAsInaprropriated(this.metadata.instanceId)
          },

          togglePinMe: function (wantToPin, groupHrefId) {
            return togglePinDefaultOrGroupSequence(this, wantToPin, groupHrefId);
          },

          toggleEnableShareMe: function () {
            return toggleEnableShareable(this);
          },

          toggleEnableMyComments: function (enable) {
            return toggleEnableMyComments(this, enable)
          },

          updateMyTitle: function (form) { //actualiza descripcion
            //return updateSequenceTitle(this, form)
            return updateMyTitle(this, form)
          },

          updateMyDescription: function (form) {
            return updateSequenceDescription(this, form)
          },

          updateMyUrl: function (form) {
            return updateMyUrl(this, form)
          },

          //updateMyContent:updateSequenceContent,

          updateMyAdConfig: function () {
            return updateSequenceAdConfig(this);
          },

          deleteMe: function () {
            return deleteSequence(this)
          },

          likeMe: function () {
            return setLikeToSequence(this)
          },

          reactMe: function (reactionImage) {
            return reactToSequence(this, reactionImage);
          },

          shareMe: function (part) {
            return shareScreen(this, part)
          }
        };

        return sequenceObj;
      }

      /**
       * Pins a sequence on top of a Group timeline
       * @param sequence {SequenceType}
       * @param desiredPinState {boolean}
       * @param {number?} groupHrefId
       */
      function togglePinDefaultOrGroupSequence(sequence, desiredPinState, groupHrefId) {
        return Utils.$q(function (resolve, reject) {
          cqnzService.togglePinDefaultOrGroupSequenceWS(desiredPinState, sequence.metadata.instanceId, groupHrefId).then(function () {
            if (desiredPinState) {
              sequence.metadata.isPinned = true;
              Utils.toast.success(Utils.translate('SEQUENCE.DIALOG.PIN_CQNZ_OK'));
            }

            resolve(desiredPinState);
          }, function (pinError) {
            console.error("togglePinGroupCqnz", pinError);
            Utils.toast.error(
              Utils.translate('SEQUENCE.DIALOG.' + (desiredPinState ? 'PIN_CQNZ_ERR' : 'UNPIN_CQNZ_ERR'))
            );
            reject(pinError)
          });
        })
      }


      /**
       * @param sequence {SequenceType}
       * @return Promise {boolean|*}
       */
      function toggleEnableShareable(sequence) {
        console.log("sequence", sequence)
        return Utils.$q(function (resolve, reject) {
          cqnzService.toggleEnableSequenceShareWS(sequence.metadata.instanceId, !sequence.share.enabled).then(function () {
            sequence.share.enabled = !sequence.share.enabled;
            if (sequence.share.enabled)
              Utils.toast.success(Utils.translate('SEQUENCE.DIALOG.ENABLE_SHAREABLE_OK'));
            else
              Utils.toast.success(Utils.translate('SEQUENCE.DIALOG.DISABLE_SHAREABLE_OK'));

            $rootScope.$broadcast("sequence.shareable.updated", {
              sequenceMetadata: sequence.metadata,
              enabled: sequence.share.enabled
            });

            resolve(true);
          }, function (error) {
            console.error("toggleEnableSequenceShareWS", error);
            if (!sequence.share.enabled)
              Utils.toast.warning(Utils.translate('SEQUENCE.DIALOG.ENABLE_SHAREABLE_ERR'));
            else
              Utils.toast.warning(Utils.translate('SEQUENCE.DIALOG.DISABLE_SHAREABLE_ERR'));

            reject(error)
          })
        })
      }

      function setAsInaprropriated(sequenceInstanceId) {
        cqnzService.setSequenceAsInappropiateWS({
          sequenceId: sequenceInstanceId,
          userHref: UserService.get_USER_INSTANCE_HREF()
        }).then(function () {
          var text01 = Utils.translate("TIMELINE.SET_INAPPROPRIATE_01");
          var text02 = Utils.translate("TIMELINE.SET_INAPPROPRIATE_02");
          Utils.toast.info(
            text01 + '. ' + text02,
            6000,
            {allowHtml: true}
          );
        }, function (err) {
          Utils.toast.warning(Utils.translate("ERROR.UNKNOWN"));
          console.error("setSequenceasInapropiate error", err);
        });
      }

      /**
       * @param sequence {SequenceType}
       * @param desiredEnableState {boolean}
       * @return Promise {boolean|*}
       */
      function toggleEnableMyComments(sequence, desiredEnableState) {
        return Utils.$q(function (resolve, reject) {
          cqnzService.toggleEnableSequenceCommentsWS(sequence.metadata.instanceId, desiredEnableState).then(function (resp) {
            sequence.comments.enabled = desiredEnableState;
            Utils.toast.success(Utils.translate('UPLOAD.' + (desiredEnableState ? 'ENABLE_COMMENTS_OK' : 'DISABLE_COMMENTS_OK')));
            $rootScope.$broadcast('sequence.comment.enabled.updated', {
              sequenceMetadata: sequence.metadata,
              enabled: desiredEnableState
            });

            resolve(desiredEnableState);
          }, function (error) {
            console.error(error);
            Utils.toast.error(Utils.translate('UPLOAD.' + (desiredEnableState ? 'ENABLE_COMMENTS_ERROR' : 'DISABLE_COMMENTS_ERROR')))
            reject(error);
          })
        })
      }

      /**
       * the sequence values: like.active.result.value and like.counter need to be set by means of
       * OPTIMISTIC UPDATES (set them as if the query were success) inside the caller function so that the
       * returned/broadcasted values are consistent
       * @param sequence {SequenceType}
       * @return Promise
       */
      function setLikeToSequence(sequence) {
        return Utils.$q(function (resolve, reject) {
          //this is the new state because it was set previously (optimistic updates) on the caller
          var newActiveState = sequence.like.active.result.value;
          var newLikeCouter = sequence.like.counter;
          console.log("pause", sequence);

          if (sequence.metadata.groupId) {
            cqnzService.setLikeToSequenceGroupWS({
              instanceId: sequence.metadata.instanceId,
              userHref: UserService.get_USER_INSTANCE_HREF(),
              groupId: sequence.metadata.groupId
            }).then(function (response) {
              $rootScope.$broadcast('sequence.counters.updated', {
                type: 'sequence.like',
                sequenceMetadata: sequence.metadata,
                //this is the new value because it was set previously (optimistic updates) on the caller
                counter: newLikeCouter,
                active: newActiveState
              });

              //Send push notification Like Timeline
              //if (response.result.value && sequence.ownerMetadata.id != UserService.get_USERSUBSCRIPTIONINSTANCE_ID()) {
              if (response.result.value && sequence.like.active.result.value && sequence.ownerMetadata.id != UserService.get_USERSUBSCRIPTIONINSTANCE_ID() ) {
                notificationsService.notifySequenceLiked(
                  sequence.ownerMetadata.id,
                  sequence.metadata.id
                )
              }

              if (sequence.like.active.result.value && sequence.ownerMetadata.id != UserService.get_USERSUBSCRIPTIONINSTANCE_ID() ) {
                //avisar que han dado like a post que han comentado
                var wsData = {
                  sequenceId: sequence.metadata.id,
                  userSubscriptionId: UserService.get_USERSUBSCRIPTION_ID(),
                  groupId: sequence.metadata.groupId === null ? 0 : sequence.metadata.groupId,
                  notificationType: 17//TYPE 17
                }
                //NEED NEW WS WITH TYPE
                UserService.getDevicesUsersCommentAPostWithType(wsData).then( function (dataWS) {
                  Utils.removeLastItemIfHasNoAttribute(dataWS, 'userDeviceId');
                  var message = UserService.get_USER_NAME() + Utils.translate('ALERTS.LIKED_POST_COMMENTED');
                  message = (message || "").replace(/\s\s+/g, ' ');
                  dataWS.forEach( function(dataDevices) {

                    AWSServices.sendNotifaboutPostCommented(
                      message, 
                      dataDevices.userDeviceId,
                      sequence.metadata.groupId === null ? 0 : sequence.metadata.groupId,
                      sequence.metadata.id
                    )

                  })
                })
                //NEED NEW WSdATA

                //avisar que han dado like a post que han dado like
                UserService.getDevicesUsersLikedAPost(wsData).then( function (dataWS) {
                  Utils.removeLastItemIfHasNoAttribute(dataWS, 'userDeviceId');
                  var message = UserService.get_USER_NAME() + Utils.translate('ALERTS.LIKED_POST_LIKED');
                  message = (message || "").replace(/\s\s+/g, ' ');
                  dataWS.forEach( function (dataDevices) {
                    AWSServices.sendNotifaboutPostLiked(
                      message, 
                      dataDevices.userDeviceId,
                      sequence.metadata.groupId === null ? 0 : sequence.metadata.groupId,
                      sequence.metadata.id
                    )
                    
                  })
                })
              }

              resolve({counter: newLikeCouter, active: newActiveState})
            }, function (error) {
              console.error("setLikeToSequence error", error);

              Utils.toast.warning(Utils.translate('SEQUENCE.DIALOG.LIKE_CQNZ_ERROR'));

              //roll back to the original values
              //because they were set previously (optimistic updates) on the caller
              newLikeCouter = sequence.like.counter + (!newActiveState ? 1 : -1);
              reject({counter: newLikeCouter, active: !newActiveState, error: error})
            });
          } else {
            cqnzService.setLikeToSequenceWS({
              instanceId: sequence.metadata.instanceId,
              userHref: UserService.get_USER_INSTANCE_HREF()
            }).then(function (response) {
              $rootScope.$broadcast('sequence.counters.updated', {
                type: 'sequence.like',
                sequenceMetadata: sequence.metadata,
                //this is the new value because it was set previously (optimistic updates) on the caller
                counter: newLikeCouter,
                active: newActiveState
              });

              //Send push notification Like Timeline
              //if (response.result.value && sequence.ownerMetadata.id != UserService.get_USERSUBSCRIPTIONINSTANCE_ID()) {
              if (response.result.value && sequence.like.active.result.value && sequence.ownerMetadata.id != UserService.get_USERSUBSCRIPTIONINSTANCE_ID() ) {
                notificationsService.notifySequenceLiked(
                  sequence.ownerMetadata.id,
                  sequence.metadata.id
                )
              }

              if (sequence.like.active.result.value && sequence.ownerMetadata.id != UserService.get_USERSUBSCRIPTIONINSTANCE_ID() ) {
                //avisar que han dado like a post que han comentado
                var wsData = {
                  sequenceId: sequence.metadata.id,
                  userSubscriptionId: UserService.get_USERSUBSCRIPTION_ID(),
                  groupId: sequence.metadata.groupId === null ? 0 : sequence.metadata.groupId,
                  notificationType: 17//TYPE 17
                }
                //NEED NEW WS WITH TYPE
                UserService.getDevicesUsersCommentAPostWithType(wsData).then( function (dataWS) {
                  Utils.removeLastItemIfHasNoAttribute(dataWS, 'userDeviceId');
                  var message = UserService.get_USER_NAME() + Utils.translate('ALERTS.LIKED_POST_COMMENTED');
                  message = (message || "").replace(/\s\s+/g, ' ');
                  dataWS.forEach( function(dataDevices) {

                    AWSServices.sendNotifaboutPostCommented(
                      message, 
                      dataDevices.userDeviceId,
                      sequence.metadata.groupId === null ? 0 : sequence.metadata.groupId,
                      sequence.metadata.id
                    )

                  })
                })
                //NEED NEW WSdATA

                //avisar que han dado like a post que han dado like
                UserService.getDevicesUsersLikedAPost(wsData).then( function (dataWS) {
                  Utils.removeLastItemIfHasNoAttribute(dataWS, 'userDeviceId');
                  var message = UserService.get_USER_NAME() + Utils.translate('ALERTS.LIKED_POST_LIKED');
                  message = (message || "").replace(/\s\s+/g, ' ');
                  dataWS.forEach( function (dataDevices) {
                    AWSServices.sendNotifaboutPostLiked(
                      message, 
                      dataDevices.userDeviceId,
                      sequence.metadata.groupId === null ? 0 : sequence.metadata.groupId,
                      sequence.metadata.id
                    )
                    
                  })
                })
              }

              resolve({counter: newLikeCouter, active: newActiveState})
            }, function (error) {
              console.error("setLikeToSequence error", error);

              Utils.toast.warning(Utils.translate('SEQUENCE.DIALOG.LIKE_CQNZ_ERROR'));

              //roll back to the original values
              //because they were set previously (optimistic updates) on the caller
              newLikeCouter = sequence.like.counter + (!newActiveState ? 1 : -1);
              reject({counter: newLikeCouter, active: !newActiveState, error: error})
            });
          }
        })
      }

      /**
       * @param sequence {SequenceType}
       * @param reactionImage {*}
       * @return Promise {boolean|*}
       */
      function reactToSequence(sequence, reactionImage) {
        var deferred = Utils.$q.defer();

        //Steps to upload
        var s3Service = new AWS.S3({
          apiVersion: '2006-03-01',
          params: {Bucket: s3.bucket}
        });

        //encrypt subscriptioninstanceID and folder name
        var userSubscriptionInstanceId = md5.createHash(UserService.get_USERSUBSCRIPTIONINSTANCE_ID().toString());
        var sequenceId = md5.createHash(sequence.metadata.instanceId.toString());
        var userId = md5.createHash(sequence.ownerMetadata.id.toString());

        //create key for each picture
        var photoKey = s3.prefix + '/' + userSubscriptionInstanceId + '/sequence/' + sequenceId + '/react/' + userId + "_" + Math.random() + '.jpg';

        //photo to blob
        var file = Utils.dataURLtoBlob(reactionImage);

        s3Service.upload({
          Key: photoKey,
          Body: file,
          ACL: 'public-read',
          ContentType: file.type
        }, function (err, data) {
          if (err) {
            console.log('There was an error uploading the small jpg:', err);
            Utils.toast.error(Utils.translate('REACTION.DIALOG.REACTION_SENT_ERROR'))
            return deferred.reject(err)
          }

          cqnzService.reactToSequenceV181WS({
            sequenceInstanceId: sequence.metadata.instanceId,
            senderInstanceId: UserService.get_USERSUBSCRIPTIONINSTANCE_ID(),
            bucketURL: "https://s3-us-west-2.amazonaws.com/" + s3.bucket + "/" + photoKey
          }).$promise.then(function (response) {

            $rootScope.$broadcast('sequence.counters.updated', {
              type: 'sequence.react',
              sequenceMetadata: sequence.metadata,
              counter: sequence.react.counter + 1
            });

            //Send push notification React
            if (response.result.value && sequence.ownerMetadata.id != UserService.get_USERSUBSCRIPTIONINSTANCE_ID())
              notificationsService.notifyReactToSequence(
                sequence.ownerMetadata.id,
                //sequence.sequenceMetadata.id
                sequence.metadata.id
              );

            Utils.toast.success(Utils.translate('REACTION.DIALOG.REACTION_SENT_OK'));
            deferred.resolve(true)
          }, function (error) {
            console.log("sendReaction error:", error);
            Utils.toast.error(Utils.translate('REACTION.DIALOG.REACTION_SENT_ERROR'));
            deferred.reject(error)
          });
        });

        return deferred.promise;
      }

      //ASK AWS TO RETRY UPLOAD UPDATING TIMESTAMP
      AWS.events.on('retry', function(response) {  
        if (response.error.name === 'RequestTimeTooSkewed') {
          console.error('User time is not correct. Handling error!');
          console.log('AWS systemClockOffset:', AWS.config.systemClockOffset);
          var serverTime = Date.parse(response.httpResponse.headers['date']);
          var timeNow = new Date().getTime();
          console.log('AWS timestamp:', new Date(serverTime));
          console.log('Browser timestamp:', new Date(timeNow));
          AWS.config.systemClockOffset = Math.abs(timeNow - serverTime);
          response.error.retryable = true;
          console.log('Setting systemClockOffset to:', AWS.config.systemClockOffset);
          console.log('Retrying uploading to S3 once more...');
        }
      });

      /**
       * @param sequence  {SequenceType}
       * @return Promise {*}
       */
      function deleteSequence(sequence) {
        return Utils.$q(function (resolve, reject) {

          if(sequence.metadata.groupId != null ) {
            //its a post in group
            var conf = {
              id: sequence.metadata.id,
              groupId: sequence.metadata.groupId,
              type: sequence.content.type,
              objects: sequence.content.items//the array of images (urls)
            }

            cqnzService.deleteSequenceAndS3ObjectsGroup(conf).then(function (resp) {
              $rootScope.$broadcast("sequence.deleted", {
                sequenceMetadata: sequence.metadata
              });
              Utils.toast.success(Utils.translate('SEQUENCE.DIALOG.DELETE_CQNZ_OK'));
              resolve(true)
            }, function (error) {
              if (error && (error.status === 504 || error.status === -1)) {
                console.log("We don't care");
                Utils.toast.success(Utils.translate('SEQUENCE.DIALOG.DELETE_CQNZ_OK'));
                resolve(true)
              } else {
                console.log("fallo en cliente");
                console.error("deleteSequenceAndS3Objects", error);
                Utils.toast.error(Utils.translate('SEQUENCE.DIALOG.DELETE_CQNZ_ERROR') + " (" + error.status + ")" );
                reject(error);
              }
            });
          } else {
            var conf = {
              id: sequence.metadata.instanceId,
              type: sequence.content.type,
              objects: sequence.content.items//the array of images (urls)
            };

            cqnzService.deleteSequenceAndS3Objects(conf).then(function (resp) {
              $rootScope.$broadcast("sequence.deleted", {
                sequenceMetadata: sequence.metadata
              });
              Utils.toast.success(Utils.translate('SEQUENCE.DIALOG.DELETE_CQNZ_OK'));
              resolve(true)
            }, function (error) {
              if (error && (error.status === 504 || error.status === -1)) {
                console.log("We don't care");
                Utils.toast.success(Utils.translate('SEQUENCE.DIALOG.DELETE_CQNZ_OK'));
                resolve(true)
              } else {
                console.log("fallo en cliente");
                console.error("deleteSequenceAndS3Objects", error);
                Utils.toast.error(Utils.translate('SEQUENCE.DIALOG.DELETE_CQNZ_ERROR') + " ( " + error.status + ")" );
                reject(error);
              }
            });
          }
        })
      }

      /**
       * @param sequence  {SequenceType}
       * @param part {string} - id of content container
       * @return Promise {boolean|*}
       */
      function shareScreen(sequence, part) {
        var deferred = Utils.$q.defer();

        var deeplink = Personalized.deepLinkPathUrl;

        var s3Service = new AWS.S3({
          apiVersion: '2006-03-01',
          params: {Bucket: s3.bucket}
        });

        var textForShow = "";
        if(sequence.description.titleBackup === "" || sequence.description.titleBackup === null || sequence.description.titleBackup === undefined) {
          textForShow = Utils.translate("SHARE_LINK.POST_ONLY.DESCRIPTION_EMPTY") + sequence.ownerMetadata.name;
        } else {
          textForShow = sequence.description.titleBackup;
        }

        var imageForShow = "";
        //check what kind of post we'll share
        if(sequence.content.type == 1) {
          //images
          imageForShow = sequence.content.items[0].url;
        } else if(sequence.content.type == 3) {
          //video
          imageForShow = sequence.content.items[0].urlVideoImage;
        }

        var branchShareApp = null;

        sequence.share.isSharing = true;
        if (sequence.content.templateId == "templateVideo") {
          var text = Utils.translate('MENU.SETTINGS.SHARE_VIDEO.MESSAGE');
          //new step to mask the url for video with new bucket
          var bucketSaved = sequence.content.items[0].url.split('/')[3];
          var url4Share = sequence.content.items[0].url;
          if(bucketSaved == s3.bucket){
            url4Share = "http://s3-us-west-2.amazonaws.com/" + s3.bucket + '/'//'http://'+ s3.maskBucket + '/'
                      + sequence.content.items[0].url.split('/')[4] + '/'
                      + sequence.content.items[0].url.split('/')[5] + '/'
                      + sequence.content.items[0].url.split('/')[6] + '/'
                      + sequence.content.items[0].url.split('/')[7] + '/'
                      + sequence.content.items[0].url.split('/')[8] + '/'
                      + sequence.content.items[0].url.split('/')[9];
          }
          window.plugins.socialsharing.share(
            text + '\n',
            null,
            null,
            //sequence.content.items[0].url,
            url4Share,
            function () {
              sequence.share.isSharing = false;
              Utils.loading.hide();
              deferred.resolve(popover);
            }, function (error) {
              sequence.share.isSharing = false;
              Utils.loading.hide();
              deferred.reject(error)
            }
          );
        }
        else {
          window.scrollTo(0,0);
          var isIOS = ionic.Platform.isIOS();
          var isAndroid = ionic.Platform.isAndroid();
          var node = document.getElementById(part);
          var nodeX = node.clientWidth;//var nodeX = node.scrollWidth;
          var nodeY = node.clientHeight;//var nodeY = node.scrollHeight;
          window.nodeY = nodeY;
          //Maximum size imageMaximum size image in IOS
          //https://discussions.apple.com/thread/4975106?tstart=0
          window.scale = isIOS && nodeX > 400 ? 3 : 4.5;

          var deepProperties = {
            canonicalIdentifier: 'myProfile',
            title: Utils.translate("SHARE_LINK.POST_ONLY.TITLE"),//'Sharing post',
            contentDescription: textForShow,
            contentImageUrl: imageForShow,
            contentIndexingMode: 'private',
            contentMetadata: { //works
              typeShare: 'Post',
              id: sequence.metadata.id,
              instanceId: sequence.metadata.instanceId
            }
          };

          var branchShareCQNZ = null;
          
          if (!window.Branch) {
            sequence.share.isSharing = false;
            Utils.toast.error(Utils.translate("PLUGINS.BRANCH.PLUGIN_NOT_INSTALLED"));
            deferred.reject()
          }

          window.Branch.createBranchUniversalObject(deepProperties).then(function (res) {
            branchShareCQNZ = res;

            var analyticsDeepLink = {
              feature: Utils.translate("SHARE_LINK.POST_ONLY.TITLE"),//"Share Post",
              campaign: "sharing",
              //deeplink_path: "#/side-menu21/MyProfile",
              stage: "post"
            };

            var propertiesDeepLink = {
              $desktop_url: "http://colmenna.com",
              //$deeplink_path: "#/side-menu21/MyProfile",
              $match_duration: 100,
              $og_image_url: imageForShow,
              $og_title: Push_Notif_Variables.title + " by niido",
              $og_description: textForShow,
              $og_app_id: "519599746650755173",
              reDirectSequenceId: sequence.metadata.id,
              reDirectSequenceInstanceId: sequence.metadata.instanceId,
            };

            branchShareCQNZ.generateShortUrl(analyticsDeepLink, propertiesDeepLink)
              .then(function (res) {
                deeplink = res.url;

                //for slideShow
                if (part.substr(0, 25) == 'imageBoxtemplateAutomate0') {
                  var photoKey = /*s3.prefix*/'Gifs/' + Utils.md5.createHash(UserService.get_USERSUBSCRIPTIONINSTANCE_ID().toString()) + '/shared/' + Utils.md5.createHash(sequence.metadata.id.toString()) + '.gif';
                  s3Service.headObject({Key: photoKey}, function (err, data) {
                    if (!err) {
                      return window.plugins.socialsharing.shareWithOptions({
                        message: deeplink,
                        url: "http://s3-us-west-2.amazonaws.com/" + s3.bucket + "/" + photoKey
                      }, function () {
                        sequence.share.isSharing = false;
                        deferred.resolve(popover);
                      }, function (error) {
                        sequence.share.isSharing = false;
                        deferred.reject(error)
                      });
                    }

                    if (err.code !== 'NotFound') {
                      sequence.share.isSharing = false;
                      return deferred.reject('There was an error creating the gif: ' + err);
                    }

                    //for slidehow
                    var imageList = [];
                    sequence.content.items.forEach(function (item) {
                      if(item.url)
                        imageList.push(item.url);
                    });

                    gifshot.createGIF({
                      gifWidth: 500,
                      gifHeight: 500,
                      images: imageList,
                      interval: .9,
                      numFrames: 10,
                      frameDuration: 1,
                      sampleInterval: 10,
                      numWorkers: 2
                    }, function (obj) {
                      if (!obj.error) {
                        var image = Utils.dataURLtoBlob(obj.image);
                        //$scope.gifData = obj.image;
                        Utils.loading.hide();
                        sequence.share.isSharing = false;
                        s3Service.upload({
                          Key: photoKey,
                          Body: image,
                          ACL: 'public-read',
                          ContentType: image.type
                        }, function (err, data) {
                          if (err) {
                            Utils.loading.hide();
                            sequence.share.isSharing = false;
                            return deferred.reject('There was an error uploading the gif: ', err);
                          }
                          //console.log('Successfully uploaded gif');
                          Utils.loading.hide();
                          sequence.share.isSharing = false;
                          window.plugins.socialsharing.shareWithOptions({
                            //data.Location send a link with amazon's url
                            //url: data.Location
                            message: deeplink,
                            url: "http://s3-us-west-2.amazonaws.com/" + s3.bucket + "/" + photoKey,
                            //files: [$scope.gifData]
                          }, function () {
                            Utils.loading.hide();
                            sequence.share.isSharing = false;
                            deferred.resolve(true)
                          }, function (error) {
                            Utils.loading.hide();
                            sequence.share.isSharing = false;
                            console.error('There was an unknown error creating the gif', error);
                            deferred.reject(error);
                          });
                        });
                      }
                      else {
                        Utils.loading.hide();
                        sequence.share.isSharing = false;
                        return deferred.reject('There was an unknown error creating the gif');
                      }
                    })
                  })
                }
                else {
                  if(isIOS) {
                    html2canvas(node, {
                      taintTest: true,
                      width: nodeX * window.scale,
                      height: nodeY * window.scale,
                      useCORS: true,
                      allowTaint: true
                    }).then(function (canvas) {

                      //console.log("canvas",canvas);
                      var context = canvas.getContext("2d");
                      var logo = new Image();
                      logo.addEventListener('load', imageFound);
                      logo.addEventListener('error', imageNotFound);
                      logo.src = 'img/LogoShare.png';

                      function imageFound() {
                        console.log('That image is found and loaded', canvas);
                        var x = canvas.width - 75 * window.scale;
                        var y = canvas.height - 70 * window.scale;
                        context.drawImage(logo, x, y, 70 * window.scale, 70 * window.scale);
                        var imgDataUrl = canvas.toDataURL();

                        var url = isAndroid === true ? deeplink : null;
                        Utils.loading.hide();
                        window.plugins.socialsharing.shareWithOptions({
                          files: [imgDataUrl],
                          url: url
                        }, function () {
                          Utils.loading.hide();
                          sequence.share.isSharing = false;
                          deferred.resolve(true)
                        }, function (error) {
                          sequence.share.isSharing = false;
                          Utils.loading.hide();
                          console.error('There was an error while sharing the sequence', error);
                          deferred.reject(error);
                        });
                      }

                      function imageNotFound() {
                        console.log('That image was not found.', canvas);
                      }

                    }, function (errorCanvas) {
                      console.log("can't generate image to work in");
                      Utils.loading.hide();
                    });
                  } else {
                    html2canvas(node, {
                      allowTaint: true,
                      backgroundColor: null,
                      onclone: true,
                      removeContainer: true,
                      scale: 1,
                      useCORS: true,
                      width: nodeX,
                      height: nodeY,
                      scrollX: 0,
                      scrollY: 0,
                      windowWidth: nodeX,
                      windowHeight: nodeY
                    }).then(function (canvas) {
                      console.log("canvas",canvas);
                      var context = canvas.getContext("2d");
                      var logo = new Image();
                      logo.addEventListener('load', imageFound);
                      logo.addEventListener('error', imageNotFound);
                      logo.src = 'img/LogoShare.png';

                      function imageFound() {
                        console.log('That image is found and loaded', canvas);
                        var x = (parseInt(canvas.style.width.split("px")[0]) - 60);
                        var y = (parseInt(canvas.style.height.split("px")[0]) + 75);
                        console.log("width and height "+x+","+y);
                        context.drawImage(logo, x, y, 70, 70);
                        var imgDataUrl = canvas.toDataURL();

                        var url = isAndroid === true ? deeplink : null;
                        Utils.loading.hide();
                        window.plugins.socialsharing.shareWithOptions({
                          files: [imgDataUrl],
                          url: url
                        }, function () {
                          Utils.loading.hide();
                          sequence.share.isSharing = false;
                          deferred.resolve(true)
                        }, function (error) {
                          sequence.share.isSharing = false;
                          Utils.loading.hide();
                          console.error('There was an error while sharing the sequence', error);
                          deferred.reject(error);
                        });
                      }
                      function imageNotFound() {
                        console.log('That image was not found.', canvas);
                      }
                    }, function (errorCanvas) {
                      console.log("can't generate image to work in");
                      Utils.loading.hide();
                    });
                  }
                }
              }).catch(function (err) {
                Utils.loading.hide();
                sequence.share.isSharing = false;
                console.error(err);
                console.error("Can't create link to App", err);
              });
          }).catch(function (err) {
            Utils.loading.hide();
            sequence.share.isSharing = false;
            console.error(err);
            console.error("Can't create branch universal object");
          });
        }

        return deferred.promise;
      }

      function updateMyTitle(sequence, form) {
        var deferred = Utils.$q.defer();

        cqnzService.updateSequenceTitle({
          sequenceId: sequence.metadata.instanceId,
          description: form.description || " "
        }).$promise.then(function () {
          var newTitle = {
            title: sequence.title.titleBackup,
            titleEditable: form.description,
            titleBackup: form.description,
            hashesList: [],
            tagsList: [],
            isUpdating: false
          };
          var oldHastagList = sequence.title.hashesList;
          //first unklink all hashs and tags
          var unlinkHashtagPromises = TagsService.unlinkHashtagFromPostDescription(sequence.title.hashesList);
          var unlinkTagsFromPostPromises = TagsService.unlinkTagsFromPostDescription(sequence.title.tagsList);

          //wait for the deletion
          Utils.$q.all({
            deleteTags: Utils.$q.all(unlinkTagsFromPostPromises),
            deleteHashs: Utils.$q.all(unlinkHashtagPromises)
          }).then(function (resolvedDelPromises) {

            //check for any new hash created by the user
            var hashtagInTextList = TagsService.extractHashTagsFromText(form.description);
            var selectedHashtagList = TagsService.getCommonHTagsInLists(
              form.savedHashes, hashtagInTextList
            );
            var hTagsToCreate = TagsService.getHTagsFromList1_NOT_IN_List2(
              TagsService.getHTagsFromList1_NOT_IN_List2(hashtagInTextList, sequence.title.hashesList),
              selectedHashtagList
            );
            var createHashtagPromiseArray = TagsService.createHashtags(hTagsToCreate);

            //made list for all hash inside the post
            var newHashtags = [];
            selectedHashtagList.forEach(function (tag) {
              newHashtags.push({
                hash: tag.hash,
                sequenceHashTag: {href: ''}
              })
            });

            hTagsToCreate.forEach(function (tag) {
              newHashtags.push({
                hash: tag.hash,
                sequenceHashTag: {href: ''}
              })
            });

            //wait to add new hashs for get the id
            Utils.$q.all({
              createHashTags: Utils.$q.all(createHashtagPromiseArray)
            }).then( function (resolvedPromises) {
              //we need to link the selected (already exist) and new (created) tags
              var hashtagIdList = [];

              //becareful! you need to keep the same order as before...
              //first the selected (already exist) then the new created ones
              selectedHashtagList.forEach(function (tag) {
                hashtagIdList.push(tag.id)
              });

              resolvedPromises.createHashTags.forEach(function (createdTagResponse) {
                console.log("createdTagResponse", createdTagResponse);
                hashtagIdList.push(createdTagResponse.result.members.idHashTag.value)
              });

              //LINK hashtags to the sequence title
              var linkHashtagToPostPromiseArray = TagsService.linkHashtagToPostDescription({
                //hashtagIdList: hashtagIdList,
                hashtagIdList: form.savedHashes,
                sequenceInstanceId: sequence.metadata.instanceId,
                notificationType: 7,
                commentID: 0
              });

              //LINK all the tags to the sequence
              var linkTagsToPostPromisesObj = TagsService.linkTagsToPostDescription({
                tagList: form.savedTags,
                sequenceInstanceId: sequence.metadata.instanceId
              });

              Utils.$q.all({
                linkedTagsResolved: Utils.$q.all(linkTagsToPostPromisesObj),
                linkedHashtagsResolved: Utils.$q.all(linkHashtagToPostPromiseArray)
              }).then( function (promiseTagData) {

                //maybe we need the sequenceHashTag attribute to delete the tag later
                //(if user wants to update the comment before reloading)
                promiseTagData.linkedHashtagsResolved.forEach(function (promiseResolved, index) {
                  console.log("promiseResolved", promiseResolved);
                  var resu = promiseResolved.hashTag ? promiseResolved.hashTag.value.href : promiseResolved.result.members.hashTag.value.href
                  //newHashtags[index].sequenceHashTag.href = resu.;
                  newHashtags[index].id = Utils.getLastUrlComponent(resu);
                  newHashtags[index].sequenceHashTag.href = promiseResolved.result.links[0].href;
                });

                /**
                 * @type {{hash: string, id:number}[]}
                 */
                var oldHastagListClean = TagsService.getCommonHTagsInLists(
                  oldHastagList, hashtagInTextList
                );
                console.log("oldHastagListClean", oldHastagListClean);

                //replace old #hashtag list
                newTitle.hashesList = oldHastagListClean.concat(newHashtags);
                console.log("Final hashtagList", newTitle.hashesList);

                //beautify the new title
                newTitle.title = cqnzService.getTextWithAnchors(
                  TagsService.getTextWithFormatedHashes(
                    TagsService.getTextWithFormatedTags(form.description, form.savedTags),
                    newTitle.hashesList
                  )
                );

                //maybe we need the saved id for delete the tag
                //(if user wants to update the comment before reloading)
                Object.keys(promiseTagData.linkedTagsResolved).forEach(function (userId) {
                  form.savedTags.some(function (tag) {
                    if ((tag.id || Utils.getLastUrlComponent(tag.userSubscription.href)) == userId) {
                      tag.sequenceTag = promiseTagData.linkedTagsResolved[userId].result.members.sequenceTag.value;
                      return true;
                    }
                  })
                });

                //replace old @tag list
                newTitle.tagsList = form.savedTags;

                $rootScope.$broadcast('sequence.title.edited', {
                  sequenceId: sequence.metadata.id,
                  description: newTitle
                });

                //finds if the users want to be notified
                var saveTagsCommentNotifsPromises = {};
                Object.keys(promiseTagData.linkedTagsResolved).forEach(function (userId) {
                  saveTagsCommentNotifsPromises[userId] = TagsService.saveTagDescriptionNotificationWS({
                    sequenceId: sequence.metadata.instanceId,
                    userId: userId
                  })
                });

                Utils.$q.all(saveTagsCommentNotifsPromises).then(function (promiseTagCommentNotifData) {
                  //console.log("promiseTagCommentNotifData", promiseTagCommentNotifData);
                  //notifies the users who want to be notified
                  notificationsService.notifyUsersTaggedInSequence(
                    promiseTagCommentNotifData,
                    sequence.metadata.id
                  ).then(function (totalUsers) {
                    if (totalUsers) console.log("Notification was sent to " + totalUsers + " tagged user");
                  }, function () {
                    console.error("Couldn't sent notification(s) to new tagged user", error);
                  });
                }, function (error) {
                  console.error("saveTagsCommentNotifsPromises", error)
                });

                deferred.resolve(newTitle);

              }, function (promiseAddTagError) {
                console.error("Can't add tags", promiseAddTagError);
                Utils.toast.error(Utils.translate('SEQUENCE.DIALOG.EDIT_POST_TITLE_ERROR'));
                deferred.reject(promiseAddTagError);
              })

            }, function (errorCreateHash) {
              console.error("Created hash error", errorCreateHash);
              Utils.toast.error(Utils.translate('SEQUENCE.DIALOG.EDIT_POST_TITLE_ERROR'));
              deferred.reject(errorCreateHash);
            })
          }, function (errorDelPromises) {
            console.error("Deleting tags error", errorDelPromises);
            Utils.toast.error(Utils.translate('SEQUENCE.DIALOG.EDIT_POST_TITLE_ERROR'));
            deferred.reject(errorDelPromises);
          })
        }, function (updateError) {
          console.error("changeTitle error", updateError);
          Utils.toast.error(Utils.translate('SEQUENCE.DIALOG.EDIT_POST_TITLE_ERROR'));
          deferred.reject(updateError);
        });

        return deferred.promise;
      }

      /**
       * @param sequence {SequenceType}
       * @param form {{description:string, savedTags:Array, savedHashes:Array}}
       * @return Promise {*}
       */
      function updateSequenceDescription(sequence, form) {
        var deferred = Utils.$q.defer();

        //Save new title
        cqnzService.updateSequenceDescription({
          sequenceId: sequence.metadata.instanceId,
          description: form.description || " "
        }).$promise.then(function () {
          var newDescription = {
            title: sequence.description.titleBackup,//<-- New formated title will go here
            titleEditable: form.description,
            titleBackup: form.description,
            isUpdating: false,
            hashesList: [],//<-- This will be set later on
            tagsList: []//<-- This will be set later on
          };
          var oldHastagList = sequence.description.hashesList;
          //first unklink all hashs and tags
          var unlinkHashtagPromises = TagsService.unlinkHashtagFromPostDescription(sequence.description.hashesList);
          var unlinkTagsFromPostPromises = TagsService.unlinkTagsFromPostDescription(sequence.description.tagsList);

          //wait for the deletion
          Utils.$q.all({
            deleteTags: Utils.$q.all(unlinkTagsFromPostPromises),
            deleteHashs: Utils.$q.all(unlinkHashtagPromises)
          }).then(function (resolvedDelPromises) {

            //check for any new hash created by the user
            var hashtagInTextList = TagsService.extractHashTagsFromText(form.description);
            var selectedHashtagList = TagsService.getCommonHTagsInLists(
              form.savedHashes, hashtagInTextList
            );
            var hTagsToCreate = TagsService.getHTagsFromList1_NOT_IN_List2(
              TagsService.getHTagsFromList1_NOT_IN_List2(hashtagInTextList, sequence.description.hashesList),
              selectedHashtagList
            );
            var createHashtagPromiseArray = TagsService.createHashtags(hTagsToCreate);

            //made list for all hash inside the post
            var newHashtags = [];
            selectedHashtagList.forEach(function (tag) {
              newHashtags.push({
                hash: tag.hash,
                sequenceHashTag: {href: ''}
              })
            });

            hTagsToCreate.forEach(function (tag) {
              newHashtags.push({
                hash: tag.hash,
                sequenceHashTag: {href: ''}
              })
            });

            //wait to add new hashs for get the id
            Utils.$q.all({
              createHashTags: Utils.$q.all(createHashtagPromiseArray)
            }).then( function (resolvedPromises) {
              //we need to link the selected (already exist) and new (created) tags
              var hashtagIdList = [];

              //becareful! you need to keep the same order as before...
              //first the selected (already exist) then the new created ones
              selectedHashtagList.forEach(function (tag) {
                hashtagIdList.push(tag.id)
              });

              resolvedPromises.createHashTags.forEach(function (createdTagResponse) {
                console.log("createdTagResponse", createdTagResponse);
                hashtagIdList.push(createdTagResponse.result.members.idHashTag.value)
              });

              //LINK hashtags to the sequence description
              var linkHashtagToPostPromiseArray = TagsService.linkHashtagToPostDescription({
                //hashtagIdList: hashtagIdList,
                hashtagIdList: form.savedHashes,
                sequenceInstanceId: sequence.metadata.instanceId,
                notificationType: 7,
                commentID: 0
              });

              //LINK all the tags to the sequence
              var linkTagsToPostPromisesObj = TagsService.linkTagsToPostDescription({
                tagList: form.savedTags,
                sequenceInstanceId: sequence.metadata.instanceId
              });

              Utils.$q.all({
                linkedTagsResolved: Utils.$q.all(linkTagsToPostPromisesObj),
                linkedHashtagsResolved: Utils.$q.all(linkHashtagToPostPromiseArray)
              }).then( function (promiseTagData) {

                //maybe we need the sequenceHashTag attribute to delete the tag later
                //(if user wants to update the comment before reloading)
                promiseTagData.linkedHashtagsResolved.forEach(function (promiseResolved, index) {
                  console.log("promiseResolved", promiseResolved);
                  var resu = promiseResolved.hashTag ? promiseResolved.hashTag.value.href : promiseResolved.result.members.hashTag.value.href
                  //newHashtags[index].sequenceHashTag.href = resu.;
                  newHashtags[index].id = Utils.getLastUrlComponent(resu);
                  newHashtags[index].sequenceHashTag.href = promiseResolved.result.links[0].href;
                });

                /**
                 * @type {{hash: string, id:number}[]}
                 */
                var oldHastagListClean = TagsService.getCommonHTagsInLists(
                  oldHastagList, hashtagInTextList
                );
                console.log("oldHastagListClean", oldHastagListClean);

                //replace old #hashtag list
                //newDescription.hashesList = oldHastagListClean.concat(newHashtags);
                newDescription.hashesList = newHashtags;
                console.log("Final hashtagList", newDescription.hashesList);

                //beautify the new title
                newDescription.title = cqnzService.getTextWithAnchors(
                  TagsService.getTextWithFormatedHashes(
                    TagsService.getTextWithFormatedTags(form.description, form.savedTags),
                    newDescription.hashesList
                  )
                );

                //maybe we need the saved id for delete the tag
                //(if user wants to update the comment before reloading)
                Object.keys(promiseTagData.linkedTagsResolved).forEach(function (userId) {
                  form.savedTags.some(function (tag) {
                    if ((tag.id || Utils.getLastUrlComponent(tag.userSubscription.href)) == userId) {
                      tag.sequenceTag = promiseTagData.linkedTagsResolved[userId].result.members.sequenceTag.value;
                      return true;
                    }
                  })
                });

                //replace old @tag list
                newDescription.tagsList = form.savedTags;

                $rootScope.$broadcast('sequence.description.edited', {
                  sequenceId: sequence.metadata.id,
                  description: newDescription
                });

                //finds if the users want to be notified
                var saveTagsCommentNotifsPromises = {};
                Object.keys(promiseTagData.linkedTagsResolved).forEach(function (userId) {
                  saveTagsCommentNotifsPromises[userId] = TagsService.saveTagDescriptionNotificationWS({
                    sequenceId: sequence.metadata.instanceId,
                    userId: userId
                  })
                });

                Utils.$q.all(saveTagsCommentNotifsPromises).then(function (promiseTagCommentNotifData) {
                  //console.log("promiseTagCommentNotifData", promiseTagCommentNotifData);
                  //notifies the users who want to be notified
                  notificationsService.notifyUsersTaggedInSequence(
                    promiseTagCommentNotifData,
                    sequence.metadata.id
                  ).then(function (totalUsers) {
                    if (totalUsers) console.log("Notification was sent to " + totalUsers + " tagged user");
                  }, function () {
                    console.error("Couldn't sent notification(s) to new tagged user", error);
                  });
                }, function (error) {
                  console.error("saveTagsCommentNotifsPromises", error)
                });

                deferred.resolve(newDescription);

              }, function (promiseAddTagError) {
                console.error("Can't add tags", promiseAddTagError);
                Utils.toast.error(Utils.translate('SEQUENCE.DIALOG.EDIT_POST_DESCRIPTION_ERROR'));
                deferred.reject(promiseAddTagError);
              })

            }, function (errorCreateHash) {
              console.error("Created hash error", errorCreateHash);
              Utils.toast.error(Utils.translate('SEQUENCE.DIALOG.EDIT_POST_DESCRIPTION_ERROR'));
              deferred.reject(errorCreateHash);
            })
          }, function (errorDelPromises) {
            console.error("Deleting tags error", errorDelPromises);
            Utils.toast.error(Utils.translate('SEQUENCE.DIALOG.EDIT_POST_DESCRIPTION_ERROR'));
            deferred.reject(errorDelPromises);
          })
        }, function (updateError) {
          console.error("changeDescription error", updateError);
          Utils.toast.error(Utils.translate('SEQUENCE.DIALOG.EDIT_POST_DESCRIPTION_ERROR'));
          deferred.reject(updateError);
        });

        return deferred.promise;
      }

      function updateSequenceTitle2(sequence, form) {
        var deferred = Utils.$q.defer();

        //Save new title
        cqnzService.updateSequenceDescription({
          sequenceId: sequence.metadata.instanceId,
          description: form.description || " "
        }).$promise.then(function () {
          var newDescription = {
            title: sequence.description.titleBackup,//<-- New formated title will go here
            titleEditable: form.description,
            titleBackup: form.description,
            isUpdating: false,
            hashesList: [],//<-- This will be set later on
            tagsList: []//<-- This will be set later on
          };

          /**
           * @type {{hash: string, sequenceHashTag: {href: string}}[]}
           */
          var oldHastagList = sequence.description.hashesList;
          console.log("oldHastagList", oldHastagList);

          /**
           * We get a standardized array of hashtags out of a string
           * @type {{hash: string, sequenceHashTag: {href: null}, id: null}[]}
           */
          var hashtagInTextList = TagsService.extractHashTagsFromText(form.description);
          console.log("hashtagInTextList", hashtagInTextList);

          var hTagsToRemove = TagsService.getHTagsFromList1_NOT_IN_List2(
            oldHastagList, hashtagInTextList
          );
          console.log("hTagsToRemove", hTagsToRemove);

          /**
           * @type {{hash: string, id:number}[]}
           */
          var selectedHashtagList = TagsService.getCommonHTagsInLists(
            form.savedHashes, hashtagInTextList
          );
          console.log("selectedHashtagList", selectedHashtagList);

          /**
           * @type {{hash: string, sequenceHashTag: {href: (null)}, id: (null)}[]}
           */
          var hTagsToCreate = TagsService.getHTagsFromList1_NOT_IN_List2(
            TagsService.getHTagsFromList1_NOT_IN_List2(hashtagInTextList, oldHastagList),
            selectedHashtagList
          );
          console.log("hTagsToCreate", hTagsToCreate);


          //Remove all the users previously tagged in the sequence title
          var unlinkTagsFromPostPromises = TagsService.unlinkTagsFromPostDescription(sequence.description.tagsList);

          //Remove unused hashtags in the sequence's title
          var unlinkHashtagPromises = TagsService.unlinkHashtagFromPostDescription(hTagsToRemove);

          var increaseHashtagsCountPromises = TagsService.increaseHashtagsCount(selectedHashtagList);

          var createHashtagPromiseArray = TagsService.createHashtags(hTagsToCreate);

          /**
           * @type {{hash:string, sequenceHashTag:{href:string}}[]}
           */
          var newHashtags = [];

          //becareful! you need to keep this order later...
          //first the selected (already exist) then the new created ones
          selectedHashtagList.forEach(function (tag) {
            newHashtags.push({
              hash: tag.hash,
              sequenceHashTag: {href: ''}
            })
          });

          hTagsToCreate.forEach(function (tag) {
            newHashtags.push({
              hash: tag.hash,
              sequenceHashTag: {href: ''}
            })
          });

          //after all the olds @tags were deleted and the new #hashtags were created
          Utils.$q.all({
            deleteTags: Utils.$q.all(unlinkTagsFromPostPromises),
            createHashTags: Utils.$q.all(createHashtagPromiseArray)
          }).then(function (resolvedPromises) {
            console.log('resolvedPromises', resolvedPromises);

            //Remove unused previous tagged users from description
            /*sequence.description.tagsList = TagsService.removeTagsNotFoundInText(
              form.description,
              sequence.description.tagsList
            );*/

            //join old tags with the new ones
            //form.savedTags.push.apply(form.savedTags, sequence.description.tagsList);


            //we need to link the seleceted (already exist) and new (created) tags
            var hashtagIdList = [];

            //becareful! you need to keep the same order as before...
            //first the selected (already exist) then the new created ones
            selectedHashtagList.forEach(function (tag) {
              hashtagIdList.push(tag.id)
            });

            resolvedPromises.createHashTags.forEach(function (createdTagResponse) {
              console.log("createdTagResponse", createdTagResponse);
              hashtagIdList.push(createdTagResponse.result.members.idHashTag.value)
            });

            //link hashtags to the sequence description
            var linkHashtagToPostPromiseArray = TagsService.linkHashtagToPostDescription({
              //hashtagIdList: hashtagIdList,
              hashtagIdList: form.savedHashes,
              sequenceInstanceId: sequence.metadata.instanceId,
              notificationType: 7,
              commentID: 0
            });


            //link tags to the sequence description
            var linkTagsToPostPromisesObj = TagsService.linkTagsToPostDescription({
              tagList: form.savedTags,
              sequenceInstanceId: sequence.metadata.instanceId
            });

            Utils.$q.all({
              linkedTagsResolved: Utils.$q.all(linkTagsToPostPromisesObj),
              linkedHashtagsResolved: Utils.$q.all(linkHashtagToPostPromiseArray)
            }).then(function (promiseTagData) {

              //maybe we need the sequenceHashTag attribute to delete the tag later
              //(if user wants to update the comment before reloading)
              promiseTagData.linkedHashtagsResolved.forEach(function (promiseResolved, index) {
                console.log("promiseResolved", promiseResolved);
                var resu = promiseResolved.hashTag ? promiseResolved.hashTag.value.href : promiseResolved.result.members.hashTag.value.href
                //newHashtags[index].sequenceHashTag.href = resu.;
                newHashtags[index].id = Utils.getLastUrlComponent(resu);
                newHashtags[index].sequenceHashTag.href = promiseResolved.result.links[0].href;
              });

              /**
               * @type {{hash: string, id:number}[]}
               */
              var oldHastagListClean = TagsService.getCommonHTagsInLists(
                oldHastagList, hashtagInTextList
              );
              console.log("oldHastagListClean", oldHastagListClean);

              //replace old #hashtag list
              newDescription.hashesList = oldHastagListClean.concat(newHashtags);
              console.log("Final hashtagList", newDescription.hashesList);

              //beautify the new title
              newDescription.title = cqnzService.getTextWithAnchors(
                TagsService.getTextWithFormatedHashes(
                  TagsService.getTextWithFormatedTags(form.description, form.savedTags),
                  newDescription.hashesList
                )
              );

              //maybe we need the saved id for delete the tag
              //(if user wants to update the comment before reloading)
              Object.keys(promiseTagData.linkedTagsResolved).forEach(function (userId) {
                form.savedTags.some(function (tag) {
                  if ((tag.id || Utils.getLastUrlComponent(tag.userSubscription.href)) == userId) {
                    tag.sequenceTag = promiseTagData.linkedTagsResolved[userId].result.members.sequenceTag.value;
                    return true;
                  }
                })
              });

              //replace old @tag list
              newDescription.tagsList = form.savedTags;

              $rootScope.$broadcast('sequence.description.edited', {
                sequenceId: sequence.metadata.id,
                description: newDescription
              });

              //finds if the users want to be notified
              var saveTagsCommentNotifsPromises = {};
              Object.keys(promiseTagData.linkedTagsResolved).forEach(function (userId) {
                saveTagsCommentNotifsPromises[userId] = TagsService.saveTagDescriptionNotificationWS({
                  sequenceId: sequence.metadata.instanceId,
                  userId: userId
                })
              });

              Utils.$q.all(saveTagsCommentNotifsPromises).then(function (promiseTagCommentNotifData) {
                //console.log("promiseTagCommentNotifData", promiseTagCommentNotifData);
                //notifies the users who want to be notified
                notificationsService.notifyUsersTaggedInSequence(
                  promiseTagCommentNotifData,
                  sequence.metadata.id
                ).then(function (totalUsers) {
                  if (totalUsers) console.log("Notification was sent to " + totalUsers + " tagged user");
                }, function () {
                  console.error("Couldn't sent notification(s) to new tagged user", error);
                });
              }, function (error) {
                console.error("saveTagsCommentNotifsPromises", error)
              });

              deferred.resolve(newDescription);
            }, function (promiseAddTagError) {
              console.error("Can't add tags", promiseAddTagError);
              Utils.toast.error(Utils.translate('SEQUENCE.DIALOG.EDIT_POST_DESCRIPTION_ERROR'));
              deferred.reject(promiseAddTagError);
            });
          }, function (promiseDeleteError) {
            console.error("Deleting tags", promiseDeleteError);
            Utils.toast.error(Utils.translate('SEQUENCE.DIALOG.EDIT_POST_DESCRIPTION_ERROR'));
            deferred.reject(promiseDeleteError);
          });
        }, function (updateError) {
          console.error("changeDescription error", updateError);
          Utils.toast.error(Utils.translate('SEQUENCE.DIALOG.EDIT_POST_DESCRIPTION_ERROR'));
          deferred.reject(updateError);
        });

        return deferred.promise;
      }

      function updateMyUrl(sequence, form) {
        var deferred = Utils.$q.defer();
        var realUrl = "";
        if(form.description === null || form.description === undefined) {
          realUrl = form.url
        } else {
          realUrl = form.description;
        }
        cqnzService.updateSequenceUrl({
          sequenceId: sequence.metadata.instanceId,
          description: realUrl || " "
        }).$promise.then(function () {
          var newUrl = {
            have: true,
            url: realUrl,
            urlBackup: realUrl
          }

          $rootScope.$broadcast('sequence.url.edited', {
            sequenceId: sequence.metadata.id,
            description: newUrl
          });

          deferred.resolve(newUrl);
        }, function (updateError) {
          console.error("changeUrl error", updateError);
          Utils.toast.error(Utils.translate('SEQUENCE.DIALOG.EDIT_POST_URL_ERROR'));
          deferred.reject(updateError);
        })

        return deferred.promise;
      }

      /**
       * @param sequence {SequenceType}
       * @return Promise {*|-1}
       */
      function updateSequenceAdConfig(sequence) {
        return Utils.$q(function (resolve) {
          var dialog = Utils.alert.getCtrl();
          dialog.show({
            templateUrl: 'templates/partials/adsChangeSettingsModal.html',
            controller: 'adConfigEditionCtrl',
            fullscreen: true,
            locals: {sequence: sequence, dialog: dialog},
            clickOutsideToClose: false,
            multiple: true
          }).then(function (newSequenceData) {
            resolve(newSequenceData)
          }, function (error) {
            resolve(-1)
          });
        })
      }

      return {
        buildSequenceObject: buildSequenceObject
      }
    }]);

/**
 * @typedef {Object} SequenceMetadataType
 * @property {number} id
 * @property {number} instanceId
 * @property {string} href
 * @property {*} dateTime
 * @property {(string|finalDate|{value}|Date|*)} finalDate
 * @property {(boolean|string)} lifetime
 * @property {(pinned|{value})} isPinned
 * @property {(boolean|isAdvertisement|{value})} isAdvertisement
 * @property {boolean} mustBeDeleted
 * @property {boolean} belongsMe
 * @property {string} reviewStatus
 */

/**
 * @typedef {Object} SequenceOwnerMetadataType
 * @property {number} id
 * @property {string} name
 * @property {string} avatar
 */

/**
 * @typedef {Object} SequenceType
 * @property {{header: {color: string, bgColor: string}, counters: {views: *, clicks: *}}} advertisement
 * @property {{enabled: boolean, counter: number, items: CommentStructureType[]}} comments,
 * @property {{
 *   type: number,
 *   templateId: string,
 *   templateIdBackup: string,
 *   items: *,
 *   videoPlayedCounter: number
 * }} content
 * @property {{
 *   title: string,
 *   titleEditable: string,
 *   titleBackup: string,
 *   hashesList: {hash:string, sequenceHashTag:{href:string}}[],
 *   tagsList: Array,
 *   isUpdating: boolean
 * }} description
 * @property {SequenceMetadataType} metadata
 * @property {number[]} allowedGroups
 * @property {{active: *, counter: number}} like
 * @property {{counter: number, enabled: *}} react
 * @property {{isSharing: boolean, isShared: boolean, enabled: boolean, counter: number}} share
 * @property {{meter aqui los atributos/propiedades que tendrá el QRCode}} QRCode
 * @property {{lockerId: number, sequence:{href:string, title:string}, sequenceLocker:{href:string}}} locker
 * @property {SequenceOwnerMetadataType} ownerMetadata
 * @property {function(boolean, number?):Promise} togglePinMe
 * @property {function():Promise} toggleEnableShareMe
 * @property {function} reportMe
 * @property {function(boolean):Promise} toggleEnableMyComments
 * @property {function():Promise} likeMe
 * @property {(function():Promise)} deleteMe
 * @property {(function(string):Promise)} reactMe
 * @property {function(*=): Promise} shareMe
 * @property {function(*): Promise} updateMyTitle
 * @property {function(): Promise} updateMyAdConfig
 */
