"use strict";
appControllers
  .controller("chatInboxCtrl2", ['$rootScope', '$scope', '$state', 'UserService', 'ChatFactory2', 'Utils', function($rootScope, $scope, $state, UserService, ChatFactory2, Utils) {

    $scope.inboxChats = [];

    $scope.goSearchUsers = function() {
      $state.go('chatSelectUsers', {}, { reload: true });
    };

    function loadChatRooms() {
      ChatFactory2.DB.getInboxContacts().then(function(contactList) {
        //console.log("loadChatRooms", contactList);
        $scope.inboxChats = contactList;
      }, function(dbError) {
        console.error("couldn't load contacts", dbError)
      })
    }

    $scope.joinRoom = function(inboxChat) {
      $state.go('chatPrivate', { friend: inboxChat, exists: true }, { reload: true });
    };


    function onInboxNewMessageReceived(ev, data) {
      console.log("Reloading inbox...");
      if($state.current.name === 'chatPrivate') {
        console.log("aqui no")
      } else {
        $state.go('menu.messagesInbox', {}, { reload: true });
      }
    }

    function onInboxConnectionChanges(ev, isConnected) {
      console.log("--- SocketIO.connection connected: %s ------", isConnected);
      $scope.isChatConnected = isConnected;

      if (!isConnected) {
        $scope.inboxChats.forEach(function(chat) {
          chat.socket.isOnline = false;
        });
      } else {
        var connStates = ChatFactory2.getContactConnStates();
        console.log("Conn states:", connStates);
        $scope.inboxChats.forEach(function(chat) {
          connStates.some(function(conn) {
            if (conn.id === chat.user.id) {
              chat.socket.isOnline = conn.isOnline;
              return true;
            }
          });
        });
      }

      Utils.$timeout(function() {
        $scope.$apply();
      })
    }

    $scope.sendFriendProfile = function(userInstanceId) {
      if (!userInstanceId) return;

      if (userInstanceId === +UserService.get_USERSUBSCRIPTIONINSTANCE_ID())
        $state.go('menu.myProfile', {}, { reload: true });
      else
        $state.go('friendProfile', { instanceId: userInstanceId }, { reload: true });
    };

    //--------------- EVENT LISTENERS/HANDLERS ---------------
    var socketUnsubscribers = [];
    var rootSubscribers = [];

    $scope.$on("$ionicView.beforeEnter", function() {
      console.log("$ionicView.beforeEnter...");
      $scope.isChatConnected = ChatFactory2.socket.isConnected();

      unsubscribeListeners();

      loadChatRooms();


      socketUnsubscribers.push(
        ChatFactory2.socket.setOnPeerConnChange(function(data) {
          console.log("****---- INBOX: Your peer connection has changed, ids:");
          $scope.inboxChats.some(function(chat) {
            if (chat.user.id === data.userAppId) {
              chat.socket.isOnline = data.isOnline;
              $scope.$apply();
              console.log("peer found, updating online status...");
              return true;
            }
          })
        })
      );

      rootSubscribers.push(
        $rootScope.$on('message.private:new', function(ev, data) {
          if($state.current.name === 'chatPrivate') {
            console.log("aqui no")
          } else {
            $state.go('menu.messagesInbox', {}, { reload: true });
          }
        })
      );

      rootSubscribers.push($rootScope.$on('message.private:new', onInboxNewMessageReceived));

      rootSubscribers.push($rootScope.$on("socketIO.connection", onInboxConnectionChanges));
    });

    $scope.$on("$ionicView.beforeLeave", function() {
      //console.log("$ionicView.beforeLeave");
      unsubscribeListeners();
    });

    function unsubscribeListeners() {
      console.log("Unsubscribing from socket & root events...");
      socketUnsubscribers.forEach(function(unsubscribe) {
        (unsubscribe || angular.noop)()
      });
      socketUnsubscribers = [];

      rootSubscribers.forEach(function(unsubscribe) {
        (unsubscribe || angular.noop)()
      });
      rootSubscribers = [];
    }

    //============= END EVENT LISTENERS/HANDLERS =============
  }])

  .controller('chatPrivateCtrl2', ['$rootScope', '$scope', '$state', 'UserService', 'ImageService3', 'ChatFactory2', 'Utils', 'ImageService2', 'FileService3', '$ionicActionSheet', 'FileService2', '$jrCrop', '$timeout', function($rootScope, $scope, $state, UserService, ImageService3, ChatFactory2, Utils, ImageService2, FileService3, $ionicActionSheet, FileService2, $jrCrop, $timeout) {
    console.log("inside chatPrivateCtrl2", $state.params);

    /**
     * @type {{socket:{isOnline:boolean}, user:{id:number, name:string, avatar:string, description:string}, message:{}}}
     */
    var isIOS = ionic.Platform.isIOS();
    var msglistLen = 0;
    var socketUnsubscribers = [];
    $scope.friend = $state.params.friend;
    $scope.addToContacts = $state.params.addToContacts;
    $scope.MEDIATYPE = ChatFactory2.MEDIA_TYPE;
    $scope.todayDate = new Date().toISOString().slice(0, 10);
    $scope.STATUS = ChatFactory2.STATUS;
    $scope.form = { message: '' };
    $scope.myAppId = UserService.get_USERSUBSCRIPTION_ID();
    $scope.iconOff = `<svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24">
        <path fill="none" d="M24 .01c0-.01 0-.01 0 0L0 0v24h24V.01zM0 0h24v24H0V0zm0 0h24v24H0V0z"/>
        <path d="M22.99 9C19.15 5.16 13.8 3.76 8.84 4.78l2.52 2.52c3.47-.17 6.99 1.05 9.63 3.7l2-2zm-4 4c-1.29-1.29-2.84-2.13-4.49-2.56l3.53 3.53.96-.97zM2 3.05L5.07 6.1C3.6 6.82 2.22 7.78 1 9l1.99 2c1.24-1.24 2.67-2.16 4.2-2.77l2.24 2.24C7.81 10.89 6.27 11.73 5 13v.01L6.99 15c1.36-1.36 3.14-2.04 4.92-2.06L18.98 20l1.27-1.26L3.29 1.79 2 3.05zM9 17l3 3 3-3c-1.65-1.66-4.34-1.66-6 0z"/>
    </svg>`;
    $scope.iconOn = `<svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24">
        <path fill="none" d="M0 0h24v24H0z"/>
        <path d="M1 9l2 2c4.97-4.97 13.03-4.97 18 0l2-2C16.93 2.93 7.08 2.93 1 9zm8 8l3 3 3-3c-1.65-1.66-4.34-1.66-6 0zm-4-4l2 2c2.76-2.76 7.24-2.76 10 0l2-2C15.14 9.14 8.87 9.14 5 13z"/>
    </svg>`;
    $scope.messageList = [];
    $scope.canLoadMore = true;
    var start = 0,
        count = 10;
    var kbEl = document.getElementById('sendMsgBtn');
    FileService2.cleanImages();
    $scope.imageURI = FileService2.images();
    $scope.imageURIShow = FileService2.images();
    $scope.uploadingPhoto = false;
    $scope.openCrop = false;
    $scope.rndCache = 0;
    $scope.selectedChatImage = '';
    $scope.activeSlide = 0;
    $scope.zoomStatus = 0;
    $scope.zoomMin = 1;

    if (!$scope.friend) {
      Utils.alert.show("friend info is not available");
      return;
    }

    if ($rootScope.mensaje != "") { //OmRo shared
      $scope.form.message = $rootScope.mensaje;
    }

    if (kbEl) {
      console.log("setting keep keyboard open");
      Utils.keepKeyboardOpenWhenHitting(kbEl, function() {
        if ($scope.form.message && $scope.isChatConnected)
          sendMessage($scope.form.message);
      });
    }

    /**
     * @param contactId {number}
     */
    function joinRoom(contactId) {
      ChatFactory2.socket.joinRoom(contactId).then(
        /**@param joinResp {{isPeerOnLine: boolean, roomId: number, isInSameRoom:boolean}}*/
        function(joinResp) {
          console.log("You are now joined to room", joinResp.roomId);
          ChatFactory2.setChatWithUserId(joinResp.roomId);

          $scope.friend.socket.isOnline = joinResp.isPeerOnLine;

          socketUnsubscribers.push(ChatFactory2.socket.setOnMsgACKByPeer(function(ackObj) {
            console.log("---- (setOnMsgACKByPeer) Your peer has %s your message", ackObj.status === $scope.STATUS.MSG_RECEIVER_SAVED ? 'received' : 'read');
            $scope.messageList.some(function(item) {
              if (item.msg.uniqueId === ackObj.uniqueId) {
                item.msg.status = ackObj.status;
                return true;
              }
            })
          }));
          socketUnsubscribers.push(ChatFactory2.socket.setOnPeerConnChange(function(data) {
            //console.log("****---- CHAT ROOM: Your peer connection has changed", data);
            if ($scope.friend.user.id === data.userAppId) {
              $scope.friend.socket.isOnline = data.isOnline;
              Utils.$timeout(function() {
                $scope.$apply();
              })
            }
          }));
          socketUnsubscribers.push(ChatFactory2.socket.setOnPeerLeftRoom(function(resp) {
            console.log("*---- Your peer has left the room")
          }));

          //only when server thinks that receiver is online but the delivery timed out
          socketUnsubscribers.push(ChatFactory2.socket.setOnMsgArchived(function(resp) {
            console.log("*---- Your msg was archived", resp);
            $scope.messageList.some(function(item) {
              if (item.msg.uniqueId === resp.uniqueId) {
                item.msg.status = $scope.STATUS.MSG_ARCHIVED;
                item.msg.processedAt = resp.processedAt;
                return true;
              }
            })
          }));

          Utils.$timeout(function() {
            $scope.$apply();
          })
        },
        function(error) {
          console.error("Couldn't joint to room :(", error);
          Utils.toast.error(Utils.translate("ERROR.UNKNOWN"));
        });
    }

    /**
     * Changes the in-memory room in socket server, never leaves the room
     */
    function leaveRoom() {
      //console.log("Trying to leave room %s", $scope.friend.user.id);
      ChatFactory2.socket.leavePrivateRoom($scope.friend.user.id).then(function(oldRoomId) {
        console.log("You have left the room %s", oldRoomId);
      }, function(error) {
        console.error("Could't leave the room %s", $scope.friend.user.id, error);
      });
    }

    function loadAllMsgsFromFriend() {
      ChatFactory2.DB.getAllContactMsgs(
        +$scope.friend.user.id
      ).then(function(messageList) {
        start = start - count > 0 ? start - count : 0;
        $scope.messageList = messageList;
        msglistLen = $scope.messageList.length;
        $scope.canLoadMore = messageList.length === count;
      }, function(error) {
        console.error("load AllMessagesFromFriend", error)
      })
      //})
    }

    $scope.loadMore = function() {
      if (!$scope.canLoadMore) {
        console.log("There are no more old messages");
        return;
      }

      ChatFactory2.DB.getAllContactMsgs(
        +$scope.friend.user.id, start, count
      ).then(function(messageList) {
        $scope.canLoadMore = messageList.length === count;
        start = start - count > 0 ? start - count : 0;
        $scope.messageList = messageList.concat($scope.messageList);
        msglistLen = $scope.messageList.length;
      }, function(error) {
        console.error("loadMore", error)
      })
    };

    loadAllMsgsFromFriend();

    $scope.backInbox = function() {
      Keyboard.hide();
      setTimeout(function () {
        $state.go('menu.messagesInbox', {}, { reload: true });
        $rootScope.mensaje = ''
      },100)
    };

    //image selection
    Utils.$ionicModal.fromTemplateUrl('templates/Chat/chat-msg-image-modal.html', {
      scope: $scope,
      animation: 'slide-in-up'
    }).then(function(modal) {
      $scope.modal = modal;
    });

    $scope.closeModal = function() {
      $scope.modal.hide();
    };
    // Cleanup the modal when we're done with it!
    $scope.$on('$destroy', function() {
      $scope.modal.remove();
    });

    $scope.checkPermissionsBefore = function() {
      if (isIOS === true) {
        //getPermission(0);
        $scope.openImageOptions();
      } else {
        getMultiplePermissions();
      }
    }

    /**
     *function to get multiple permission for camera
     */
    function getMultiplePermissions() {
      cordova.plugins.diagnostic.getPermissionsAuthorizationStatus(function(statuses) {
        var havePerm = 0;
        var deniedPerm = false;
        for (var permission in statuses) {
          switch (statuses[permission]) {
            case cordova.plugins.diagnostic.permissionStatus.GRANTED:
              console.log("Permission granted to use " + permission);
              havePerm++;
              break;
            case cordova.plugins.diagnostic.permissionStatus.NOT_REQUESTED:
              console.log("Permission to use " + permission + " has not been requested yet");
              havePerm--;
              break;
            case cordova.plugins.diagnostic.permissionStatus.DENIED:
              console.log("Permission denied to use " + permission + " - ask again?");
              havePerm--;
              break;
            case cordova.plugins.diagnostic.permissionStatus.DENIED_ALWAYS:
              console.log("Permission permanently denied to use " + permission + " - guess we won't be using it then!");
              deniedPerm = true;
              break;
          }
        }
        if (!deniedPerm) {
          if (havePerm == 2) {
            //have both permissions granted
            $scope.openImageOptions();
          } else if (havePerm < 2) {
            //both or one permission hasn't granted
            askMultiplePermissions();
          }
        } else {
          Utils.toast.error(Utils.translate('PLUGINS.PERMISSIONS.DENIED_PERMISSION') +
            Utils.translate('PLUGINS.PERMISSIONS.TYPE.CAMERA') + ' ' +
            Utils.translate('PLUGINS.PERMISSIONS.AND') + ' ' +
            Utils.translate('PLUGINS.PERMISSIONS.TYPE.GALLERY') + '. \n' +
            Utils.translate('PLUGINS.PERMISSIONS.ALWAYS_DENIED_PERMISSION'));
        }
      }, function(error) {
        console.error("The following error occurred: " + error);
      }, [
        cordova.plugins.diagnostic.permission.CAMERA,
        cordova.plugins.diagnostic.permission.READ_EXTERNAL_STORAGE
      ])
    }

    /**
     *function to get multiple permission for camera
     */
    function askMultiplePermissions() {
      cordova.plugins.diagnostic.requestRuntimePermissions(function(statuses) {
        var havePerm = 0;
        var deniedPerm = false;
        for (var permission in statuses) {
          switch (statuses[permission]) {
            case cordova.plugins.diagnostic.permissionStatus.GRANTED:
              console.log("Permission granted to use " + permission);
              havePerm++;
              break;
            case cordova.plugins.diagnostic.permissionStatus.NOT_REQUESTED:
              console.log("Permission to use " + permission + " has not been requested yet");
              havePerm--;
              break;
            case cordova.plugins.diagnostic.permissionStatus.DENIED:
              console.log("Permission denied to use " + permission + " - ask again?");
              havePerm--;
              break;
            case cordova.plugins.diagnostic.permissionStatus.DENIED_ALWAYS:
              console.log("Permission permanently denied to use " + permission + " - guess we won't be using it then!");
              deniedPerm = true;
              break;
          }
        }
        if (!deniedPerm) {
          if (havePerm == 2) {
            //have both permissions granted
            $scope.openImageOptions();
          } else if (havePerm < 2) {
            //both or one permission hasn't granted
            Utils.toast.error(Utils.translate('PLUGINS.PERMISSIONS.DENIED_PERMISSION') +
              Utils.translate('PLUGINS.PERMISSIONS.TYPE.CAMERA') + ' ' +
              Utils.translate('PLUGINS.PERMISSIONS.AND') + ' ' +
              Utils.translate('PLUGINS.PERMISSIONS.TYPE.GALLERY') + '.');
          }
        } else {
          Utils.toast.error(Utils.translate('PLUGINS.PERMISSIONS.DENIED_PERMISSION') +
            Utils.translate('PLUGINS.PERMISSIONS.TYPE.CAMERA') + ' ' +
            Utils.translate('PLUGINS.PERMISSIONS.AND') + ' ' +
            Utils.translate('PLUGINS.PERMISSIONS.TYPE.GALLERY') + '. \n' +
            Utils.translate('PLUGINS.PERMISSIONS.ALWAYS_DENIED_PERMISSION'));
        }
      }, function(error) {
        console.error("The following error occurred: " + error);
      }, [
        cordova.plugins.diagnostic.permission.CAMERA,
        cordova.plugins.diagnostic.permission.READ_EXTERNAL_STORAGE
      ])
    }

    //bottom list for image picker
    $scope.openImageOptions = function() {
      $scope.hideSheet = $ionicActionSheet.show({
        buttons: [
          { text: 'Take photo' },
          { text: 'Photo from library' }
        ],
        titleText: 'Add images',
        cancelText: 'Cancel',
        buttonClicked: function(index) {
          $scope.getMessageImage(index);
        }
      });
    };

    $scope.getMessageImage = function(index) {
      $scope.hideSheet();
      $scope.imageURI = [];
      if (index === 0) { // camera
        ImageService2.saveCameraMediaMessagePhoto(index).then(function() {
        });
      } else { // library
        ImageService2.saveAlbumMediaMessagePhoto(index).then(function() {
        });
      }
    }

    $rootScope.$on("addedImageMsg", function() { // LISTENER FOR IMAGE LOADED
      showImageMsg();
    });

    function showImageMsg() {
      $scope.imageURI = [];
      $scope.$apply();
      var img = FileService2.images();
      var exifTemp = FileService2.exifImage();
      if (img.length === 0) {
        $timeout(function() {
          showImageUser();
        }, 1000);
      } else {
        if ($scope.imageURI === null || $scope.imageURI === undefined || $scope.imageURI.length === 0) {
          var opt = {};
          if (isIOS === true) {
            $scope.imageURI = img[0];
            $scope.modal.show();
            $scope.$apply();
          } else {
            var urlToUse = null;
            if (img[0].uri) {
              urlToUse = img[0].uri;
            } else {
              urlToUse = img[0];
            }
            $scope.imageURI = urlToUse;
            $scope.modal.show();
            $scope.$apply();
          }
        } else {
          $scope.openCrop = false;
          console.log("finish loading");
        }
      }
    }

    $scope.sendFromModal = function() {
      $scope.modal.hide();
      sendMessage($scope.form.message, { URI: $scope.imageURI, type: ChatFactory2.MEDIA_TYPE.IMAGE });
    };

    function sendMessage(text, media) {
      media = media || { URI: '', type: '' };
      console.log("media", media);

      var tempText = $scope.form.message;
      $scope.form.message = '';

      var newMsg = new ChatFactory2.PrivateMessageToSend({
        toContactId: $scope.friend.user.id,
        text: text
      });

      //add new attributes for using in template
      if (media.URI) {
        newMsg.fileSrc = media.URI;
        newMsg.fileType = ChatFactory2.MEDIA_TYPE.IMAGE;
      }

      var dDiv = '';
      if (!msglistLen)
        dDiv = new Date().toISOString().slice(0, 10);
      else if (newMsg.createdAt.slice(0, 10) !== $scope.messageList[msglistLen - 1].msg.createdAt.slice(0, 10)) {
        dDiv = newMsg.createdAt.slice(0, 10)
      }

      var finalMsg = {
        dividerDate: dDiv,
        msg: newMsg,
        mediaProgress: 0
      };

      msglistLen = $scope.messageList.push(finalMsg);

      //console.log("finalMsg", finalMsg);

      if (media.URI) {
        media.URI = media.URI.substr(0, media.URI.indexOf('?') > -1 ? media.URI.indexOf('?') : media.URI.length)
        ChatFactory2.util.uploadPrivateMsgImage({
          fromId: $scope.myAppId,
          toId: $scope.friend.user.id,
          msgId: newMsg.uniqueId.replace(/:/g, '_'),
          imgURI: media.URI,
        }).then(function(uploadS3Response) {
          console.log("Message image has been uploaded successfully");

          finalMsg.mediaProgress = 100;

          var ext = media.URI.split('.');
          ext = ext.length > 1 ? ext.pop() : '';

          var fileInfo = {
            fileType: ChatFactory2.MEDIA_TYPE.IMAGE,
            fileSrc: uploadS3Response.Location,
            fileMimeType: 'image/' + ext,
            fileExtension: ext
          };

          newMsg.fileSrc = fileInfo.fileSrc;

          console.log("File uploaded!", fileInfo);

          ChatFactory2.DB.insertNewFile(fileInfo).then(function(id) {
            newMsg.fileId = id;

            //Insert new attributes for sending to socket
            newMsg.fileType = ChatFactory2.MEDIA_TYPE.IMAGE;
            newMsg.fileSrc = uploadS3Response.Location;
            newMsg.fileMimeType = 'image/' + ext;

            if ($scope.addToContacts)
              _insertContactAndSendMsg(newMsg);
            else
              doSendMSG(newMsg);
          }, function(error) {
            //error printed in factory
          })
        }, function( /*error*/ ) {
          //error printed inside function

          //roll data back
          $scope.messageList.pop();
          $scope.form.message = tempText;
          Utils.alert.show(Utils.translate('GROUPS.SETTINGS.DIALOG.UPLOAD_IMAGE_ERR'), { title: "Error" })
        });
      } else {
        if ($scope.addToContacts)
          _insertContactAndSendMsg(newMsg);
        else
          doSendMSG(newMsg);
      }

      Utils.$ionicScrollDelegate.scrollBottom();
      $rootScope.mensaje = ''; //OmRo
    }

    /**
     *
     * @param newMsg
     * @private
     */
    function _insertContactAndSendMsg(newMsg) {
      ChatFactory2.DB.insertNewContact($scope.friend.user).then(function(id) {
        //console.log("New contact added with id %s", id);
        ChatFactory2.socket.joinToContactRoom($scope.friend.user.id).then(function(resp) {
          ChatFactory2.socket.createContactRoom($scope.friend.user.id, resp.isOnline, true /*skipCheck*/ );

          $scope.addToContacts = false;
          doSendMSG(newMsg);
        }, function(error) {
          console.error("joinToContactRoom %d, %s", $scope.friend.user.id, $scope.friend.user.name, error);
          Utils.toast.error(Utils.translate("ERROR.UNKNOWN"));
          doSendMSG(newMsg);
        })
      }, function(dbError) {
        console.error("Couldn't create the contact", $scope.friend.user, dbError);
        Utils.toast.error(Utils.translate("ERROR.UNKNOWN"))
      });
    }

    /**
     *
     * @param {PrivateMessageToSend} newMsg
     */
    function doSendMSG(newMsg) {
      ChatFactory2.socket.send(newMsg).then(function(ackObj) {
        //console.log("the msg has been processed by server...", ackObj);

        if (ackObj.status === ChatFactory2.STATUS.MSG_ARCHIVED)
          ChatFactory2.util.sendPushNotification($scope.friend.user.id).then();

        newMsg.status = ackObj.status;
        newMsg.processedAt = ackObj.processedAt;
      }, function(error) {
        console.error("Can't send message", error);
        Utils.toast.error("El mensaje no pudo ser enviado");
      });
    }

    function onNewMessageReceived(ev, data) {
      console.log("--- new message detected in friend %s !", $scope.friend.user.id, data);

      var dDiv = '';
      if (!msglistLen)
        dDiv = new Date().toISOString().slice(0, 10);
      else if (data.msg.createdAt.slice(0, 10) !== $scope.messageList[msglistLen - 1].msg.createdAt.slice(0, 10)) {
        dDiv = data.msg.createdAt.slice(0, 10)
      }

      msglistLen = $scope.messageList.push({
        dividerDate: dDiv,
        msg: data.msg,
        mediaProgress: 100
      });

      Utils.$timeout(function() {
        $scope.$apply();
        Utils.$ionicScrollDelegate.$getByHandle('private-chat-container').scrollBottom();
      }, 800);
    }
    
    $scope.showImageViewer = function(imageSrc) {
      $scope.selectedChatImage = imageSrc;
      Utils.$ionicModal.fromTemplateUrl('templates/Chat/image-viewer.html', {
        scope: $scope,
        animation: 'zoom-from-center'
      }).then(function(modal) {
        $scope.imageViewer = modal;
        $scope.imageViewer.show();
      });
    };

    $scope.closeImageViewer = function() {
      $scope.selectedChatImage = '';
      $scope.imageViewer.hide();
      $scope.imageViewer.remove();
    };
    
    $scope.doubleTapped = function(slide) {
      if ($scope.zoomStatus === 0) {
        Utils.$ionicScrollDelegate.$getByHandle('scrollHandle' + slide).zoomBy(2, true);
        $scope.zoomStatus++;
      } else {
        Utils.$ionicScrollDelegate.$getByHandle('scrollHandle' + slide).zoomBy(0, true);
        $scope.zoomStatus--;
      }
    };

    $scope.updateSlideStatus = function(slide) {
      var zoomFactor = Utils.$ionicScrollDelegate.$getByHandle('scrollHandle' + slide).getScrollPosition().zoom;
      if (zoomFactor === $scope.zoomMin) {
        Utils.$ionicSlideBoxDelegate.enableSlide(true);
      } else {
        Utils.$ionicSlideBoxDelegate.enableSlide(false);
      }
    };

    $scope.trustAsHtml = function(text) {
      return Utils.$sce.trustAsHtml(ChatFactory2.util.generateAnchors(text));
    };

    $scope.openInappBrowser = function(url) {
      var isIOS = ionic.Platform.isIOS();
      var zone = '_blank';
      var opt = '';
      if(isIOS){
        zone = '_blank';
        opt = 'clearsessioncache=yes,usewkwebview=yes,toolbar=yes,presentationstyle=fullscreen,hidespinner=yes,closebuttoncaption=Close,closebuttoncolor=#ffffff4,navigationbuttoncolor=#ffffff,lefttoright=yes,location=no';
      }
      else{
        zone ='_blank';
        opt = 'clearsessioncache=yes,usewkwebview=yes,toolbar=yes,presentationstyle=fullscreen,hidespinner=yes,closebuttoncaptionSet=Close,closeButtonColorSet=#ffffff4,navigationbuttoncolorSet=#ffffff,lefttoright=yes,location=no';
      }
      var ref = cordova.InAppBrowser.open( url, zone, opt);
    };
    
    //--------------- EVENT LISTENERS/HANDLERS ---------------
      
    var rootSubscribers = [];
    $scope.$on('$ionicView.beforeEnter', function() {
      $scope.isChatConnected = ChatFactory2.socket.isConnected();
      Keyboard.setResizeMode('native');
      if ($scope.isChatConnected) joinRoom($scope.friend.user.id);

      rootSubscribers.push(
        $rootScope.$on('message.private:new', onNewMessageReceived)
      );

      rootSubscribers.push(
        $rootScope.$on("socketIO.connection", onConnectionChanges)
      );
    });

    var textArea;
    $scope.$on('$ionicView.afterEnter', function() {
      setTimeout(function() {
        if (!textArea)
          textArea = document.getElementsByClassName('privateTextArea')[0];
        //added class to find object in view
        if (textArea) {
          textArea.focus();
        } else {
          console.warn("can't find message input")
        }

        Utils.$timeout(function() {
          Utils.$ionicScrollDelegate.$getByHandle('private-chat-container').scrollBottom();
        }, 650);
      }, 100);
    });

    $scope.$on("$ionicView.beforeLeave", function() {
      console.log("before leave clean all");
      Keyboard.setResizeMode('ionic');
      leaveRoom();
      
      removeSocketSubscribers();

      if (kbEl)
        Utils.doNotKeepKeyboardOpenWhenHitting(kbEl);

      rootSubscribers.forEach(function(unsubscribe) {
        (unsubscribe || angular.noop)()
      });

      rootSubscribers = [];
    });

    function removeSocketSubscribers() {
      socketUnsubscribers.forEach(function(unsubscribe) {
        (unsubscribe || angular.noop)()
      });
      socketUnsubscribers = [];
    }

    function onConnectionChanges(ev, isConnected) {
      console.log("--- SocketIO.connection connected: %s ------", isConnected);
      $scope.isChatConnected = isConnected;
      if ($scope.isChatConnected)
        joinRoom($scope.friend.user.id);
      else {
        removeSocketSubscribers();
        $scope.friend.socket.isOnline = false;
        Utils.toast.warning(Utils.translate('CHAT.OFFLINE_WARN_MESSAGE'));
      }

      Utils.$timeout(function() {
        $scope.$apply();
      })
    }

    //============= END EVENT LISTENERS/HANDLERS =============
  }])

  .controller("chatSearchUsersCtrl2", ['$scope', '$rootScope', '$state', 'UserService', 'AsyncSearch', 'ChatFactory2', 'Utils', 'searchFactory',
    function($scope, $rootScope, $state, UserService, AsyncSearch, ChatFactory2, Utils, searchFactory) {

      var lastSearch, pendingSearch;

      $scope.currentContacts = [];
      $scope.usersFound = [];
      $scope.isSearching = false;
      $scope.actionType = "suggested";

      $scope.data = {
        userName: "",
        selectedUsers: [],
        showSelectedContactsList: false,
        readyForMessage: false,
        multipleUsers: false,
        maxLength: 0,
        typeMessage: 'Grupal',
        friendsIdList: [],
        friendsIdListclean: [],
        listDataSendMessage: [],
        listMessagesData: {},
        messageText: "",
        searchTerm: ""
      };

      if ($rootScope.mensaje != "") { //OmRo Shared Groups
        $scope.data.messageText = $rootScope.mensaje;
      }

      ChatFactory2.DB.getCurrentRooms().then(function(items) {
        $scope.currentContacts = items;
      }, function(error) {
        console.error("Couldn't get the current rooms", error)
      }).finally(function() {
        $scope.findSuggestedFriends();
      });

      //console.log("roster", $scope.rosterList);

      //suggested users params
      var pagination1 = {
        start: 0,
        count: 10,
        increase: function(inc) {
          this.count += inc || 1
        },
        reset: function() {
          this.start = 0;
          this.count = 10;
        }
      };

      //Search users params
      var pagination2 = {
        start: 0,
        count: 10,
        increase: function(inc) {
          this.count += inc || 1
        },
        reset: function() {
          this.start = 0;
          this.count = 10;
        }
      };
      $scope.canGetMoreUsers = true;

      $scope.loadMoreResults = function() {
        console.log("search by scroll");
        if ($scope.actionType === "suggested")
          $scope.findSuggestedFriends();
        else
          performSearchN(false);
        //performSearch();
      };

      $scope.usersFound = [];
      //funciones para el manejo de usuarios, selección y deselección
      $scope.findSuggestedFriends = function() {
        //console.log("finding suggested users...")
        $scope.isSearching = true;
        UserService.findSuggestedFriends({
          userId: UserService.get_USERSUBSCRIPTIONINSTANCE_ID(),
          start: pagination1.start,
          count: pagination1.count
        }).$promise.then(function(suggestedFriends) {
          //console.log("suggestedFriends", suggestedFriends);
          Utils.removeLastItemIfHasNoAttribute(suggestedFriends, 'name');
          var totalRetrieved = suggestedFriends.length;

          $scope.canGetMoreUsers = totalRetrieved >= pagination1.count;
          $scope.isSearching = false;

          if (!totalRetrieved) {
            //$scope.$broadcast('scroll.refreshComplete');
            $scope.$broadcast('scroll.infiniteScrollComplete');
            return;
          }

          pagination1.start += totalRetrieved;

          suggestedFriends.forEach(function(user) {
            $scope.usersFound.push({
              avatar: Utils.getUserImgUrl(user),
              userSubscription: +Utils.getLastUrlComponent(user.userSubscription.href),
              href: user.userSubscription.href,
              //verified: user.verifyUser,
              name: user.name,
              _lowername: user.name.toLowerCase(),
              bio: user.userBio,
              //dataForPush: promise,
              isSelected: false
            });
          });

          $scope.data.showSelectedContactsList = false;
          $scope.$broadcast('scroll.infiniteScrollComplete');
        }, function(error) {
          console.error("couldn't get suggested user list", error);
          $scope.errorLoading = true;
        }).finally(function() {
          $scope.isSearching = false;
          $scope.$broadcast('scroll.infiniteScrollComplete');
        });
      };

      $scope.beginSearch = function() {
        if (!$scope.data.searchTerm || !debounceSearch()) return;

        pagination2.reset();

        $scope.usersFound = [];

        //performSearchN();
        performSearchN(true);

        $scope.actionType = "search";
      }

      function performSearchN(userMade) {
        console.log("search by user");
        $scope.isSearching = true;
        searchFactory.madeMinSearch(
          $scope.data.searchTerm,
          pagination2.start,
          pagination2.count
        ).then(function(UserResults) {
          Utils.removeLastItemIfHasNoAttribute(UserResults, 'name');
          var totalRetrieved = UserResults.length;

          $scope.canGetMoreUsers = totalRetrieved >= pagination2.count;
          $scope.isSearching = false;

          if (!totalRetrieved) {
            //$scope.$broadcast('scroll.refreshComplete');
            $scope.$broadcast('scroll.infiniteScrollComplete');
            Utils.$ionicScrollDelegate.resize();
            return;
          }

          pagination2.start += totalRetrieved;
          if (userMade)
            $scope.usersFound = [];

          UserResults.forEach(function(user) {
            var isInList = false;
            var photoDisplay = "img/ProfilePhoto.svg";
            if ($scope.data.selectedUsers.length > 1) {
              isInList = $scope.data.selectedUsers.some(function(prevUser) {
                return prevUser.userSubscription === +Utils.getLastUrlComponent(user.href);
              });
            }
            //var promise = UserService.getDevices(Utils.getLastUrlComponent(user.userSubscription.href));
            if (user.photo)
              photoDisplay = user.photo;
            else {
              photoDisplay = "img/ProfilePhoto.svg";
            }
            $scope.usersFound.push({
              avatar: photoDisplay,
              userSubscription: +Utils.getLastUrlComponent(user.href),
              href: user.href,
              //verified: user.verifyUser,
              name: user.name,
              _lowername: user.name.toLowerCase(),
              //dataForPush: promise,
              isSelected: isInList
            });
          });
          console.log("list length", $scope.usersFound);
          $scope.data.showSelectedContactsList = false;
          Utils.$ionicScrollDelegate.resize();
          refreshDebounce();
        }, function(error) {
          console.error("couldn't get user list", error);
          $scope.errorLoading = true;
        }).finally(function() {
          $scope.isSearching = false;
          $scope.$broadcast('scroll.infiniteScrollComplete');
        })
      }

      $scope.searchUsers = function() {
        if (!$scope.data.userName || !debounceSearch()) return;

        pagination2.reset();

        $scope.usersFound = [];

        performSearch();

        $scope.actionType = "search";
      };

      function performSearch() {
        //console.log("performing search...")
        $scope.isSearching = true;
        UserService.searchUsersByPartialNameOrIncludingSpaces({
          term: $scope.data.userName,
          start: pagination2.start,
          count: pagination2.count
        }).then(function(UserResults) {
          Utils.removeLastItemIfHasNoAttribute(UserResults, 'name');
          var totalRetrieved = UserResults.length;

          $scope.canGetMoreUsers = totalRetrieved >= pagination2.count;
          $scope.isSearching = false;

          if (!totalRetrieved) {
            //$scope.$broadcast('scroll.refreshComplete');
            $scope.$broadcast('scroll.infiniteScrollComplete');
            Utils.$ionicScrollDelegate.resize();
            return;
          }

          pagination2.start += totalRetrieved;

          UserResults.forEach(function(user) {
            var isInList = false;
            var photoDisplay = "img/ProfilePhoto.svg";
            if ($scope.data.selectedUsers.length > 1) {
              isInList = $scope.data.selectedUsers.some(function(prevUser) {
                return prevUser.userSubscription === +Utils.getLastUrlComponent(user.href);
              });
            }
            //var promise = UserService.getDevices(Utils.getLastUrlComponent(user.userSubscription.href));
            if (user.photo)
              photoDisplay = user.photo;
            else {
              photoDisplay = "img/ProfilePhoto.svg";
            }
            $scope.usersFound.push({
              avatar: photoDisplay,
              userSubscription: +Utils.getLastUrlComponent(user.href),
              href: user.href,
              //verified: user.verifyUser,
              name: user.name,
              _lowername: user.name.toLowerCase(),
              //dataForPush: promise,
              isSelected: isInList
            });
          });

          $scope.data.showSelectedContactsList = false;
          Utils.$ionicScrollDelegate.resize();
          refreshDebounce();
        }, function(error) {
          console.error("couldn't get user list", error);
          $scope.errorLoading = true;
        }).finally(function() {
          $scope.isSearching = false;
          $scope.$broadcast('scroll.infiniteScrollComplete');
        })

        /*$scope.isSearching = true;
        AsyncSearch.findFriendUserSubscriptionsByUserNamev157({
          userName: $scope.data.userName,
          withStart: pagination2.start,
          withCount: pagination2.count
        }).then(function (UserResults) {
          Utils.removeLastItemIfHasNoAttribute(UserResults, 'name');
          var totalRetrieved = UserResults.length;

          $scope.canGetMoreUsers = totalRetrieved >= pagination2.count;
          $scope.isSearching = false;

          if (!totalRetrieved) {
            //$scope.$broadcast('scroll.refreshComplete');
            $scope.$broadcast('scroll.infiniteScrollComplete');
            Utils.$ionicScrollDelegate.resize();
            return;
          }

          pagination2.start += totalRetrieved;

          UserResults.forEach(function (user) {
            var isInList = false;
            if ($scope.data.selectedUsers.length > 1) {
              isInList = $scope.data.selectedUsers.some(function (prevUser) {
                return prevUser.userSubscription === +Utils.getLastUrlComponent(user.userSubscription.href);
              });
            }
            //var promise = UserService.getDevices(Utils.getLastUrlComponent(user.userSubscription.href));
            $scope.usersFound.push({
              avatar: Utils.getUserImgUrl(user),
              userSubscription: +Utils.getLastUrlComponent(user.userSubscription.href),
              href: user.userSubscription.href,
              //verified: user.verifyUser,
              name: user.name,
              _lowername: user.name.toLowerCase(),
              //dataForPush: promise,
              isSelected: isInList
            });
          });

          $scope.data.showSelectedContactsList = false;
          Utils.$ionicScrollDelegate.resize();
          refreshDebounce();
        }, function (error) {
          console.error("couldn't get user list", error);
          $scope.errorLoading = true;
        }).finally(function () {
          $scope.isSearching = false;
          $scope.$broadcast('scroll.infiniteScrollComplete');
        });*/
      }

      $scope.selectUser = function(user) {
        //verificar que no sea mi mismo usuario
        var itsMe = user.userSubscription === +Utils.getLastUrlComponent(UserService.get_USER_INSTANCE_HREF());

        //Verificar que no este en lista previa
        var isInList = $scope.data.selectedUsers.some(function(prevUser) {
          return prevUser.userSubscription === user.userSubscription;
        });

        var whatIndex = $scope.data.selectedUsers.findIndex(function(prevUser) {
          return prevUser.userSubscription === user.userSubscription;
        });

        var idx = -1;
        if (!itsMe && !isInList) {
          //Select user
          user.isSelected = true;
          $scope.data.selectedUsers.push(user);
        } else if (itsMe) {
          //Utils.toast.info(Utils.translate("GROUPS.MEMBERS_MGMT.DIALOG.YOULL_BE_THE_ADMIN"));
          Utils.toast.info("Can't add yourself");
        } else if (isInList) {
          //Deselect user
          user.isSelected = false;
          //idx = $scope.data.selectedUsers.indexOf(user);
          $scope.data.selectedUsers.splice(whatIndex, 1);
        }
      };

      $scope.unselectUser = function(user, listIndex) {
        $scope.data.selectedUsers.splice(listIndex, 1);
        if (!$scope.data.selectedUsers.length)
          $scope.hideSelectedUsersList();
      };

      $scope.showSelectedUsersList = function() {
        $scope.data.showSelectedContactsList = $scope.data.selectedUsers.length > 0;
      };

      $scope.hideSelectedUsersList = function() {
        $scope.data.showSelectedContactsList = false;
        focusSearchInput();
      };

      $scope.okClicked = function() {
        $scope.data.showSelectedContactsList = false;

        if ($scope.data.selectedUsers.length > 1) {
          $scope.data.readyForMessage = true;
          $scope.data.multipleUsers = true;
          return;
        }

        if ($scope.data.selectedUsers.length === 1) {
          var exists = $scope.currentContacts.some(function(contact) {
            return contact.id === $scope.data.selectedUsers[0].userSubscription;
          });

          var friend = {};
          if (exists) {
            ChatFactory2.DB.getInboxContact(
              $scope.data.selectedUsers[0].userSubscription
            ).then(function(contact) {
              $state.go('chatPrivate', { friend: contact, addToContacts: false }, { reload: true });
            })
          } else {
            friend.socket = { isOnline: false };
            friend.user = {
              id: $scope.data.selectedUsers[0].userSubscription,
              name: $scope.data.selectedUsers[0].name,
              avatar: $scope.data.selectedUsers[0].avatar,
              bio: $scope.data.selectedUsers[0].bio
            };
            friend.message = {};
            $state.go('chatPrivate', { friend: friend, addToContacts: true }, { reload: true });
          }
        }
      };

      $scope.sendMsgAll = function() {
        if (!$scope.data.messageText) return;

        Utils.loading.show(Utils.translate('PRIVATEMESSAGE.SENDINGMSG'));

        var receiversList = [];
        var insertPromises = {};
        $scope.data.selectedUsers.forEach(function(selUser) {
          var exists = $scope.currentContacts.some(function(contact) {
            return contact.id === selUser.userSubscription;
          });

          var user = {
            id: selUser.userSubscription,
            name: selUser.name,
            avatar: selUser.avatar,
            bio: selUser.bio
          };

          if (!exists) {
            //console.log("--- New contact (id: %d, name: %s) found, adding him/her to your contact list", selUser.userSubscription, selUser.name);
            insertPromises[selUser.userSubscription] = ChatFactory2.DB.insertNewContact(user).then(function( /*id*/ ) {
              return ChatFactory2.socket.joinToContactRoom(selUser.userSubscription).then(function(resp) {
                ChatFactory2.socket.createContactRoom(selUser.userSubscription, resp.isOnline, true /*skipCheck*/ );
                receiversList.push(user);
                return Utils.$q.resolve();
              }, function(error) {
                console.error("joinToContactRoom %d, %s", selUser.userSubscription, selUser.name, error);
                return Utils.$q.reject(error);
              });
            }, function(error) {
              console.error("can't insert new contact on DB", error);
            })
          } else {
            console.log("--- Contact (id: %d, name: %s) already exists", selUser.userSubscription, selUser.name);
            receiversList.push(user);
          }
        });



        Utils.$q.all(insertPromises).then(function(responses) {

          var sendMsgPromises = {};
          //finally send message to every receiver
          receiversList.forEach(function(usrObj, idx) {
            var msg = new ChatFactory2.PrivateMessageToSend({
              toContactId: usrObj.id,
              text: $scope.data.messageText
            });

            //because we need different unique ids
            msg.uniqueId += idx; //this wont add up idx but concatenate it to string

            sendMsgPromises[usrObj.id] = (ChatFactory2.socket.send(msg));
          });

          Utils.$q.all(sendMsgPromises).then(function(responses) {
            Object.keys(responses).forEach(function(userId) {
              ChatFactory2.util.sendPushNotification(userId).then();
            });

            $state.go('menu.messagesInbox', {}, { reload: true });
          }, function(errors) {
            console.error("sendMsgPromises", errors);
            Utils.toast.error(Utils.translate("El mensaje no pudo ser enviado"));
          }).finally(function() {
            Utils.loading.hide();
          })
        }, function(errors) {
          console.error("insertPromises", errors);
          Utils.loading.hide();
        });
        $rootScope.mensaje = '';
      };

      $scope.backClicked = function() {
        if ($scope.data.showSelectedContactsList)
          $scope.hideSelectedUsersList();
        else if ($scope.data.readyForMessage) {
          $scope.data.readyForMessage = false;
          $scope.data.multipleUsers = false;
          focusSearchInput();
        } else {
          //$ionicHistory.goBack(1);
          //window.history.back();
          $state.go('menu.messagesInbox', {}, { reload: true });
          $rootScope.mensaje = ''
          //locals.dialog.cancel();
        }
      };

      function debounceSearch() {
        var now = new Date().getMilliseconds();
        lastSearch = lastSearch || now;
        return ((now - lastSearch) < 300);
      }

      function refreshDebounce() {
        lastSearch = 0;
        pendingSearch = null;
        //cancelSearch  = angular.noop;
      }

      var searchInput;

      function focusSearchInput() {
        setTimeout(function() {
          if (!searchInput)
            searchInput = document.getElementById('search-input');

          if (searchInput) {
            //searchInput = searchInput[searchInput.length - 1];
            searchInput.selectionStart = $scope.data.userName.length;
            searchInput.selectionEnd = $scope.data.userName.length;
            searchInput.focus();
          } else
            console.log("can't find search input")
        }, 900);
      }

      focusSearchInput();

      var deregisterHardBack = Utils.$ionicPlatform.registerBackButtonAction(
        $scope.cancelClicked, 101
      );

      // cancel custom back behaviour
      $scope.$on('$destroy', function() {
        deregisterHardBack();
      });

    }
  ])

;
