"use strict";
appControllers
  .controller('vodCtrl', ['$rootScope', '$scope', '$state', 'UserService', 'Utils', 'StreamingService', '$ionicScrollDelegate', 'AntMediaServer', 'cqnzService', 'SequenceService', 'SequenceFactory', 'adsService', 'permissionValidationService', 'timelinesService',
    function($rootScope, $scope, $state, UserService, Utils, StreamingService, $ionicScrollDelegate, AntMediaServer, cqnzService, SequenceService, SequenceFactory, adsService, permissionValidationService, timelinesService) {
      var permissions = permissionValidationService;
      var myUserId = UserService.get_USERSUBSCRIPTIONINSTANCE_ID();
      $scope.showStream = permissions.canStream();
      $scope.iAmAdvertiser = permissions.canAdvertisementPermit();

      $scope._controllerName = "vodCtrl";
      $scope._groupHrefId = 0; //Default timeline

      $scope.goBack = function() {
        $state.go("menu.timeline", { groupIndex: 0 }, { reload: true })
      }

      $scope.data = {};

      var pagination = {
        start: 0,
        count: 5,
        reset: function() {
          this.start = 0;
          this.count = 5;
        },
        increaseCount: function(step) {
          this.count += step || 5
        }
      };

      $scope._sequenceList = [];
      $scope.isRefreshing = false;
      $scope.isLoading = false;
      $scope.canGetMoreItems = true;
      $scope.errorLoading = false;

      $scope.refreshPage = function() {
        $scope.isRefreshing = true;
        $scope._sequenceList = [];
        pagination.reset();
        $scope.findStreamingPublications(true);
      }

      $scope.loadingComplete = function() {
        $scope.isLoading = false;
        $scope.isRefreshing = false;
        $scope.$broadcast('scroll.infiniteScrollComplete');
        setTimeout(function() {
          Utils.$ionicScrollDelegate.resize();
        }, 800);
      }

      $scope.findStreamingPublications = function(newRound) {
        if (newRound)
          pagination.reset();

        if ($scope.isLoading)
          return;

        $scope.isLoading = true;

        StreamingService.findAllStreamingPublications(pagination.start, pagination.count).$promise.then(
          function(streamingPublications) {
            var listLength = streamingPublications.length;
            $scope.canGetMoreItems = listLength >= pagination.count;

            if (!listLength) {
              $scope.loadingComplete();
              return;
            }

            pagination.start += listLength;
            pagination.increaseCount();

            streamingPublications.forEach(function(seqBasicData) {

              var commentsCounts = cqnzService.countSequenceCommentsByGroupWS({
                sequenceId: seqBasicData.sequenceId,
                groupId: $scope._groupHrefId
              }).$promise;

              var tagsList = SequenceService.obtainTagwithIdandTypev166({
                sequenceId: seqBasicData.sequenceId,
                notificationType: 7
              });

              //LuMa HashTags in sequence
              //var hashesList = SequenceService.getHashFromSequence(seqBasicData.sequenceId);
              var hashesList = SequenceService.findSequenceHashTagComment({
                sequenceId: seqBasicData.sequenceId,
                notificationType: 7,
                commentId: 0
              });

              Utils.$q.all({
                commentsCounts: commentsCounts,
                tagsList: tagsList,
                hashesList: hashesList
              }).then(function(streamingSequence) {
                streamingSequence.basicData = seqBasicData;

                streamingSequence.sequenceOwnerId = Utils.getLastUrlComponent(seqBasicData.userSubscriptionId.href);
                streamingSequence.sequenceItems = cqnzService.getItemsToSequenceV165WS(seqBasicData.sequenceId);
                streamingSequence.likeActive = SequenceService.findSmileysCount({
                  sequenceId: Utils.getLastUrlComponent(seqBasicData.sequence.href),
                  userId: myUserId
                });
                streamingSequence.followEachOther = myUserId == streamingSequence.sequenceOwnerId ? { result: { value: true } } : SequenceService.followEachOther({
                  userId: myUserId,
                  friendId: streamingSequence.sequenceOwnerId
                });
                streamingSequence.viewsCounter = seqBasicData.isAdvertisement && streamingSequence.sequenceOwnerId == myUserId ? adsService.getViewsCount(seqBasicData.sequenceId) : null;
                streamingSequence.clicksCounter = seqBasicData.isAdvertisement && streamingSequence.sequenceOwnerId == myUserId ? adsService.getClicksCount(seqBasicData.sequenceId) : null;
                streamingSequence.URLClicksCounter = streamingSequence.basicData.isAdvertisement && streamingSequence.sequenceOwnerId == myUserId ? adsService.getURLClicksCount(seqBasicData.sequenceId) : null;

                streamingSequence.basicData.sequenceCommentCount = streamingSequence.commentsCounts.result.value;

                var commentsPromises = {};
                if (streamingSequence.basicData.sequenceCommentCount) {
                  commentsPromises[$scope._groupHrefId] = cqnzService.getCommentsOfSequenceWS({
                    sequenceId: seqBasicData.sequenceId,
                    start: 0,
                    count: 2,
                    groupId: $scope._groupHrefId
                  }).$promise
                }

                streamingSequence.comments = [];
                Utils.$q.all(commentsPromises).then(function(resolvedComments) {
                  Object.keys(resolvedComments).forEach(function(groupId) {
                    Utils.removeLastItemIfHasNoAttribute(resolvedComments[groupId], 'commentId');
                    if (resolvedComments[groupId].length) {
                      resolvedComments[groupId].groupId = groupId;
                      streamingSequence.comments = streamingSequence.comments.concat(resolvedComments[groupId]);
                    }
                  });
                  $scope._sequenceList.push(SequenceFactory.buildSequenceObject(streamingSequence));
                  $scope.loadingComplete();
                }, function(error) {
                  console.log("all(commentsPromises)", error);
                  $scope._sequenceList.push(SequenceFactory.buildSequenceObject(streamingSequence));
                  $scope.loadingComplete();
                })
              }, function(errors) {
                console.error("Comments-Tags-Hashes error", errors);
                $scope.errorLoading = true;
                $scope.canGetMoreItems = false;
                $scope.loadingComplete();
              })
            });
          },
          function(streamingPublicationsError) {
            console.log('findAllStreamingPublications error: ', streamingPublicationsError);
            $scope.errorLoading = true;
            $scope.canGetMoreItems = false;
            $scope.loadingComplete();
          }
        );
      }

      timelinesService.defaultTimeline.events.sequence.onDelete = function (data) {
        console.log("defaultTimeline.events.sequence.onDelete", data);
        $scope._sequenceList.some(function (sequence, index) {
          if (sequence.metadata.id == data.sequenceMetadata.id) {
            $scope._sequenceList.splice(index, 1);
            pagination.count = 5;
            //pagination.start--;
            $scope.findStreamingPublications();
            //pagination.count = 5;
            return true;
          }
        });
      };

      $scope.findStreamingPublications(true);
    }
  ]);

/*.controller('vodCtrl', ['$rootScope', '$scope', '$state', 'UserService', 'Utils', 'StreamingService', '$ionicScrollDelegate', 'AntMediaServer',
  function($rootScope, $scope, $state, UserService, Utils, StreamingService, $ionicScrollDelegate, AntMediaServer) {

    var myUserId = UserService.get_USERSUBSCRIPTIONINSTANCE_ID();
    $scope.poster = AntMediaServer.s3Bucket + '/splash.png'

    $scope.goBack = function() {
      $state.go('menu.timeline', { groupIndex: "0" }, { reload: true });
    }

    $scope.data = {};

    var pagination = {
      start: 0,
      count: 5,
      reset: function() {
        this.start = 0;
        this.count = 5;
      },
      increaseCount: function(step) {
        this.count += step || 5
      }
    };

    $scope.streamingList = [];

    $scope.isRefreshing = false;
    $scope.isLoading = false;
    $scope.canGetMoreItems = true;
    $scope.errorLoading = false;

    $scope.refreshPage = function() {
      $scope.isRefreshing = true;
      $scope.streamingList = [];
      pagination.reset();
      $scope.findAllStreaming(true);
      $scope.isRefreshing = false;
    }

    $scope.finishLoading = function() {
      $scope.isLoading = false;
      $scope.$broadcast('scroll.infiniteScrollComplete');
      setTimeout(function() {
        Utils.$ionicScrollDelegate.resize();
      }, 800);
    }

    $scope.findAllStreaming = function(newRound) {
      if (newRound)
        pagination.reset();

      if ($scope.isLoading)
        return;

      $scope.isLoading = true;

      StreamingService.findAllStreamingv2(pagination.start, pagination.count).$promise.then(
        function(responseList) {
          var listLength = responseList.length;
          $scope.canGetMoreItems = listLength >= pagination.count;

          if (!listLength) {
            $scope.isLoading = false;
            $scope.$broadcast('scroll.infiniteScrollComplete');
            return;
          }

          pagination.start += responseList.length;
          pagination.increaseCount();

          responseList.forEach(function(userStreaming) {
            $scope.streamingList.push(userStreaming);
          });
          $scope.finishLoading();
        },
        function(error) {
          console.log('FindAllStreamingv2 Error:', error);
          $scope.finishLoading();
        });
    }

    $scope.findAllStreaming(true);
  }
])*/
