"use strict";

appControllers
  .controller("groupsTabsCtrl", ['$rootScope', '$scope', '$state', 'GroupFactory', 'Utils', function($rootScope, $scope, $state, GroupFactory, Utils) {
    Utils.clearHistory();

    $scope.activeTab = GroupFactory.getCurrentGroupTab() === 0 ? 'myGroups' : 'otherGroups';

    $scope.activateTab = function(tabname) {
      $scope.activeTab = tabname;
      var view1 = document.getElementsByClassName('group-view-myGroups');
      var view2 = document.getElementsByClassName('group-view-otherGroups');

      var i;

      if (tabname === 'myGroups') {
        for (i = 0; i < view1.length; i++) {
          if (view1[i].children[0])
            view1[i].children[0].setAttribute('nav-view', 'active')
        }

        for (i = 0; i < view2.length; i++) {
          if (view2[i].children[0])
            view2[i].children[0].setAttribute('nav-view', 'cached')
        }
      } else {
        for (i = 0; i < view1.length; i++) {
          if (view1[i].children[0])
            view1[i].children[0].setAttribute('nav-view', 'cached')
        }

        for (i = 0; i < view2.length; i++) {
          if (view2[i].children[0])
            view2[i].children[0].setAttribute('nav-view', 'active')
        }
      }
    };

    $scope.$on('$ionicView.beforeEnter', function() {
      if ($state.current.name === 'menu.groupsTabs.myGroups')
        $scope.activeTab = 'myGroups';

      Utils.$timeout(function() {
        $scope.activateTab($scope.activeTab);
      }, 100);
    })
  }])

  .controller("myGroupsTabCtrl", ['$rootScope', '$scope', '$state', 'UserService', 'GroupsService', 'GroupFactory', 'permissionValidationService', 'Utils','ShareApp',
    function($rootScope, $scope, $state, UserService, GroupsService, GroupFactory, permissionValidationService, Utils, ShareApp) {
      //console.log("myGroupsTabCtrl created!!!")
      var isIOS = ionic.Platform.isIOS();//OmRo
      var isAndroid = ionic.Platform.isAndroid();
      $scope.totalMyGroups = 0;
      $scope.myGroupList = GroupFactory.getCurrentAvailableUserGroupsList();
      $scope.myPendingGroupList = GroupFactory.getPendingUserGroupsList();
      $scope.data = { searchTerm: "" };

      $scope.isRefreshing = false;
      $scope.canGetMoreOfMyGroups = true;
      $scope.isLoadingMyGroups = false;
      $scope.isLoadingPendingGroups = false;
      var firstLoad = true;
      var pagination = {
        start: 0,
        count: 10,
        reset: function() {
          this.start = 0;
          this.count = 10;
        },
        increaseCount: function(step) {
          this.count += step || 5
        }
      };

      $scope.loadMyGroups = function() {
        if ($scope.isLoadingMyGroups) return;

        $scope.isLoadingMyGroups = true;

        //console.log("pagination", pagination.start, pagination.count);

        GroupFactory.loadAvailableUserGroups({
          userSubscriptionHref: UserService.get_USER_INSTANCE_HREF(),
          start: pagination.start,
          count: pagination.count
        }).then(function(groups) {
          var totalRetrieved = groups.length;
          groups.forEach(function(group) {
            if (group.administrator && $_canReadReceivedJoinRequests())
              group.pendingJoinReqs = GroupsService.getNumJoiningRequestsByGroup(group.id);
          });

          if (firstLoad) {
            $scope.myGroupList = [];
            firstLoad = false;
          }

          $scope.myGroupList = $scope.myGroupList.concat(groups);
          GroupFactory.setCurrentAvailableUserGroupsList($scope.myGroupList);

          $scope.totalMyGroups = $scope.myGroupList.length;
          $scope.canGetMoreOfMyGroups = totalRetrieved >= pagination.count;
          pagination.start += totalRetrieved;
          pagination.increaseCount();
        }, function( /*error*/ ) {
          $scope.canGetMoreOfMyGroups = false;
          Utils.alert.show(Utils.translate('GROUPS.MAIN_PAGE.DIALOG.GET_USER_GROUPS_ERR'));
        }).finally(function() {
          $scope.isLoadingMyGroups = false;
          $scope.isRefreshing = false;
          $scope.$broadcast('scroll.refreshComplete');
          $scope.$broadcast('scroll.infiniteScrollComplete');
        });
      };

      $scope.loadMyGroups();

      function loadMyPendigGroups() {
        if ($scope.isLoadingPendingGroups) return;
        //console.log("Loading pending groups...");
        GroupFactory.loadPendingUserGroups(UserService.get_USER_INSTANCE_HREF()).then(function(pendigGroupsList) {
          $scope.myPendingGroupList = pendigGroupsList;
        }, function(error) {
          console.error("loadMyPendigGroups", error);
          Utils.alert.show(Utils.translate('GROUPS.MAIN_PAGE.DIALOG.GET_USER_JOIN_REQS_ERR'));
        }).finally(function() {
          $scope.isLoadingPendingGroups = false;

          $scope.isRefreshing = false;
          $scope.$broadcast('scroll.refreshComplete');
        });
      }

      loadMyPendigGroups();

      $scope.refreshMyGroups = function() {
        console.log("refreshMyGroups...");
        $scope.totalMyGroups = 0;
        $scope.myGroupList = [];
        $scope.myPendingGroupList = [];

        $scope.isRefreshing = true;
        $scope.canGetMoreOfMyGroups = true;
        pagination.reset();

        $scope.loadMyGroups();
        loadMyPendigGroups();
      };

      $scope.confirmRejectRequest = function(listIndex) {
        Utils.confirm.show(
          Utils.translate("GROUPS.SENT_REQS.DIALOG.REJECT_REQ_CONF") +
          " <b>" + $scope.myPendingGroupList[listIndex].name + "</b>?"
        ).then(function(resp) {
          if (resp) {
            Utils.loading.show();
            GroupsService.removeMember($scope.myPendingGroupList[listIndex].groupMemberHrefId).then(function() {
              Utils.toast.success(Utils.translate("GROUPS.SENT_REQS.DIALOG.REJECT_REQ_OK"));
              $scope.myPendingGroupList.splice(listIndex, 1);
              //$rootScope.$broadcast('groups.others.reload');
              //$scope.refreshMyGroups();
            }, function(removeError) {
              console.error(removeError);
              Utils.toast.error(Utils.translate("GROUPS.SENT_REQS.DIALOG.REJECT_REQ_ERR"))
            }).finally(function() {
              Utils.loading.hide();
            })
          }
        })
      };

      $scope.openUsersOptionsMenu = function(group) {
        var buttons = [],
          buttonsIds = [];

        buttons.push({ text: Utils.translate('GROUPS.MEMBERS_MGMT.CREATE_CODE') });
        buttonsIds.push("CREATE_CODE");

        Utils.$ionicActionSheet.show({
          buttons: buttons,
          titleText: Utils.translate('SEQUENCE.OPTIONS'),
          buttonClicked: function(buttonIndex) {
            switch (buttonsIds[buttonIndex]) {
              case 'ADD_USERS':
                $scope.openSearchModal();
                break;
              case 'CREATE_CODE':
                GroupsService.createGroupInvitation({
                  ownerSubscription: UserService.get_USER_INSTANCE_HREF(),
                  groupSubscription: group.href
                }).$promise.then(function(resp) {
                  //console.log("resp", resp);
                                        
                    if(isAndroid){
                          $scope.compaket = ShareApp.shareAndroid;
                    }else if(isIOS){
                          $scope.compaket = ShareApp.shareIos;
                    }
                    
                  $scope.messageShare = Utils.translate("GROUPS.MEMBERS_MGMT.DIALOG.CODE_GENERATED_OK2") +
                    " " + group.name + " " +
                    Utils.translate('GROUPS.MEMBERS_MGMT.DIALOG.USING_THIS_CODE') + ":  " + resp.result.value + " ";
                  var options = {
                    message: $scope.messageShare // not supported on some apps (Facebook, Instagram)

                  };

                  var onSuccess = function(result) { //OmRo Shared Groups
                    console.log("Share completed? " + result.completed); // On Android apps mostly return false even while it's true
                    console.log("Shared to app: " + result.app);

                    if (result.app === $scope.compaket) {
                      $state.go('chatSelectUsers')
                      $rootScope.mensaje = $scope.messageShare;
                    } // On Android result.app since plugin version 5.4.0 this is no longer empty. On iOS it's empty when sharing is cancelled (result.completed=false)
                  };

                  var onError = function(msg) {
                    console.log("Sharing failed with message: " + msg);
                  };

                  window.plugins.socialsharing.shareWithOptions(options, onSuccess, onError);


                  // window.plugins.socialsharing.share(
                  //   // Text, some apps not support this
                  //   Utils.translate("GROUPS.MEMBERS_MGMT.DIALOG.CODE_GENERATED_OK1") + " " + group.name + " " +
                  //   Utils.translate('GROUPS.MEMBERS_MGMT.DIALOG.USING_THIS_CODE') + ": " +
                  //   (Utils.platformName() == 'ios' ? (resp.result.value) : ''),

                  //   Utils.translate("MENU.GROUPS.TITLE"), // subject
                  //   null, //image needed
                  //   resp.result.value
                  // );

                  // Utils.alert.show(
                  //   Utils.translate("GROUPS.MEMBERS_MGMT.DIALOG.CODE_GENERATED_OK2") +
                  //   " <b>" + group.name + "</b> " +
                  //   Utils.translate('GROUPS.MEMBERS_MGMT.DIALOG.USING_THIS_CODE') + ": <b>" + resp.result.value + "</b>", { class: 'selectable', title: Utils.translate("GLOBAL.INFORMATION") }
                  // );

                }, function(error) {
                  console.error("cant create Group Invitation", error);
                  Utils.translate("GROUPS.MEMBERS_MGMT.DIALOG.CODE_GENERATED_ERROR")
                });
                break;
              default:
                Utils.toast.warning("Invalid option '" + buttonsIds[buttonIndex] + "'");
            }
            return true;
          }
        });
      };

      $scope.gotoGroupTimeline = function(group) {
        Utils.clearCache();
        Utils.clearHistory();

        var index = 0;
        index = $scope.myGroupList.map(function(d) {
          return d['id'];
        }).indexOf(group.id)
        index = index + 1;
        console.log("gotten", index);

        Utils.$timeout(function() {
          $rootScope.$apply();
          $state.go("menu.timeline", { groupIndex: index }, { reload: true })
        }, 100);
      };

      $scope.sendToPreview = function(group, origin) {
        if(group)
          $state.go("groupSettings", { group: group, origin: origin });
      }

      $scope.showPrivateGroupError = function() {
        Utils.toast.warning(Utils.translate('GROUPS.MAIN_PAGE.DIALOG.GROUP_IS_PRIVATE'));
      };

      // ------------------------- PERMISSIONS VALIDATION SECTION ---------------------------
      $scope.$_canCreateGroup = function() {
        return UserService.USER_IS_SUPERADMIN() || permissionValidationService.canCreateGroup();
      };

      $scope.$_canSearchGroup = function() {
        return UserService.USER_IS_SUPERADMIN() || permissionValidationService.canSearchGroup();
      };

      $scope.$_canReadOwnGroups = function() {
        return UserService.USER_IS_SUPERADMIN() || permissionValidationService.canReadOwnGroups();
      };

      function $_canReadReceivedJoinRequests() {
        return UserService.USER_IS_SUPERADMIN() || permissionValidationService.canReadReceivedJoinRequests();
      };

      $scope.$_canManageGroupMembers = function() {
        return UserService.USER_IS_SUPERADMIN() || permissionValidationService.canManageGroupMembers();
      };

      $scope.$_canCancelSentJoinRequest = function() {
        return UserService.USER_IS_SUPERADMIN() || permissionValidationService.canCancelSentJoinRequest();
      };

      $scope.$_canViewGroupInfo = function() {
        return UserService.USER_IS_SUPERADMIN() || permissionValidationService.canViewGroupInfo();
      };

      $scope.$_canViewSentRequests = function() {
        return UserService.USER_IS_SUPERADMIN() || permissionValidationService.canReadSentJoinRequests();
      };


      // ------------------------- EVENTS ------------------------
      var isFirstime = true;
      var unsubscribers = [];

      $scope.$on('$ionicView.beforeEnter', function() {
        GroupFactory.setCurrentGroupTab(0); //my groups

        unsubscribers[0] = $rootScope.$on('groups.mine.reload', function() {
          $scope.refreshMyGroups();
        });

        //console.log("$ionicView.beforeEnter");

        if (isFirstime) {
          isFirstime = false;
          return;
        }
        console.log("re-Loading your groups");

        $scope.isLoadingMyGroups = true;
        GroupFactory.loadAvailableUserGroups({
          userSubscriptionHref: UserService.get_USER_INSTANCE_HREF()
        }).then(function(myTempGroups) {

          var totalGroupsTemp = myTempGroups.length;

          var differencesFound = false;

          if ($scope.myGroupList.length === totalGroupsTemp) {
            //check if has been changes in groups ids
            differencesFound = $scope.myGroupList.some(function(group) {
              var found = myTempGroups.some(function(tempGroup) {
                return tempGroup.id === group.id;
              });
              return !found;
            })
          } else {
            differencesFound = true;
            if (totalGroupsTemp > $scope.myGroupList.length)
              Utils.toast.info(Utils.translate("GROUPS.MAIN_PAGE.DIALOG.NEW_GROUPS_DETECTED"));
          }

          if (differencesFound) {
            console.log("RE-setting myGroupList[]");
            $scope.myGroupList = myTempGroups;
          }

          $scope.isLoadingMyGroups = false;
        }, function(error) {
          console.error("get User Groups", error);
          $scope.isLoadingMyGroups = false;
        }).finally(function() {
          loadMyPendigGroups();
        })

      });

      $scope.$on("$destroy", function() {
        console.log("unsubscribing groups listeners...");
        unsubscribers.forEach(function(unsubscribe) {
          if (typeof unsubscribe === 'function') {
            unsubscribe();
          }
        });
      });
    }
  ])

  .controller("otherGroupsTabCtrl", ['$rootScope', '$scope', '$state', 'UserService', 'GroupsService', 'GroupFactory', 'permissionValidationService', 'Utils',
    function($rootScope, $scope, $state, UserService, GroupsService, GroupFactory, permissionValidationService, Utils) {
      $scope.totalOtherGroups = 0;
      $scope.otherGroupsList = [];
      $scope.data = { searchTerm: "" };

      $scope.isRefreshing = false;
      $scope.canGetMoreOfOtherGroups = true;
      $scope.isLoadingOtherGroups = false;
      var pagination = {
        start: 0,
        count: 10,
        reset: function() {
          this.start = 0;
          this.count = 10;
        },
        increaseCount: function(step) {
          this.count += step || 5
        }
      };

      $scope.loadOtherGroups = function() {
        if ($scope.isLoadingOtherGroups) return;

        $scope.isLoadingOtherGroups = true;

        //console.log("pagination", pagination.start, pagination.count);

        GroupFactory.loadOtherGroups({
          userSubscriptionHref: UserService.get_USER_INSTANCE_HREF(),
          start: pagination.start,
          count: pagination.count
        }).then(function(otherGroupsList) {
          var totalRetrieved = otherGroupsList.length;
          $scope.otherGroupsList = $scope.otherGroupsList.concat(otherGroupsList); //clear current list
          $scope.totalOtherGroups = $scope.otherGroupsList.length;

          $scope.canGetMoreOfOtherGroups = totalRetrieved >= pagination.count;
          pagination.start += totalRetrieved;
          pagination.increaseCount();
        }, function( /*error*/ ) {
          $scope.canGetMoreOfOtherGroups = false;
          Utils.alert.show(Utils.translate("GROUPS.MAIN_PAGE.DIALOG.GET_OTHER_GROUPS_ERR"));
        }).finally(function() {
          $scope.isLoadingOtherGroups = false;
          $scope.isRefreshing = false;
          $scope.$broadcast('scroll.refreshComplete');
          $scope.$broadcast('scroll.infiniteScrollComplete');
        });
      };

      $scope.loadOtherGroups();

      $scope.refreshOtherGroups = function() {
        //console.log("refreshOtherGroups...")
        $scope.totalOtherGroups = 0;
        $scope.otherGroupsList = [];

        $scope.isRefreshing = true;
        $scope.canGetMoreOfOtherGroups = true;
        pagination.reset();

        $scope.loadOtherGroups();
      };

      $scope.joinToPublicGroup = function(group) {
        Utils.loading.show();
        GroupsService.addMember({
          groupId: group.hrefId,
          adminHref: UserService.get_USER_INSTANCE_HREF(),
          friendHref: UserService.get_USER_INSTANCE_HREF()
        }).then(function() {
          //console.log(joinResponse);
          GroupsService.getUserGroups(UserService.get_USER_INSTANCE_HREF()).then(function(myGroups) {
            var foundIdx = -1;
            myGroups.some(function(resource, idx) {
              if (resource.group) {
                if (group.hrefId === Utils.getLastUrlComponent(resource.group.href)) {
                  foundIdx = idx;
                  return true;
                }
              }
            });

            GroupFactory.getCurrentAvailableUserGroupsList().splice(foundIdx, 0, group);
            group.isPreview = false;
            Utils.loading.hide();
            $rootScope.$broadcast('cacheAndHistory.clear');
            group.requestSent = true;
            GroupFactory.setCurrentGroupIndex(foundIdx + 1);
            $state.go("menu.timeline", { prepareStatement: 0, groupIndex: foundIdx + 1 }, { reload: true });

            setTimeout(function() {
              $scope.refreshOtherGroups();
            }, 800);
          }, function(myGroupsError) {
            Utils.loading.hide();
            console.error(myGroupsError);
            //Utils.toast.error(Utils.translate('GROUPS.MAIN_PAGE.DIALOG.GET_USER_GROUPS_ERR'));

            //if couldn't get the new group id, send to general timeline
            GroupFactory.setCurrentGroupIndex(0);
            $state.go("menu.timeline", { prepareStatement: 0, groupIndex: 0 })
          });
        }, function(joinError) {
          console.error(joinError);
          Utils.loading.hide();
          Utils.alert.show(Utils.translate('GROUPS.MAIN_PAGE.DIALOG.JOIN_TO_THIS_ERR'));
        })
      };

      $scope.requestJoinToPrivateGroup = function(otherGroup) {
        Utils.loading.show();
        GroupsService.addMember({
          groupId: otherGroup.hrefId,
          adminHref: UserService.get_USER_INSTANCE_HREF(),
          friendHref: UserService.get_USER_INSTANCE_HREF()
        }).then(function() {
          Utils.toast.success(Utils.translate('GROUPS.MAIN_PAGE.DIALOG.REQUEST_SENT_OK'));
          $rootScope.$broadcast('groups.mine.reload');
          otherGroup.requestSent = true;
          $scope.refreshOtherGroups();
        }, function(joinError) {
          Utils.alert.show(Utils.translate('GROUPS.MAIN_PAGE.DIALOG.REQUEST_SENT_ERR'));
          console.error(joinError);
        }).finally(function() {
          Utils.loading.hide();
        })
      };

      $scope.showPublicTimelinePreview = function(otherGroup) {
        if (otherGroup.privacy === "Private") {
          //Utils.toast.warning(Utils.translate('GROUPS.MAIN_PAGE.DIALOG.GROUP_IS_PRIVATE'));
          return
        }

        //otherGroup.isPreview = false;
        //$state.go("groupTimelinePreview", { groupData: otherGroup });
      };

      $scope.gotoSentRequests = function() {
        $scope.popover.hide();
        setTimeout(function() {
          $state.go("groupSentRequests");
        }, 200)
      };

      $scope.gotoReceivedRequests = function() {
        $scope.popover.hide();
        setTimeout(function() {
          $state.go("groupReceivedRequests");
        }, 200)
      };
                                     
      $scope.sendToPreview = function(group, origin) {
        if(group)
          $state.go("groupSettings", { group: group, origin: origin });
      }

      // ------------------------- PERMISSIONS VALIDATION SECTION ---------------------------

      $scope.$_canSearchGroup = function() {
        return UserService.USER_IS_SUPERADMIN() || permissionValidationService.canSearchGroup();
      };

      $scope.$_canReadOthersGroups = function() {
        return UserService.USER_IS_SUPERADMIN() || permissionValidationService.canReadOthersGroups();
      };

      //for public groups
      $scope.$_canJoinToPublicGroup = function() {
        return UserService.USER_IS_SUPERADMIN() || permissionValidationService.canJoinToPublicGroup();
      };

      //for private groups
      $scope.$_canSendJoinRequest = function() {
        return UserService.USER_IS_SUPERADMIN() || permissionValidationService.canSendJoinRequest();
      };

      // ------------------------- EVENTS ------------------------
      $rootScope.$on('groups.others.reload', function() {
        $scope.loadOtherGroups();
      });

      $scope.$on("$ionicView.beforeEnter", function() {
        GroupFactory.setCurrentGroupTab(1); //other groups
      })
    }
  ])

  .controller("groupCreateStep1Ctrl", ["$rootScope", "$scope", "$state", 'AsyncSearch', "UserService", "GroupsService", 'Utils',
    function($rootScope, $scope, $state, AsyncSearch, UserService, GroupsService, Utils) {
      $scope.$on("$ionicView.enter", function() {
        //the final selected members
        $scope.form = GroupsService.getGroupForm();
      });

      $rootScope.$on('groupForm.cleared', function() {
        $scope.form = GroupsService.getGroupForm();
        $scope.form.name = '';
        setTimeout(function() {
          $scope.$apply()
        });
      });

      /**
       * the random users list
       * @type {{id:string, href:string, name:string, photo:string}[]}
       */
      $scope.randomUsersList = [];

      $scope.showNextForm = function() {
        $state.go("groupsCreateStep2")
      };

      $scope.isLoadingRandomUsers = false;
      (function getRandomUsers() {
        $scope.isLoadingRandomUsers = true;
        UserService.findUsersRandomWS({
          userInstanceId: UserService.get_USERSUBSCRIPTIONINSTANCE_ID(),
          start: 0,
          count: 10
        }).$promise.then(
          /**
           * @param resp {{
           *   creationDate:string,
           *   name:string,
           *   profilePicture:string|null,
           *   profilePictureUrl:string|null,
           *   role:*,
           *   userBio:string,
           *   userSubscription:*
           * }|*}
           */
          function(resp) {
            $scope.isLoadingRandomUsers = false;
            Utils.removeLastItemIfHasNoAttribute(resp, 'userSubscription');
            //console.log(resp)

            resp.forEach(function(item) {
              resp.id = Utils.getLastUrlComponent(item.userSubscription.href);
              //item.name: <-- it's already in the resp obj
              item.href = item.userSubscription.href;
              item.avatar = Utils.getUserImgUrl(item);
              item.isSelected = false;
            });

            $scope.randomUsersList = resp;
          },
          function(error) {
            console.log("getRandomUsers", error);
            $scope.isLoadingRandomUsers = false;
          })
      })();

      $scope.toggleSelectContact = function(user) {
        user.isSelected = !user.isSelected;

        var foundAt = null;

        //because the member could be created in different contexts, then their references could not be the same
        $scope.form.members.some(function(member, index) {
          if (member.href == user.href) return foundAt = index;
        });

        foundAt !== null ? $scope.form.members.splice(foundAt, 1) : $scope.form.members.push(user);
      };

      $scope.sendFriendProfile = function(userInstanceId) {
        var userInstanceId = +userInstanceId.split('/')[6];
        if (!userInstanceId) return;

        if (userInstanceId === +UserService.get_USERSUBSCRIPTIONINSTANCE_ID())
          $state.go('menu.myProfile', {}, { reload: true });
        else
          $state.go('friendProfile', { instanceId: userInstanceId }, { reload: true });
      };

      //Zone to open new page for add users
      $scope.openSearchModal = function() {
        var dialog = Utils.alert.getCtrl();
        dialog.show({
          templateUrl: 'templates/groups/group-members-selection-modal.html',
          controller: 'GroupSearchUsersCtrl',
          locals: {
            previousUsers: $scope.form.members, //users previously selected in this modal
            dialog: dialog
          },
          clickOutsideToClose: false,
          fullscreen: true // Only for -xs, -sm breakpoints.
        }).then(
          /**
           * @param resp {[{
           *  id:string,
           *  href:string
           *  name:string,
           *  photo:string,
           * }] | []}
           */
          function(resp) {
            $scope.form.members = resp;

            //check if in modal some selected users were unselected and update their state
            $scope.randomUsersList.forEach(function(member) {
              member.isSelected = resp.some(function(user) {
                return user.href === member.href;
              })
            })
          });
      }
    }
  ])

  .controller("groupCreateStep2Ctrl", ['$rootScope', '$scope', '$state', "ImageService3", "GroupsService", "GroupFactory", "s3", "UserService", 'Utils', 'ImageService2', '$ionicActionSheet', 'FileService2', '$jrCrop', '$timeout',
    function($rootScope, $scope, $state, ImageService3, GroupsService, GroupFactory, s3, UserService, Utils, ImageService2, $ionicActionSheet, FileService2, $jrCrop, $timeout) {
      $scope.form = GroupsService.getGroupForm();
      //console.log($scope.form);

      var resetAfterLeave = true;
      var isIOS = ionic.Platform.isIOS();
      //INSTANCES JCROP
        var cropBack = $jrCrop;
        var cropPhoto = $jrCrop;
      //INSTANCES JCROP END
      $scope.$on("$ionicView.afterLeave", function() {
        if (resetAfterLeave) {
          GroupsService.resetGroupForm();
          $rootScope.$broadcast('groupForm.cleared', {});
        }
      });

      FileService2.cleanImages();
      $scope.form.backgroundPhoto = FileService2.images();
      $scope.form.image = FileService2.images();

      $scope.checkPermissionsBefore = function(type) {
        if (isIOS === true) {
          //getPermission(0);
          $scope.showMenu(type);
        } else {
          getMultiplePermissions(type);
        }
      }

      /**
       *function to get multiple permission for camera
       */
      function getMultiplePermissions(type) {
        cordova.plugins.diagnostic.getPermissionsAuthorizationStatus(function(statuses) {
          var havePerm = 0;
          var deniedPerm = false;
          for (var permission in statuses) {
            switch (statuses[permission]) {
              case cordova.plugins.diagnostic.permissionStatus.GRANTED:
                console.log("Permission granted to use " + permission);
                havePerm++;
                break;
              case cordova.plugins.diagnostic.permissionStatus.NOT_REQUESTED:
                console.log("Permission to use " + permission + " has not been requested yet");
                havePerm--;
                break;
              case cordova.plugins.diagnostic.permissionStatus.DENIED:
                console.log("Permission denied to use " + permission + " - ask again?");
                havePerm--;
                break;
              case cordova.plugins.diagnostic.permissionStatus.DENIED_ALWAYS:
                console.log("Permission permanently denied to use " + permission + " - guess we won't be using it then!");
                deniedPerm = true;
                break;
            }
          }
          if (!deniedPerm) {
            if (havePerm == 2) {
              //have both permissions granted
              $scope.showMenu(type);
            } else if (havePerm < 2) {
              //both or one permission hasn't granted
              askMultiplePermissions(type);
            }
          } else {
            Utils.toast.error(Utils.translate('PLUGINS.PERMISSIONS.DENIED_PERMISSION') +
              Utils.translate('PLUGINS.PERMISSIONS.TYPE.CAMERA') + ' ' +
              Utils.translate('PLUGINS.PERMISSIONS.AND') + ' ' +
              Utils.translate('PLUGINS.PERMISSIONS.TYPE.GALLERY') + '. \n' +
              Utils.translate('PLUGINS.PERMISSIONS.ALWAYS_DENIED_PERMISSION'));
          }
        }, function(error) {
          console.error("The following error occurred: " + error);
        }, [
          cordova.plugins.diagnostic.permission.CAMERA,
          cordova.plugins.diagnostic.permission.READ_EXTERNAL_STORAGE
        ])
      }

      /**
       *function to get multiple permission for camera
       */
      function askMultiplePermissions(type) {
        cordova.plugins.diagnostic.requestRuntimePermissions(function(statuses) {
          var havePerm = 0;
          var deniedPerm = false;
          for (var permission in statuses) {
            switch (statuses[permission]) {
              case cordova.plugins.diagnostic.permissionStatus.GRANTED:
                console.log("Permission granted to use " + permission);
                havePerm++;
                break;
              case cordova.plugins.diagnostic.permissionStatus.NOT_REQUESTED:
                console.log("Permission to use " + permission + " has not been requested yet");
                havePerm--;
                break;
              case cordova.plugins.diagnostic.permissionStatus.DENIED:
                console.log("Permission denied to use " + permission + " - ask again?");
                havePerm--;
                break;
              case cordova.plugins.diagnostic.permissionStatus.DENIED_ALWAYS:
                console.log("Permission permanently denied to use " + permission + " - guess we won't be using it then!");
                deniedPerm = true;
                break;
            }
          }
          if (!deniedPerm) {
            if (havePerm == 2) {
              //have both permissions granted
              $scope.showMenu(type);
            } else if (havePerm < 2) {
              //both or one permission hasn't granted
              Utils.toast.error(Utils.translate('PLUGINS.PERMISSIONS.DENIED_PERMISSION') +
                Utils.translate('PLUGINS.PERMISSIONS.TYPE.CAMERA') + ' ' +
                Utils.translate('PLUGINS.PERMISSIONS.AND') + ' ' +
                Utils.translate('PLUGINS.PERMISSIONS.TYPE.GALLERY') + '.');
            }
          } else {
            Utils.toast.error(Utils.translate('PLUGINS.PERMISSIONS.DENIED_PERMISSION') +
              Utils.translate('PLUGINS.PERMISSIONS.TYPE.CAMERA') + ' ' +
              Utils.translate('PLUGINS.PERMISSIONS.AND') + ' ' +
              Utils.translate('PLUGINS.PERMISSIONS.TYPE.GALLERY') + '. \n' +
              Utils.translate('PLUGINS.PERMISSIONS.ALWAYS_DENIED_PERMISSION'));
          }
        }, function(error) {
          console.error("The following error occurred: " + error);
        }, [
          cordova.plugins.diagnostic.permission.CAMERA,
          cordova.plugins.diagnostic.permission.READ_EXTERNAL_STORAGE
        ])
      }

      $scope.uploadingPhoto = false;
      $scope.openCrop = false;
      $scope.showMenu = function(typePhoto) {
        $scope.hideSheet = $ionicActionSheet.show({
          buttons: [
            { text: 'Take photo' },
            { text: 'Photo from library' }
          ],
          titleText: 'Add images',
          cancelText: 'Cancel',
          buttonClicked: function(index) {
            if (typePhoto === 'b')
              $scope.selectGroupBgImage(index);
            else
              $scope.selectGroupImage(index);
          }
        });
      }

      // BEGIN PROCESS TO SELECT GROUP PHOTO
        $scope.selectGroupImage = function(index) {
          $scope.hideSheet(); //USING NEW PLUGIN MediaPicker $scope.form.image
          $scope.form.image = [];
          if (index === 0) { // camera
            ImageService2.saveCameraMediaProfilePhoto(index).then(function() {
            });
          } else { // library
            ImageService2.saveAlbumMediaProfilePhoto(index).then(function() {
            });
          }
        };

        $rootScope.$on("addedImageB", function() {
          showImageUser();
        })

        function showImageUser() {
          if($scope.openCrop) {
            return true;
          }
          $scope.openCrop = true;
          var img = FileService2.images();
          var exifTemp = FileService2.exifImage();
          if(img.length === 0) { //ask again in 1 sec
            $timeout(function () {
              showImageUser();
            }, 1000);
          } else {
            if($scope.form.image === null || $scope.form.image === undefined || $scope.form.image.length === 0) {
              var opt = {};
              if(isIOS === true) {
                //prepare IOS
                opt = {
                  url: img[0],
                  width: 200,
                  height: 200,
                  circle: true,
                  allowRotation: true,
                  title: 'Usa tus dedos para ajustar la imagen'
                }
                cropPhoto.crop(opt).then(function(canvas) {
                  // success!
                  //$scope.$apply();
                  $scope.form.image[0] = canvas.toDataURL();
                  $timeout(function() {
                    $scope.openCrop = false;
                    $scope.$apply();
                  }, 0);
                }, function(err) {
                  console.error(err);
                  $scope.form.image[0] = img[0];
                  $scope.openCrop = false;
                  $scope.$apply();
                });
              } else {
                var urlToUse = null;
                if(img[0].uri) {
                  urlToUse = img[0].uri;
                } else {
                  urlToUse = img[0];
                }
                opt = {
                  url: urlToUse,
                  width: 200,
                  height: 200,
                  circle: true,
                  allowRotation: true,
                  title: 'Usa tus dedos para ajustar la imagen'
                }
                cropPhoto.crop(opt).then(function(canvas) {
                  // success!
                  //$scope.$apply();
                  $scope.form.image[0] = canvas.toDataURL();
                  $timeout(function() {
                    $scope.openCrop = false;
                    $scope.$apply();
                  }, 0);
                }, function(err) {
                  console.error(err);
                  $scope.form.image[0] = img[0];
                  $scope.openCrop = false;
                  $scope.$apply();
                });
              }
            } else {
              $scope.openCrop = false;
              console.log("finish loading");
            }
          }
        }
      // END PROCESS TO SELECT GROUP PHOTO

      // BEGIN PROCESS TO SELECT BACKGROUND GROUP PHOTO
        $scope.selectGroupBgImage = function(index) {
          $scope.hideSheet(); //USING NEW PLUGIN MediaPicker $scope.form.backgroundPhoto
          $scope.form.backgroundPhoto = [];
          if(index === 0) { // camera
            ImageService2.saveCameraMediaBackgroundPhoto(index).then(function() {
            });
          } else { // library
            ImageService2.saveAlbumMediaBackgroundPhoto(index).then(function() {
            });
          }
        };

        $rootScope.$on("addedImageC", function() {
          showBackgroundImage();
        })

        function showBackgroundImage() {
          if($scope.openCrop) {
            return true;
          }
          $scope.openCrop = true;
          var img = FileService2.images();
          var exifTemp = FileService2.exifImage();
          if(img.length === 0) {
            $timeout(function() {
              showBackgroundImage();
            }, 1000);
          } else {
            if($scope.form.backgroundPhoto === null || $scope.form.backgroundPhoto === undefined || $scope.form.backgroundPhoto.length === 0) {
              var opt = {};
              if(isIOS === true) {
                //prepare IOS
                opt = {
                  url: img[0],
                  width: 393,
                  height: 230,
                  circle: true,
                  allowRotation: true,
                  title: 'Usa tus dedos para ajustar la imagen'
                }
                cropBack.crop(opt).then(function(canvas) {
                  // success!
                  //$scope.$apply();
                  $scope.form.backgroundPhoto[0] = canvas.toDataURL();
                  $timeout(function() {
                    $scope.openCrop = false;

                    $scope.$apply();
                  }, 0);
                }, function(err) {
                  console.error(err);
                  $scope.form.backgroundPhoto[0] = img[0];
                  $scope.openCrop = false;
                  $scope.$apply();
                });
              } else {
                var urlToUse = null;
                if(img[0].uri) {
                  urlToUse = img[0].uri;
                } else {
                  urlToUse = img[0];
                }
                opt = {
                  url: urlToUse,
                  width: 393,
                  height: 230,
                  circle: true,
                  allowRotation: true,
                  title: 'Usa tus dedos para ajustar la imagen'
                }
                cropBack.crop(opt).then(function(canvas) {
                  // success!
                  //$scope.$apply();
                  $scope.form.backgroundPhoto[0] = canvas.toDataURL();
                  $timeout(function() {
                    $scope.openCrop = false;
                    $scope.changeProfilePicture();
                    $scope.$apply();
                  }, 0);
                }, function(err) {
                  console.error(err);
                  $scope.form.backgroundPhoto[0] = img[0];
                  $scope.openCrop = false;
                  $scope.$apply();
                });
              }
            } else {
              $scope.openCrop = false;
              console.log("finish loading background")
            }
          }
        }
      // END PROCESS TO SELECT BACKGROUND GROUP PHOTO

      /**
       * Creates a group (w/o photo),
       * Uploads and generates s3 image URL,
       * Updates the group's photo,
       * Adds members to the group
       */
      $scope.createGroup = function() {
        Utils.loading.show();
        if ($scope.form.image === null || $scope.form.image === undefined  || $scope.form.image.length === 0 ||  $scope.form.image === "img/photoFrame.svg") {
          Utils.loading.hide();
          Utils.alert.show(Utils.translate('GROUPS.CREATE_S2.DIALOG.NEED_G_IMAGE_ERR'));
          return true;
        }
        if ($scope.form.backgroundPhoto === null || $scope.form.backgroundPhoto === undefined || $scope.form.backgroundPhoto.length === 0 || $scope.form.backgroundPhoto === "img/postBg.png") {
          Utils.loading.hide();
          Utils.alert.show(Utils.translate('GROUPS.CREATE_S2.DIALOG.NEED_G_IMAGE_ERR02'));
          return true;
        }
        //REGISTER GROUP

        registerGroup().then(function(createResponse) {
          var groupHrefIdHash = Utils.md5.createHash(
            Utils.getLastUrlComponent(createResponse.result.links["0"].href)
          );
          var photoKey = s3.prefix + '/grupo/' + groupHrefIdHash + '/portada/foto.jpg';
          var backPhotoKey = s3.prefix + '/grupo/' + groupHrefIdHash + '/background/foto.jpg';
          //UPLOAD IMAGE TO AWS
          uploadFileToS3(photoKey, $scope.form.image[0]).then(function(uploadResponse) {
            //UPLOAD BACKGROUND TO AWS
            updateGroupImageDBRecord(createResponse.result.instanceId, uploadResponse.Location).then(function() {
              //LINK BACKGROUND TO GROUP
              uploadFileToS3(backPhotoKey, $scope.form.backgroundPhoto[0]).then(function(uploadBackResponse) {
                //ADD MEMBERS LIST TO GROUP
                updateGroupBackgroundDBRecord(createResponse.result.instanceId, uploadBackResponse.Location).then(function() {

                  //asynchronous call to add and approve members
                  addMembers(createResponse.result.instanceId).then(function(addResponses) {
                    approveMembers(addResponses).then(function() {
                      console.log("All goood");
                    }, function(approveError) {
                      rollBackGroup(7, createResponse.result.instanceId, addMembersErrors, uploadResponse, uploadBackResponse);
                    })
                  }, function(addMembersErrors) {
                    rollBackGroup(6, createResponse.result.instanceId, addMembersErrors, uploadResponse, uploadBackResponse);
                  });

                  //APPROVE MEMBERS LIST TO GROUP
                  $timeout( function () {
                    GroupsService.resetGroupForm();
                    //LOAD ALL GROUPS INCLUDE THE NEW ONE
                    GroupFactory.loadAvailableUserGroups({
                      userSubscriptionHref: UserService.get_USER_INSTANCE_HREF()
                    }).then(function(myGroups) {
                      var gId = -1;
                      myGroups.some(function(group, idx) {
                        if (group.hrefId === createResponse.result.instanceId)
                          return gId = idx + 1;
                      });
                      Utils.loading.hide();
                      GroupFactory.setCurrentGroupIndex(gId);
                      //GO TO NEW GROUP
                      $state.go("menu.timeline", { groupIndex: gId }, { reload: true })
                    }, function(errorLoad) {
                      //IF FAIL SEND TO TIMELINE
                      $state.go("menu.timeline", { prepareStatement: 0, groupIndex: 0 })
                    })
                  }, 1000);
                  
                }, function(updateGroupBackError) {
                  rollBackGroup(5, createResponse.result.instanceId, updateGroupBackError, uploadResponse, uploadBackResponse);
                })
              }, function(uploadS3Error) {
                rollBackGroup(4, createResponse.result.instanceId, uploadS3Error, uploadResponse, null);
              });
            }, function(updateGroupError) {
              rollBackGroup(3, createResponse.result.instanceId, updateGroupError, uploadResponse, null);
            })
          }, function(uploadS3Error) {
            rollBackGroup(2, createResponse.result.instanceId, uploadS3Error, null, null);
          });
        }, function(createGroupError) {
          rollBackGroup(1, null, createGroupError, null, null);
        }).finally(function() {
          //console.log("createGroup().finally");
        });
      };

      /**
       *  FUNCTION TO ROLLBACK GROUP CREATION
       *
       **/
      function rollBackGroup(step, id, msgError, key01, key02) {
        //Utils.loading.hide();
        console.error("Can't create group because: ", msgError);
        switch (step) {
          case 1:
            //fails to even create the group
            Utils.loading.hide();
            Utils.alert.show(Utils.translate('GROUPS.CREATE_S2.DIALOG.CREATE_GROUP_ERR'));
            break;
          case 2:
            //Fails to upload the first Photo to aws s3
            //delete the group, hide the loading spinner
            GroupsService.removeGroup(id).then(function(resClearGroup) {
              Utils.loading.hide();
              Utils.alert.show(Utils.translate('GROUPS.CREATE_S2.DIALOG.ADD_G_IMAGE_ERR'));
            }, function(errClearGroup) {
              Utils.loading.hide();
            });
            break;
          case 3:
          case 4:
            //fails to link the photo to the group case 3
            // delete the group, delete the photo, hide the loading spinner
            //or
            //fails to upload the background to aws s3 case 4
            // delete the group, delete the photo, hide the loading spinner
            GroupsService.removeGroup(id).then(function(resClearGroup) {
              //DELETE PHOTO FROM AWS
              s3Service.deleteObjects({
                Bucket: s3.bucket,
                Delete: { Objects: [{ Key: key01.Key }] }
              }, function(err, data) {
                if (err)
                  console.error('deleteS3Objects', err);
                Utils.loading.hide();
                if (step === 3) {
                  Utils.alert.show(Utils.translate('GROUPS.CREATE_S2.DIALOG.SET_G_IMAGE_ERR'));
                } else if (step === 4) {
                  Utils.alert.show(Utils.translate('GROUPS.CREATE_S2.DIALOG.ADD_G_IMAGE_ERR02'));
                }

              });
            }, function(errClearGroup) {
              Utils.loading.hide();
            });
            break;
          case 5:
            //fails to link the background to the group
            // delete group, delete photo and delete background
            GroupsService.removeGroup(id).then(function(resClearGroup) {
              //DELETE PHOTO FROM AWS
              s3Service.deleteObjects({
                Bucket: s3.bucket,
                Delete: { Objects: [{ Key: key01.Key }] }
              }, function(err, data) {
                if (err)
                  console.error('deleteS3Objects', err);
                //DELETE BACKGROUND FROM AWS
                s3Service.deleteObjects({
                  Bucket: s3.bucket,
                  Delete: { Objects: [{ Key: key02.Key }] }
                }, function(err, data) {
                  if (err)
                    console.error('deleteS3Objects', err);
                  Utils.loading.hide();
                  Utils.alert.show(Utils.translate('GROUPS.CREATE_S2.DIALOG.SET_G_IMAGE_ERR02'));
                })
              });
            }, function(errClearGroup) {
              Utils.loading.hide();
            });
            break;
          case 6:
          case 7:
            //fails to add the member list to the group
            //show alert message to the user but continue with the process 
            //
            //O
            //Fails to approbe members into the group
            //show alert message to the user but continue with the process
            GroupsService.resetGroupForm();
            GroupFactory.loadAvailableUserGroups({
              userSubscriptionHref: UserService.get_USER_INSTANCE_HREF()
            }).then(function(myGroups) {
              var gId = -1;
              myGroups.some(function(group, idx) {
                if (group.hrefId === createResponse.result.instanceId)
                  return gId = idx + 1;
              });
              Utils.loading.hide();
              GroupFactory.setCurrentGroupIndex(gId);
              if (step === 6)
                Utils.alert.show(Utils.translate('GROUPS.CREATE_S2.DIALOG.ADD_MEMBERS_ERR'));
              else
                Utils.alert.show(Utils.translate("GROUPS.CREATE_S2.DIALOG.APPROVE_MEMBERS_ERR"));
              $state.go("menu.timeline", { groupIndex: gId }, { reload: true })
              //$state.go("menu.timeline", {newGroupId: createResponse.result.instanceId}, {reload:true});
            }, function(errorLoad) {
              //Fails to get all avalible groups for the user
              Utils.loading.hide();
              if (step === 6)
                Utils.alert.show(Utils.translate('GROUPS.CREATE_S2.DIALOG.ADD_MEMBERS_ERR'));
              else
                Utils.alert.show(Utils.translate("GROUPS.CREATE_S2.DIALOG.APPROVE_MEMBERS_ERR"));
              $state.go("menu.timeline", { prepareStatement: 0, groupIndex: 0 })
            })
            break;
        }
      }

      function deletePostRollback() {
        //GroupsService.removeGroup
      }

      /**
       * Creates a group without image (s3 url), it will be added afterwards
       * @returns {Promise}
       */
      function registerGroup() {
        return Utils.$q(function(resolve, reject) {
          var groupForm = GroupsService.getGroupForm();

          GroupsService.createGroup({
            userHref: UserService.get_USER_INSTANCE_HREF(),
            name: groupForm.name,
            privacy: groupForm.privacy,
            description: groupForm.description,
            url: ""
          }).then(function(createResponse) {
            resolve(createResponse);
          }, function(createGroupError) {
            console.error(createGroupError);
            reject(createGroupError);
          });
        });
      }

      /**
       * Uploads a file to s3
       * @param photoKey {string}
       * @param imageURI {string}
       * @return {Promise}
       */
      function uploadFileToS3(photoKey, imageURI) {
        return Utils.$q(function(resolve, reject) {
          ImageService3.convertUriToImage64(imageURI).then(function(imageB64) {
            var s3Service = new AWS.S3({
              apiVersion: '2006-03-01',
              params: { Bucket: s3.bucket }
            });

            var file = Utils.dataURLtoBlob(imageB64);

            //time to upload the group's image...
            s3Service.upload({
              Key: photoKey,
              Body: file,
              ACL: 'public-read',
              ContentType: file.type
            }, function(uploadError, data) {
              uploadError ? reject(uploadError) : resolve(data);
            });
          }, function(error) {
            reject(error);
          })
        });
      }

      /**
       * @param groupInstanceId
       * @param photoLocation
       * @returns {Promise}
       */
      function updateGroupImageDBRecord(groupInstanceId, photoLocation) {
        return Utils.$q(function(resolve, reject) {
          GroupsService.updateGroupsPhoto(groupInstanceId, photoLocation).then(function(updateResponse) {
            if (updateResponse.result.value == true)
              resolve(updateResponse);
            else
              reject(updateResponse);
          }, function(updateError) {
            console.error("Can't update group's image url", updateError);
            reject(updateError);
          });
        });
      }

      /**
       *
       * @param groupInstanceId
       * @param photoLocation
       * @returns {Promise}
       */
      function updateGroupBackgroundDBRecord(groupInstanceId, photoLocation) {
        return Utils.$q(function(resolve, reject) {
          GroupsService.updateGroupsBackground(groupInstanceId, photoLocation).then(function(updateResponse) {
            if (updateResponse.$resolved == true)
              resolve(updateResponse);
            else
              reject(updateResponse);
          }, function(updateError) {
            console.error("Can't update group's background url", updateError);
            reject(updateError);
          });
        });
      }

      /**
       * Adds members to a given group;
       * if there was at least 1 insert error: reject, otherwise: resolve
       * @param groupInstanceId
       * @returns {Promise} - [responses] || [errors]
       */
      function addMembers(groupInstanceId) {
        return Utils.$q(function(resolve, reject) {
          var promises = [];
          $scope.form.members.forEach(function(member, idx) {
            //console.log("GroupInstanceId: " + groupInstanceId);
            promises[idx] = GroupsService.addMember({
              groupId: groupInstanceId,
              adminHref: UserService.get_USER_INSTANCE_HREF(),
              friendHref: member.href
            });
          });

          Utils.$q.all(promises).then(function(responses) {
            //console.log(responses);
            resolve(responses);
          }, function(errors) {
            reject(errors);
          });
        });
      }

      /**
       * Only for those users that have been added to a private group
       * @param {Object[]} addResponses
       * @returns {Promise}
       */
      function approveMembers(addResponses) {
        return Utils.$q(function(resolve, reject) {
          var promises = [];

          addResponses.forEach(function(resource) {
            var groupHrefId = Utils.getLastUrlComponent(resource.result.members.groupMember.value.href);
            promises.push(GroupsService.approveJoinRequest(groupHrefId));
          });

          Utils.$q.all(promises).then(function(responses) {
            resolve(responses);
          }, function(errors) {
            reject(errors);
          });
        })
      }

      $scope.goBack = function() {
        resetAfterLeave = false;
        $rootScope.$ionicGoBack();
      }

      //ASK AWS TO RETRY UPLOAD UPDATING TIMESTAMP
      AWS.events.on('retry', function(response) {  
        if (response.error.name === 'RequestTimeTooSkewed') {
          console.error('User time is not correct. Handling error!');
          console.log('AWS systemClockOffset:', AWS.config.systemClockOffset);
          var serverTime = Date.parse(response.httpResponse.headers['date']);
          var timeNow = new Date().getTime();
          console.log('AWS timestamp:', new Date(serverTime));
          console.log('Browser timestamp:', new Date(timeNow));
          AWS.config.systemClockOffset = Math.abs(timeNow - serverTime);
          response.error.retryable = true;
          console.log('Setting systemClockOffset to:', AWS.config.systemClockOffset);
          console.log('Retrying uploading to S3 once more...');
        }
      });
      
      $scope.KeyBoardIsShow = null;
      
      window.addEventListener('keyboardWillShow', (event) => {
          console.log("va a abrir");
          $scope.KeyBoardIsShow = '1';
          $scope.$apply();
      });
      
      window.addEventListener('keyboardWillHide', () => {
          console.log("va a cerrar")
          $scope.KeyBoardIsShow = '2';
          $scope.$apply();
      });
    }
  ])

  .controller("groupMembersManagementCtrl", ['$scope', '$state', 'AsyncSearch', 'UserService', 'GroupFactory', 'GroupsService', 'permissionValidationService', 'Utils',
    function($scope, $state, AsyncSearch, UserService, GroupFactory, GroupsService, permissionValidationService, Utils) {
      console.info("groupMembersManagementCtrl created!", $state.params);

      //Para validar si muestro o no el btn de remover user
      $scope.mySubscriptionHref = UserService.get_USER_INSTANCE_HREF();

      $scope.group = $state.params.group;
      $scope.imTheAdmin = $scope.group.administrator;
      $scope.imTheOwner = !!$scope.group.owner;

      $scope.isRefreshing = false;
      $scope.canGetMoreGroupMembers = true;
      $scope.isLoadingMembers = false;
      var pagination = {
        start: 0,
        count: 10,
        reset: function() {
          this.start = 0;
          this.count = 10;
        },
        increaseCount: function(step) {
          this.count += step || 5
        }
      };
      var paginationSpecific = {
        start: 0,
        count: 10,
        reset: function() {
          this.start = 0;
          this.count = 10;
        },
        increaseCount: function(step) {
          this.count += step || 5
        }
      };
      $scope.specificText;
      /**@type {GroupMemberType[]}*/
      $scope.membersList = [];
      $scope.totalMembers = 0;

      // ZONE for searching, filtering  & adding contacts
      $scope.asyncContacts = [];
      //$scope.filterSelected = true;


      //ZONE for querying group's members
      var bannedList = [];

      $scope.loadMembers = function() {
        if ($scope.isLoadingMembers) return;

        GroupsService.countMembersByGroupIdWS($scope.group.id).$promise.then(function(response) {
          $scope.totalMembers = response.result.value;
        }, function(error) {
          console.error("Cant get the num of members", error);
          $scope.totalMembers = $scope.membersList.length;
        });

        $scope.isLoadingMembers = true;

        //console.log("pagination", pagination.start, pagination.count);

        GroupFactory.getGroupMembers({
          groupId: $scope.group.id,
          start: pagination.start,
          count: pagination.count
        }).then( /**@param {GroupMemberType[]} membersList*/ function(membersList) {
          //console.log("Group Members", membersList);
          var totalRetrieved = membersList.length;
          $scope.membersList = $scope.membersList.concat(membersList);
          $scope.canGetMoreGroupMembers = totalRetrieved >= pagination.count;
          pagination.start += totalRetrieved;
          pagination.increaseCount();
        }, function( /*error*/ ) {
          $scope.canGetMoreGroupMembers = false;
          Utils.alert.show(Utils.translate('GROUPS.MEMBERS_MGMT.DIALOG.GET_MEMBERS_LIST_ERR'));
        }).finally(function() {
          $scope.isRefreshing = false;
          $scope.isLoadingMembers = false;
          $scope.$broadcast('scroll.refreshComplete');
          $scope.$broadcast('scroll.infiniteScrollComplete');
        });
      };

      $scope.loadMembers();

      //NEW SEARCH, SPECIFIC MEMBERS
        $scope.loadSpecificMembers = function(textSearch) {
          if(textSearch === "" || textSearch === undefined || textSearch === null) {
            $scope.loadMembers();
          }

          if ($scope.isLoadingMembers) return;

          GroupsService.countMembersByGroupIdWS($scope.group.id).$promise.then(function(response) {
            $scope.totalMembers = response.result.value;
          }, function(error) {
            console.error("Cant get the num of members", error);
            $scope.totalMembers = $scope.membersList.length;
          });

          $scope.isLoadingMembers = true;

          //CLEAN LIST TO SHOW ONLY SPECIFIC BALA
          $scope.membersList = null;
          //CLEAN LIST TO SHOW ONLY SPECIFIC END


          GroupFactory.getGroupSpecificMembers({
            groupId: $scope.group.id,
            start: 0,
            count: 0,
            txtSearch: textSearch
          }).then( /**@param {GroupMemberType[]} membersList*/ function(membersList) {
            var totalRetrieved = membersList.length;
            $scope.membersList = membersList;
            $scope.canGetMoreGroupMembers = false;
            pagination.reset();
          }, function( /*error*/ ) {
            $scope.canGetMoreGroupMembers = false;
            Utils.alert.show(Utils.translate('GROUPS.MEMBERS_MGMT.DIALOG.GET_MEMBERS_LIST_ERR'));
          }).finally(function() {
            $scope.isRefreshing = false;
            $scope.isLoadingMembers = false;
            $scope.$broadcast('scroll.refreshComplete');
            $scope.$broadcast('scroll.infiniteScrollComplete');
          });
        }
      //NEW SEARCH, SPECIFIC MEMBERS END

      $scope.refreshGroupMembers = function() {
        $scope.isRefreshing = true;
        $scope.canGetMoreGroupMembers = true;
        $scope.membersList = [];
        $scope.asyncContacts = [];
        pagination.reset();
        $scope.totalMembers = 0;

        $scope.loadMembers();
      };


      //ZONE for adding members
      $scope.addMoreMembers = function() {
        Utils.loading.show();
        var addPromises = [];
        $scope.asyncContacts.forEach(function(member) {
          addPromises.push(
            GroupsService.addMember({
              groupId: $scope.group.hrefId,
              adminHref: UserService.get_USER_INSTANCE_HREF(),
              friendHref: member.href
            })
          );
        });

        Utils.$q.all(addPromises).then(function(allResponses) {
          //console.log("$q.all(addPromises): All members were added",allResponses);
          $scope.asyncContacts = [];

          var approvalPromises = [];
          allResponses.forEach(function(resource) {
            approvalPromises.push(GroupsService.approveJoinRequest(resource.result.instanceId));
          });

          Utils.$q.all(approvalPromises).then(function(responses) {
            //console.log("$q.all(approvalPromises)", responses)
            Utils.toast.success(Utils.translate('GROUPS.MEMBERS_MGMT.DIALOG.APPROVE_JOIN_OK'))
          }, function(error) {
            Utils.alert.show(Utils.translate('GROUPS.MEMBERS_MGMT.DIALOG.APPROVE_JOIN_ERR'));
            console.error("$q.all(approvalPromises)", error);
          }).finally(function() {
            $scope.refreshGroupMembers();
            Utils.$timeout(function() {
              Utils.loading.hide();
            }, 800);
          })
        }, function(allErrors) {
          Utils.loading.hide();
          $scope.refreshGroupMembers();
          Utils.alert.show(
            Utils.convertHTMLEntity(
              Utils.translate('GROUPS.MEMBERS_MGMT.DIALOG.ADD_MEMBERS_ERR')
            )
          ).then(function() {
            $state.go("groupReceivedRequests");
          });
          console.error("$q.all(addPromises): Can't add member to group (" + $scope.group.id + ") because: ", allErrors);
        });
      };
      //ZONE for adding members

      $scope.openMemberPopup = function($mdMenu, ev) {
        $mdMenu.open(ev)
      };

      /*/Prompt-like dialog (with checkboxes)
      $scope.openMemberOptions = function (member) {
        setTimeout(function () {
          Utils.alert.getCtrl().show({
            templateUrl: "memberOptions.html",
            locals: {
              name: member.name,
              state: member.state,
              isAdmin: member.administrator,
              isModerator: member.moderator,
              isBanned: member.banned,
              isLocked: member.locked,
              delete: false
            },
            controller: function ($scope, $mdDialog, locals) {
              $scope.locals = locals;
              $scope.stateChange = function () {
                if (locals.administrator)
                  locals.moderator = true;
                locals.roleFlag=
                  locals.administrator * 8 +
                  locals.moderator * 4 +
                  locals.banned * 2 +
                  locals.locked
              };
              $scope.ok = function () {
                $mdDialog.hide($scope.locals)
              };
              $scope.cancel = function () {
                $mdDialog.cancel()
              }
            }
          }).then(function (locals) {
            if (locals.delete)
              $scope.confirmRemoveMember(member);
            else
              applyNewState(member.state, locals);
          })
        }, 10);
      };
      function applyNewState(oldState, locals) {
        console.log(oldState + "=>" + locals.state)
      }*/

      /**
       * turns a given user into an admin
       * @param listIndex {number}
       */
      $scope.confirmToggleAdmin = function(listIndex) {
        var willBeAdmin = !$scope.membersList[listIndex].administrator;
        Utils.confirm.show(
          willBeAdmin ?
          Utils.translate('GROUPS.MEMBERS_MGMT.DIALOG.TURN_INTO_ADMIN_CONF') :
          Utils.translate('GROUPS.MEMBERS_MGMT.DIALOG.REVOKE_ADMIN_CONF'), { okClass: 'md-accent', cancelClass: 'md-primary' }
        ).then(function(resp_bool) {
          if (resp_bool) {
            Utils.loading.show();
            GroupsService.changeAdministrator($scope.membersList[listIndex].groupMemberHrefId, willBeAdmin).then(function() {
              $scope.membersList[listIndex].administrator = willBeAdmin;
              $scope.membersList[listIndex].roleFlag = GroupFactory.calcUserRoleFlag($scope.membersList[listIndex]);
            }, function(turnAdminError) {
              Utils.translate('GROUPS.MEMBERS_MGMT.DIALOG.CHANGE_USER_ROL_ERR');
              console.error(turnAdminError)
            }).finally(function() {
              Utils.loading.hide();
            })
          }
        });
      };

      /**
       *
       * @param listIndex {number}
       */
      $scope.confirmToggleModerator = function(listIndex) {
        var willBeModerator = !$scope.membersList[listIndex].moderator;
        Utils.confirm.show(
          Utils.translate('GROUPS.MEMBERS_MGMT.DIALOG.TURN_INTO_' + (willBeModerator ? 'MODERATOR_CONF' : 'REGULAR_USER_CONF')), { okClass: 'md-accent', cancelClass: 'md-primary' }
        ).then(function(resp_bool) {
          if (resp_bool) {
            Utils.loading.show();
            GroupsService.changeModerator($scope.membersList[listIndex].groupMemberHrefId, willBeModerator).then(function() {
              $scope.membersList[listIndex].moderator = willBeModerator;
              $scope.membersList[listIndex].roleFlag = GroupFactory.calcUserRoleFlag($scope.membersList[listIndex]);
            }, function(turnModeratorError) {
              Utils.translate('GROUPS.MEMBERS_MGMT.DIALOG.CHANGE_USER_ROL_ERR');
              console.error(turnModeratorError)
            }).finally(function() {
              Utils.loading.hide();
            })
          }
        });
      };

      $scope.confirmToggleLockUser = function(listIndex) {
        Utils.confirm.show(
          $scope.membersList[listIndex].locked ?
          Utils.translate('GROUPS.MEMBERS_MGMT.DIALOG.UNLOCK_USER_CONF') :
          Utils.convertHTMLEntity(
            Utils.translate('GROUPS.MEMBERS_MGMT.DIALOG.LOCK_USER_CONF')
          ), { okClass: 'md-accent', cancelClass: 'md-primary' }
        ).then(function(resp_bool) {
          if (resp_bool) {
            $scope.membersList[listIndex].locked ?
              doUnlockUser($scope.membersList[listIndex]) :
              doLockUser($scope.membersList[listIndex], $scope.group.href);
          }
        });
      };

      function doLockUser(member, groupHref) {
        Utils.loading.show();
        GroupsService.lockUserFromGroup(member.userSubscription.href, groupHref).then(function(resp) {
          member.locked = true;
          member.userGroupLocked = resp.userGroupLocked;
          member.roleFlag = GroupFactory.calcUserRoleFlag(member);
        }, function(turnModeratorError) {
          Utils.alert.show(Utils.translate('GROUPS.MEMBERS_MGMT.DIALOG.LOCK_USER_ERR'));
          console.error(turnModeratorError)
        }).finally(function() {
          Utils.loading.hide();
        })
      }

      function doUnlockUser(member) {
        if (!member.userGroupLocked || !member.userGroupLocked.href)
          return Utils.alert.show("Sorry, we don't know which user to unlock :(");

        Utils.loading.show();
        GroupsService.unlockUserFromGroup(
          Utils.getLastUrlComponent(member.userGroupLocked.href)
        ).then(function() {
          member.locked = false;
          delete member.userGroupLocked;
          member.roleFlag = GroupFactory.calcUserRoleFlag(member);
          Utils.loading.hide();
        }, function(turnModeratorError) {
          Utils.alert.show(Utils.translate('GROUPS.MEMBERS_MGMT.DIALOG.UNLOCK_USER_ERR'));
          console.error(turnModeratorError);
          Utils.loading.hide();
        });
      }

      $scope.confirmBanUser = function(listIndex, groupHref) {
        Utils.confirm.show(
          Utils.convertHTMLEntity(
            Utils.translate('GROUPS.MEMBERS_MGMT.DIALOG.BAN_USER_CONF')
          ), { okClass: 'md-accent', cancelClass: 'md-primary' }
        ).then(function(resp_bool) {
          if (resp_bool) {
            Utils.loading.show();
            GroupsService.banUserFromGroup(
              $scope.membersList[listIndex].userSubscription.href, groupHref
            ).then(function() {
              $scope.membersList[listIndex].banned = true;
              $scope.membersList[listIndex].roleFlag = GroupFactory.calcUserRoleFlag($scope.membersList[listIndex]);

              //if user try to add it again, this list must have the banner item
              /*bannedList.push({
                userSubscription: {href: $scope.membersList[listIndex].userSubscription.href}
              });*/
            }, function(turnModeratorError) {
              Utils.alert.show(Utils.translate('GROUPS.MEMBERS_MGMT.DIALOG.BAN_USER_ERR'));
              console.error(turnModeratorError)
            }).finally(function() {
              Utils.loading.hide();
            })
          }
        });
      };


      //ZONE for removing members
      $scope.confirmRemoveMember = function(listIndex) {
        Utils.confirm.show(
          Utils.translate('GROUPS.MEMBERS_MGMT.DIALOG.DELETE_USER_CONF'), { okClass: 'md-accent', cancelClass: 'md-primary' }
        ).then(function(resp_bool) {
          if (resp_bool) {
            Utils.loading.show();
            GroupsService.removeMember($scope.membersList[listIndex].groupMemberHrefId).then(function() {
              $scope.membersList.splice(listIndex, 1);
              Utils.toast.success(Utils.translate('GROUPS.MEMBERS_MGMT.DIALOG.DELETE_USER_OK'))
              //console.log("removeMember", removeResponse);
            }, function(removeError) {
              console.error("removeMember", removeError);
              Utils.alert.show(Utils.translate('GROUPS.MEMBERS_MGMT.DIALOG.DELETE_USER_ERR') + " " + $scope.membersList[listIndex].name)
            }).finally(function() {
              Utils.loading.hide();
            })
          }
        });
      };
      //END ZONE for removing members

      $scope.openSearchModal = function() {
        var dialog = Utils.alert.getCtrl();
        dialog.show({
          templateUrl: 'templates/groups/group-members-selection-modal.html',
          controller: 'GroupSearchUsersCtrl',
          locals: {
            previousUsers: $scope.asyncContacts, //users previously selected in this modal
            fixedUsers: $scope.membersList, //users that can't be unselected
            bannedList: bannedList,
            dialog: dialog
          },
          clickOutsideToClose: false,
          fullscreen: true // Only for -xs, -sm breakpoints.
        }).then(
          /**
           * @param resp {[{
           *  id:string,
           *  href:string
           *  name:string,
           *  photo:string,
           * }] | []}
           */
          function(resp) {
            //console.log("resp", resp);
            $scope.asyncContacts = resp;
            $scope.addMoreMembers();
          });
      };

      $scope.sendFriendProfile = function(userInstanceId) {
        var userInstanceId = +userInstanceId.href.split('/')[6];
        if (!userInstanceId) return;

        if (userInstanceId === +UserService.get_USERSUBSCRIPTIONINSTANCE_ID())
          $state.go('menu.myProfile', {}, { reload: true });
        else
          $state.go('friendProfile', { instanceId: userInstanceId }, { reload: true });
      };


      // ------------------------- PERMISSIONS VALIDATION SECTION ---------------------------
      $scope.$_canManageGroupAdminMemberStatus = function(member) {
        return member.userSubscription.href != $scope.mySubscriptionHref && (
          UserService.USER_IS_SUPERADMIN() ||
          ($scope.imTheOwner || $scope.imTheAdmin && permissionValidationService.canManageGroupMemberStatus())
        );
      };

      $scope.$_canManageGroupRegularMemberStatus = function(member) {
        return UserService.USER_IS_SUPERADMIN() || (
          $scope.imTheAdmin &&
          member.userSubscription.href != $scope.mySubscriptionHref &&
          permissionValidationService.canManageGroupMemberStatus()
        );
      };

      $scope.$_canUpdateGroupMemberAsAdmin = function() {
        //Only the group's owner should be able to promote sb as an admin
        return UserService.USER_IS_SUPERADMIN() || ($scope.imTheOwner && permissionValidationService.canUpdateGroupMemberAsAdmin());
      };

      $scope.$_canUpdateGroupMemberAsModerator = function() {
        return UserService.USER_IS_SUPERADMIN() || permissionValidationService.canUpdateGroupMemberAsModerator();
      };

      $scope.$_canUpdateGroupMemberLockedStatus = function() {
        return UserService.USER_IS_SUPERADMIN() || permissionValidationService.canUpdateGroupMemberLockedStatus();
      };

      $scope.$_canUpdateGroupMemberBannedStatus = function() {
        return UserService.USER_IS_SUPERADMIN() || permissionValidationService.canUpdateGroupMemberBannedStatus();
      };

      $scope.$_canAddGroupMember = function() {
        return UserService.USER_IS_SUPERADMIN() || $scope.imTheAdmin && permissionValidationService.canAddGroupMember();
      };

      $scope.$_canDeleteGroupMember = function() {
        return UserService.USER_IS_SUPERADMIN() || permissionValidationService.canDeleteGroupMember();
      };
    }
  ])

  .controller("groupSettingsCtrl", ["$rootScope", "$scope", '$state', '$ionicHistory', 's3', 'ImageService3', 'GroupsService', 'GroupFactory', 'UserService', 'permissionValidationService', 'Utils', '$timeout', 'ImageService2', 'FileService2', '$jrCrop', 'AWSServices',
    function($rootScope, $scope, $state, $ionicHistory, s3, ImageService3, GroupsService, GroupFactory, UserService, permissionValidationService, Utils, $timeout, ImageService2, FileService2, $jrCrop, AWSServices) {
      console.info("groupSettingsCtrl created!", $state.params);

      /**
       * {
       *  id:         number
       *  hrefId:     string
       *  name:       string
       *  description:string
       *  image:      string (url)
       * }
       */
      $scope.group = $state.params.group;

      var groupHasBeenModified = false;
      var orgValues = {
        name: $scope.group.name,
        desc: $scope.group.description,
        priv: $scope.group.privacy
      };
      FileService2.cleanImages();
      var tempGroupImageB64 = FileService2.images();;
      var tempBackImageB64 = FileService2.images();;
      $scope.myGroupSubscriptionHrefId = null;
      $scope.numOfUsersAvailablesForBecomingAdmin = 0;
      $scope.imTheAdmin = $scope.group.administrator;
      $scope.imModertor = $scope.group.moderator;
      $scope.imMember = $scope.group.imMember;
      $scope.hasValidChanges = false;
      var effectiveGroupMembers = [];
      var numOfOtherAdmins = 0;
      //$scope.enabledComments = $scope.group.enableComment;

      /* we must show the "Leave group" button when the group has more than 1 user (counting out the banned ones)*/
      $scope.totalUsers = 0;
      GroupFactory.getGroupMembers({
        groupId: $scope.group.id
      }).then(function(membersList) {
        membersList.forEach(function(user) {
          if (user.userSubscription.href === UserService.get_USER_INSTANCE_HREF())
            $scope.myGroupSubscriptionHrefId = Utils.getLastUrlComponent(user.groupMember.href);
          else {
            if (!user.banned) {
              effectiveGroupMembers.push(user);

              if (user.administrator)
                numOfOtherAdmins++;
              else if (!user.locked)
                $scope.numOfUsersAvailablesForBecomingAdmin++;
            }
          }
        });

        $scope.totalUsers = membersList.length;
      }, function( /*error*/ ) {});


      $scope.checkPermissionsBefore = function(type) {
        if (ionic.Platform.isIOS() === true) {
          //getPermission(0);
          if (type === 'p') { //group main photo
            $scope.selectAndUpdateGroupImage();
          } else if (type === 'b') { //group background photo
            $scope.selectAndUpdateBgGroupImage();
          }
        } else {
          getMultiplePermissions(type);
        }
      }

      /**
       *function to get multiple permission for camera
       */
      function getMultiplePermissions(type) {
        cordova.plugins.diagnostic.getPermissionsAuthorizationStatus(function(statuses) {
          var havePerm = 0;
          var deniedPerm = false;
          for (var permission in statuses) {
            switch (statuses[permission]) {
              case cordova.plugins.diagnostic.permissionStatus.GRANTED:
                console.log("Permission granted to use " + permission);
                havePerm++;
                break;
              case cordova.plugins.diagnostic.permissionStatus.NOT_REQUESTED:
                console.log("Permission to use " + permission + " has not been requested yet");
                havePerm--;
                break;
              case cordova.plugins.diagnostic.permissionStatus.DENIED:
                console.log("Permission denied to use " + permission + " - ask again?");
                havePerm--;
                break;
              case cordova.plugins.diagnostic.permissionStatus.DENIED_ALWAYS:
                console.log("Permission permanently denied to use " + permission + " - guess we won't be using it then!");
                deniedPerm = true;
                break;
            }
          }
          if (!deniedPerm) {
            if (havePerm == 2) {
              //have both permissions granted
              if (type === 'p') { //group main photo
                $scope.selectAndUpdateGroupImage();
              } else if (type === 'b') { //group background photo
                $scope.selectAndUpdateBgGroupImage();
              }
            } else if (havePerm < 2) {
              //both or one permission hasn't granted
              askMultiplePermissions(type);
            }
          } else {
            Utils.toast.error(Utils.translate('PLUGINS.PERMISSIONS.DENIED_PERMISSION') +
              Utils.translate('PLUGINS.PERMISSIONS.TYPE.CAMERA') + ' ' +
              Utils.translate('PLUGINS.PERMISSIONS.AND') + ' ' +
              Utils.translate('PLUGINS.PERMISSIONS.TYPE.GALLERY') + '. \n' +
              Utils.translate('PLUGINS.PERMISSIONS.ALWAYS_DENIED_PERMISSION'));
          }
        }, function(error) {
          console.error("The following error occurred: " + error);
        }, [
          cordova.plugins.diagnostic.permission.CAMERA,
          cordova.plugins.diagnostic.permission.READ_EXTERNAL_STORAGE
        ])
      }

      /**
       *function to get multiple permission for camera
       */
      function askMultiplePermissions(type) {
        cordova.plugins.diagnostic.requestRuntimePermissions(function(statuses) {
          var havePerm = 0;
          var deniedPerm = false;
          for (var permission in statuses) {
            switch (statuses[permission]) {
              case cordova.plugins.diagnostic.permissionStatus.GRANTED:
                console.log("Permission granted to use " + permission);
                havePerm++;
                break;
              case cordova.plugins.diagnostic.permissionStatus.NOT_REQUESTED:
                console.log("Permission to use " + permission + " has not been requested yet");
                havePerm--;
                break;
              case cordova.plugins.diagnostic.permissionStatus.DENIED:
                console.log("Permission denied to use " + permission + " - ask again?");
                havePerm--;
                break;
              case cordova.plugins.diagnostic.permissionStatus.DENIED_ALWAYS:
                console.log("Permission permanently denied to use " + permission + " - guess we won't be using it then!");
                deniedPerm = true;
                break;
            }
          }
          if (!deniedPerm) {
            if (havePerm == 2) {
              //have both permissions granted
              if (type === 'p') { //group main photo
                $scope.selectAndUpdateGroupImage();
              } else if (type === 'b') { //group background photo
                $scope.selectAndUpdateBgGroupImage();
              }
            } else if (havePerm < 2) {
              //both or one permission hasn't granted
              Utils.toast.error(Utils.translate('PLUGINS.PERMISSIONS.DENIED_PERMISSION') +
                Utils.translate('PLUGINS.PERMISSIONS.TYPE.CAMERA') + ' ' +
                Utils.translate('PLUGINS.PERMISSIONS.AND') + ' ' +
                Utils.translate('PLUGINS.PERMISSIONS.TYPE.GALLERY') + '.');
            }
          } else {
            Utils.toast.error(Utils.translate('PLUGINS.PERMISSIONS.DENIED_PERMISSION') +
              Utils.translate('PLUGINS.PERMISSIONS.TYPE.CAMERA') + ' ' +
              Utils.translate('PLUGINS.PERMISSIONS.AND') + ' ' +
              Utils.translate('PLUGINS.PERMISSIONS.TYPE.GALLERY') + '. \n' +
              Utils.translate('PLUGINS.PERMISSIONS.ALWAYS_DENIED_PERMISSION'));
          }
        }, function(error) {
          console.error("The following error occurred: " + error);
        }, [
          cordova.plugins.diagnostic.permission.CAMERA,
          cordova.plugins.diagnostic.permission.READ_EXTERNAL_STORAGE
        ])
      }

      $scope.uploadingPhoto = false;
      $scope.openCrop = false;
      // BEGIN PROCESS TO UPDATE GROUP PHOTO
        $scope.selectAndUpdateGroupImage = function() {
          tempGroupImageB64 = [];
          ImageService3.showImageSelectionOptions().then(function(index) {
            if (index === -1) return; //cancel/close the menu tempGroupImageB64
            if(index === 0) { // camera
              ImageService2.saveCameraMediaProfilePhoto(index).then(function () {
              });
            } else { // library tempGroupImageB64
              ImageService2.saveAlbumMediaProfilePhoto(index).then(function () {
              });
            }
          });
        };

        $rootScope.$on("addedImageB", function() {
          showImageUser();
        })

        function showImageUser() {
          if($scope.openCrop) {
            return true;
          }
          $scope.openCrop = true;
          var img = FileService2.images();
          var exifTemp = FileService2.exifImage();
          if(img.length === 0) {
            $timeout(function () {
              showImageUser();
            }, 1000);
          } else {
            if(tempGroupImageB64 === null || tempGroupImageB64 === undefined || tempGroupImageB64.length === 0) {
              var opt = {};
              if(ionic.Platform.isIOS() === true) {
                //prepare IOS
                opt = {
                  url: img[0],
                  width: 200,
                  height: 200,
                  circle: true,
                  allowRotation: true,
                  title: 'Usa tus dedos para ajustar la imagen'
                }
                $jrCrop.crop(opt).then(function(canvas) {
                  // success!
                  //$scope.$apply();
                  tempGroupImageB64[0] = canvas.toDataURL();
                  $timeout(function() {
                    $scope.openCrop = false;
                    updateGroupImage();
                    $scope.$apply();
                  }, 0);
                }, function(err) {
                  console.error(err);
                  tempGroupImageB64[0] = img[0];
                  $scope.openCrop = false;
                  $scope.$apply();
                });
              } else {
                var urlToUse = null;
                if(img[0].uri) {
                  urlToUse = img[0].uri;
                } else {
                  urlToUse = img[0];
                }
                opt = {
                  url: urlToUse,
                  width: 200,
                  height: 200,
                  circle: true,
                  allowRotation: true,
                  title: 'Usa tus dedos para ajustar la imagen'
                }
                $jrCrop.crop(opt).then(function(canvas) {
                  // success!
                  //$scope.$apply();
                  tempGroupImageB64[0] = canvas.toDataURL();
                  $timeout(function() {
                    $scope.openCrop = false;
                    updateGroupImage();
                    $scope.$apply();
                  }, 0);
                }, function(err) {
                  console.error(err);
                  tempGroupImageB64[0] = img[0];
                  $scope.openCrop = false;
                  $scope.$apply();
                });
              }
            } else {
              $scope.openCrop = false;
              console.log("finish loading");
            }
          }
        }

        function updateGroupImage() {
          console.log("Actualizando imagen del grupo...");
          Utils.loading.show();
          //var photoKey = s3.prefix + '/grupo/' + Utils.md5.createHash($scope.group.hrefId) + '/portada/foto.jpg';
          var indexRandom = Math.floor(Math.random() * 1000);
          var photoKey = s3.prefix + '/grupo/' + Utils.md5.createHash($scope.group.hrefId.toString()) + '/portada/foto' + indexRandom + '.jpg';

          uploadImageToS3(photoKey, tempGroupImageB64).then(function(uploadS3Response) {
            updateGroupImageDBRecord($scope.group.hrefId, uploadS3Response.Location).then(function() {
              $scope.group.image = uploadS3Response.Location;
              tempGroupImageB64 = null;
              groupHasBeenModified = true;
              $rootScope.$broadcast('groups.mine.reload');
              Utils.toast.success(Utils.translate('GROUPS.SETTINGS.DIALOG.UPDATE_IMAGE_OK'));
            }, function(updateDbError) {
              console.error(updateDbError);
              Utils.alert.show(Utils.translate('GROUPS.SETTINGS.DIALOG.UPDATE_IMAGE_ERR'))
            }).finally(function() {
              Utils.loading.hide();
            })
          }, function(uploadS3Error) {
            Utils.loading.hide();
            console.error(uploadS3Error);
            Utils.alert.show(Utils.translate('GROUPS.SETTINGS.DIALOG.UPLOAD_IMAGE_ERR'), { title: "Error" })
          });
        }

        function updateGroupImageDBRecord(groupHrefId, imageUrl) {
          return Utils.$q(function(resolve, reject) {
            GroupsService.updateGroupsPhoto(groupHrefId, imageUrl).then(function(updateResponse) {
              updateResponse.result.value === true ? resolve(updateResponse) : reject(updateResponse);
            }, function(updateError) {
              reject(updateError);
            });
          });
        }
      // END PROCESS TO UPDATE GROUP PHOTO

      // BEGIN PROCESS TO UPDATE GROUP BACKGROUND
        $scope.selectAndUpdateBgGroupImage = function() {
          tempBackImageB64 = [];
          ImageService3.showImageSelectionOptions().then(function(index) {
            if (index === -1) return; //cancel/close the menu
            if(index === 0) { // camera
              ImageService2.saveCameraMediaBackgroundPhoto(index).then(function () {
              });
            } else { // library 
              ImageService2.saveAlbumMediaBackgroundPhoto(index).then(function () {
              });
            }
          });
        };

        $rootScope.$on("addedImageC", function () {
          showBackImageUser();
        })

        function showBackImageUser() {
          if($scope.openCrop) {
            return true;
          }
          $scope.openCrop = true;
          var img = FileService2.images();
          var exifTemp = FileService2.exifImage();
          if(img.length === 0) {
            $timeout(function() {
              showBackImageUser();
            }, 1000);
          } else {
            if(tempBackImageB64 === null || tempBackImageB64 === undefined || tempBackImageB64.length === 0) {
              var opt = {};
              if(ionic.Platform.isIOS() === true) {
                //prepare IOS
                opt = {
                  url: img[0],
                  width: 393,
                  height: 230,
                  circle: true,
                  allowRotation: true,
                  title: 'Usa tus dedos para ajustar la imagen'
                }
                $jrCrop.crop(opt).then(function(canvas) {
                  // success!
                  //$scope.$apply();
                  tempBackImageB64[0] = canvas.toDataURL();
                  $timeout(function() {
                    $scope.openCrop = false;
                    updateGroupBackgroundImage();
                    $scope.$apply();
                  }, 0);
                }, function(err) {
                  console.error(err);
                  $scope.newBackImage[0] = img[0];
                  $scope.openCrop = false;
                  $scope.$apply();
                });
              } else {
                var urlToUse = null;
                if(img[0].uri) {
                  urlToUse = img[0].uri;
                } else {
                  urlToUse = img[0];
                }
                opt = {
                  url: urlToUse,
                  width: 393,
                  height: 230,
                  circle: true,
                  allowRotation: true,
                  title: 'Usa tus dedos para ajustar la imagen'
                }
                $jrCrop.crop(opt).then(function(canvas) { // success!
                  // success!
                  //$scope.$apply();
                  tempBackImageB64[0] = canvas.toDataURL();
                  $timeout(function() {
                    updateGroupBackgroundImage();
                    $scope.openCrop = false;
                    $scope.$apply();
                  }, 0);
                }, function(err) {
                  console.error(err);
                  tempBackImageB64[0] = img[0];
                  $scope.openCrop = false;
                  $scope.$apply();
                });
              }
            } else {
              $scope.openCrop = false;
              console.log("finish loading background")
            }
          }
        }

        function updateGroupBackgroundImage() {
          console.log("Actualizando background...");
          Utils.loading.show();
          var indexRandom = Math.floor(Math.random() * 1000);
          var photoKey = s3.prefix + '/grupo/' + Utils.md5.createHash($scope.group.hrefId.toString()) + '/background/foto' + indexRandom + '.jpg';

          uploadImageToS3(photoKey, tempBackImageB64).then(function(uploadS3Response) {
            updateGroupBackgroundDBRecord($scope.group.hrefId, uploadS3Response.Location).then(function() {
              $scope.group.backgroundImage = uploadS3Response.Location;
              tempBackImageB64 = null;
              groupHasBeenModified = true;
              sendNotifToMembers();
              //NOTIF GROUP BUT LEAVE USER FREE BALA
                //sendNotifAppMembers();
              $rootScope.$broadcast('groups.mine.reload');
              Utils.toast.success(Utils.translate('GROUPS.SETTINGS.DIALOG.UPDATE_IMAGE_OK'));
            }, function(updateDbError) {
              console.error(updateDbError);
              Utils.alert.show(Utils.translate('GROUPS.SETTINGS.DIALOG.UPDATE_IMAGE_ERR'))
            }).finally(function() {
              Utils.loading.hide();
            })
          }, function(uploadS3Error) {
            Utils.loading.hide();
            console.error(uploadS3Error);
            Utils.alert.show(Utils.translate('GROUPS.SETTINGS.DIALOG.UPLOAD_IMAGE_ERR'), { title: "Error" })
          });
        }
      // END PROCESS TO UPDATE GROUP BACKGROUND

      //ZONE FOR UPLOAD IMAGES
      /**
       * @param photoKey
       * @param imageB64
       * @return {Promise}
       */
      function uploadImageToS3(photoKey, imageB64) {
        return Utils.$q(function(resolve, reject) {
          var s3Service = new AWS.S3({
            apiVersion: '2006-03-01',
            params: { Bucket: s3.bucket }
          });

          var file = Utils.dataURLtoBlob(imageB64[0]);

          s3Service.upload({
            Key: photoKey,
            Body: file,
            ACL: 'public-read',
            ContentType: file.type
          }, function(uploadError, data) {
            if (uploadError) {
              console.log("Can't upload the image to S3: ", uploadError);
              return reject(uploadError);
            }

            resolve(data);
          });
        });
      }
      //END ZONE FOR UPLOAD IMAGES

      $scope.checkForChanges = function() {
        if (!$scope.group.name || !$scope.group.description)
          return $scope.hasValidChanges = false;

        return $scope.hasValidChanges = (
          $scope.group.name !== orgValues.name ||
          $scope.group.description !== orgValues.desc ||
          $scope.group.privacy !== orgValues.priv
        )
      };

      function sendNotifToMembers() {
        var wsData = {
          groupId: $scope.group.id,
          userSubscriptionId: UserService.get_USERSUBSCRIPTION_ID(),
          notification: 16 //TYPE 16
        };
        //SE OCUPA WS QUE MANDA TIPO DE NOTIFICACIÓN
        GroupsService.getEndPointMemberswithNotif(wsData).then(function(dataWS) {
          //GroupsService.getEndPointMembers(wsData).then(function (dataWS) {
          //prepare message to send
          //console.log("complete", dataWS);
          var messageForShow = Utils.translate('ALERTS.NEW_GROUP_BACKGROUND_01') +
            $scope.group.name +
            Utils.translate('ALERTS.NEW_GROUP_BACKGROUND_02');
          //send to each one endpoint
          dataWS.forEach(function(dataDevices) {

            //console.log("for each", dataDevices);
            AWSServices.sendNewGroupCoverNotification(
              messageForShow,
              dataDevices.userDeviceId,
              $scope.group.id,
              $scope.group.name
            )

          })
        }, function(errorWS) {
          console.log("complete error", errorWS);
        })
      }

      function updateGroupBackgroundDBRecord(groupHrefId, imageUrl) {
        return Utils.$q(function(resolve, reject) {
          GroupsService.updateGroupsBackground(groupHrefId, imageUrl).then(function(updateResponse) {
            updateResponse.$resolved === true ? resolve(updateResponse) : reject(updateResponse);
          }, function(updateError) {
            reject(updateError);
          });
        });
      }

      $scope.updateGroupFields = function() {
        //console.log($scope.group);
        Utils.loading.show();
        GroupsService.updateGroupFields({
          groupInstanceHrefId: $scope.group.hrefId,
          name: $scope.group.name,
          description: $scope.group.description,
          privacy: $scope.group.privacy
        }).then(function() {
          groupHasBeenModified = true;
          $rootScope.$broadcast('groups.mine.reload');
          Utils.toast.success(Utils.translate('GROUPS.SETTINGS.DIALOG.UPDATE_DATA_OK'));
          $scope.hasValidChanges = false;
        }, function(updateError) {
          console.error("updateGroupFields", updateError);
          Utils.alert.show(Utils.translate('GROUPS.SETTINGS.DIALOG.UPDATE_DATA_ERR'));
        }).finally(function() {
          Utils.loading.hide();
        })
      };

      $scope.confirmLeave = function() {
        Utils.confirm.show(
          Utils.translate('GROUPS.SETTINGS.DIALOG.LEAVE_GROUP_CONF'), { okClass: 'md-accent', cancelClass: 'md-primary' }).then(function(resp_bool) {
          if (resp_bool) {

            if (!$scope.imTheAdmin || numOfOtherAdmins > 0)
              leaveGroup();
            else {
              showModalWithUsers();
            }
          }
        })
      };

      //Execute this fnc when the user is not the group's admin
      function leaveGroup() {
        Utils.loading.show();
        GroupsService.removeMember($scope.myGroupSubscriptionHrefId).then(function() {
          //console.log("removeMember", removeResponse);
          GroupFactory.setCurrentGroupIndex(0);
          Utils.clearCache();
          Utils.clearHistory();
          $rootScope.$broadcast('groups.mine.reload');
          setTimeout(function() {
            Utils.loading.hide();
            Utils.toast.success(Utils.translate('GROUPS.SETTINGS.DIALOG.LEAVE_GROUP_OK'));
            $state.go('menu.groupsTabs');
          }, 800);
        }, function(removeError) {
          Utils.loading.hide();
          console.error("removeMember", removeError);
          Utils.alert.show(Utils.translate('GROUPS.SETTINGS.DIALOG.LEAVE_GROUP_ERR'))
        });
      }

      //Execute this fnc when the group has more than one user and the YOU are an admin
      function showModalWithUsers() {
        /*GroupFactory.getGroupMembers({
          groupId: $scope.group.id,
          start: 0,
          count: 0
        }).then(/**@param {GroupMemberType[]} membersList*/
        /*function (membersList) {*/
        //console.log("Group Members", $scope.membersList);
        Utils.modal.show({
          templateUrl: 'templates/groups/group-delegate-admin-modal.html',
          locals: {
            membersList: effectiveGroupMembers, //membersList,
            selectedUserHrefId: null
          }
        }).then(function(newData) {
          if (newData.selectedUserHrefId != null)
            changeAdminAndRemoveMe(newData.selectedUserHrefId);
        })
        /*}, function () {
          Utils.alert.show(Utils.translate('GROUPS.MEMBERS_MGMT.DIALOG.GET_MEMBERS_LIST_ERR'));
        }).finally(function () {
          $scope.isLoadingMembers = false;
        });*/
      }

      function changeAdminAndRemoveMe(groupMemberId) {

        Utils.loading.show();

        //make sure the group always has an admin
        GroupsService.changeAdministrator(groupMemberId /*number*/ , true).then(function( /*updateResponse*/ ) {
          //console.log("changeAdministrator", updateResponse);
          Utils.loading.hide();

          //finally remove yor self...
          leaveGroup(); //because i'm not the admin anymore
        }, function(updateError) {
          Utils.loading.hide();
          Utils.alert.show(Utils.translate('GROUPS.SETTINGS.DIALOG.CHANGE_ADMIN_ERR'));
          console.error(updateError);
        })
      }

      $scope.confirmRemove = function() {
        Utils.confirm.show(
          Utils.translate('GROUPS.SETTINGS.DIALOG.REMOVE_GROUP_CONF'), {
            okClass: 'md-accent',
            cancelClass: 'md-primary'
          }).then(function(resp_bool) {
          if (resp_bool) {
            removeGroup($scope.group.hrefId)
          }
        })
      };


      $scope.changePostStatus = function() {

      }

      /**
       * @param {string|number} groupHrefId
       */
      function removeGroup(groupHrefId) {
        Utils.loading.show();
        GroupsService.removeGroup(groupHrefId).then(function() {
          GroupFactory.setCurrentGroupIndex(0);
          Utils.clearCache();
          Utils.clearHistory();
          $rootScope.$broadcast('groups.mine.reload');

          setTimeout(function() {
            Utils.loading.hide();
            Utils.toast.success(Utils.translate('GROUPS.SETTINGS.DIALOG.REMOVE_GROUP_OK'));
            $state.go("menu.groupsTabs");
          }, 800);
        }, function(removeError) {
          Utils.loading.hide();
          console.error(removeError);
          Utils.alert.show(Utils.translate('GROUPS.SETTINGS.DIALOG.REMOVE_GROUP_ERR'))
        });
      }

      $scope.openPageForPush = function() {
        // GROUP NAME $scope.group.name
        // GROUP ID $scope.group.id
        //call WS
        var dialog = Utils.alert.getCtrl();
        dialog.show({
          templateUrl: 'templates/groups/group-pushNotification-membersTpl.html',
          controller: "sendPushGroupMembersCtrl",
          locals: {
            idForUse: $scope.group.hrefId,
            groupName: $scope.group.name,
            dialog: dialog
          },
          clickOutsideToClose: false,
          fullscreen: true // Only for -xs, -sm breakpoints.
        }).then(function() {
          console.log("opened dialog");
        }, function() {
          console.log("opened dialog");
        });
      }

      /*
       * Not used for now...
       *
      function deletePendingRequests() {
        lookForPendingJoinReqs($scope.group.hrefId).then(function (requestList) {
          if(requestList.length){
            Utils.loading.hide();
            Utils.confirm.show(
              "There are some pending request to approve, do you wanto to..."
            ).then(function (boolResp) {
              if(boolResp){
                Utils.loading.show();
                removePendingRequests(requestList).then(function () {
                  removeGroup($scope.group.hrefId).then(function () {
                  }, function () {
                    Utils.alert.show(Utils.translate('GROUPS.SETTINGS.DIALOG.REMOVE_GROUP_ERR'))
                  });
                }, function (error) {
                  reject(error);
                  Utils.alert.show(Utils.translate("GROUPS.RECEIVED_REQS.DIALOG.REJECT_REQ_ERR"));
                })
              }
            })
          }
          else{resolve(true);}//<-- no pending requests
        }, function (requestsError) {
          reject(requestsError);
          Utils.alert.show(Utils.translate("GROUPS.RECEIVED_REQS.DIALOG.GET_RECEIVED_REQS_ERR"))
        });
      }
      /*
       * Verifies whether or not are requests for joining to a given group
       * @returns {Promise} - onSuccess(Group Join Requests Array), onError(error{*})
       *
      function lookForPendingJoinReqs(groupHrefId) {
        return Utils.$q(function (resolve, reject) {
          //This gets all the requests for all the user's groups
          GroupsService.getRequestsToAprove(UserService.getUserInfo().INSTANCE_URL).then(function (requestsList) {
            //console.log(requestsList);
            //remove the last item because it's normally a useless object
            if (!requestsList[requestsList.length - 1].userSubscription)
              requestsList.splice(requestsList.length - 1, 1);
            var groupReqs = [];
            requestsList.forEach(function (request) {
              //todo: el atributo: request.groupHrefId no existe, solicitar a adida lo agregue
              if(request.groupHrefId === groupHrefId){
                groupReqs.push({
                  group:{
                    hrefId:request.groupHrefId
                    //name:request.groupName,
                    //image: request.url
                  },
                  user:{
                    //avatar: Utils.getUserImgUrl(request),
                    userSubscriptionHref: request.userSubscription.href,
                    groupMemberHrefId: Utils.getLastUrlComponent(request.groupMember.href),
                    //bio: request.userBio,
                    //name: request.name
                  }
                });
              }
            });
            resolve(groupReqs);
          }, function (error) {
            console.error(error);
            reject(error);
          })
        })
      }
      /*
       * @param {Object[]} requestList
       * @returns {Promise} - onSuccess(true), onError(error)
       *
      function removePendingRequests(requestList) {
        return Utils.$q(function (resolve, reject) {
          var rmUserPromises = [];
          requestList.forEach(function (request) {
            rmUserPromises.push(
              GroupsService.removeMember(Utils.getLastUrlComponent(request.groupMember.href))
            );
          });
          Utils.$q.all(rmUserPromises).then(function () {
            resolve(true);
          }, function (rmUserErrors) {
            console.error(rmUserErrors);
            reject(rmUserErrors);
          })
        })
      }
      */

      $scope.goBack = function() {
        if (!$ionicHistory.backView().stateName) {
          $ionicHistory.goBack();
          if (groupHasBeenModified) $rootScope.$broadcast('cacheAndHistory.clear');
          return;
        }

        var lastStateName = $ionicHistory.backView().stateName;
        if (groupHasBeenModified) {
          //console.log("Group has been modified, going to: " + lastStateName);

          //image/fields were modified
          $state.go(
            lastStateName, {
              group: $scope.group,
              groupIndex: $state.params.groupIndex ? $state.params.groupIndex + 1 : null
            }, { reload: true }
          );

          $rootScope.$broadcast('cacheAndHistory.clear');
        } else {
          //console.log("Group has not been modified Going to " + lastStateName);
          $state.go(
            lastStateName, {
              group: $state.params.group,
              groupIndex: $state.params.groupIndex ? $state.params.groupIndex + 1 : null
            }, { reload: false });
        }
      };

      // override hard back
      // registerBackButtonAction() returns a function which can be used to deregister it
      var deregisterHardBack = Utils.$ionicPlatform.registerBackButtonAction(
        $scope.goBack, 101
      );

      // cancel custom back behaviour
      $scope.$on('$destroy', function() {
        deregisterHardBack();
        //deregisterSoftBack();
      });

      //ASK AWS TO RETRY UPLOAD UPDATING TIMESTAMP
      AWS.events.on('retry', function(response) {  
        if (response.error.name === 'RequestTimeTooSkewed') {
          console.error('User time is not correct. Handling error!');
          console.log('AWS systemClockOffset:', AWS.config.systemClockOffset);
          var serverTime = Date.parse(response.httpResponse.headers['date']);
          var timeNow = new Date().getTime();
          console.log('AWS timestamp:', new Date(serverTime));
          console.log('Browser timestamp:', new Date(timeNow));
          AWS.config.systemClockOffset = Math.abs(timeNow - serverTime);
          response.error.retryable = true;
          console.log('Setting systemClockOffset to:', AWS.config.systemClockOffset);
          console.log('Retrying uploading to S3 once more...');
        }
      });


      // ------------------------- PERMISSIONS VALIDATION SECTION ---------------------------
      $scope.$_canUpdateGroupName = function() {
        return UserService.USER_IS_SUPERADMIN() || permissionValidationService.canUpdateGroupName();
      };

      $scope.$_canUpdateGroupImage = function() {
        return UserService.USER_IS_SUPERADMIN() || ($scope.imTheAdmin && permissionValidationService.canUpdateGroupImage());
      };

      $scope.$_canUpdateGroupPrivacy = function() {
        return UserService.USER_IS_SUPERADMIN() || permissionValidationService.canUpdateGroupPrivacy();
      };

      $scope.$_canLeaveGroup = function() {
        return $scope.imMember && (
          (!$scope.imTheAdmin && (UserService.USER_IS_SUPERADMIN() || permissionValidationService.canLeaveOthersGroup())) ||
          ($scope.numOfUsersAvailablesForBecomingAdmin > 0 && (UserService.USER_IS_SUPERADMIN() || permissionValidationService.canLeaveOwnGroup()))
        );
      };

      $scope.$_canRemoveGroup = function() {
        return UserService.USER_IS_SUPERADMIN() || (
          ($scope.imTheAdmin && permissionValidationService.canRemoveOwnGroup()) ||
          (permissionValidationService.canRemoveOthersGroup())
        );
      };
    }
  ])

  .controller("GroupSentRequests", ['$rootScope', "$scope", "GroupsService", "UserService", "Utils",
    function($rootScope, $scope, GroupsService, UserService, Utils) {

      $scope.loadingContent = true;
      $scope.groupsList = [];
      GroupsService.getUserJoinRequests(UserService.get_USER_INSTANCE_HREF()).then(function(requestsList) {
        //console.log(requestsList);

        //remove the last item because it's normally a useless object
        if (!requestsList[requestsList.length - 1].group)
          requestsList.splice(requestsList.length - 1, 1);

        requestsList.forEach(function(request) {
          $scope.groupsList.push({
            description: request.description,
            groupMemberHrefId: Utils.getLastUrlComponent(request.groupMember.href),
            href: request.group.href,
            id: request.groupId,
            image: request.url || 'img/photoFrame.svg',
            backgroundImage: request.backgroundUrl || './img/postBg.png',
            name: request.groupName,
            privacy: request.privacy
          })
        })
      }, function(requestsError) {
        console.error(requestsError);
        Utils.alert.show(Utils.translate("GROUPS.MAIN_PAGE.DIALOG.GET_USER_JOIN_REQS_ERR"));
      }).finally(function() {
        $scope.loadingContent = false;
      });

      $scope.confirmRejectRequest = function(listIndex) {
        Utils.confirm.show(
          Utils.translate("GROUPS.SENT_REQS.DIALOG.REJECT_REQ_CONF") +
          " <b>" + $scope.groupsList[listIndex].name + "</b>?"
        ).then(function(resp) {
          if (resp) {
            Utils.loading.show();
            GroupsService.removeMember($scope.groupsList[listIndex].groupMemberHrefId).then(function() {
              Utils.toast.success(Utils.translate("GROUPS.SENT_REQS.DIALOG.REJECT_REQ_OK"));
              $scope.groupsList.splice(listIndex, 1);
              $rootScope.$broadcast('groups.others.reload');
            }, function(removeError) {
              console.error(removeError);
              Utils.toast.error(Utils.translate("GROUPS.SENT_REQS.DIALOG.REJECT_REQ_ERR"))
            }).finally(function() {
              Utils.loading.hide();
            })
          }
        })
      }
    }
  ])

  .controller("GroupReceivedRequests", ['$rootScope', "$scope", "$state", "GroupsService", "UserService", 'permissionValidationService', '$mdDialog', 'Utils', 'AWSServices',
    function($rootScope, $scope, $state, GroupsService, UserService, permissionValidationService, $mdDialog, Utils, AWSServices) {
      //console.log("inside GroupReceivedRequests", $state.params);
      $scope.groupId = $state.params.groupId;

      if ($state.params.totalRequest)
        $scope.totalRequest = $state.params.totalRequest;
      else
        GroupsService.getNumJoiningRequestsByGroup($scope.groupId).$promise.then(function(resp) {
          $scope.totalRequest = resp.result.value;
        });

      $scope.requestList = [];
      var pagination = {
        start: 0,
        count: 10,
        reset: function() {
          this.start = 0;
          this.count = 10;
        },
        increaseCount: function(step) {
          this.count += step || 5
        }
      };
      $scope.isRefreshing = false;
      $scope.isLoadingContent = false;
      $scope.canGetMoreItems = true;
      $scope.lookFor = null;
      $scope.specificSearchOn = false;
      $scope.textSaved = "";
      $scope.loadRequests = function() {
        $scope.specificSearchOn = false;
        $scope.loadingContent = true;
        GroupsService.getRequestsToAproveWS({
          groupId: $scope.groupId,
          start: pagination.start,
          count: pagination.count
        }).$promise.then(function(requestsList) {
          Utils.removeLastItemIfHasNoAttribute(requestsList, 'userSubscription');
          //console.log(requestsList);
          var totalRetrieved = requestsList.length;
          $scope.canGetMoreItems = totalRetrieved >= pagination.count;

          if (!totalRetrieved) {
            onLoadingEnd();
            return;
          }

          requestsList.forEach(function(request) {
            $scope.requestList.push({
              group: {
                id: $scope.groupId,
                name: request.groupName,
                image: request.url
              },
              applicantMetadata: {
                avatar: Utils.getUserImgUrl(request),
                userSubscriptionHref: request.userSubscription.href,
                groupMemberHrefId: Utils.getLastUrlComponent(request.groupMember.href),
                bio: request.userBio,
                name: request.name,
                city: request.residenceCity || Utils.translate("GROUPS.RECEIVED_REQS.NOT_AVAILABLE"),
                facebookUser: request.facebookUser || Utils.translate("GROUPS.RECEIVED_REQS.NOT_AVAILABLE"),
                numInvitations: request.invitation || 0
              }
            })
          });

          pagination.start += totalRetrieved;
          pagination.increaseCount();

          onLoadingEnd();
        }, function(requestsError) {
          onLoadingEnd();
          console.error(requestsError);
          Utils.alert.show(Utils.translate("GROUPS.RECEIVED_REQS.DIALOG.GET_RECEIVED_REQS_ERR"));
        });
      };
      $scope.loadRequests();

      $scope.loadSpecificRequests = function() {
        $scope.specificSearchOn = true;
        $scope.loadingContent = true;
        GroupsService.getSpecificRequestsToApproveWS({
          groupId: $scope.groupId,
          start: pagination.start,
          count: pagination.count,
          search: $scope.textSaved
        }).$promise.then(function(requestsList) {
          Utils.removeLastItemIfHasNoAttribute(requestsList, 'userSubscription');
          //console.log(requestsList);
          var totalRetrieved = requestsList.length;
          $scope.canGetMoreItems = totalRetrieved >= pagination.count;

          if (!totalRetrieved) {
            onLoadingEnd();
            return;
          }

          requestsList.forEach(function(request) {
            $scope.requestList.push({
              group: {
                id: $scope.groupId,
                name: request.groupName,
                image: request.url
              },
              applicantMetadata: {
                avatar: Utils.getUserImgUrl(request),
                userSubscriptionHref: request.userSubscription.href,
                groupMemberHrefId: Utils.getLastUrlComponent(request.groupMember.href),
                bio: request.userBio,
                name: request.name,
                city: request.residenceCity || Utils.translate("GROUPS.RECEIVED_REQS.NOT_AVAILABLE"),
                facebookUser: request.facebookUser || Utils.translate("GROUPS.RECEIVED_REQS.NOT_AVAILABLE"),
                numInvitations: request.invitation || 0
              }
            })
          });

          pagination.start += totalRetrieved;
          pagination.increaseCount();

          onLoadingEnd();
        }, function(requestsError) {
          onLoadingEnd();
          console.error(requestsError);
          Utils.alert.show(Utils.translate("GROUPS.RECEIVED_REQS.DIALOG.GET_RECEIVED_REQS_ERR"));
        })
      }

      function onLoadingEnd() {
        $scope.isRefreshing = false;
        $scope.isLoadingContent = false;
        $scope.$broadcast('scroll.refreshComplete');
        $scope.$broadcast('scroll.infiniteScrollComplete');
      }

      $scope.refreshRequests = function(x) {
        if (!x) return;

        $scope.canGetMoreItems = true;
        $scope.isRefreshing = true;
        $scope.requestList = [];
        pagination.reset();
        $scope.specificSearchOn = false;
        $scope.loadRequests();
      };

      $scope.confirmRejectRequest = function(listIndex) {
        Utils.confirm.show(
          Utils.translate("GROUPS.RECEIVED_REQS.DIALOG.REJECT_REQ_CONF") +
          " <b>" + $scope.requestList[listIndex].applicantMetadata.name + "</b>?"
        ).then(function(resp) {
          if (!resp) return;

          Utils.loading.show();
          rejectOneRequest($scope.requestList[listIndex].applicantMetadata.groupMemberHrefId).then(function() {
            Utils.toast.success(Utils.translate("GROUPS.RECEIVED_REQS.DIALOG.REJECT_REQ_OK"));
            $scope.requestList.splice(listIndex, 1);
            $rootScope.$broadcast('groups.mine.reload');
          }, function(removeError) {
            console.error(removeError);
            Utils.toast.error(Utils.translate("GROUPS.RECEIVED_REQS.DIALOG.REJECT_REQ_ERR"))
          }).finally(function() {
            Utils.loading.hide();
          })
        })
      };

      $scope.approveRequest = function(listIndex) {
        Utils.loading.show();
        acceptOneRequest($scope.requestList[listIndex].applicantMetadata.groupMemberHrefId).then(function(value) {
          Utils.toast.success(Utils.translate("GROUPS.RECEIVED_REQS.DIALOG.APPROVE_REQ_OK"));
          sendPushAcceptedUser($scope.requestList[listIndex]);
          $scope.requestList.splice(listIndex, 1);
          $rootScope.$broadcast('groups.mine.reload');
        }, function(approveRequestError) {
          console.error("approveRequestError", approveRequestError);
          Utils.toast.error(Utils.translate("GROUPS.RECEIVED_REQS.DIALOG.APPROVE_REQ_ERR"))
        }).finally(function() {
          Utils.loading.hide();
        })
      };

      function sendPushAcceptedUser(userData) {
        // FIRST CHECK STATUS
        var wsData = {
          userSubs: Utils.getLastUrlComponent(userData.applicantMetadata.userSubscriptionHref),
          type: 12 //TYPE 12
        };
        UserService.chkStatusForPush(wsData).$promise.then(
          function(responseChk) {
            console.log("got", responseChk);
            if (responseChk.result.value) {
              UserService.getDevices(
                Utils.getLastUrlComponent(userData.applicantMetadata.userSubscriptionHref)
              ).$promise.then(
                function(userPoints) { //userData.applicantMetadata.name 
                  var message = userData.group.name + Utils.translate("ALERTS.ACCEPTED_INTO_GROUP")
                  userPoints.result.value.forEach(function(endpointArn) {
                    AWSServices.sendNotifAcceptedGroup(
                      message,
                      endpointArn.title,
                      $scope.groupId,
                      userData.group.name
                    )
                  })
                },
                function(errorPoints) {
                  console.error("User Service get Devices", error);
                  reject(error)
                }
              )
            }
          },
          function(errorChk) {
            console.error("User Service get Devices", errorChk);
            reject(error)
          }
        )
        // WITH getSendUserNotification
        // USERSUBSINSTANCE Y TYPE (12)

      }

      /**
       * @param groupMemberHrefId
       * @return Promise
       */
      function acceptOneRequest(groupMemberHrefId) {
        return GroupsService.approveJoinRequest(groupMemberHrefId)
      }

      function acceptAllRequests() {
        Utils.loading.show();

        var resolvedPromises = [];
        /*$scope.requestList.forEach(function (request) {
          resolvedPromises.push(acceptOneRequest(request.applicantMetadata.groupMemberHrefId))
        });*/
        /*Utils.$q.all(resolvedPromises).then(function (responses) {
          $scope.requestList = [];
          $rootScope.$broadcast('groups.mine.reload');
          Utils.toast.success(Utils.translate('GROUPS.RECEIVED_REQS.DIALOG.APPROVE_ALL_REQ_OK'));
          Utils.loading.hide();
        }, function (errors) {
          Utils.loading.hide();
          console.error("acceptAllRequests", errors);
          Utils.toast.error(Utils.translate('GROUPS.RECEIVED_REQS.DIALOG.REJECT_ALL_REQ_ERR'));
        })*/
        GroupsService.approveAllJoinRequests(
          $scope.groupId
        ).$promise.then(function(response) {
          //leave the process to be
          $rootScope.$broadcast('groups.mine.reload');
          $rootScope.$ionicGoBack();
          Utils.toast.success(Utils.translate('GROUPS.RECEIVED_REQS.DIALOG.APPROVE_ALL_REQ_OK'));
          Utils.loading.hide();
        }, function(error) {
          //Catch some errors to know what to do
          if (error && (error.status === 504 || error.status === -1)) {
            console.log("We don't care", error);
            $rootScope.$broadcast('groups.mine.reload');
            $rootScope.$ionicGoBack();
            Utils.toast.success(Utils.translate('GROUPS.RECEIVED_REQS.DIALOG.APPROVE_ALL_REQ_OK'));
            Utils.loading.hide();
          } else {
            console.log("fallo en cliente", error);
            Utils.loading.hide();
            Utils.toast.error(Utils.translate('GROUPS.RECEIVED_REQS.DIALOG.REJECT_ALL_REQ_ERR'));
          }
        })
      }

      /**
       * @param groupMemberHrefId
       * @return {Promise}
       */
      function rejectOneRequest(groupMemberHrefId) {
        return GroupsService.removeMember(groupMemberHrefId)
      }

      function rejectAllRequests() {
        Utils.loading.show();

        var resolvedPromises = [];
        $scope.requestList.forEach(function(request) {
          resolvedPromises.push(rejectOneRequest(request.applicantMetadata.groupMemberHrefId))
        });

        Utils.$q.all(resolvedPromises).then(function(responses) {
          $scope.requestList = [];
          $rootScope.$broadcast('groups.mine.reload');
          Utils.toast.success(Utils.translate('GROUPS.RECEIVED_REQS.DIALOG.REJECT_ALL_REQ_OK'));
          Utils.loading.hide();
        }, function(errors) {
          Utils.loading.hide();
          console.error("rejectAllRequests", errors);
          Utils.toast.error(Utils.translate('GROUPS.RECEIVED_REQS.DIALOG.REJECT_ALL_REQ_ERR'));
        })
      }

      $scope.openBottomSheet = function() {
        Utils.$ionicActionSheet.show({
          buttons: [
            { text: Utils.translate('GROUPS.RECEIVED_REQS.DIALOG.RECEIVED_ACCEPT_ALL') + ' <i class="icon myIcon-line-weight"></i>' }
          ],
          titleText: Utils.translate('SEQUENCE.OPTIONS'),
          buttonClicked: function(buttonIndex) {
            switch (buttonIndex) {
              case 0:
                Utils.confirm.show(
                  Utils.translate('GROUPS.RECEIVED_REQS.DIALOG.ACCEPT_ALL_REQ_CONF')
                ).then(function(boolResp) {
                  if (!boolResp) return;
                  acceptAllRequests();
                });
                break;
              default:
                Utils.toast.warning("invalid option");
            }
            return true;
          },
          destructiveText: $scope.$_canAcceptJoinRequests() ? Utils.translate('GROUPS.RECEIVED_REQS.DIALOG.RECEIVED_REJECT_ALL') + ' <i class="icon ion-close-round"></i>' : null,
          destructiveButtonClicked: $scope.$_canAcceptJoinRequests() ? function() {
            Utils.confirm.show(
              Utils.translate('GROUPS.RECEIVED_REQS.DIALOG.REJECT_ALL_REQ_CONF')
            ).then(function(boolResp) {
              if (!boolResp) return;
              rejectAllRequests()
            });
            return true;
          } : null
        })
      }

      $scope.showInvitations = function(ev, request) {
        console.log(request)
        if (!request.applicantMetadata.numInvitations) return;

        Utils.loading.show();
        GroupsService.findUsersWhoInvitedWS(
          request.applicantMetadata.userSubscriptionHref,
          request.group.id
        ).$promise.then(function(resp) {
          Utils.removeLastItemIfHasNoAttribute(resp, "userSubscription");
          console.log(resp);
          Utils.modal.show({
            templateUrl: 'templates/groups/invitationsList-modal.html',
            parent: angular.element(document.body),
            //targetEvent: ev,
            clickOutsideToClose: false,
            fullscreen: true, // Only for -xs, -sm breakpoints.
            locals: { usersList: resp },
            controller: function($scope, $mdDialog, locals) {
              $scope.usersList = { /*26:{name, avatar, etc}, etc...*/ };
              var usrHrefIdsList = { /*26:0, 55:4, etc...*/ };
              locals.usersList.forEach(function(user) {
                var uid = Utils.getLastUrlComponent(user.userSubscription.href);
                if (usrHrefIdsList.hasOwnProperty(uid))
                  usrHrefIdsList[uid]++;
                else
                  usrHrefIdsList[uid] = 1;

                if (usrHrefIdsList[uid] === 1) {
                  $scope.usersList[uid] = {
                    name: user.name,
                    avatar: Utils.getUserImgUrl(user),
                    subscription: user.userSubscription.href,
                    numSentInvitations: usrHrefIdsList[uid]
                  }
                } else {
                  $scope.usersList[uid].numSentInvitations = usrHrefIdsList[uid];
                }
              });

              $scope.hide = function() {
                $mdDialog.hide();
              };

              $scope.cancel = function() {
                $mdDialog.cancel();
              };
            }
          }).then(function(answer) {
            console.log(answer);
          });
        }, function(error) {
          console.error("cant find Users Who Invited", error);
          Utils.toast.error(Utils.translate("ERROR.UNKNOWN"))
        }).finally(function() {
          Utils.loading.hide();
        })
      };

      $scope.specificSearch = function(textForSearch) {
        $scope.canGetMoreItems = true;
        $scope.isRefreshing = true;
        $scope.requestList = [];
        pagination.reset();
        if (textForSearch === null || textForSearch === "" || textForSearch === undefined) {
          $scope.textSaved = "";
          $scope.specificSearchOn = false;
          $scope.loadRequests();
        } else {
          $scope.textSaved = textForSearch;
          $scope.specificSearchOn = true;
          $scope.loadSpecificRequests();
        }
      }

      $scope.differTypeSearchs = function() {
        if ($scope.specificSearchOn) {
          $scope.loadSpecificRequests();
        } else {
          $scope.loadRequests();
        }
      }

      $scope.sendFriendProfile = function(userInstanceId) {
        if (!userInstanceId) return;
        var idAlone = Utils.getLastUrlComponent(userInstanceId)
        if (idAlone === +UserService.get_USERSUBSCRIPTIONINSTANCE_ID())
          $state.go('menu.myProfile', {}, { reload: true });
        else
          $state.go('friendProfile', { instanceId: idAlone }, { reload: true });
      };

      //---------------- PERMISSIONS ------------------
      $scope.$_canAcceptJoinRequests = function() {
        return UserService.USER_IS_SUPERADMIN() || permissionValidationService.canAcceptJoinRequests();
      };

      $scope.$_canRejectJoinRequests = function() {
        return UserService.USER_IS_SUPERADMIN() || permissionValidationService.canRejectJoinRequests();
      };
    }
  ])

  .controller("GroupSearchUsersCtrl", ["$scope", 'locals', 'UserService', 'GroupsService', 'TagsService', 'Utils', 'searchFactory',
    function($scope, locals, UserService, GroupsService, TagsService, Utils, searchFactory) {
      var lastSearch, pendingSearch;

      var myUserHref = UserService.get_USER_INSTANCE_HREF();

      //makes sure that is an array instance
      locals.previousUsers = locals.previousUsers || [];
      locals.fixedUsers = locals.fixedUsers || [];
      locals.bannedList = locals.bannedList || [];
      console.log("locals.bannedList", locals.bannedList);

      /**
       * @type {[{
       *  id:string,
       *  href:string
       *  name:string,
       *  photo:string
       * }]}
       */
      $scope.usersFound = [];

      $scope.data = {
        //userName: "",
        searchTerm: "",
        selectedUsers: [],
        showSelectedContactsList: false
      };

      //because the previous list is passed by reference and if users are removed
      //and click on cancel the modal, the original selected users list would be
      //modified as well
      locals.previousUsers.forEach(function(user) {
        $scope.data.selectedUsers.push(user);
      });

      $scope.isSearching = false;
      $scope.searchEnded = false;

      var pagination = {
        start: 0,
        count: 25
      };

      $scope.beginSearch = function() {
        $scope.usersFound = [];
        if (!$scope.data.searchTerm || !debounceSearch()) return;
        $scope.isSearching = true;
        $scope.searchEnded = false;

        searchFactory.madeMinSearch(
          $scope.data.searchTerm,
          pagination.start,
          pagination.count
        ).then(function(UserResults) {
          console.log("us", UserResults);
          $scope.usersFound = UserResults;
          $scope.data.showSelectedContactsList = false;
          refreshDebounce();
        }, function(error) {
          console.error("couldn't get user list", error);
          $scope.errorLoading = true;
        }).finally(function() {
          $scope.isSearching = false;
          $scope.searchEnded = true;
        })
      }

      $scope.searchUsers = function() {
        $scope.usersFound = [];
        if (!$scope.data.userName || !debounceSearch()) return;
        $scope.isSearching = true;
        $scope.searchEnded = false;

        //console.log("Searching for " + $scope.data.userName);
        UserService.searchUsersByPartialNameOrIncludingSpaces({
          term: $scope.data.userName,
          start: pagination.start,
          count: pagination.count
        }).then(
          function(UserResults) {
            //console.log("UserResults", UserResults);
            $scope.usersFound = UserResults;
            $scope.data.showSelectedContactsList = false;
            refreshDebounce();
          },
          function(error) {
            console.error("couldn't get user list", error);
            $scope.errorLoading = true;
          }).finally(function() {
          $scope.isSearching = false;
          $scope.searchEnded = true;
        });
      };

      /**
       * @param user {{
       *  id:string,
       *  href:string
       *  name:string,
       *  photo:string
       * }}
       * @param listIndex
       */
      $scope.selectUser = function(user, listIndex) {
        var itsMe = user.href === myUserHref;

        //only non existing users will be added
        var isInList = $scope.data.selectedUsers.some(function(prevUser) {
          return prevUser.href === user.href;
        });

        var isInGroup = locals.fixedUsers.some(function(fixedUser) {
          return fixedUser.userSubscriptionHref === user.href;
        });

        var isBannedFromGroup = GroupsService.checkIfIsBannedOrLockedUser(locals.bannedList, user.href);

        if (!itsMe && !isInList && !isInGroup && !isBannedFromGroup) {
          $scope.data.selectedUsers.unshift(user); // = $scope.data.selectedUsers.concat(user);
          $scope.usersFound.splice(listIndex, 1);
        } else if (itsMe)
          Utils.toast.info(Utils.translate("GROUPS.MEMBERS_MGMT.DIALOG.YOULL_BE_THE_ADMIN"));
        else if (isInList)
          Utils.toast.info(Utils.translate("GROUPS.MEMBERS_MGMT.DIALOG.USER_ALREADY_IN_LIST"));
        else if (isBannedFromGroup)
          Utils.toast.warning(Utils.translate('GROUPS.MEMBERS_MGMT.DIALOG.USER_BANNED_FROM_GROUP'));
        else if (isInGroup)
          Utils.toast.info(Utils.translate('GROUPS.MEMBERS_MGMT.DIALOG.USER_ALREADY_IN_GROUP'));

        if (!$scope.usersFound.length) {
          //$scope.data.userName = "";
          $scope.data.searchTerm = "";
          focusSearchInput();
        }
      };

      $scope.unselectUser = function(user, listIndex) {
        $scope.data.selectedUsers.splice(listIndex, 1);

        if (!$scope.data.selectedUsers.length)
          $scope.hideSelectedUsersList();

      };

      $scope.isInListOrGroup = function(userHref) {
        var isInList = $scope.data.selectedUsers.some(function(user) {
          return userHref === user.href;
        }) || userHref === myUserHref;

        var isInGroup = locals.fixedUsers.some(function(user) {
          return userHref === user.userSubscriptionHref;
        });

        return isInList || isInGroup;
      };

      $scope.isUserBlocked = function(userHref) {
        return GroupsService.checkIfIsBannedOrLockedUser(locals.bannedList, userHref);
      };

      $scope.showSelectedUsersList = function() {
        $scope.data.showSelectedContactsList = $scope.data.selectedUsers.length > 0;
      };

      $scope.hideSelectedUsersList = function() {
        $scope.data.showSelectedContactsList = false;
        focusSearchInput();
      };

      $scope.okClicked = function() {
        locals.dialog.hide($scope.data.selectedUsers);
      };

      $scope.cancelClicked = function() {
        if ($scope.data.showSelectedContactsList)
          $scope.hideSelectedUsersList();
        else
          locals.dialog.cancel();
      };

      function debounceSearch() {
        var now = new Date().getMilliseconds();
        lastSearch = lastSearch || now;
        return ((now - lastSearch) < 300);
      }

      function refreshDebounce() {
        lastSearch = 0;
        pendingSearch = null;
        //cancelSearch  = angular.noop;
      }

      var searchInput;

      function focusSearchInput() {
        setTimeout(function() {
          if (!searchInput)
            searchInput = document.getElementById('search-input');

          if (searchInput) {
            //searchInput = searchInput[searchInput.length - 1]; searchTerm
            /*searchInput.selectionStart = $scope.data.userName.length;
            searchInput.selectionEnd = $scope.data.userName.length;*/
            searchInput.selectionStart = $scope.data.searchTerm.length;
            searchInput.selectionEnd = $scope.data.searchTerm.length;
            searchInput.focus();
          } else
            console.log("can't find search input")
        }, 900);
      }

      focusSearchInput();

      var deregisterHardBack = Utils.$ionicPlatform.registerBackButtonAction(
        $scope.cancelClicked, 101
      );

      // cancel custom back behaviour
      $scope.$on('$destroy', function() {
        deregisterHardBack();
      });

      $scope.getCounter = function() {
        var total = $scope.data.selectedUsers.length;
        return total < 100 ? total : '+99';
      }
    }
  ])

  .controller('sendPushGroupMembersCtrl', ['$scope', 'ApiEndPoint', '$state', 'reDirectService', 'Utils', 'locals', '$rootScope', 'userAttendanceService', 'UserService', 'AWSServices', 'GroupsService',
    function($scope, ApiEndPoint, $state, reDirectService, Utils, locals, $rootScope, userAttendanceService, UserService, AWSServices, GroupsService) {
      $scope.nameGroup = locals.groupName;
      $scope.idGroup = locals.idForUse;
      $scope.textForPush = "";
      $scope.savingTime = false;
      var toastCtrl = Utils.toast.getCtrl();
      var newNotifToast;

      $scope.closeDialog = function() {
        locals.dialog.hide();
      };


      $scope.sendPush = function() {
        if ($scope.textForPush === "") {
          return true;
        } else {
          $scope.savingTime = true;
          newNotifToast = toastCtrl.info(Utils.translate('Sending'), 0, 'top');
          GroupsService.getAllDevicesFromGroup(
            $scope.idGroup
          ).$promise.then(function(response) {
            console.log("response", response);
            response.forEach(function(endpointArn) {
              AWSServices.sendNotificationGroupMembers(
                $scope.textForPush,
                $scope.nameGroup,
                endpointArn.userDeviceId,
                true,
                'GROUP_PUSH',
                UserService.get_USERSUBSCRIPTIONINSTANCE_ID()
              );
            })
            $scope.savingTime = false;
            toastCtrl.clear(newNotifToast);
            $scope.closeDialog();
            Utils.toast.success(Utils.translate('QRSCANPAGE.ATTENDANT_INFO.SENT_PUSH'));
          }, function(error) {
            toastCtrl.clear(newNotifToast);
            console.error("Cant get the devices", error);
          });
        }
      }
    }
  ]);
